#!/bin/bash

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

#
#  Isaias Sanchez -Cortina.
#

# DEFAULTS
# help
HELP=" [options] <inputdir> <prjdir>                    
  <inputdir> The directory with the input data   
  <prjdir> The project dir which will be created 
                                                 
  Options:                                       
                                                 
    -h: Shows this help                          
    -q: quiet                                    
"
# 

export CONTINUE=${CONTINUE:-1}

export INPUTFORMAT=${INPUTFORMAT:-'trs'}
export PARSE_TR_FORMAT=${PARSE_TR_FORMAT:--HTMLescapes}
export PARSE_LM_FORMAT=${PARSE_LM_FORMAT:- --Regex ortographic --SIL remove --CurlyTokens remove --SpellID remove --Apostrophe destokenize +dash}
export PARSE_AM_FORMAT=${PARSE_AM_FORMAT:- --Regex acoustic --SIL keep --CurlyTokens remove --SpellID keep --Apostrophe destoken +dash}
export NoLongSilence=${NoLongSilence:-1}
export CORPUS_NAME=${CORPUS_NAME:-`basename "$INPUT"`}


# Load helper functions
source "$TLUTILS"

# Parse and check the remaining command line arguments :
if [ ! $# -eq 2 ] ; then 
    help "$HELP" && error "SYNTAX ERROR: wrong number of arguments: $@."
fi
export INPUT="$1" ; [ -d "$INPUT" ] || error $"Could not find input directory '$INPUT'"
export PRJD="$2" 
export TLLANG=${!3:-ca}

# Guess 
require "$TL_PREPROCESS_LANG/parse.perl" "$TL_PREPROCESS_LANG/transcriure.sh" "$TL_PREPROCESS_COMMON/extract_samples__$INPUTFORMAT.sh" "$TL_PREPROCESS_COMMON/clustering-heuristics.perl"

# Varariables to be saved (All  vars will be saved)
export G2P_model=${G2P_model:-$TL_PREPROCESS_LANG/train+dev+cmu6.dic-9.g2p-model}
export train_DICT=${train_DICT:-"$TL_PREPROCESS_LANG/train+dev+cmu6.dic-numbered"}
## OPCIONS PER AL SEGRE PER DEFECTE, ES POT GASTAR PER A PASSAR OPCIONS EXTRES DES DE FORA
export PARSE_PHO_FORMAT=${PARSE_PHO_FORMAT:-}
export SPH2PIPE=${SPH2PIPE:-$TL_PREPROCESS_EXTERNAL/sph2pipe/bin/sph2pipe}

export LISTS="$PRJD/lists"
export train_tLfea_LIST="$LISTS/samples.lst"

#Clustering
export CLUSTERS_table=${CLUSTERS_table:-$LISTS/samples.clustering}
export CLUSTERS_list=${CLUSTERS_list:-$LISTS/samples-clusters.lst}
export CLUSTERS_SAMPLES_list=${CLUSTERS_SAMPLES_list:-$LISTS/samples-clusters-samples.lst}
export CLUSTERS_heuristic=${CLUSTERS_heuristic:-byDir}
export CLUSTERS_heuristic_select=${CLUSTERS_heuristic_select:-MEDIA_NAME SPEAKER}
export CLUSTERS_PATH=${CLUSTERS_PATH:-$LISTS/clusters}



# Delete previous runs unless CONTINUE flag is set
if [ "$CONTINUE" -eq 0 ] ; then  
    rm -rf "$LISTS" 2>/dev/null   
fi


# Generate directories
mkdir -p "$PRJD" "$PRJD/samples" "$LISTS" 
[ -d "$PRJD" ]  || error $"Could not find input directory '$PRJD'"

set -u


#####################################
# Extract samples and create speaker lists
if [ "$CONTINUE" -gt 0   ]  && [ -s "$train_tLfea_LIST" ]  ; then 
    log $"Found list of features '$train_tLfea_LIST' : Skipping the extraction of audio segments and the compututation of features  from '$INPUT' "
else 
    log $"Extracting samples from audio files"
    debug "$TL_PREPROCESS_COMMON/extract_samples__$INPUTFORMAT.sh" "$INPUT" "$PRJD" \> "$train_tLfea_LIST"
     "$TL_PREPROCESS_COMMON/extract_samples__$INPUTFORMAT.sh" "$INPUT" "$PRJD" > "$train_tLfea_LIST"
fi




#####################################
# Parse transcriptions to adapt them for AM an LM training.
Nsamples=`wc -l < "$train_tLfea_LIST"`
log "Parsing $Nsamples .tr files listed in '$train_tLfea_LIST'"
myn=0
mytenpercent=$((Nsamples/10))
[ $mytenpercent -eq 0 ] && mytenpercent=1
mytens=0
myskipped=0
train_tLfea_LIST_to_be_transliterated="$train_tLfea_LIST.to_be_transliterated"
> "$train_tLfea_LIST_to_be_transliterated"
while read i ; do
        file="$PRJD/${i%%.tLfea}"
        myn=$((myn+1))
        if [ "$CONTINUE" -eq 1 ] ; then 
           if [ -s "$file.am.txt" ] && [ -s "$file.lm.txt" ] ; then
                myskipped=$((myskipped+1))
                if [ ! -s "$file.pho" ] ; then
                    echo "$i" >> "$train_tLfea_LIST_to_be_transliterated"
                fi
                continue
           fi
         
        fi 
         if [ ! $(( myn % $mytenpercent )) ] ; then 
           mytens=$((mytens+1))
           log_ne "$((mytens*10))% of $Nsamples files"
         fi
        if [ "$PARSE_TR_FORMAT" == '-HTMLescapes' ]  ;  then
            perl -i -npe 's/(\&(amp;)?quot;)//g' "$file.tr"
        fi
        debug "# Text for the Acoustic Model : $TL_PREPROCESS_LANG/parse.perl  $PARSE_AM_FORMAT   < $file.tr > $file.am.txt"
        "$TL_PREPROCESS_LANG/parse.perl"  $PARSE_AM_FORMAT   < "$file.tr" > "$file.am.txt"
        debug "# Text for the Language Model : $TL_PREPROCESS_LANG/parse.perl  $PARSE_LM_FORMAT < $file.tr > $file.lm.txt "
        "$TL_PREPROCESS_LANG/parse.perl"  $PARSE_LM_FORMAT < "$file.tr" > "$file.lm.txt" # Text for the Language Model :
        echo "$i" >> "$train_tLfea_LIST_to_be_transliterated"
 done < "$train_tLfea_LIST"
 log $"Parsed $myn of $Nsamples files (skipped $myskipped)"
 
 if [ "$CONTINUE" -eq 1 ] && [ `wc -l < "$train_tLfea_LIST_to_be_transliterated"` -eq "0" ] ; then
    log $"Transliteration was already performed. To force re-calculation, erase the corresponding .pho files or use -f"
    debug "Erase the corresponding .pho files; use -f ; Or run the followinf command line : 
        \e[34m$TL_PREPROCESS_LANG/transcriure.sh  $train_tLfea_LIST $PRJD  $PRJD  $PARSE_PHO_FORMAT >  $PRJD/monophonemes.list\e[0m    "
 else 
    log $"Transliterating `wc -l <"$train_tLfea_LIST_to_be_transliterated" ` " samples of $Nsamples""
    
     "$TL_PREPROCESS_LANG/transcriure.sh" \
         "$train_tLfea_LIST_to_be_transliterated" "$PRJD"  \
         "$PRJD" $PARSE_PHO_FORMAT > "$PRJD/monophonemes.list"
fi
        

## Remove samples with no transliteration from the train list:
log $"Removing samples with empty (SP) transliteration or no .pho"
perl -i -ne '
    $pho="$ENV{PRJD}/$_"; chomp $pho ; $pho=~ s/\.tLfea$//;  $pho.=".pho"; 
    unless (-s $pho){
        print STDERR $_;
        next;
    }
    open(A,$pho);
    my $skip=1; 
    while (my $t=<A>){
        for $w (split(" " , $t)) {
            unless ($w =~ /^SP$/i) {
                $skip=0; 
                last;
            }
        }
        last unless $skip; 
    }
    close(A);
    if ($skip){
        print STDERR $_;
        next;
    }
    print $_;
    ' $train_tLfea_LIST  2> $train_tLfea_LIST.removed ; 
Nsamples=`wc -l < "$train_tLfea_LIST"`
log $"Train list now has $Nsamples samples"
rm "$train_tLfea_LIST_to_be_transliterated"
#####################################
#Generate phonemes list:
log $"Generating phonemes list: $LISTS/samples-uniq-monophos.lst"
gawk -v dir=$PRJD '{
f= dir"/"$0
while ( getline < f )
  for(i=1;i<=NF;++i) syms[$i]= 1
close(f)
}
END {for(s in syms)print s|"sort"}' "$PRJD/monophonemes.list" > "$LISTS/samples-uniq-monophos.lst"

CONTINUE=0
#####################################
## cluster media
if  [ "$CONTINUE" -gt 0 ] && [ -s "$CLUSTERS_table"  ]  ; then 
    log $"Found cluster table '$CLUSTERS_table' : Skipping clustering step"
else 
    log $"Clustering media files '$CLUSTERS_heuristic' to '$CLUSTERS_PATH' "
    debug "\e[34m$TL_PREPROCESS_COMMON/clustering-heuristics.perl $train_tLfea_LIST \
`[ $DEBUG -eq 1 ] && echo -v` \
-H  $CLUSTERS_heuristic  \
--outlist $CLUSTERS_list --clusters  $CLUSTERS_PATH \
--Prepend "$PRJD/" --select $CLUSTERS_heuristic_select -- > $CLUSTERS_table\e[0m" ;
    "$TL_PREPROCESS_COMMON/clustering-heuristics.perl" "$train_tLfea_LIST" \
        -H  "$CLUSTERS_heuristic"   \
      `[ $DEBUG -eq 1 ] && echo -v` \
        --out-clusters-samples-list $CLUSTERS_SAMPLES_list \
        --outlist "$CLUSTERS_list" --clusters  "$CLUSTERS_PATH" \
        --Prepend "$PRJD/" --select $CLUSTERS_heuristic_select -- > "$CLUSTERS_table" ; # Output
fi

#####################################
# Save Configuration
VARSmandatory="TLLANG INPUTFORMAT INPUT PRJD"
SaveConf "$PRJD/config"
log $"Preprocessing done"
