#!/usr/bin/perl -w

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

#Load libraries
# Locate Library Main dir:

use strict 'vars';

our $DEBUG=0; 

unless (@ARGV){
   help ("Splits lines in a text file into separate files (blocks)",
   "$0 <Number of blocks> <FILE> [options]","
Mandatory:
          Number of Blocks: Integer
          FILE: Text file to split (default: read from standard input)
                   ( use '-' to read from standard input )
Options
   -quote 
   -stdout
   -stdoutquiet
   --out   <Path/Basename>   :
   --parts <List of integers>:
   "); 
   die ("Syntax ERROR. Number of blocks is mandatory")
}

my $bQUOTE=0; 
my $nBlocks=shift; 
$nBlocks =~ /^\d+$/ || die ( "First argument must be an integer, the number of parts, instead of $nBlocks ");
my @Parts=(1..$nBlocks);
my $sFile='-'; 
my $bToSTDOUT=0;
my $BASE_OUT; 
my $bSTDOUTQUIET=0;
while ($_=shift) {
      /^-quote$/ && ($bQUOTE=1,next); 
      /^-stdout$/ && ($bToSTDOUT=1,next);
      /^-stdoutquiet$/ && ($bToSTDOUT=1, $bSTDOUTQUIET=1 , next);  
      /^--out$/ && ( $BASE_OUT=shift,next);
      /^--parts$/ && ( @Parts=@ARGV , last);  
      $sFile=$_;
}
$BASE_OUT=($sFile eq '-')?'stdin':$sFile unless defined($BASE_OUT); 




# Read input file                                                               
my $f=myopen($sFile);                      # Open and return pointer to file    
my @data=grep {! /^\s*$/} (<$f>);          # Get non-empty lines from file      

if ($bQUOTE) {
   for (0..$#data) {
      chomp $data[$_];
      $data[$_]="\"$data[$_]\"\n";
   }    
} else  {
   chomp $data[$#data];  $data[$#data].="\n"; # Ensure last line ends with new line
}
close($f) unless ($f eq \*STDIN);          # Close file                         

# Share Samples                                                                 
#   There will be $nBlocks                                                       
# a)The first $nRemaingSamples will have $nMinSamplesPerBlock + 1 samples       
# b)The rest just only $nRemaingSamples                                         
my $n=scalar(@data); 
my $nMinSamplesPerBlock=int($n/$nBlocks);
my $nSamples=$nMinSamplesPerBlock+1;
my $nRemaningSamples=$n-($nBlocks*$nMinSamplesPerBlock); 
my $nNextSample=$nRemaningSamples*$nSamples;



for (@Parts) {
   my $nStart;
   my $nEnd;
   if ($_ <= $nRemaningSamples)  { # a) Print the firsts $nRemaingSamples samples                                  
      $nStart=($_-1)*$nSamples;
      $nEnd=($_)*$nSamples-1;
   } else {  # b) Print remaining samples                                       
      $nStart=($_-$nRemaningSamples-1)*$nMinSamplesPerBlock+$nNextSample;
      $nEnd=($_-$nRemaningSamples)*$nMinSamplesPerBlock-1+$nNextSample;
   }
   my $fOUT=undef;
   if ($bToSTDOUT) {
      $fOUT=\*STDOUT;
      print $fOUT "# $_ of $nBlocks (Samples in [".($nStart+1)."..".($nEnd+1)."])  \n" unless $bSTDOUTQUIET;
   } else {
      open ($fOUT, ">", "$BASE_OUT.part${_}of$nBlocks");
   }
   print $fOUT @data[$nStart..$nEnd]; 
   close($fOUT) unless ($fOUT eq \*STDOUT); 
}


sub myopen {
   my $STR_FILE_Reference=shift;
   my $fail=1;  $fail=shift if @ARGV; 
   my $FILE_Reference;
   if ($STR_FILE_Reference eq '-'){
      $FILE_Reference=\*STDIN;
      $STR_FILE_Reference="(from standar input)";
   } else {
      if ($STR_FILE_Reference =~ /\n$/)  {
         unless (open ($FILE_Reference,$STR_FILE_Reference)) {
            chomp $STR_FILE_Reference;
            open ($FILE_Reference,$STR_FILE_Reference) or do {
               die("File '$STR_FILE_Reference', ending or not with newline character, could not be opened") if $fail;
               print STDERR "[WW] File '$STR_FILE_Reference', ending or not with newline character, could not be opened\n" ;
               return undef; 
            }
         }
      } else {
         open ($FILE_Reference,$STR_FILE_Reference) or do {
               die("File '$STR_FILE_Reference' could not be opened") if $fail;
               print STDERR "[WW] File '$STR_FILE_Reference' could not be opened\n";
               return undef; 
            }
      }
   }
   return $FILE_Reference;
}
