#!/bin/bash

#
#  Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# Defaults
CONTINUE=${CONTINUE:-1}
DEBUG=${DEBUG:-0}
VERBOSE=${VERBOSE:-1}
TMP=""
export TLUTILS_PATH=`dirname "${BASH_SOURCE[0]}"`
export TLK_PATH
export TLPATH

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

# declare -a conf_KNOWNPARAMETERS
# declare -a conf_KNOWNPARAMETERS_HELP
##########################
# FORMATER VARIABLES
PRGNAME=${PRGNAME:-`basename "$0"`}
REALPRGNAME=`basename "$0"`
PRGDIR=`dirname "$0"`
HELP=${HELP:-$PRGNAME
SYNTAX:
$NAME_PROGRAM <arguments use the common Utils.bash syntax>

MANDATORY ARGUMENTS:     $VARSmandatory
OPTIONS:                 $VARSoptionaly}
export DATE=`date +%Y%m%d.%H%M%S`
export uniqID="$DATE.$RANDOM"
FORMAT_HIGHLIGHT=`echo -e "\e[41;37m"`
FORMAT_DEFAULT=`echo -e "\e[0m"`

#########################
#
# function Add_KNOWNPARAMETERS {
#     conf_KNOWNPARAMETERS+=( "$1" );
#     shift;
#     conf_KNOWNPARAMETERS_HELP+=( "$@" );
# }

#########################
# Messages 
function log {
    (( "$VERBOSE" )) && echo -e "[II] $PRGNAME : $@" >&2 || return 0 
}
function log_ne {
    (( "$VERBOSE" )) && echo -ne "[II] $PRGNAME : $@\r" >&2  || return 0 
}
function success {
    (( "$VERBOSE" )) && echo -e "[II] $PRGNAME : OK! $@" >&2 || return 0 
}
function debug {
    (( "$DEBUG" ))  &&  echo -e "[GG] $REALPRGNAME:`caller 0` $@" >&2 || return 0 
}
function _debug {
    (( "$DEBUG" ))  &&  echo -e "[GG] $REALPRGNAME:`caller 1` $@" >&2 || return 0 
}
function warning {
    if (( "$DEBUG" )) ; then
        echo -e "[WW] $REALPRGNAME:`caller 0` $@" >&2    
    else
        echo -e "[WW] $REALPRGNAME: $@" >&2
    fi
}
function error {
    echo -e "[EE] $PRGNAME:`caller 0` $@" >&2
    exit 1
}
function _error {
    echo -e "[EE] $REALPRGNAME:`caller 1` $@" >&2
    exit 1
}
function help {
    local myhelp=${!1:-$HELP}
#   echo -e "\n---              Help  for $PRGNAME              ---\n" 
    echo "$myhelp" >&2
#   echo "PARAMETERS:" >&2
#   local tmpvar
#     for (( tmpvar=1 ; tmpvar<=${#conf_KNOWNPARAMETERS[*]} ; tmpvar++ )) ; do 
#         echo -e "\e[41;30m${#conf_KNOWNPARAMETERS[$tmpvar]}\e[0m: ${#conf_KNOWNPARAMETERS_HELP[$tmpvar]}" 
#     done
#    cat<<EOS>&2
#                                                                               
#COMMON OPTIONS                                                                 
#-h                : Show this help                                             
#                                                                               
#-q or -VERBOSE    : Quiet (Do not print [II] messages)          (def. +VERBOSE)
#-g or -DEBUG      : Debug (Print [GG] messages).                  (def. -DEBUG)
#-f or -CONTINUE   : Do not re-use previous generated files.    (def. +CONTINUE)
#--conf_file=<file>: Add options found in <file>. Options are declared as bash  
#                    variables, for instance: CONTINUE=1                        
#                                                                               
#GENERAL SYNTAX FOR OPTIONS                                                     
#-OPTION           : Disables OPTION.        (internally sets variable OPTION=0)
#+OPTION           : Enables  OPTION.        (internally sets variable OPTION=1)
#--OPTION=<VALUE>  : Sets variable OPTION to <VALUE>                            
#                                                                               
#TLK BINARIES                                                                   
    cat<<EOS>&2
Other options:
  -h                Show this help                                             
  -q or -VERBOSE    Quiet (Do not print [II] messages)
  -g or -DEBUG      Debug (Print [GG] messages)
  -f or -CONTINUE   Do not re-use previous generated files
  --TLPATH=<bin/>   Path to tLK binaries. (def. they are expected in the PATH)
EOS

}
function end {
    local status=$1
    shift
    (( "$status" == "0" )) || echo -e "$PRGNAME: [EE] EXIT $status" >&2
}


#########################
# Remote?
function PathIsLocal {
  if [[ `stat -f -L -c %T  "$1" ` =~ ^nfs ]]  ; then
    return 0
  else
    return 1
  fi
  
}
#########################
# Variables handling
function init_var {
    if  [ $# -lt 1  ]  ; then 
        return 
    fi
    local var
    for var in "$@" ; do
      eval $var='"'${!var:-''}'"'
    done
    _debug "Initialized variables to empty : $@"
}
#########################
# requirements
function require {
    local prog
    for prog in "$@" ; do 
        [ -x "$prog" ] || _error "Missing utility $prog"
    done
}

function require_var {
    # Useful only no set -u
    local var
    for var in "$@" ; do 
        local tmp=${!var}
        [ -z "$tmp" ] && _error "Missing parameter --$var=... "
        _debug "$var is set to $tmp"

    done
}
function require_file {
    local file
    for file in "$@" ; do 
        [ -s "$file" ] || _error "Missing or empty required file : '$file' "
    done
}
function require_var_is_nonemptyfile {
    local var
    for var in "$@" ; do 
        local tmp=`eval echo -n \\$$var`
        [ -z "$tmp" ] &&  _error "Missing parameter --$var=<file> "
        [ -s "$tmp" ] ||  _error "--$var='$tmp' : File is missing or empty. "
        _debug "$var is set to '$tmp'"
    done
}

#########################
# Manage Configuration
function SaveConf {
    set +u
    local myvar;
    if [ $# -eq 0 ] ; then 
        for myvar in $VARSmandatory $VARSoptional; do 
            echo "$myvar='"`eval echo "\\$$myvar"`"'"
        done
    else 
        for myvar in $VARSmandatory $VARSoptional ; do
            echo "$myvar='"`eval echo "\\$$myvar"`"'"
        done > "$1"
    fi
    # NOTE: compgen -A variable | grep ^conf_ | while ... gets all vars beggining with conf_
}
function AddConf {
    local myvar;
    compgen -A variable | grep ^conf_ | while read myvar ; do
        echo "$myvar='"`eval echo "\\$$myvar"`"'"
    done >> "$1"
}


#########################
# Directories
function MakeDirs {
    local Dir
    for Dir in "$@" ; do 
        mkdir -p "$Dir" 2>/dev/null || _error "<(line `caller 0`) Could not create directory '$Dir'"
        _debug  Made dir $Dir
    done
}
function MakeTMP  {
    # Make a temporary directory
    TMP="`mktemp -d -t tmp.XXXXX`"
    if [ -d "$TMP" ] ; then
        _debug "Temporary directory set to '$TMP'"
        return
    fi
    TMP="/tmp/tmp.$USER.$PRGNAME.$uniqID"
    mkdir -p "$TMP" 2>/dev/null 
    if [ -d "$TMP" ] ; then
        _debug "Temporary directory set to '$TMP'"
        return
    fi
    TMP="$PWD/tmp.$USER.$PRGNAME.$uniqID"
    TMP=`$READLINK "$TMP"`
    if [ -d "$TMP" ] ; then
        _debug "Temporary directory set to '$TMP'"
        return
    fi
    _error "Could not create a temporary directory at '$TMP' , neither in /tmp/ nor /home/tmp"

}
function DelTMP  {
    # delete  temporary directory
    set +e
    local myTMP=${!1:-$TMP}
    rm -rf "$myTMP"/ 2>/dev/null
    _debug "<(line `caller 0`) Deleted temporary directory '$myTMP'"
}

function DelTMPunlessDebug  {
    # Make a temporary directory
    if [ -z "$TMP" ] ; then
      _debug "<(line `caller 0`) \e[41;37m[WW]\e[0m No temporary directory has been set before"
      return
    fi
    set +e
    local myTMP=${!1:-$TMP}
    if [ "$DEBUG" -gt 0 ] ; then 
        _debug "<(line `caller 0`) temporary directory '$myTMP' has not been deleted."
    else 
        rm -rf "$myTMP"/ 2>/dev/null
        _debug "<(line `caller 0`) Deleted temporary directory '$myTMP' ."
    fi
}

###########################
# Execution
export SUBMIT=${SUBMIT:-"$TLUTILS_PATH/submit.sh"}
export QSUBMIT="$TLUTILS_PATH/submit.sh"
export PARALLEL_MAXPROC=${PARALLEL_MAXPROC:--1}
if [ $PARALLEL_MAXPROC ==  '-1' ] ; then
    export PARALLEL_MAXPROC=`perl -e 'my $cpu=2;  if (open(C,"/proc/cpuinfo")) {  $cpu=scalar(grep {/^processor/} <C>); $cpu=($cpu>0)?$cpu:2      } ;     print STDOUT $cpu ' `
fi
parallel () {
    awk "BEGIN{print \"all: ALL_TARGETS\\n\"}{print \"TARGET_\"NR\":\\n\\t@-\"\$0\"\\n\"}END{printf \"ALL_TARGETS:\";for(i=1;i<=NR;i++){printf \" TARGET_%d\",i};print\"\\n\"}" | tee /dev/stderr | make -j $@ -f - all
}

#################################################################################### 
# PARSE arguments
function ParseArguments {
    ParseArguments_shift_n=0
    _debug "Parsing arguments: $@"
    local tmpvar;
    local ARG_last;
    local ARG_next;
    for ((tmpvar=1;tmpvar<=$#;tmpvar++)) ; do
        case "${!tmpvar}" in
            -h)  help "$HELP"; let ++ParseArguments_shift_n   ; exit 0;; 
            -q)  export VERBOSE=0;    let ++ParseArguments_shift_n   ;; 
            -g)  export DEBUG=1;      let ++ParseArguments_shift_n   ;; 
            -f)  export CONTINUE=0;   let ++ParseArguments_shift_n   ;; 
            --)  let ++ParseArguments_shift_n ; break ;;
            --*=*) ARG_last="${!tmpvar:2}" ;  ARG_last="${ARG_last/=/=\"}\"" ;
                ARG_last=`echo $ARG_last | sed -e 's+""\(.\)+"\1+' -e 's+\([^=]\)""$+\1"+'`
                eval export "${ARG_last}" || error "while evaluating '${!tmpvar}'" ;
                if [ ${ARG_last/=*/}  == 'conf_file' ] ; then  source "$conf_file" ;   log source "$conf_file" ;  fi ;
                let ++ParseArguments_shift_n  ;;
             --*) ARG_last="${!tmpvar:2}" ;
                eval export "${ARG_last/%\"\"=/\"}" || echo "while evaluating '${!tmpvar}'" ;
                if [ ${ARG_last/=*/}  == 'conf_file' ] ; then  source "$conf_file" ;   log source "$conf_file" ;  fi ;
                let ParseArguments_shift_n+=2  ;;
            -*) ARG_last="${!tmpvar:1}" ; eval export "$ARG_last=0" || warn " argument  '${!tmpvar}' : Could not set variable '${!tmpvar}=1' " ; let ++ParseArguments_shift_n  ;;
            +*) ARG_last="${!tmpvar:1}" ; eval export "$ARG_last=1" || warn " argument  '${!tmpvar}' : Could not set variable '${!tmpvar}=1' " ; let ++ParseArguments_shift_n  ;;
            *)         break ;; 
        esac           # NOTE: BASH: 'break' exits the loop, not only the case stament
    done
    shift $ParseArguments_shift_n
    if [ $# -gt 0 ] ; then _debug "Unparsed arguments: $@" ; fi
}

# Defaults:
# Check binary dependencies
export TLPATH=${TLK_PATH:-$PRGDIR/../../bin}
# Better if not using "readlink -f" here

if [ ! -d "$TLPATH" ] ; then
    tLEXTRACT=`which tLextract`
    if [ -e "$tLEXTRACT" ] ; then
        export TLPATH=`dirname "$tLEXTRACT"`
    else 
        TLPATH="tL/bin"
        if [ ! -d "$TLPATH" ] ; then
            warning "Could not find tL binaries path: Export environment variable TLPATH to the proper location or place binaries in a default PATH"
        fi
    fi
fi
#export TLK_RECOGNISE="$TLPATH/tLrecognise"
#export TLK_CMLLR="$TLPATH/tLcmllr"
#export TLK_CMLLRFEAS="$TLPATH/tLcmllrfeas"

init_var $VARSmandatory $VARSoptional

# parse actual command line
ParseArguments "$@"

require_var $VARSmandatory

shift $ParseArguments_shift_n

############### 
# SIGNALING
trap "error $FORMAT_HIGHLIGHT KILLED     $FORMAT_DEFAULT" SIGKILL
trap "error $FORMAT_HIGHLIGHT TERMINATED $FORMAT_DEFAULT" TERM
#trap "error $FORMAT_HIGHLIGHT INTERRUPTED$FORMAT_DEFAULT" INT
trap 'end $?' EXIT

set -u

