#!/usr/bin/perl -w

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

use strict; 
use File::Basename;

my $PROG=basename($0);


my $verbose=0;
my $sheuristic='';
my @filelists;
my @fhs;
my $sfOutList='lists/clusters.lst';
my $sfOutClustersSamplesList='lists/train-clusters-samples.lst';
my $sfOutClusters='lists/clusters';
my $sPREPENDPATH='./';
############################
# ByProps heuristic:
my $propsext='.props';
# REGEX List to select the Properties in the .props file  
our @sPROPS=('MEDIA_NAME' ,'SPEAKER' );  


###################
# PARSE ARGUMENTS
while (@ARGV)  {
    $_=shift(@ARGV); 
    /^(-v|--verbose)$/                            && ( $verbose = 1 , next ) ;
    /^(-H|--heuristic)$/                          && ( $sheuristic = shift(@ARGV)   , next ) ;
    /^--?h(elp)?$/                                && ( help()                       , exit ) ; 
    /^--?o(utlist)?$/                             && ( $sfOutList= shift(@ARGV)     , next ) ; 
    /^--out-clusters-samples-list$/               && ( $sfOutClustersSamplesList= shift(@ARGV)     , next ) ; 
    /^--?c(lusters)?$/                            && ( $sfOutClusters= shift(@ARGV) , next ) ; 
    /^--?p(ropsext)?$/                            && ( $propsext =shift             , next ) ; 
    /^--?P(repend)?$/                             && ( $sPREPENDPATH =shift          , next ) ; 
    /^--?s(elect(ion)?)?$/                        && ( do {
        @sPROPS=();
        while ($_=shift(@ARGV)) {
           next  if /^--$/ ;
           push (@sPROPS , $_)
        }
                                }                         , next ) ; 
    push @filelists, $_ ;
}


###################
# Check heuristic
my $heuristic=\&{$sheuristic}; 
die("Unknown subroutine $sheuristic. (See help) \n")  unless exists &$sheuristic; 

###################
# Prepare output
system ("mkdir -p '$sfOutClusters'") && die ("Could not create output directory for the cluster lists $sfOutClusters\n") ; 

############################
# ByProps heuristic: Pre-compile regex:
@sPROPS=map{uc} sort (@sPROPS);
our @PROPS=map{ qr/^\s*$_\s*=\s*\"?([^\n\"]*)\"?/ }  @sPROPS;


@filelists=('-') unless (@filelists); 
for (@filelists) {
    if ($_ eq '-') {
        push @fhs,\*STDIN ; 
    } else {
        open( my $fh , "$_" ) or die ("Could not open media list '$_'\n"); 
        push @fhs, $fh; 
    }
}


my $n=0; 
my $cluster; 
my %h; 
my $orig;

if ($verbose) {
  print STDERR "[GG] $PROG: Reading samples...\n";
  print STDERR "[GG] $PROG: Progress ";
}

open(C,">$sfOutClustersSamplesList") or die ("cannot created $sfOutClustersSamplesList");
for my $fh (@fhs) {
    while (<$fh>) {
        chomp;
        $orig=$_;
        s,\.tLfea,,;
        # STrategy: Cluster by the 3 first letters of the name
        $cluster=$heuristic->("$sPREPENDPATH$_");
        print C "$cluster\n";
        push @{$h{$cluster}} , $orig; 
        print STDOUT "$orig $cluster\n" ; 
        # VERBOSE: 
        $n++;
        if ($verbose) {
          print STDERR '=' unless $n % 100 ;
          print STDERR "$n\n" unless $n % 5000 ; 
        }
    }
}
close(C);


$n=0; 
my @k=keys %h;
if ($verbose) {
  print STDERR "\n[GG] $PROG: Writing ".scalar(@k)." cluster ($sheuristic) lists...\n";
  print STDERR "[GG] $PROG: Progress ";
}
open(B,">$sfOutList") ; 
for (@k) {
    open(A,">$sfOutClusters/$_.lst") or die "Could not create cluster '$sfOutClusters/$_.lst'\n";  
    print A join("\n",@{$h{$_}})."\n"  ; 
    close(A) ;
    print B "$_\n";
    $n++;
    if ($verbose) {
      print STDERR "=" unless $n % 100 ;
      print STDERR "$n\n" unless $n % 5000 ; 
    }
}
close(B); 
if ($verbose) {
  print STDERR "\n";
}

###################
# CLUSTER SAMPLES BY ....

# ... directory path (up to the 3rd parent dir.) :
sub byDir {
    $_=dirname(shift); 
    s,^.*/(?=[^/]+/[^/]+/[^/]+$),,;
    s,\/+,_,g;
    return  $_;
}

# ... the file name of the sample:
sub byName {
    $_=shift; 
    s,\tLfea$,,; 
    return basename($_) ;
}

# ... the 3 first letters of the sample file
sub byNameFirst3Letters{
    $_=shift; 
    s,\tLfea$,,;
    return substr(basename($_),0,3) ;
}


# ... 
sub byProps {
    $_=shift;
    s,\tLfea$,,; 
    $_.='.props';
    open(P,$_) or do {
        print STDERR ("Could not open properties file : '$_' . Using byDir heuristic ... \n") ;
        return byDir($_);
    };

    my @RES=map{$_.'_'} @sPROPS;
    for my $r (0 ..$#RES) {
        $RES[$r]=~  s,[\W],_,g;
    }
    while(<P>) {
        for my $r (0..$#PROPS) {
            if (/$PROPS[$r]/ig) {
                if (defined($1)) {
                    $RES[$r].=$1;
                    $RES[$r]=~  s,[\W],_,g;
                    last;
                }
            }
        }
    }
    close(P);
    my $cluster=join('.', @RES); 
    return $cluster; 
}

###################
# HELP
sub help {

    print STDERR "
[HH] $0:   Generates lists  of files from the given lists of files and a two colum list file associating each
original file to a cluster list.
Files in a same of the generated list are considered to belong to a same 'cluster'.
Clusters are generated using a rule set stablished by the current clustering heristics (-H option).

SYNTAX: 
    S$0: list1 [... listN] [options] 
 
OPTIONS
    -H or --heuristic    : one of byDir , byName , byNameFirst3Letters , byProps
    -o or --outlist      : Output 2-column list file, with format  [FILE1 CLUSTERLIST]
    --out-clusters-samples-list    : Output 1-column list file, with format  CLUSTERLIST
    -P or --repend       : Prepend path to each sample sub-path (just for clustering - not to appear in the resulting lists)
    -c or --clusters     : Ouput path for the lists for eacg cluster
    -p or --propsext     : Extension of the properties files used by --H byProps (default: .props)
    -s or --select       : List of regular expresion to select properties in the .props file. 
                           Use -- to end the list. Regexs must be separated by blanks
                           The default selection properties list is : 
                                'MEDIA_NAME' 'SPEAKER'
    -h or --help          Show this help.
";
    
    
}
