#!/usr/bin/env python
# -*- coding: utf-8 -*-

# dfxp2stm.py: Converts a dfxp reference to SCLITE stm format
#
#  Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#


from sys import argv, exit, stdin, stdout, stderr
import os
import argparse
import re
import string
import unicodedata
import sys
import codecs
import locale

import libdfxp

def dfxp2stm(dfxp, wavname, outputfile):

  channel = '1'
  speaker = dfxp.authorId if dfxp.authorId != None else 'UNK'

  #remove all characters marked as punctuation symbols in UNICODE
  tbl = dict.fromkeys(i for i in xrange(sys.maxunicode)
                          if unicodedata.category(unichr(i)).startswith('P'))

  i=1
  for segment in dfxp.data:
    trans = segment.toText()
    trans = trans.translate(tbl)
    samplename = "samples/%s/sp/%09.3f_%09.3f-%s.tLfea" %(
        wavname, segment.begin, segment.end, wavname )
    #remove punctuation
    line = "%s %s %s %s %s %s" %(
          samplename, channel, speaker, segment.begin, segment.end, trans.lower()
          )
    outputfile.write(line)
    i+=1


#####################################################################
#####################################################################
#####################################################################
def main():

  parser = argparse.ArgumentParser(description=
      'Returns the best transcriptions of a dfxp in a suitable format for tLalign')

  parser.add_argument("input_dfxp", help='Input .dfxp file to be processed')
  parser.add_argument("output_stm", help='Input .dfxp file to be processed')

  args = parser.parse_args(argv[1:])
  basename = os.path.basename(args.input_dfxp)
  wavname = re.sub(r'.dfxp$', '', basename)

  input_dfxp = libdfxp.parseDfxp(args.input_dfxp)
  input_dfxp.data = libdfxp.filterDfxp(input_dfxp)
  output_file = codecs.open(args.output_stm, "w", "utf-8")

  dfxp2stm(input_dfxp, wavname, output_file)

####################################################################################################
if __name__ == "__main__":
  main()
