#!/bin/bash

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

HELP="<INPUT> <PROJD>
  <inputdir> The directory with the input .dfxp   
  <prjdir> The project dir which will be created 
                                                 
  Options:                                       
                                                 
    -h: Shows this help                          
    -q: quiet                                    
"
export SAMPLE_minduration=${SAMPLE_minduration:-0.1}
export EXTRA_TLEXTRACT_OPTS=${EXTRA_TLEXTRACT_OPTS:-""}
# Load helper functions
source "$TLUTILS"

# PARSE AND CHECK ARGUMENTS
if [ ! $# -eq 2 ] ; then
    error "SYNTAX ERROR: wrong number of arguments:  `help`" 
fi

INPUT=$1
PROJD=$2


function check_is_iso {
    DFXP=$1
    INFO=`grep '<?xml ' $DFXP` || return 1
    ENC=$(echo $INFO |
    sed 's|<?xml||g' |
    sed 's|?>||g' |
    gawk -F= '{ret=$1;for(i=1;i<=NF;++i)ret=ret" "$i;print ret}' |
    gawk '{for(i=1;i<NF;++i)if($i=="encoding"){print $(i+1);exit}}')
    [ -z $ENC ] && return 1
    ENC=$(echo $ENC | perl -pe 's/\r\n/\n/' | gawk '{s=tolower($1);gsub("\"","",s);
                                      gsub("-","",s);print s}')
    [ $ENC != "iso88591" ] && [ $ENC != "iso885915" ] && return 1
    return 0
}




set -eu



log $"Extracting audio and transcription samples from $INPUT"

for DFXP in "$INPUT/"*.dfxp ; do
    
    MEDIA_NAME=`basename "$DFXP" .dfxp`
    # Check if already done
    skip=0
    if [ "$CONTINUE" -eq 1 ] && [ -s "$PROJD/samples/$MEDIA_NAME/sp/sp_fea.lst"  ] ; then
        debug "Checking whether to skip or not the segmentation of $MEDIA_NAME"
        skip=1
        while read j ; do   if [ ! -s "$PROJD/samples/$MEDIA_NAME/sp/$j" ] ; then skip=0 ; break ; fi ;  done < "$PROJD/samples/$MEDIA_NAME/sp/sp_fea.lst"
        if [ "$skip" -eq 1 ] ; then
            log $"Skipping feature extraction of $MEDIA_NAME. (Set -f to force recomputation)"
            echo "$MEDIA_NAME"
            continue
        fi
    fi

    MakeTMP # Make temporary directory and store path in $TMP

    # Segmenting data
    WAV=${DFXP%%.dfxp}.wav
    debug " - $WAV: Extracting samples... \e[34m$TL_PREPROCESS_COMMON/segment_dfxp_wav.py $WAV $DFXP $TMP\e[0m"
    BASENAME=`basename $DFXP .dfxp`
    "$TL_PREPROCESS_COMMON/segment_dfxp_wav.py" "$WAV" "$DFXP" "$TMP" \
	-i "$BASENAME"
	REFERENCES=${REFERENCES:-''}
	if [ ! -z "$REFERENCES" ]
	then
	    debug $"Extracting reference transcription"
	    debug "$TL_PREPROCESS_COMMON/dfxp2stm.sh" "$INPUT" "$REFERENCES"/"$BASENAME.stm"
	    "$TL_PREPROCESS_COMMON/dfxp2stm.py" "$DFXP" "$REFERENCES"/"$BASENAME".stm
	    gawk '{for(i=6;i<NF;i++) printf("%s ",$i); printf("%s\n", $NF)}' \
		< "$REFERENCES"/"$BASENAME".stm >"$REFERENCES"/"$BASENAME".txt
	fi
    
    case $CMVN in
	spk)
	    
	    rm -f "$TMP"/spk_*.lst
            while read SPK
            do
                read SAMPLE
                echo "$TMP/sp/$SAMPLE.wav" >> "$TMP/spk_$SPK.lst"
                echo "$TMP/sp/$SAMPLE.tLfea" >> "$TMP/spk_$SPK.fea.lst"
                echo "$SPK"
            done < "$TMP/spks.lst" |
            sort -u > "$TMP/spks"
            while read l
            do
                tLextract "$TMP/spk_$l.lst" "$TMP/spk_$l.fea.lst" \
		    -n $EXTRA_TLEXTRACT_OPTS
		cat "$TMP/spk_$l.fea.lst"
            done < "$TMP/spks" > "$TMP/sp_fea.lst"
	    ;;
	video|none) ## Vídeo|none (default)
    # Extracting speech features
    # Select samples:
	    if [[ $CMVN == "none" ]]
	    then
		NORMOPTS=""
	    else
		NORMOPTS="-n"
	    fi
	    perl -ne '
    BEGIN{our $rej=0; $N=0}
    # Select only speech segments (which lay inside the sp/ directory) :
        next unless /\/sp\// ; 
        $N++;
    # Select samples with duration > 0.5 . Warn if the sample is rejected in this way
        $d=`soxi -D $_`; chomp $d ; 
        if ($d<$ENV{SAMPLE_minduration} ) {
            $rej++;
            next;
        }
    # Ensure each item in the list ends with a newline character
        chomp; $_.="\n";
    print; 
    END{
        print STDERR "\e[43;37m[WW]\e[0m Rejected $rej of $N samples of dur. < $ENV{SAMPLE_minduration}  "   
        if $rej;
    }
    '     "$TMP/wavs_sequential.lst"  > "$TMP/sp.lst"
    
	    sed 's|[.]wav|.tLfea|g' "$TMP/sp.lst" > "$TMP/sp_fea.lst"
	    debug "$TLPATH/tLextract $TMP/sp.lst $TMP/sp_fea.lst" $NORMOPTS $EXTRA_TLEXTRACT_OPTS
	    if [ `wc -l < "$TMP/sp.lst"` -eq 0 ] ; then
		warning "Empty sample list $TMP/sp.lst! . Ending extraction"
		DelTMPunlessDebug # Clean $TMP
		exit 1
	    fi
    
	    "$TLPATH/tLextract" "$TMP/sp.lst" "$TMP/sp_fea.lst" \
		$NORMOPTS $EXTRA_TLEXTRACT_OPTS
	    ;;
	*) echo "unknown value for: CMVN"; exit 1;;
    esac
    
    # move features to output directory
    MakeDirs "$PROJD/samples/$MEDIA_NAME/sp" 
    while read f ; do  mv "$f" "$PROJD/samples/$MEDIA_NAME/sp";   done < "$TMP/sp_fea.lst"
    debug " - $WAV: Moved features files to $PROJD/samples/$MEDIA_NAME/sp"
    
    while read f ; do  
        mv "${f%%tLfea}tr" "$PROJD/samples/$MEDIA_NAME/sp";   
        echo "samples/$MEDIA_NAME/sp/`basename \"$f\"`"
    done < "$TMP/sp_fea.lst"
    cat "$TMP/sp_fea.lst" | while read i ; do basename "$i" ; done | tee > "$PROJD/samples/$MEDIA_NAME/sp/sp_fea.lst"
    debug " - $WAV: Moved .tr files to $PROJD/samples/$MEDIA_NAME/sp..."
    perl -we '
        use File::Basename;
        use strict; 
        open(SP, $ARGV[0]) or die "\e[41;37m[EE]\e[0m Could not open $ARGV[0]\n";
        my %h;
        while(!eof(SP)) {
            my $spk=<SP>;  my $sample=<SP>; 
            chomp  $spk ;  chomp $sample      ;
            $h{$sample}=$spk;
        }
        close(SP); 
        open(LST, $ARGV[1]) or die "\e[41;37m[EE]\e[0m Could not open tLfea lists $ARGV[1]\n";
        my $OUT=$ARGV[2] ;
        die "Could not access directory $OUT when trying to write speech properties file for each sample\n" unless (-d $OUT);
        my $MEDIA_NAME="";   $MEDIA_NAME=$ARGV[3] if @ARGV>3;
        my $rej=0; my $N=0;
        while(<LST>){
            chomp; 
            $_=basename($_, ".tLfea");
            $N++;
            unless  ( exists($h{$_} )) {
                $rej++;
                print STDERR "\e[41;37m[EE]\e[0m not found: $_ \n";
                next;
            };
            open (SPEECHPROPS, ">$OUT/$_.props") or die "Could not create speech properties file $OUT/$_\n";
            print SPEECHPROPS "MEDIA_NAME=\"$MEDIA_NAME\";\nSPEAKER=\"$h{$_}\";\n";
            close(SPEECHPROPS);
        }
        END{
         print STDERR "\e[41;37m[EE]\e[0m Found $rej out of $N samples in $ARGV[1] not listed in the speakers file, $ARGV[0]\n" 
         if $rej;
        }
    ' "$TMP/spks.lst"  "$TMP/sp_fea.lst"  "$PROJD/samples/$MEDIA_NAME/sp" "$MEDIA_NAME"  || error "Could not parse speakers ($TMP/spks.lst , $TMP/sp_fea.lst)"
    debug " - $WAV: Generated speech properties files at > $PROJD/samples/$MEDIA_NAME/sp ( $TMP/spks.lst  $TMP/sp_fea.lst  $PROJD/samples/$MEDIA_NAME/sp $MEDIA_NAME  )"
    DelTMPunlessDebug # Clean $TMP
    log " - $WAV: Done"
    
done
