#!/bin/bash

#
#  Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# SYNTAX #################################################
# $0 "$INPUT" "$PRJD" > "$LISTS/videos.lst"

# ARGUMENTS ##############################################
source "$TLUTILS"

export INPUT=${INPUT:-$1}
export PRJD=${PRJD:-$2}

export CORPUS_NAME=${CORPUS_NAME:-`basename "$INPUT"`}

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

# DEFAULTS
#Applications:
PROGRAM_PATH=`dirname "$0"`
PROGRAM_PATH=`$READLINK "$PROGRAM_PATH"`
PROGRAM_NAME=`basename "$0"`

export INPUT_AUDIO_EXT=${INPUT_AUDIO_EXT:-wv1}
export INPUT_TEXT_EXT=${INPUT_TEXT_EXT:-tra}
export SPH2PIPE=${SPH2PIPE:-$PROGRAM_PATH/../external/sph2pipe/bin/sph2pipe}
export tLextract=${tLextract:-$TLPATH/tLextract}


set -eu


log "Converting in  $INPUT .wv1 to wav  to `pwd`/samples"
find "$INPUT"/ -name "*.$INPUT_AUDIO_EXT"  | while read i ; do
    OUT_WAV="$PRJD/samples/"`echo "$i" | sed -e "s,^$INPUT/*,," -e "s,$INPUT_AUDIO_EXT\$,wav,"`
    OUT_WAV_DIR=`dirname "$OUT_WAV"`
    debug mkdir -p "$OUT_WAV_DIR" ;
    mkdir -p "$OUT_WAV_DIR" ;
    # Convert from sphere format (sph, wv1, etc. ) to wav:
    debug $SPH2PIPE "$i" -f rif  \> "$OUT_WAV"  ;
    $SPH2PIPE "$i" -f rif  > "$OUT_WAV"  ;
    # Copy transcription file :
    debug cp "${i%%$INPUT_AUDIO_EXT}$INPUT_TEXT_EXT" "${OUT_WAV%%wav}tr"
    cp "${i%%$INPUT_AUDIO_EXT}$INPUT_TEXT_EXT" "${OUT_WAV%%wav}tr"
    echo $OUT_WAV_DIR 
done | sort -u  > "$PRJD/dirs"

log $"Converting audio .wav segments at `pwd`/samples to cepstrals features. Mean and std normalization is peformed over each set of samples in a same subdirectory :"


nsegments=`echo -e "$PRJD/dirs" | wc -l`
onepercent=$((nsegments/100+1))
mysegment=0
echo  -n "[" >&2

cat "$PRJD/dirs" | while read mydir ; do
    # Progress bar:
    mysegment=$((mysegment+1))
    if (( $mysegment % $onepercent ))  ; then
        echo -n "=" >&2
    fi
    # Normalize a set of wavs in the same subdirectory and extract the cepstrals features:
    wavs=`ls "$mydir/"*.wav`
    outs=`echo -e "$wavs" | sed 's,\.wav$,\.tLfea,' `
    "$tLextract" -n <( echo -e "$wavs" ) <( echo -e "$outs" )
    echo -e "$outs" | sed -e "s,^$PRJD/*,,"
done

echo "]" >&2




# deprecated : get vocabulary 
#| perl -ne 'for (split) {s/(\d+)$// ; $h{$_}=0}; END { for (sort(keys(%h)))  {print "$_\n"} }  ' > "$PRJD/vocabulary.voc"
