#!/bin/bash

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# SYNTAX #################################################
# $0 "$INPUT" "$PRJD" > "$LISTS/videos.lst"

# ARGUMENTS ##############################################
export INPUT=${INPUT:-$1}
export PRJD=${PRJD:-$2}

export CORPUS_NAME=${CORPUS_NAME:-`basename "$INPUT"`}

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

# DEFAULTS
#Applications:
PROGRAM_PATH=`dirname "$0"`
PROGRAM_PATH=`$READLINK "$PROGRAM_PATH"`
PROGRAM_NAME=`basename "$0"`
SEGMENT_STR="$PROGRAM_PATH/segment_srt_wav.perl"
SEGMENT_TR="$PROGRAM_PATH/segment_rt_wav.perl"
SEGMENT_STM="$PROGRAM_PATH/segment_stm_wav.perl"
SPLIT="$PROGRAM_PATH/Split.perl"


export SPH2PIPE=${SPH2PIPE:-$PROGRAM_PATH/../external/sph2pipe/bin/sph2pipe}
tLextract=${tLextract:-$TLPATH/tLextract}

myPID=$$
. "$PROGRAM_PATH/Utils.bash"

set -eu

log  $"Segmenting .srt|.rt|.stm transcriptions and their corresponding audio files from $INPUT to $PRJD/samples"

( 
    if [ -d "$INPUT" ] ; then 
        find "$INPUT/" \( -name "*.rt" -o -name "*.srt" -o -name "*.stm"  \) -type f
    else
        cat "$INPUT"
    fi
) > "$PRJD/samples/$PROGRAM_NAME.$myPID.lst"

"$SPLIT" $PARALLEL_MAXPROC "$PRJD/samples/$PROGRAM_NAME.$myPID.lst"



cat >"$PRJD/samples/$PROGRAM_NAME.$myPID.sh" <<EOS
cat "$PRJD/samples/$PROGRAM_NAME.$myPID.lst.part\${SGE_TASK_ID}of$PARALLEL_MAXPROC" | while read i ; do
#     set -x
    set -u
    # Check whether a more convinient format of the file exists_
    extension=\${i##*.}
    filename=\${i%.*}
    MEDIA_NAME=\`echo -n "\$i" | perl -pne 'm:(([^/]+/){1,4})[^/]+\$: ; \$_=\$1'\`
    OUT_VIDEO="$PRJD/samples/\$MEDIA_NAME/sp"
    DIR_i=\`dirname "\$i"\`
    mkdir -p "\$OUT_VIDEO"
    FILE_MEDIA=\`ls "\$DIR_i/"*.wav "\$DIR_i/"*.mp3 "\$DIR_i/"*.mp4 "\$DIR_i/"*.flv "\$DIR_i/"*.aac "\$DIR_i/"*.mv4 2>/dev/null |head -n1 \`
    [ -z "\$FILE_MEDIA" ] && exit 100
    echo "\$FILE_MEDIA" > "\$OUT_VIDEO/MEDIA.orig"
    case "\$extension" in
        rt)
            mkdir -p "\$OUT_VIDEO" ;
            echo "[GG] cat \$i | $SEGMENT_TR --dir \$OUT_VIDEO --audio \$FILE_MEDIA" >&2
            cat "\$i" | "$SEGMENT_TR" --dir "\$OUT_VIDEO" --audio "\$FILE_MEDIA" ;
        ;;
        srt)
            mkdir -p "\$OUT_VIDEO" ;
            # SPECIFIC TO VL ...................................................................v
            echo "[GG] cat \$i | $SEGMENT_STR --dir \$OUT_VIDEO --audio \$FILE_MEDIA" >&2
            cat "\$i" | "$SEGMENT_STR" --dir "\$OUT_VIDEO" --audio "\$FILE_MEDIA" ;
            ;;
        stm)
            audio=\${i%%stm}sph  "$SEGMENT_STM"   "\$i" ;;
        *) echo "$PROGRAM_NAME [EE] Unknown transcript format '\$extension' for file \$i " >&2 ;;
    esac
done > "$PRJD/samples/$PROGRAM_NAME.$myPID.lst.part\${SGE_TASK_ID}pf$PARALLEL_MAXPROC".processed
EOS
chmod +x "$PRJD/samples/$PROGRAM_NAME.$myPID.sh"
for ((p=1 ; p<=$PARALLEL_MAXPROC ; p++)) ; do echo  SGE_TASK_ID="$p"  "$PRJD/samples/$PROGRAM_NAME.$myPID.sh" ; done |   parallel $PARALLEL_MAXPROC

log $"Converting audio .wav segments at $PRJD/samples to cepstrals features. Mean and std normalization is peformed over each set of samples of the same speaker and media file"

dirs=`find "$PRJD/samples/" -name "*.wav" -printf "%h\n"| sort -u`
nsegments=`echo -e "$dirs" | wc -l`
onepercent=$((nsegments/100+1))
mysegment=0
echo  -n "[" >&2

echo -e "$dirs" | while read mydir ; do
    mysegment=$((mysegment+1))
    if [ $((mysegment%onepercent)) -eq 0 ]  ; then  echo -n "=" >&2 ; fi
    wavs=`ls "$mydir/"*.wav`
    outs=`echo -e "$wavs" | sed -e 's,\.wav$,\.tLfea,' `
    "$tLextract" -n <( echo -e "$wavs" ) <( echo -e "$outs" )
    echo -e "$outs" | sed -e "s,^$PRJD/*,,"
done

echo "]" >&2



cat "$PRJD/samples/$PROGRAM_NAME.$myPID.lst"

# deprecated : get vocabulary 
#| perl -ne 'for (split) {s/(\d+)$// ; $h{$_}=0}; END { for (sort(keys(%h)))  {print "$_\n"} }  ' > "$PRJD/vocabulary.voc"
