#!/bin/bash

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# SYNTAX #################################################
# $0 "$INPUT" "$PRJD" > "$LISTS/videos.lst"

# ARGUMENTS ##############################################
export INPUT=${INPUT:-$1}
export PRJD=${PRJD:-$2}

export CORPUS_NAME=${CORPUS_NAME:-`basename "$INPUT"`}

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

# DEFAULTS
#Applications:
PROGRAM_PATH=`dirname "$0"`
PROGRAM_PATH=`$READLINK "$PROGRAM_PATH"`
PROGRAM_NAME=`basename "$0"`

export SPH2PIPE=${SPH2PIPE:-$PROGRAM_PATH/../external/sph2pipe/bin/sph2pipe}
tLextract=${tLextract:-$TLPATH/tLextract}

export extract_samples__stm__AUDIO_USENAMES=${extract_samples__stm__AUDIO_USENAMES:-0}

set -eu

echo '[II]' $"Segmenting .stm transcriptions and their corresponding .sph audio files at $INPUT to $PRJD/samples" >&2

if [ "$extract_samples__stm__AUDIO_USENAMES" == '1' ] ; then
find "$INPUT/" -name "*.stm"  | while read i ; do
perl -w -e ' 
      use Scalar::Util qw(looks_like_number);
      use strict;  use File::Basename; our $inputdir=dirname($ARGV[0]) ;  my $oldaudionotfound="";
      #no warnings "numeric"; 
      open (A,$ARGV[0]);
      while(<A>) {
       print STDERR "." unless ( $. % 100);
       next  if /ignore_time_segment_in_scoring/;
       next if /^\s*;;/ ;
       @_=split;
       (my $name , my $nchannels, my $speaker , my $start ,my  $end , my $spkdef ) = splice (@_ , 0 , 6 );
       #print STDERR "[GG]  ( $name , $nchannels,  $speaker , $start , $end ,  $spkdef ) \n" ;
       $spkdef =~ s/[<>]//g ;
       $spkdef =~ s/[^\w]+/_/g ;
       $name =~ s/[^\w]+/_/g   ;
       $speaker =~ s/[^\w]+/_/g;
       $speaker="unk"  if ( $speaker =~ /^\s*$/ );
       $spkdef="unk"  if ( $spkdef =~ /^\s*$/ );
       $name="unk"  if ( $spkdef =~ /^\s*$/ );
       #print STDERR "[GG]  ( $name , $nchannels,  $speaker , $start , $end ,  $spkdef ) \n";
       
       unless ( looks_like_number($end) ) {
            print STDERR "[WW] Could not get the end time value from line $. of $ARGV[0] : $_ ->  ( $name , $nchannels,  $speaker , $start , $end ,  $spkdef ) \n";
            next;
       };
       unless ( looks_like_number($start) ) {
            print STDERR "[WW] Could not get the start time value from line $. of $ARGV[0] : $_ ->  ( $name , $nchannels,  $speaker , $start , $end ,  $spkdef ) \n";
            next;
       };
       my $dir_out="$ENV{PRJD}/samples/$spkdef/$speaker/$name/sp/";
       system("mkdir -p '"'"'$dir_out'"'"' ") && die "[EE] Could not create output directory for sample in line $. of $ARGV[0] at $dir_out/\n" ;

       pop @_ if $_[$#_] =~ /^\(/ && /\)$/ ;
       $_="@_\n";
       if (/^\s*$/) {
            print "[WW] Skipping line $. in $ARGV[0] : Empty transcription\n" ;
            next
       }
       open (F, ">$dir_out/$start-$end.tr") or die "[EE] Could not create output transcription for sample in line $. of $ARGV[0] at $dir_out/$start-$end.tr\n" ;
       print F "@_\n"; 
       #print STDOUT "$spkdef/$speaker/$name\n";
       close(F);

        my $audio="$inputdir/$name";
       if ( -s "$audio.sph" ) {
        $audio.=".sph";
        `$ENV{SPH2PIPE} -t $start:$end "$audio" -f rif  > "$dir_out/$start-$end.wav" ` ;
       } elsif ( -s "$audio.wav" ) {
        $audio.=".wav";
        my $trimwav="sox '"'"'$audio'"'"' -2 -c 1 -r 16k '"'"'$dir_out/$start-$end.wav'"'"' trim $start =$end" ;
        system ($trimwav) && die ("[EE] \e[41m*\e[0m Could not trim wav with $trimwav\n");
       } else {
        print STDERR "[EE] Could not find audio file $audio .sph neither .wav\n" unless ($oldaudionotfound eq $audio); 
        $oldaudionotfound=$audio;
        next;
       }
      };
      close(A);
    ' "$i"

done
echo >&2

else
find "$INPUT/" -name "*.stm"  | while read i ; do
    if [ -s "${i%%stm}sph"  ] ; then
        audio=${i%%stm}sph
    elif [ -s "${i%%stm}wav" ] ; then
        audio=${i%%stm}wav
    else
        echo "[EE] Could not find a suitable audio file fortranscirption '$i'. Please copy or link the proper .sph or .wav file at the transcript directiory " >&2
        continue
    fi
    audio="$audio"    perl -ne '
     BEGIN{        print STDERR "."   ;}
     next  if /ignore_time_segment_in_scoring/ or /^\s*;/ ; 
       @_=split;
       ($name , $nchannels, $speaker , $start , $end , $spkdef ) = splice (@_ , 0 , 6 ); 
       $spkdef =~ s/[<>]//g ;
       $spkdef =~ s/\W+/_/g ;
       $name =~ s/\W+/_/g   ;
       $speaker =~ s/\W+/_/g;
       my $dir_out="$ENV{PRJD}/samples/$spkdef/$speaker/$name/sp/";
       `mkdir -p "$dir_out"`;
      # print STDERR "$ENV{SPH2PIPE} -t $start:$end $ENV{audio} -f rif  > $dir_out/$start-$end.wav\n" ;
       `$ENV{SPH2PIPE} -t $start:$end "$ENV{audio}" -f rif  > "$dir_out/$start-$end.wav" ` ;
       open (F, ">$dir_out/$start-$end.tr");
       pop @_ if $_[$#_] =~ /^\(/ && /\)$/ ;
       $_="@_\n"; 
       print F; 
       #print STDOUT "$spkdef/$speaker/$name\n";
       close(F);
    ' "$i"
done
echo >&2

fi
echo '[II]'  $"Converting audio .wav segments at $PRJD/samples to cepstrals features. Mean and std normalization is peformed over each set of samples of the same speaker and media file:" >&2

dirs=`find "$PRJD/samples/" -name "*.wav" -printf "%h\n"| sort -u`
nsegments=`echo -e "$dirs" | wc -l`
onepercent=$((nsegments/100+1))
mysegment=0
echo  -n "[" >&2

echo -e "$dirs" | while read mydir ; do
    mysegment=$((mysegment+1))
    if [ $((mysegment%onepercent)) -eq 0 ]  ; then  echo -n "=" >&2 ; fi
    wavs=`ls "$mydir/"*.wav`
    outs=`echo -e "$wavs" | sed -e 's,\.wav$,\.tLfea,' `
    "$tLextract" -n <( echo -e "$wavs" ) <( echo -e "$outs" )
    echo -e "$outs" | sed -e "s,^$PRJD/*,,"
done

echo "]" >&2

# deprecated : get vocabulary 
#| perl -ne 'for (split) {s/(\d+)$// ; $h{$_}=0}; END { for (sort(keys(%h)))  {print "$_\n"} }  ' > "$PRJD/vocabulary.voc"

