#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from trs import *
import sys
import re
from acronims import acronims

trs= load_trans_file(sys.stdin)

class Poison:
    pass

def prep_event(event):
    etype= event.attrs['type']
    if etype == 'noise': return ' /SF//'
    elif etype == 'pronounce': return ''
    elif etype == 'lexical': return ''
    elif etype == 'language': return Poison()
    else:
        sys.exit("Tipus d'event '%s' desconegut"%etype)

def prep_text(text):
    ret= re.sub('([ ]*/SF//[ ]*)+',' /SF// ',text)
    ret= re.sub('(^|[ ])(\S+)-(?=[ ]|$)','\g<1>/\g<2>//',ret)
    ret= re.sub('(^|[ ])m+(?=[ ]|$)','\g<1>/m//',ret)
    ret= re.sub('(^|[ ])eh+(?=[ ]|$)','\g<1>/e//',ret)
    ret= re.sub('(^|[ ])ah+(?=[ ]|$)','\g<1>/a//',ret)
    for k,v in acronims.iteritems():
        ret= re.sub('(^|[ ])%s(?=[ ]|$)'%k,'\g<1>/%s/%s/'%(v,k),ret)
    ret= ret.strip()
    if ret=='/SF//': ret= '[sonido de fondo]'
    return ret

def prep_turn(turn):
    ncontent= []
    prev= ""
    for c in turn.content:
        if isinstance(c,TransSync):
            if prev != "": ncontent.append(prep_text(prev.strip()))
            ncontent.append(c)
            prev= ""
        elif isinstance(c,TransEvent):
            aux= prep_event(c)
            if type(aux)==str : prev+= prep_event(c)
            else:
                ncontent.append(prep_text(prev.strip()))
                prev= ""
                ncontent.append(aux)
        else: prev+= ' '+c
    if prev != "": ncontent.append(prep_text(prev.strip()))
    turn.content= ncontent

# Elimina els syncs amb Poison
def remove_poison(turn):
    ncontent= []
    length= len(turn.content)
    s= None; i= 0
    while i < length:
        s= turn.content[i]
        while isinstance(s,TransSync):
            ncontent.append(s)
            i+= 1
            if i >= length: break
            s= turn.content[i]
        if i >= length: break
        aux= []
        t= turn.content[i]
        poisoned= False
        while not isinstance(t,TransSync):
            if isinstance(t,Poison): poisoned= True
            aux.append(t)
            i+= 1
            if i>= length: break
            t= turn.content[i]
        if poisoned: ncontent.append('[sonido de fondo]')
        elif aux==[] : i+= 1
        else: ncontent+= aux
    turn.content= ncontent

# Elimina syncs buits o 
def reduce_turn(turn):
    ncontent= []
    length= len(turn.content)
    s= None; i= 0
    prev_sf= False
    while i < length:
        if s==None:
            s= turn.content[i]
            i+= 1
            if i >= length: break
        t= turn.content[i]
        if not isinstance(s,TransSync): sys.exit('Oo???!!')
        if isinstance(t,TransSync): t= '[sonido de fondo]'
        else                      : i+= 1
        cur_sf= (t=='[sonido de fondo]')
        if prev_sf and cur_sf:
            s= None
            continue
        ncontent.append(s)
        ncontent.append(t)
        prev_sf= cur_sf
        s= None
    if s != None:
        ncontent.append(s)
        ncontent.append('[sonido de fondo]')
    turn.content= ncontent

for e in trs.episodes:
    for s in e.sections:
        for t in s.turns:
            prep_turn(t)
            remove_poison(t)
            reduce_turn(t)

trs.write(sys.stdout)
