#!/usr/bin/python

#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from libdfxp import *
import sys
import commands
from optparse import OptionParser
import os
import cStringIO

SPECIAL_TOKENS = [ "[HESITATION]", "<unk>" ]

def filterSegment(obj,filt=1):
  if filt == 0: # No filtering
    return obj
  if filt == 1: # filter special tokens and syntax
    new_data = []
    for d in obj.data:
      if isinstance(d,TlWord):
        if d.txt not in SPECIAL_TOKENS:
          new_data.append(d)
      elif isinstance(d,TlRawText):
        d.txt = re.sub(r'\s+',r' ', re.sub(r'/([^/]+)/([^/]*)/',r'\2',d.txt))
        new_data.append(d)
      else:
        new_data.append(d)
    obj.data = new_data
    return obj

def filterAlt(obj,doc_ts,policy=2):
  if policy == 1: # return last alternative
    last = None
    last_ts = datetime(1970,01,01,00,00)
    for d2 in obj.data:
      ts = d2.timeStamp
      if ts == None: ts = doc_ts
      if ts == None: ts = datetime(2010,01,01,00,00)
      if ts > last_ts: last = d2
    return last
  elif policy == 2: # return best alternatives
    max_confMeasure = -1
    max_authorConf = -1
    last_ts = datetime(1970,01,01,00,00)
    max_seg = None
    human = False
    for d2 in obj.data:
      if d2.authorType == "human" or (d2.authorType == None and pdfxp.authorType == "human"):
        human = True
      ts = d2.timeStamp
      if ts == None: ts = doc_ts
      if ts == None: ts = datetime(2010,01,01,00,00)
      if not(human):
        if d2.confMeasure > max_confMeasure or (d2.confMeasure == max_confMeasure and ts > last_ts):
          max_confMeasure = d2.confMeasure
          last_ts = ts
          max_seg = d2
      elif human and d2.authorType == "human":
        if d2.authorConf > max_authorConf or (d2.authorConf == max_authorConf and ts > last_ts):
          max_authorConf = d2.authorConf
          last_ts = ts
          max_seg = d2
    return max_seg
  else:
    return None
    
def filterDfxp(pdfxp, altFiltPol=2, segFiltPol=1):
  if altFiltPol == 0: # return all stuff
    return pdfxp.data
  else:
    filt_data = []
    doc_ts = pdfxp.timeStamp
    for d1 in pdfxp.data:
      if isinstance(d1,TlSegment):
        fs = filterSegment(d1,segFiltPol)
        if fs == None: return None
        filt_data.append(fs)
      elif isinstance(d1,TlAlt):
        seg = filterAlt(d1,doc_ts,altFiltPol)
        if seg == None: return None
        fs = filterSegment(seg,segFiltPol)
        if fs == None: return None
        filt_data.append(fs)
      else:
        return None
    return filt_data

def segment(pdfxp, wav, outd, vid):
  wavsfd= file("%s/wavs_sequential.lst"%outd,'w')
  spksfd= file("%s/spks.lst"%outd,'w')
  for i in xrange(len(pdfxp.data)):
    d1 = pdfxp.data[i]
    if isinstance(d1,TlSegment):
      processTlSegment(d1, wav, outd, vid, wavsfd, spksfd)
    else:
      sys.stderr.write("[EE] Unexpected element, expecting tl:segment.\n")
      sys.exit(1)
  spksfd.close()
  wavsfd.close()

def processTlSegment(s, wav, outd, vid, wavsfd, spksfd ):
  stype = "sp"
  bname= "%s_%s-%s"%(str("{0:.3f}".format(s.begin).zfill(9)),str("{0:.3f}".format(s.end).zfill(9)),str(vid))
  sample_fn="%s/%s/%s"%(outd,stype,bname)
  tr = cStringIO.StringIO()
  for d in s.data:
    if isinstance(d,TlWord) or isinstance(d,TlRawText):
      tr.write("%s "%(d.txt.encode('utf-8')))
    elif isinstance(d,TlGroup):
      for d1 in d.data: tr.write("%s "%(d1.txt.encode('utf-8')))
  tr.write("\n")
  seg_tr_fn="%s.tr"%sample_fn
  seg_tr_fd=open(seg_tr_fn,'w')
  seg_tr_fd.write(tr.getvalue()) # write transcription
  seg_tr_fd.close()
  
  seg_wav_fn="%s.wav"%sample_fn
  res = commands.getstatusoutput('sox %s %s trim %.2f %.2f'%(wav_fn,seg_wav_fn,s.begin,s.end-s.begin))
  if res[0] != 0:
    sys.stderr.write("[EE]: Failed to trim wav: %s\n"%(res[1]))
    sys.exit(1)
  wavsfd.write('%s\n'%seg_wav_fn)
  spksfd.write('spk1\n')
  spksfd.write('%s\n'%bname)

usage = "usage: %prog [options] <i_wav> <i_dfxp> <o_dir>"
parser = OptionParser(usage)
parser.add_option("-i", "--video_id", dest="vid", type="string", default=None, help="manually specify the current sample ID")

(options, args) = parser.parse_args()

if len(args) != 3:
	parser.error("[EE]: Incorrect number of arguments.")

wav_fn=args[0]
dfxp_fn=args[1]
out_dir=args[2]

pdfxp = parseDfxp(dfxp_fn)
pdfxp.data = filterDfxp(pdfxp,2,0)

if os.path.isdir('%s/sp'%(out_dir)) == False:
  os.makedirs('%s/sp'%(out_dir))
  os.makedirs('%s/bg'%(out_dir))

vid = pdfxp.videoId
if options.vid != None:
  vid = options.vid

segment(pdfxp, wav_fn, out_dir, vid)
