#!/usr/bin/perl -w 

#
#  Copyright 2012,2013,2014 transLectures-UPV Team
#  Copyright 2011 Isaías Sánchez Cortina
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# Segment srt subtitle files for further ASR processing.
#         v0.1 July 2013 , by  isanchez@dsic.upv.es
# Modified from convert-srt_to_ak , v0.1 Feb 2011 by isanchez@dsic.upv.es


use strict;

# binmode(STDOUT, ":utf8");
# binmode(STDERR, ":utf8");
# binmode(STDIN, ":utf8");
use File::Basename;
# use Encode;


(our $prog , our $script_path) = fileparse($0);

my $TotalDur;
my $fAudio; 
my $DIR_out='./';
our $DEBUG=0;

our $OUT_WAV_SampleRate=16000;
our $OUT_WAV_BitsPerSample=16;
our $OUT_WAV_BitRate=($OUT_WAV_BitsPerSample*$OUT_WAV_SampleRate);

while (@ARGV){
    $_= shift;
    # Operation modes:
    /^-h$/ && (help(), exit); # HELP
    # Paths:
    /^--dir$/ && ($DIR_out=shift, next);
    /^--audio$/ && ($fAudio=shift, next);
    /^--ends$/ && ($TotalDur=shift,next); 
    /^-g$/ && ($DEBUG=1, next); 
}

my $AVDEBUG=$DEBUG?'':'2>/dev/null';

our $AU;
if ( defined($fAudio)) {
    {
        open (RAW, "avconv -i '$fAudio' -ac 1 -ab $OUT_WAV_BitRate -ar $OUT_WAV_SampleRate -f wav - | tail -c +45 |");
        binmode(RAW,":bytes");
        local $/ = undef;
        $AU = <RAW>;
        close(RAW);

    }
    if (!defined($TotalDur))  {
        #my $readTotalDur=(sort { $b <=> $a }  (grep {s,^duration=,,} (`avprobe -show_streams "$fAudio" $AVDEBUG`)))[0];
        {
            use bytes;
            $TotalDur=length($AU)*8/$OUT_WAV_BitRate;
            no bytes;
        }
        unless ($TotalDur >0 ){
            print STDERR "$prog: [WW] Failed:(sort { \$b <=> \$a }  (grep {s,^duration=,,} (\`avprobe -show_streams $fAudio $AVDEBUG\`)))[0]
            Could not find total duration for the suplied media file '$fAudio': Last segment will be probably skipped\n "
        }
    }
}


# Precompile  regex
my $REGEX_time=qr/(\d+):(\d+):(\d+),(\d+) --> (\d+):(\d+):(\d+),(\d+)/;
my $REGEX_next=qr/^\s*$/;
my $rTag=qr/<([^<>]*)>/;
my $rTimeBegin=qr/time\s+begin\s*=\s*\"((?'hh'\d+):)?(?'mm'\d+):(?'ss'\d+(\.\d+)?)\"/i;
# Helper vars
my $REAL_lastTime=0.;
my $STR_text=" ";

my $init=1;

# Main loop : for each line in a srt-formated  file:
my $text="";
my $REAL_startTime;

MAIN: while(<STDIN>){
    gg ("$. : $_");
    while ($_ =~ s/$rTag//){
    gg ("$. removing one  <> : $_");
        my $tag=$1; 
        if ($tag =~  $rTimeBegin )  {
            my $hh=defined($+{hh})?$+{hh}*3600.0:0;
            $REAL_startTime=sprintf("%.3f", ( $hh + 60.0 * $+{mm}+ $+{ss} ));
            gg ("$. matched time: $+{mm} : $+{ss}: = $REAL_startTime\n");
            if ($init) {
                    $REAL_lastTime=$REAL_startTime;
                    $init=0;
                    next;
            }
            my $dur=$REAL_startTime-$REAL_lastTime;
            if ($dur <0.2) {
                print STDERR "$prog: [WW] skipping  segment with less than 0.2s from $REAL_lastTime to $REAL_startTime ($DIR_out)";
                $text="";
                $REAL_lastTime=$REAL_startTime;
                next;
            }
            
            # Print text turn to .tr segment
            if ($text =~ $REGEX_next ) {
                print STDERR "$prog: [WW] skipping  segment with empty text from $REAL_lastTime to $REAL_startTime ($DIR_out)";
                $text="";
                $REAL_lastTime=$REAL_startTime;
                next;
            }
            print STDERR ".";
            open (F, ">$DIR_out/$REAL_startTime-$REAL_lastTime.tr");
            $text=~s,\s+, ,g;
            print F "$text\n";
            close(F);
            gg ("Wrote $DIR_out/$REAL_startTime-$REAL_lastTime.tr with content: $text");
            # extract audio segment
            if (defined( $fAudio)) {
                Audio_FromRAWToWAV($REAL_startTime, $dur , "$DIR_out/$REAL_lastTime-$REAL_startTime.wav" ) ;

            }
            print STDERR "=";
            $REAL_lastTime=$REAL_startTime;
            $text="";

        }
        last MAIN if eof(STDIN);
    }
    gg ("$. after  remove : $_");

    # Increase turn text
    next if $_ =~ $REGEX_next;
    $text.=$_;

}

if (defined($TotalDur) && ($TotalDur>($REAL_lastTime+0.2)) && (length( $text)) ) {
    open (F, ">$DIR_out/$REAL_startTime-$REAL_lastTime.tr");
    print F "$text\n";
    close(F);
    my $dur=$REAL_startTime-$REAL_lastTime;
    unless ($dur <0.2) {
        Audio_FromRAWToWAV($REAL_startTime, $dur , "$DIR_out/$REAL_lastTime-$REAL_startTime.wav" ) ;
     }
}

# DEPRECATED:
#`avconv -y  -ss $REAL_lastTime -t $REAL_lastTime -i "$fAudio" "$DIR_out/$REAL_lastTime-$REAL_startTime.wav" $AVDEBUG`;
#############
sub gg {
 return unless $DEBUG;
 print STDERR "$prog: [GG] @_\n";
}
sub help{
print STDERR "
\e[1m$prog:\e[0m
\e[47;30m".( "-" x 80)."
".( "-" x 80)."\e[0m 
TODO
\e[1mSYNTAX\e[0m
    cat FILE_STDIN | $prog  [Options] > OUTPUT_HUGEPHRASE_TXT 
Help:
-h                : Show this help.

Audio file to be segmented:


Paths:
--dir (PATH)      : Path to drop the  text (.tr) and audio (.wav) segments from the srt;

Optional:

#         v0.1 July 2013 , by  isanchez\@dsic.upv.es
# Modified from convert-srt_to_ak , v0.1 Feb 2011 by isanchez\@dsic.upv.es


";
}


sub Audio_FromRAWToWAV{
   (my $start, my $dur, my $fOut ) =@_;
   
   my $Bstart=int($start*$OUT_WAV_BitRate/8/2)*2; # Ensure number is even
   my $N_bytesRAWaudio=int($dur*$OUT_WAV_BitRate/8)*2; # Ensure number is even
   
   use bytes;
   # Bytes sematics:
   # The RAW bytes-size will be the bytes-size of AD file minus its ASCII hea8der
   # AD ASCII header ends with \nData\n                                         
   
   #                    R    I    F    F
   my $N_SubChunk1Size=36+$N_bytesRAWaudio;
   
   #                                                        |Lit-End 16 for PCM  |LE #channels         |LE-4B SampleRate=16KHz                  |LE-2Bchan*BPS/8     d     a    t    a    
   my $BYTES_WAVheader="RIFF".pack("l<",$N_SubChunk1Size)."WAVEfmt \x{10}\x{00}\x{00}\x{00}\x{01}\x{00}\x{01}\x{00}\x{80}\x{3e}\x{00}\x{00}\x{00}\x{7d}\x{00}\x{00}\x{02}\x{00}\x{10}\x{00}\x{64}\x{61}\x{74}\x{61}".pack("l<",$N_bytesRAWaudio);
   #                       ^Total WAV file size-8 bytes, packed little-endian 4-bytes       |LE 1=Linear                   |LE-4B BitRate-------           |LE-2B BPS                      ^Total RAW Audio size-8 bytes, packed little-endian 4-bytes
   #                                        SampleRate 16000Sample/s * #Nchannels 1Chan * BitsPerSample 16bit/(Sample·Chan) / 8bit/Byte     --^
   
   
   # print header
   open (WAVout, ">$fOut"); 
   print WAVout $BYTES_WAVheader;
   print WAVout substr($AU,$Bstart,$N_bytesRAWaudio);
   no bytes; 
   close (WAVout); 
   
#    # buffer 512KB
#    while (read ($FH_IN,$_,524288)){
#       print $FH_OUT $_; 
#    }
     gg ("Extracted $dur seconds starting from {$start}s to $fOut");   
}

