#!/usr/bin/perl -w

#
#  Copyright 2012,2013,2014 transLectures-UPV Team
#  Copyright 2011 Isaías Sánchez Cortina
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# Segment srt subtitle files for further ASR processing.
#         v0.1 July 2013 , by  isanchez@dsic.upv.es
# Modified from convert-srt_to_ak , v0.1 Feb 2011 by isanchez@dsic.upv.es


use strict;

# binmode(STDOUT, ":utf8");
# binmode(STDERR, ":utf8");
# binmode(STDIN, ":utf8");
use File::Basename;
# use Encode;


(our $prog , our $script_path) = fileparse($0);

my $fAudio; 
my $DIR_out='./';
our $DEBUG=0;

while (@ARGV){
    $_= shift;
    # Operation modes:
    /^-h$/ && (help(), exit); # HELP
    # Paths:
    /^--dir$/ && ($DIR_out=shift, next);
    /^--audio$/ && ($fAudio=shift, next);
        /^-g$/ && ($DEBUG=1, next); 

}

my $AVDEBUG=$DEBUG?'':'2>/dev/null';



# Precompile  regex
my $REGEX_time=qr/(\d+):(\d+):(\d+)[,.](\d+)\s*-->\s*(\d+):(\d+):(\d+)[,.](\d+)/;
my $REGEX_next=qr/^\s*$/;

# Helper vars
my $REAL_lastTime=0.;
my $STR_text=" ";

gg ( "No input wav defined") unless (defined( $fAudio) && length($fAudio)) ;

# Main loop : for each line in a srt-formated  file:
MAIN: while(<STDIN>){
    while ($_ =~ $REGEX_next){
        $_ = <STDIN>; # skips blank lines or number
        last MAIN if eof(STDIN);
        # 	 print STDERR "Dropping... $_";
    }
    $_ = <STDIN>; # skips trans number
    die ("!!!! $_") unless ($_ =~ $REGEX_time) ;
    gg ("==> $_ : $1 , $2 , $3 , $4 ; $5 , $6, $7, $8");

    #     print STDERR "looking... for time on $_";
    my $REAL_startTime=sprintf("%.3f",3600.0 * $1 + 60.0 * $2 + $3 + $4/1000);
    $REAL_startTime=$REAL_lastTime+0.1  if $REAL_startTime<$REAL_lastTime ;
    my $REAL_endTime=sprintf("%.3f",3600.0 * $5 + 60.0 * $6 + $7 + $8/1000 );
    $REAL_lastTime=$REAL_endTime;
    # Check sanity
    my $dur=$REAL_endTime-$REAL_startTime;
    gg ("From $REAL_startTime -> $REAL_endTime ..");
    
    # read turn text
    my $STR_turn=<STDIN>;
    while(<STDIN>){
        last if $_ =~ $REGEX_next;
        $STR_turn.=<STDIN>;
    }
    
    $STR_turn =~ s/\s+/ /g;     # Join by silences every line in the turn
    $STR_turn=~ tr/\000-\01F/ /;
    # Print text turn to .tr segment
    if ($dur <0.1) {
        print STDERR "$prog: [WW] $DIR_out: Skipped  segment [$REAL_startTime to $REAL_endTime]<0.1s with phrase '$STR_turn '\n";
        next;
    }
    gg ("... tr: '$STR_turn'");
    open (F, ">$DIR_out/$REAL_startTime-$REAL_endTime.tr");
    print F "$STR_turn\n";
    close(F);
    # extract audio segment
    if (defined( $fAudio) && length($fAudio)) {
#         gg ("avconv -y  -ss $REAL_startTime -t $dur -i $fAudio $DIR_out/$REAL_startTime-$REAL_endTime.wav $AVDEBUG");
#         `avconv -y  -ss $REAL_startTime -t $dur -i "$fAudio" "$DIR_out/$REAL_startTime-$REAL_endTime.wav" $AVDEBUG ` ;c
        my $trimwav="sox '$fAudio' -2 -c 1 -r 16k '$DIR_out/$REAL_startTime-$REAL_endTime.wav' trim $REAL_startTime =$REAL_endTime" ;
        gg ("Trimming audio $trimwav..."); 
        system ($trimwav) && die ("[EE] \e[41m*\e[0m Could not trim wav with $trimwav\n");
    }
}


#############

#############
sub gg {
 return unless $DEBUG;
 print STDERR "\e[34m$prog: [GG] @_\e[0m\n";
}
sub help{
print STDERR "
\e[1m$prog:\e[0m
\e[47;30m".( "-" x 80)."
".( "-" x 80)."\e[0m 
TODO
\e[1mSYNTAX\e[0m
    cat FILE_STDIN | $prog  [Options] > OUTPUT_HUGEPHRASE_TXT 
Help:
-h                : Show this help.

Audio file to be segmented:


Paths:
--dir (PATH)      : Path to drop the  text (.tr) and audio (.wav) segments from the srt;

Optional:

#         v0.1 July 2013 , by  isanchez\@dsic.upv.es
# Modified from convert-srt_to_ak , v0.1 Feb 2011 by isanchez\@dsic.upv.es


";
}
