#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
#  Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

# Juan Daniel Valor Miro, 2014
#
# NOTE: This phonetic transcriptor is based on IPA rules.
# http://www.ipasource.com/diction_help
# Main differences with Italian Eutrans:
#   - Collapsed phonemes market in the code (see code).
#   - Repeated phonemes are two different (mm -> m m).
#   - Phonemes of gli are [g][l][i] and not [l][l].
#
import sys, re, unicodedata

#=== INI: NUMBER TO ITALIAN TEXT ===#

def int2ita(num):
	units = ['','uno','due','tre','quattro','cinque','sei','sette','otto','nove']
	teens = ['','undici','dodici','tredici','quattordici','quindici','sedici','diciasette','diciotto','diciannove']
	tens = ['','dieci','venti','trenta','quaranta','cinquanta','sessanta','settanta','ottanta','novanta']
	thousands = ['','mille','milione',"miliardo","trilione","quadrilione","quintilione"]
	words = []
	if num==0: words.append('zero')
	else:
		numStr = '%d'%num
		numStrLen = len(numStr)
		groups = int((numStrLen+2)/3)
		numStr = numStr.zfill(groups*3)
		for i in range(0,groups*3,3):
			h,t,u = int(numStr[i]),int(numStr[i+1]),int(numStr[i+2])
			g = int(groups-(i/3+1))
			if h>=1:
				if u>1: words.append(units[h])
				words.append('cento')
			if t>1:
				words.append(tens[t])
				if u>=1: words.append(units[u])
			elif t==1:
				if u>=1: words.append(teens[u])
				else: words.append(tens[t])
			else:
				if u>=1: words.append(units[u])
			if (g>=1) and ((h+t+u)>0):
				if words[-1]=="uno": words[-1]=thousands[g]
				else: words.append(thousands[g])
	return words

def trynum(w):
	try:
		if w.isdigit(): return int2ita(int(w))
		else:
			nums=w.split(",")
			if len(nums)==2:
				if nums[0].isdigit() and nums[1].isdigit():
					text=int2ita(int(nums[0]))
					text.append("e")
					text.extend(int2ita(int(nums[1])))
					return text
	except: pass
	return []

#=== END: NUMBER TO ITALIAN TEXT ===#

#=== INI: CLEAN ITALIAN TEXT ===#

def deleteaccents(s,doit=True):
	if doit==False: return s
	else: return ''.join(c for c in unicodedata.normalize('NFD',s) if unicodedata.category(c)!='Mn')

def expandwords(s):
	s=s.replace('$',' dollari ')
	s=s.replace('%',' per cento ')
	s=s.replace('€',' euro ')
	s=s.replace('&',' e ')
	s=s.replace('=',' uguale ')
	s=s.replace('+',' piu ')
	s=s.replace('-',' meno ')
	s=s.replace('/',' diviso ')
	s=s.replace('*',' moltiplicato ')
	return s

def parseline(text,doit=True):
	text=text.replace(';',' ').replace(':',' ').replace('!',' ').replace('?',' ').replace('¿',' ').replace('¡',' ').replace('/',' ').replace('-',' ').replace('_',' ').replace(')',' ').replace('(',' ')
	if doit==True: text=text.replace('\'','')
	text=text.replace('.','').replace('\"','').replace('º','').replace('ª','').replace('“','').replace('”','')
	text=expandwords(text)
	text=re.sub(r'\s+',' ',text.replace("\n",""))
	text=text.lower()
	words,newwords=text.split(" "),[]
	for w in words:
		numx=trynum(w)
		if len(numx)==0:
			if w=='b': newwords.append("bi")
			elif w=='c': newwords.append("ci")
			elif w=='d': newwords.append("di")
			elif w=='f': newwords.append("effe")
			elif w=='g': newwords.append("gi")
			elif w=='h': newwords.append("acca")
			elif w=='l': newwords.append("elle")
			elif w=='m': newwords.append("emme")
			elif w=='n': newwords.append("enne")
			elif w=='p': newwords.append("pi")
			elif w=='q': newwords.append("cu")
			elif w=='r': newwords.append("erre")
			elif w=='s': newwords.append("esse")
			elif w=='t': newwords.append("ti")
			elif w=='v': newwords.append("vi")
			elif w=='z': newwords.append("zeta")
			elif w=='j': newwords.extend(["i","lunga"])
			elif w=='k': newwords.append("cappa")
			elif w=='w': newwords.extend(["vi","doppia"])
			elif w=='x': newwords.append("ics")
			elif w=='y': newwords.append("i")
			else: newwords.append(w)
		else: newwords.extend(numx)
	newline=" ".join(newwords)
	newline=expandwords(newline)
	newline=re.sub(r'\s+',' ',newline.replace("\n",""))
	newline=deleteaccents(newline.replace(',',''),doit)
	return newline.lower()

#=== END: CLEAN ITALIAN TEXT ===#

#=== INI: ITALIAN TRANSLITERATOR ===#

def simplifier(t):
	p,l=[],"-"
	for i in range(len(t)):
		if t[i]!=l:
			p.append(t[i])
			l=t[i]
		else: pass
	return p

def transliterate(l):
	global phonmode
	if l=="": return ""
	t,e,l=[],0,"          "+l+"          "
	for w in range(len(l)):
		if e>0:
			e=e-1
			continue
		if l[w]==" " and ( phonmode=="fullsp" or phonmode=="compactsp" ):
			if len(t)==0: t.append("SP")
			elif t[-1]!="SP": t.append("SP")
		elif l[w]=="a":
			if l[w+1]=="i" and l[w+2] not in ["a","e","o","u"]:
				t.append("a")
				t.append("i")
				e=1
			elif l[w+1]=="u" and l[w+2] not in ["a","e","i","o","u"]:
				t.append("a")
				t.append("u")
				e=1
			else:
				t.append("a")
		elif l[w]=="b": t.append("b")
		elif l[w]=="c":
			if l[w+1]=="e" or l[w+1]=="i":
				t.append("c")
			elif l[w+1]=="c":
				if l[w+2]=="e" or l[w+2]=="i":
					t.append("t")
					t.append("c")
					e=1
				else:
					t.append("k")
					t.append("k")
					e=1
			elif l[w+1]=="h":
				t.append("k")
				e=1
			elif l[w+1]=="q" and l[w+2]=="u":
				t.append("k")
				t.append("k")
				t.append("w")
				e=2
			else:	t.append("k")
		elif l[w]=="d": t.append("d")
		elif l[w]=="e":
			if l[w+1]=="i":
				t.append("e")
				t.append("i")
				e=1
			elif l[w+1]=="u":
				t.append("e")
				t.append("u")
				e=1
			else:
				t.append("e")
		elif l[w]=="f": t.append("f")
		elif l[w]=="g":
			if l[w+1]=="e" or l[w+1]=="i":
				t.append("G")
			elif l[w+1]=="g":
				if l[w+2]=="e" or l[w+2]=="i":
					t.append("d")
					t.append("G")
					e=1
				else:
					t.append("g")
					t.append("g")
					e=1
			elif l[w+1]=="n":
				t.append("n") # Collapsed [n] phonemes.
				e=1
			elif l[w+1]=="u":
				t.append("g")
				t.append("w")
				e=1
			elif l[w+1]=="l" and l[w+2]=="i":
				t.append("g")
				t.append("l")
				t.append("i")
				e=2
			else:	t.append("g")
		elif l[w]=="i":
			if l[w+1]=="e" and l[w+2]=="i":
				t.append("j")
				t.append("e")
				t.append("i")
				e=2
			elif ( l[w-1]=="c" or l[w-1]=="g" ) and l[w+1] in ["a","e","i","o","u"]:
				pass
			elif l[w-2]=="s" and l[w-1]=="c" and l[w+1] in ["a","e","i","o","u"]:
				pass
			elif l[w-2]=="g" and l[w-1]=="l" and l[w+1] in ["a","e","i","o","u"]:
				pass
			elif l[w+1]=="a" or l[w+1]=="e" or l[w+1]=="o" or l[w+1]=="u":
				t.append("j")
			else:	t.append("i")
		elif l[w]=="j": t.append("j")
		elif l[w]=="k": t.append("k")
		elif l[w]=="l": t.append("l")
		elif l[w]=="m": t.append("m")
		elif l[w]=="n": t.append("n") # Collapsed [n] phonemes.
		elif l[w]=="o": t.append("o") # Collapsed all phonemes.
		elif l[w]=="p": t.append("p")
		elif l[w]=="q":
			if l[w+1]=="u":
				t.append("k")
				t.append("w")
				e=1
			else:	t.append("k")
		elif l[w]=="r": t.append("r")
		elif l[w]=="s": # Collapsed all [s] and [z] phonemes.
			if l[w+1]=="c":
				if l[w+2]=="e" or l[w+2]=="i":
					t.append("s")
				else:
					t.append("s")
					t.append("k")
				e=1
			else:
				t.append("s")
		elif l[w]=="t": t.append("t")
		elif l[w]=="u":
			if l[w+1] in ["a","e","i","o","u"]:
				t.append("w")
			else:	t.append("u")
		elif l[w]=="v": t.append("v")
		elif l[w]=="w": t.append("w")
		elif l[w]=="x": t.append("x")
		elif l[w]=="y": t.append("y")
		elif l[w]=="z": t.append("z") # Collapsed all phonemes.
	if phonmode=="compact" or phonmode=="compactsp": t=simplifier(t)
	return " ".join(t)

#=== END: ITALIAN TRANSLITERATOR ===#

#=== INI: MAIN PROGRAM EXECUTION ===#
def setphonmode(mode):
	global phonmode
	phonmode=mode

def mainloop():
	global phonmode
	phonmode=sys.argv[1]
	f=open(sys.argv[2],"r")
	for l in f:
		line=parseline(l)
		trans=transliterate(line)
		sys.stdout.write(trans+"\n")
	f.close()

def main():
	if len(sys.argv)==3 and ( sys.argv[1]=="full" or sys.argv[1]=="compact" or sys.argv[1]=="fullsp" or sys.argv[1]=="compactsp" ): mainloop()
	else: sys.stderr.write("Usage: "+sys.argv[0]+" [full|compact|fullsp|compactsp] italian.txt\n\t* compact modes does not repeat consecutive phonemes\n\t* sp modes put SP between words and phrases\n")

if __name__ == "__main__":
	main()

#=== END: MAIN PROGRAM EXECUTION ===#

