#!/bin/bash

# Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <tmpdir> <tmpdir-previous-pass> <net> <learning-rate> <momemtum>

  <tmpdir>                      Temporal directory of the alignment estimation
  <tmpdir-previous-pass>        Temporal directory of the previous pass
  <samples-list>                List of samples to be generated
  <net>                         Input Deep Neural Network to be adapted
  <epochs>                      Epochs for the adaptation
  <learning-rate>               Adaptation DNN learning rate
  <momemtum>                    Momentum used in DNN adaptation
  <weigth>                      Interpolation weight between senones

  Options:

  -h:             Print this help
  -c <path>       Path to tLdnn-classify
  -a <path>       Path to tLdnn-adapt
  -o <path>       Path to tLdnn-combine
  -n              Do not combine samples
  -l              Combine adapted samples with this list of samples.
EOF
}

NUMARGS=8
DNNCLASSIFY=tLdnn-classify
DNNADAPTATION=tLdnn-adapt
DNNCOMBINE=tLdnn-combine
DO_COMBINE=1
COMBINE_EXT_LIST=""
while getopts "hc:a:o:nl:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        c) DNNCLASSIFY=$OPTARG;;
        a) DNNADAPTATION=$OPTARG;;
        o) DNNCOMBINE=$OPTARG;;
        n) DO_COMBINE=0;;
        l) COMBINE_EXT_LIST=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
TMP=$1
TMP_PREV=$2
LST=$3
NET=$4
EPOCHS=$5
LR=$6
MOM=$7
WEIGHT=$8
shift $NUMARGS

set -e

echo "#Adapting"
COMMAND="$DNNADAPTATION -seed 7 -epochs $EPOCHS -batch 1000 -lr $LR -mom $MOM \
    -llb -lle -erre \
    -net $NET -out $TMP/adapted.net \
    -tsamples $TMP/align.labels.lst -tlabels $TMP/align.labels \
    -vsamples $TMP/align.labels.lst -vlabels $TMP/align.labels"
echo $COMMAND
$COMMAND

echo "#Generating senones"
cat $LST | awk -v dir=$TMP '{print dir"/senones.adapt/"$1}' > $TMP/dnnfeas-adapt.lst
mkdir -p $(for i in `< $TMP/dnnfeas-adapt.lst`; do 
  dirname $i
done | sort -u)

COMMAND="$DNNCLASSIFY -samples $LST \
                  -net $TMP/adapted.net -batch 200 -osamples $TMP/dnnfeas-adapt.lst"
echo $COMMAND
$COMMAND

if [ $DO_COMBINE -eq 1 ]; then
  echo "#Combining"
  cat $LST | awk -v dir=$TMP '{print dir"/senones.combined/"$1}' > $TMP/dnnfeas-combined.lst2
  mkdir -p $(for i in `< $TMP/dnnfeas-combined.lst2`; do 
    dirname $i
  done | sort -u)

  if [ "$COMBINE_EXT_LIST" != "" ]; then
    SEN2="$COMBINE_EXT_LIST"
  else
    SEN2="$TMP_PREV/senones.lst"
  fi

  WEIGHT2=`echo 1.0 $WEIGHT | awk '{print $1-$2}'`
  COMMAND="$DNNCOMBINE -prod -c1 $TMP/dnnfeas-adapt.lst -w1 $WEIGHT -c2 $SEN2 -w2 ${WEIGHT2} -out $TMP/dnnfeas-combined.lst2"
  echo $COMMAND
  $COMMAND

  mv $TMP/dnnfeas-combined.lst2 $TMP/dnnfeas-combined.lst

else

  ln -s "$TMP/dnnfeas-adapt.lst" "$TMP/dnnfeas-combined.lst"

fi

