#!/bin/bash

# Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <A_samples-list> <B_samples-list> <outdir> <weight>

  <A_samples-list>                Samples generated from system A.
  <B_samples-list>                Samples generated form system B.
  <outdir>                        Output directory
  <weigth>                        Interpolation weight between senones

  Options:

  -h:             Print this help
  -o <path>       Path to tLdnn-combine
EOF
}

NUMARGS=4
DNNCOMBINE=tLdnn-combine
while getopts "hc:a:o:n:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        o) DNNCOMBINE=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
ALIST=$1
BLIST=$2
OUTDIR=$3
WEIGHT=$4
shift $NUMARGS

set -e

echo "#Combining"
cat $ALIST | awk -v dir=$OUTDIR '{print dir"/combined.senones/"$1}' > $OUTDIR/combined_senones.lst
mkdir -p $(for i in `< $OUTDIR/combined_senones.lst`; do 
  dirname $i
done | sort -u)

WEIGHT2=`echo 1.0 $WEIGHT | awk '{print $1-$2}'`
COMMAND="$DNNCOMBINE -prod -c1 $ALIST -w1 $WEIGHT -c2 $BLIST -w2 ${WEIGHT2} -out $OUTDIR/combined_senones.lst"
echo $COMMAND
$COMMAND
