#!/bin/bash 

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

# This script is intended to give fast access through the network

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

set -eu

function cache_get_record_cache {
    line="$*";
    declare -a records
    sep=0;  # have we seen a ':' previously ?
    first=1; # is the first words?
    for ((pos=0;pos<${#line};++pos)); do
        if [ "${line:$pos:1}" == ":" ]; then
            if [ "$sep" -eq "1" ]; then
                pp=$[ ${#records[@]} -1 ];
                records[$pp]+="${line:$pos:1}";
                sep=0; first=0;
            else
                sep=1;
                first=1;
            fi;
        else
            if [ $first -eq "1" ]; then 
                records+=(${line:$pos:1});
            else
                pp=$[ ${#records[@]} -1 ];
                records[$pp]+="${line:$pos:1}";
            fi;
            first=0;  sep=0;
        fi;
    done;
    _output="("
    for i in ${!records[@]}; do 
        _output+=" \"${records[$i]}\"";
    done;
    _output+=" )";
    #echo declare -a _record=$_output;
    echo $_output;
}

function cache_get_cache_filename_dir {
 local kk;
 kk=${1%/*};
 echo ${kk##/}
}

function cache_get_cache_filename {
     echo ${1##/};
}

function cache_get_field_cache {
    if [ "$#" -ne 2 ]; then     echo "";   return 1;  fi;
    eval declare -a _record=`cache_get_record_cache "$1"`;
    echo ${_record[$[$2-1]]}
}

function cache_format_record {
 if [ $# -gt 6 ]; then return 1; fi;
  USER=`cache_preprocess_string "$1" `;
  f=`cache_preprocess_string "$2" `;
  fts=`cache_preprocess_string "$3" `;
  fc=`cache_preprocess_string "$4" `;
  fcts=`cache_preprocess_string "$5" `;
  if [ $# -eq 6 ]; then 
    list=`cache_preprocess_string "$6" `;
    echo "$USER:$f:$fts:$fc:$fcts:$list";
  else
    echo "$USER:$f:$fts:$fc:$fcts";
  fi
}

# get a record from the DB with the record $4
function cache_get_record_from_file {
    if [ "$#" -ne 2 ]; then     echo "";   return 1;  fi;
    local DB="$1"
    local file="$2"
    local _output=""
#    echo "file: $file" >/dev/stderr
#    echo "DB: $DB" >/dev/stderr
    while read line; do 
        eval declare -a _record=`cache_get_record_cache "$line"`;
#        echo "LINE: $line" >/dev/stderr
#        echo " [[ ${_record[1]} == $file ]]" >/dev/stderr
#        echo " ${_record[@]} " >/dev/stderr
        if [[ ${_record[1]} == $file ]]; then
#            echo "JAJAJAJA">/dev/stderr
            _output="$line"
#        else
#            echo "MMUAAA" >/dev/stderr
        fi;
    done<"$DB"
    echo "$_output"
}

# make sure that : occurrences are duplicated
function cache_preprocess_string {
    echo "$*" | sed -e 's/:/::/g'
}

# generates a unique url
function cache_get_unique_url {
if [ $# -ne 1 ]; then echo "error_no_path"; return 1; fi  ;

if [ ${1:0:1} == "/" ]; then url="$1"; else url="$PWD/$1"; fi;

#Is a symbolic link
if [ -h $1 ]; then 
  kk=`readlink -e $url`
  if [ -z $url ]; then
        echo "/error_asking_for_cache: file $1 points to a non-existing file";
        exit 1;
  fi
  url=$kk
fi

url=`$DB_BIN/uniq_path "$url"`
echo $url
}

function cache_write_to_cache {
  if [ $# -lt 3 ]; then return 1; fi;
  #record=`cache_preprocess_string "$1" `;# already done in get_cache
  local record="$1"
#  eval declare -a _record=`cache_get_record_cache "$record"`;

#  USER="${_record[0]}"; f="${_record[1]}"; fts="${_record[2]}"; fc="${_record[3]}"; fcts="${_record[4]}";
  local f="$2"
  local DB="$3"
  if [ $# -gt 3 ]; then 
    # todo compres DB for $f
    while read line; do
        if [ "`cache_get_field_cache "$line" 2`" != "$f" ]; then
            echo "$line"
        fi;
    done <"$DB" >"$DB".back
    ${DB_RM} "$DB"
    if [ -s "$DB.back" ]; then
      mv "$DB.back" "$DB"
    fi
  fi;
  echo "$record" >> "$DB"
}

function cache_erase_record {
  local record="$1"

  while read line; do
    if [ "`cache_get_field_cache "$line" 2`" != `cache_get_field_cache "$record" 2` ]; then
        echo "$line"
    fi;
  done <"$DB" >"$DB".back
  ${DB_RM} "$DB"
  mv "$DB.back" "$DB"
}

function mutex_in {
  ##preserve previous opts
  #local BASHOPS="$-"
  #set +e
  #if [ -f "$DB.lock" ]; then
  #   lockfile-check --use-pid "$DB"
  #   if [ $? -eq 255 ]; then #wrong lockfile
  #     warning_msg "Wrong lockfile $DB.lock (possibly killed proc?). Deleting it"
  #     lockfile-remove "$DB"
  #   fi
  # fi
  debug_msg "Trying to lock $DB"
  lockfile-create --retry $RETRY_ATTEMPS --use-pid "$DB"
  if [ $? -eq 4 ]; then #retry attemps reached
    warning_msg "Cache DB could not be locked. Aborting cache"
    echo 0
  else
    echo 1
  fi
  #lockfile-touch "$DB" &
  #BADGER="$!";
  #set "-$BASHOPS"
}

function mutex_out {
  lockfile-remove $DB
}


# Delete from cache files that were last accessed one day ago
function clean_old_cache_files {
  CACHE_DIR="$BASE_CACHE_DIR"

  if [ ! -e "$DB" ]; then 
    return
  fi

  mutex_in

  i=1; delete_fields=""
  while read line; do
    declare -a record=`cache_get_record_cache $line`

    f=${record[3]}
    f_modified=`date --date="${record[4]}" +"%s"`
    too_old=`date --date="3 day ago" +"%s"`

    #delete if older than five days
    if [ "$f_modified" -lt "$too_old" ]; then
      delete_record $record
      delete_fields="$delete_fields $i"
    fi
    i=$[i+1]
  done < "$DB"
  
  if [ ! -z "$delete_fields" ]; then
    gawk -v d="$delete_fields" \
      'BEGIN{ split(d,a," "); i=1 }
      {
        if(NR == a[i]) 
          i++; 
        else 
          print $0
      }' "$DB" > "$DB".bk
    mv "$DB".bk "$DB"
  fi
  mutex_out
}

function mutex_generic_in {
  local lock=$1
  debug_msg "Trying to lock $lock"
  lockfile -5 -r 720 $lock.lock
  if [ $? -ne 0 ]; then #retry attemps reached
    warning_msg "File $lock could not be locked. Aborting cache"
    echo 0
  else
    debug_msg "Locked $lock.lock"
    echo $HOSTNAME > $lock
    JOB_ID=${JOB_ID:=""}
    if [ "$JOB_ID" ]; then
      echo "JOB ID: $JOB_ID" >> $lock
    fi
    SGE_TASK_ID=${SGE_TASK_ID:=""}
    if [ "$SGE_TASK_ID" ]; then
      echo "SGE_TASK_ID: $SGE_TASK_ID" >> $lock
    fi
    echo 1
  fi
}

function mutex_generic_out {
  local lock=$1
  if [ ! -f $lock ]; then
    warning_msg "Missing file $lock"
  fi
  rm -rf $lock $lock.lock
}

function copy_file {
  local orig=$1
  local dest=$2

  #only one copy of a file 
  cp "$orig" "$dest"
}

function copy_files_in_list {
  local f
  local cache_file
  local file
  local list=$1
  local cache_dir=$2
  #create dirs
#for dir in `cat "$1" | sed -e 's+/[^/]*$++' | sort -u`; do
  mkdir -p $(for file in `< "$list"`; do
    f=`cache_get_unique_url $file`
    cache_file=`cache_get_cache_filename $f`
    echo "$cache_dir"/"$cache_file"
  done | sed -e 's+/[^/]*$++' | sort -u)

  #copy files
  for file in `< "$1"`; do
    f=`cache_get_unique_url $file`
    cache_file=`cache_get_cache_filename $f`
    cp "$file" "$cache_dir"/"$cache_file"
    echo "$cache_dir"/"$cache_file"
  done
}

function empty_space_in_dir {
  local dir=$1

  echo `df $dir | tail -n 1 | gawk '{print $4*1024}'`
}

function is_local_filesystem {
  local dir=$1

  IS_ORIG_NFS=`df -l "$1" &> /dev/null; echo $?`
  if [ "$IS_ORIG_NFS" -eq "0" ]; then
    echo 1
  else
    echo 0
  fi
}

function delete_record {
  declare -a record="( ${!1} )"

  if [ ${#record[@]} -eq 8 ]; then
    if [ "${record[7]}" = "list" ]; then #file is a list
      for file in `< "${record[4]}"`; do
        rm "$file"
      done
    fi
  fi
  rm "${record[4]}"

}

function delete_oldest_entry {
  local DB=$1
  while read line; do
    READ_TIME=`cache_get_field_cache "$line" 5`
    echo `date --date="$READ_TIME" +%s` $line
  done < "$DB" | sort -k 1 | cut -d ' ' -f 2- > "$DB.$$";

  #delete the first entry only 
  line=`head -n 1 "$DB.$$"`
  declare -a record=`cache_get_record_cache $line`
  debug_msg "Deleting ${record[*]}"
  delete_record "record[@]"

  #store rest 
  tail -n +2 "$DB.$$" > "$DB"
  rm -f "$DB.$$"
}

function warning_msg {
  local msg=$1
  echo "[WW] get_cache.sh `date` : $msg" 1>&2
} 

function error_msg {
  local msg=$1
  echo "[EE] get_cache.sh `date` : $msg" 1>&2
} 

function debug_msg {
  local msg=$1
  echo "[II] get_cache.sh `date` : $msg" 1>&2
}

function print_help {
    cat <<EOF
$0 [options] <file> 

  <file>        File to be cached in a local dir

  Options:

  -h:               : Print this help
  -t  <tmpdirname>  : Directory to store the files and DB (default "/home/tmp")
  -d  <dbname>      : Name of the local cache database ( default "\$HOST.\$USER.db" )
  -l                : File to put in cache contains a list of files to put in cache
  -r  <path>        : Path to the lock file. By default, a file name
                      $HOME/.${HASH_ORIG} is created. It contains the machine that is reading
  -m  <min_bytes>   : Minimum number of bytes required /home/tmp. If this number is not met
                      old files are deleted (default 1GB) .
  -x                : Remove the cache before start
  -p                : Returns a hardlink duplicate of the cached file (in order to avoid being removed).
                      Users are in charge of deleting this duplicate. Duplicates are stored in 
                      tmpdirname/files.\$USERNAME

EOF
}

###########################################################################################
###########################################################################################
###########################################################################################
###########################################################################################

USER=`whoami`
DB_DIR=/home/tmp
DB_BIN=/home/tmp/bin
DB_NAME=$HOSTNAME.$USER
DB_RM=/bin/rm
HASH_ORIG="get_cache-copy"
myHOME=${myHOME:-}
if [ ! -z "$myHOME" ]; then 
    HOME_LOCK=$myHOME/.${HASH_ORIG}
else
    HOME_LOCK=$HOME/.${HASH_ORIG}
fi;

MIN_SPACE=10737418240 #10GB
RETRY_ATTEMPS=30 # over 1 hour
LIST="no"
DELETE="no"
HARDLINK="no"

NUMARGS=1
while getopts "ht:d:lr:m:xp" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        t) DB_DIR=$OPTARG;;
        d) DB_NAME=$OPTARG;;
        l) LIST="yes";;
        r) HOME_LOCK="$OPTARG";;
        m) MIN_SPACE=$OPTARG;;
        x) DELETE="yes";;
        p) HARDLINK="yes";;
  esac
done
BASE_CACHE_DIR=${DB_DIR}/files.$USER

[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
FILE_RAW=$1
FILE=`$READLINK $1` #Unique file name
shift $NUMARGS

#Set DB name
DB="$DB_DIR/$DB_NAME.db"

if [ "$DELETE" = "yes" ]; then
  debug_msg "Removing all cache related files in this machine" 
  rm -rf $DB $DB.lock* $BASE_CACHE_DIR
fi

#Check if file is valid
if [ ! -e $FILE ] || [ ! -r $FILE ]; then
  error_msg "File not exists or it not readable"
  exit 1
fi

debug_msg "Caching file\"$FILE\""

#Check if we are in local machine and abort if so
if [ `is_local_filesystem $FILE` -eq 1 ]; then
  warning_msg "Local Machine. Aborting cache"
  echo $FILE_RAW #return the argname to allow comparison to check if its the same
  exit 0
fi

debug_msg "Locking home lock $HOME_LOCK"
if [ `mutex_generic_in "$HOME_LOCK"` -eq 0 ]; then
  echo $FILE_RAW
  exit 0
fi
TRAP_COMMAND_ERR="warning_msg \"An error ocurred. Unlocking $HOME_LOCK\"; mutex_generic_out $HOME_LOCK"
trap "$TRAP_COMMAND_ERR" ERR TERM

TRAP_COMMAND_EXIT="debug_msg \"Exiting. Killing touch and removing lock $HOME_LOCK\"; mutex_generic_out $HOME_LOCK"
trap "$TRAP_COMMAND_EXIT" EXIT

#Check if the file is already cached
debug_msg "Checking if file is already cached"
RECORD=""
if [ -s "$DB" ]; then
  RECORD=`cache_get_record_from_file "$DB" "$FILE"` 
else
  debug_msg "Database not created yet"
fi
  
#FILE IS IN CACHE
if [ "$RECORD" ]; then
  CACHE_TIME=`cache_get_field_cache "$RECORD" 5`
  CACHED_FILE=`cache_get_field_cache "$RECORD" 4`;
  a=`date --date="$CACHE_TIME" +%s`
  b=`stat -c %Y $FILE` #last modified
  
  debug_msg "CACHED: $CACHE_TIME ORIGINAL: `stat -c %y $FILE`"
  if [ "$a"  -lt  "$b" ]; then #file has been updated
    # delete its record and file and then cache it the next bucle
    debug_msg "File is outdated. Deleting it from the cache."
    cache_erase_record $RECORD
    $DB_RM $CACHED_FILE
    RECORD=""
  else
    CURRENT_TIME=`date +"%Y-%m-%d %T"`
    FILE_TIME=`cache_get_field_cache "$RECORD" 3`;
    NEW_RECORD=`cache_format_record "$USER" "$FILE" "$FILE_TIME" "$CACHED_FILE" "$CURRENT_TIME"` 
    cache_write_to_cache "$NEW_RECORD" "$FILE" "$DB" erase_duplicates

    if [ "$HARDLINK" = "yes" ]; then
      DUP_FILE="`mktemp $BASE_CACHE_DIR/tmp.XXXXXXXX`-`basename $CACHED_FILE`"
      ln -f $CACHED_FILE $DUP_FILE
      echo $DUP_FILE
    else
      echo $CACHED_FILE;
    fi
    exit 0
  fi
fi

#FILE IS NOT IN CACHE OR OUTDATED
if [ -z $RECORD ]; then

  CACHED_DIR=`cache_get_cache_filename_dir $FILE`
  mkdir -p "$BASE_CACHE_DIR/$CACHED_DIR"
  CACHED_FILENAME="$BASE_CACHE_DIR/$CACHED_DIR/`basename $FILE`"

  debug_msg "Caching file to $CACHED_FILENAME"

  CURRENT_TIME=`date +"%Y-%m-%d %T"`
  FILE_TIME=`stat -c %Y $FILE` 
  FILE_TIME=`date --date="@$FILE_TIME" +"%Y-%m-%d %T"`

  FREE_SPACE=`empty_space_in_dir $DB_DIR`
  if [ $LIST = "yes" ]; then
    FILE_SIZE=`du -s $(< $FILE) $FILE | gawk '{s+=$1}END{print s}'`
  else
    FILE_SIZE=`stat -c %s $FILE`
  fi
  SPACE=$[FREE_SPACE-FILE_SIZE-MIN_SPACE]

  debug_msg "Free space: $FREE_SPACE File size: $FILE_SIZE Min Space: $MIN_SPACE"
  debug_msg "Free space after cache $[FREE_SPACE-FILE_SIZE]"
  #Delete old files to meet space requirements
  while [ $SPACE -lt 0 ]; do #we need more space
    if [ ! -s "$DB" ]; then
      break
    fi
    delete_oldest_entry "$DB"
    FREE_SPACE=`empty_space_in_dir $DB_DIR`
    SPACE=$[FREE_SPACE-FILE_SIZE-MIN_SPACE]
    debug_msg "Free space: $FREE_SPACE File size: $FILE_SIZE Min Space: $MIN_SPACE"
    debug_msg "Free space after cache $[FREE_SPACE-FILE_SIZE]"
  done
  #Space not sufficent even after deleting
  if [ $SPACE -lt 0 ]; then
    warning_msg "Not enough free space to cache file. File will not be cached!"
    echo "$FILE_RAW"
    exit 0
  fi

  debug_msg "Copying file"  
  NEW_RECORD=""
  if [ $LIST = "yes" ]; then
    copy_files_in_list "$FILE" "$BASE_CACHE_DIR" > "$CACHED_FILENAME"
    NEW_RECORD=`cache_format_record "$USER" "$FILE" "$FILE_TIME" "$CACHED_FILENAME" "$CURRENT_TIME" "list"`
  else
    copy_file "$FILE" "$CACHED_FILENAME"
    NEW_RECORD=`cache_format_record "$USER" "$FILE" "$FILE_TIME" "$CACHED_FILENAME" "$CURRENT_TIME"`
  fi
  cache_write_to_cache "$NEW_RECORD" "$FILE" "$DB"

  if [ "$HARDLINK" = "yes" ]; then
    DUP_FILE="`mktemp $BASE_CACHE_DIR/tmp.XXXXXXXX`-`basename $CACHED_FILENAME`"
    ln -f $CACHED_FILENAME $DUP_FILE
    echo $DUP_FILE
  else
    echo $CACHED_FILENAME
  fi
  exit 0
fi

