#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#  Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

#include i18n
import gettext
_ = gettext.gettext

from optparse import OptionParser
import sys

class CWTrans:

    def __init__(self,src,dst,wc):
        self.src= src
        self.dst= dst
        self.wc= wc

    def __repr__(self):
        return u'%s %s %s'%(self.src,self.dst,self.wc)

class Entry:

    def __init__(self,w,p,seq):
        self.w= w
        self.p= p
        self.seq= seq

    def __repr__(self):
        return u'%s %g %s'%(self.w,self.p,u' '.join(self.seq))

def cmp_cw(a,b):
    if a.src<b.src   : return -1
    elif a.src>b.src : return 1
    elif a.dst<b.dst : return -1
    elif a.dst>b.dst : return 1
    elif a.wc<b.wc : return -1
    elif a.wc>b.wc : return 1
    else           : return 0

class Lexicon:

    def __init__(self,iscw=False):
        self.entries= []
        self.iscw= iscw
        if iscw:
            self.wildcards= {}
            self.cwstart= None
            self.cwstop= []
            self.cwtrans= []

    def add(self,entry):
        self.entries.append(entry)

    def write(self,fd):
        if self.iscw:
            fd.write('CWLEXICON\n')
            fd.write('WILDCARDS\n')
            keys= self.wildcards.keys()
            keys.sort()
            for wc in keys:
                aux= list(self.wildcards[wc])
                aux.sort()
                aux= u' '.join(aux)
                fd.write((u'%s %s\n'%(wc,aux)).encode('utf-8'))

            fd.write(('CWSTART %s\n'%self.cwstart).encode('utf-8'))
            tmp= list(self.cwstop)
            tmp.sort()
            aux= u' '.join(tmp)
            fd.write(('CWSTOP %s\n'%aux).encode('utf-8'))
            fd.write('CWTRANS\n')
            self.cwtrans.sort(cmp_cw)
            for c in self.cwtrans:
                fd.write((u'%s\n'%unicode(c)).encode('utf-8'))
            fd.write('ENTRIES\n')
            for e in self.entries:
                fd.write(('%s\n'%unicode(e)).encode('utf-8'))
        else:
            fd.write('LEXICON\n')
            for e in self.entries:
                fd.write(('%s\n'%unicode(e)).encode('utf-8'))

    @staticmethod
    def __get_key(wc,cont):
        l= list(cont)
        l.sort()
        tmp= reduce(lambda x,y:u'%s:%s'%(x,y),l)
        c= wc.split('-')[-1].split('+')[0]
        return u'%s##%s'%(c,tmp)

    def __get_tied_wildcards(self):
        table= {}
        tlist= {}
        nwildcards= {}
        for wc,cont in self.wildcards.iteritems():
            key= Lexicon.__get_key(wc,cont)
            aux= table.get(key)
            if aux==None:
                table[key]= wc
                tlist[wc]= wc
                nwildcards[wc]= cont
            else: tlist[wc]= aux
        self.wildcards= nwildcards
        return tlist

    def join_wildcards(self):
        if not self.iscw: return
        twcs= self.__get_tied_wildcards()
        # Modifica cwtrans
        aux= set()
        for c in self.cwtrans:
            tmp= c.src,c.dst,twcs[c.wc]
            aux.add(tmp)
        self.cwtrans= []
        for a,b,c in aux:
            self.cwtrans.append(CWTrans(a,b,c))
        # Modifica entries
        for e in self.entries:
            e.seq[0]= twcs[e.seq[0]]
            e.seq[-1]= twcs[e.seq[-1]]


def read_lexmono(fn):
    f= open(fn) if fn!=None else sys.stdin
    lex= Lexicon()
    l= f.readline()
    while l!='':
        line= l.strip()
        l= f.readline()
        if line=='': continue
        if line=='LEXICON': break
        else: sys.exit( _( u"unable to read token LEXICON" ) )
    while l!='':
        line= l.strip()
        l= f.readline()
        if line=='': continue
        line= line.decode('utf-8').split()
        if len(line)<3:
            sys.exit( _( u'incomplete lexicon entry: %s' ) %(' '.join(line)))
        lex.add(Entry(line[0],float(line[1]),line[2:]))
    return lex

def read_tlist(fn):
    f= open(fn,'r')
    tlist= {}
    l= f.readline()
    while l!='':
        line= l.strip()
        l= f.readline()
        if line=='': continue
        line= line.decode('utf-8').split()
        if len(line)==1: tlist[line[0]]= line[0]
        elif len(line)==2: tlist[line[0]]= line[1]
        else: sys.exit( _( u"too many fields in '%s': %s" ) %(fn,u' '.join(line)))
    return tlist

def mono2tri(mono,tlist,sil):
    lex= Lexicon()
    for e in mono.entries:
        aux= []
        i= 0
        length= len(e.seq)
        while i < length:
            if e.seq[i]==sil: aux.append(tlist[sil])
            else:
                if i>0 and e.seq[i-1]!=sil: sym= u'%s-%s'%(e.seq[i-1],e.seq[i])
                else:                       sym= e.seq[i]
                if i<(length-1) and e.seq[i+1]!=sil: sym= u'%s+%s'%(sym,
                                                                    e.seq[i+1])
                aux.append(tlist[sym])
            i+= 1
        lex.add(Entry(e.w,e.p,aux))
    return lex

def get_inits(mono,sil):
    ret= set();bsyms= set()
    for e in mono.entries:
        a= e.seq[0]
        bsyms.add(a)
        if a==sil: tmp= a,None,sil
        elif len(e.seq)==1: tmp= a,None,u'*-%s+*'%a
        elif e.seq[1]==sil: tmp= a,sil,u'*-%s'%a
        else:
            b= e.seq[1]
            tmp= a,b,u'*-%s+%s'%(a,b)
        ret.add(tmp)
    return bsyms,ret

def get_cwtrans_and_stops(mono,tlist,sil,wi):
    bsyms,inits= get_inits(mono,sil)
    esyms= set(); ret= {}
    # Obté finals
    for e in mono.entries:
        sym= e.seq[-1]
        if sym!=sil: esyms.add(sym)
        if sym==sil         : ret[sil]= None,sil
        elif len(e.seq)==1  : ret[u'*-%s+*'%sym]= None,sym
        elif e.seq[-2]==sil : ret[u'%s+*'%sym]= sil,sym
        else                : ret[u'%s-%s+*'%(e.seq[-2],sym)]= e.seq[-2],sym
    # Obté stops
    stops= set()
    for k,v in ret.iteritems():
        # Si el final és el silenci
        if v[1]==sil:
            tmp= sil,tlist[sil]
            stops.add(tmp)
        # Si el final no té context a l'esquerra.
        elif v[0]==None:
            tmp= k,tlist[v[1]]  # Cas especial
            stops.add(tmp)
            for e in esyms:
                if e==sil: a= v[1]
                else     : a=u'%s-%s'%(e,v[1])
                tmp= k,tlist[a]
                stops.add(tmp)
        # Si el final és normal o té silenci esquerra
        else:
            if v[0]==sil: a= v[1]
            else        : a= u'%s-%s'%(v[0],v[1])
            tmp= k,tlist[a]
            stops.add(tmp)
    # Força transicions inicials
    ret[sil]= None,sil
    # Crea transicions i stops
    # En principi les transicions que estic generant ací són
    # exclusivament transicions entre paraules (i les inicials que es
    # trauen afegint artificialment el sil com node final). Però quan
    # es transita a un principi sense context a la dreta (paraula d'un
    # únic fonema) hi ha que tindre en compter que la següent paraula
    # a eixa podria ser </s> i per tant ha de poder-se transitar al
    # trifonema sense context a la dreta.
    for k,v in ret.iteritems():
        trans= set()
        # Si el final és el silenci
        if v[1]==sil:
            for j in inits:
                if j[0]==sil:
                    tmp= tlist[sil],tlist[sil],j[2]
                    trans.add(tmp)
                elif j[1]==None:
                    # En aquest cas sempre fique esta transició
                    tmp= tlist[sil],tlist[u'%s'%j[0]],j[2]
                    trans.add(tmp)
                    for b in bsyms:
                        if b!=sil:
                            tmp= tlist[sil],tlist[u'%s+%s'%(j[0],b)],j[2]
                            trans.add(tmp)
                elif j[1]==sil:
                    tmp= tlist[sil],tlist[j[0]],j[2]
                    trans.add(tmp)
                else:
                    tmp= tlist[sil],tlist[u'%s+%s'%(j[0],j[1])],j[2]
                    trans.add(tmp)
        # Si el final no té context a l'esquerra.
        elif v[0]==None:
            for e in esyms:
                if e==sil: a= v[1]
                else     : a=u'%s-%s'%(e,v[1])
                for j in inits:
                    if j[0]==sil:
                        tmp= tlist[a],tlist[sil],j[2]
                        trans.add(tmp)
                    elif j[1]==None:
                        tri= tlist[u'%s+%s'%(a,j[0])]
                        # En aquest cas sempre fique esta transició
                        tmp= tri,tlist[u'%s-%s'%(v[1],j[0])],j[2]
                        trans.add(tmp)
                        for b in bsyms:
                            if b!=sil:
                                tmp= tri,tlist[u'%s-%s+%s'%(v[1],j[0],b)],j[2]
                                trans.add(tmp)
                    elif j[1]==sil:
                        tri= tlist[u'%s+%s'%(a,j[0])]
                        tri2= tlist[u'%s-%s'%(v[1],j[0])]
                        tmp= tri,tri2,j[2]
                        trans.add(tmp)
                    else:
                        tri= tlist[u'%s+%s'%(a,j[0])]
                        tri2= tlist[u'%s-%s+%s'%(v[1],j[0],j[1])]
                        tmp= tri,tri2,j[2]
                        trans.add(tmp)
                        # Permet nocrosswords
                        if wi:
                            tri= tlist[a]
                            tri2= tlist[u'%s+%s'%(j[0],j[1])]
                            tmp= tri,tri2,j[2]
                            trans.add(tmp)

        # Si el final és normal o té silenci esquerra
        else:
            if v[0]==sil: a= v[1]
            else        : a= u'%s-%s'%(v[0],v[1])
            for j in inits:
                if j[0]==sil:
                    tmp= tlist[a],tlist[sil],j[2]
                    trans.add(tmp)
                elif j[1]==None:
                    tri= tlist[u'%s+%s'%(a,j[0])]
                    # En aquest cas sempre fique esta transició
                    tmp= tri,tlist[u'%s-%s'%(v[1],j[0])],j[2]
                    trans.add(tmp)
                    for b in bsyms:
                        if b!=sil:
                            tmp= tri,tlist[u'%s-%s+%s'%(v[1],j[0],b)],j[2]
                            trans.add(tmp)
                elif j[1]==sil:
                    tri= tlist[u'%s+%s'%(a,j[0])]
                    tri2= tlist[u'%s-%s'%(v[1],j[0])]
                    tmp= tri,tri2,j[2]
                    trans.add(tmp)
                else:
                    tri= tlist[u'%s+%s'%(a,j[0])]
                    tri2= tlist[u'%s-%s+%s'%(v[1],j[0],j[1])]
                    tmp= tri,tri2,j[2]
                    trans.add(tmp)
                    # Permet nocrosswords
                    if wi:
                        tri= tlist[a]
                        tri2= tlist[u'%s+%s'%(j[0],j[1])]
                        tmp= tri,tri2,j[2]
                        trans.add(tmp)
        ret[k]= trans
    return ret,stops



def mono2cwtri(mono,tlist,sil,wi):
    cwtrans,stops= get_cwtrans_and_stops(mono,tlist,sil,wi)
    lex= Lexicon(True)
    lex.wildcards[sil]= set([tlist[sil]])
    lex.cwstart= tlist[sil]
    for wc,sym in stops:
        aux= lex.wildcards.get(wc)
        if aux == None:
            lex.wildcards[wc]= aux= set()
        aux.add(sym)
    lex.cwstop= set([i[1] for i in stops])
    lex.cwstop.add(tlist[sil])
    aux= set()
    for k,v in cwtrans.iteritems():
        for i in v:
            aux2= lex.wildcards.get(k)
            if aux2 == None:
                lex.wildcards[k]= aux2= set()
            aux2.add(i[0])
            aux2= lex.wildcards.get(i[2])
            if aux2 == None:
                lex.wildcards[i[2]]= aux2= set()
            aux2.add(i[1])
            tmp= i[0],i[1],i[2]
            aux.add(tmp)
    lex.cwtrans= [CWTrans(i[0],i[1],i[2]) for i in aux]
    for e in mono.entries:
        aux= []
        i= 0
        length= len(e.seq)
        sym= e.seq[0]
        if sym==sil        : aux.append(sym)
        elif length==1     : aux.append(u'*-%s+*'%sym)
        elif e.seq[1]==sil : aux.append(u'*-%s'%sym)
        else               : aux.append(u'*-%s+%s'%(sym,e.seq[1]))
        i+= 1
        while i < length-1:
            if e.seq[i]==sil: aux.append(tlist[sil])
            else:
                if e.seq[i-1]!=sil: sym= u'%s-%s'%(e.seq[i-1],e.seq[i])
                else              : sym= e.seq[i]
                if e.seq[i+1]!=sil: sym= u'%s+%s'%(sym,e.seq[i+1])
                aux.append(tlist[sym])
            i+= 1
        if length>1:
            sym= e.seq[-1]
            if sym==sil         : aux.append(sym)
            elif e.seq[-2]==sil : aux.append(u'%s+*'%sym)
            else                : aux.append(u'%s-%s+*'%(e.seq[-2],sym))
        lex.add(Entry(e.w,e.p,aux))
    lex.join_wildcards()
    return lex

parser= OptionParser(usage='%prog [options] <tlist>')
parser.add_option('-c','--crosswords',action='store_true',dest='cw',
                  default=False,help= _( u"make a cross-word lexicon" ) )
parser.add_option('-i','--input',dest='input',
                  help= _( u"monophoneme lexicon" ) ,default=None,metavar='FILE')
parser.add_option('-o','--output',dest='output',
                  help= _( u"file where the triphoneme lexicon will be written" ) ,
                  default=None,metavar='FILE')
parser.add_option('-s','--sil',dest='sil',
                  help= _( u"symbol used as silence. Default value is SP" ) ,
                  metavar='SYM',default='SP')
parser.add_option('--word-internal',action='store_true',dest="wi",
                  default=False,
          help= ( _( u"allows word internal transitions in a crossword lexicon") ) )
(opts,args)= parser.parse_args()
if len(args)!=1:
    parser.error( _( u"wrong number of arguments" ) )
lexmono= read_lexmono(opts.input)
tlist= read_tlist(args[0])
if opts.cw: lextri= mono2cwtri(lexmono,tlist,opts.sil,opts.wi)
else:       lextri= mono2tri(lexmono,tlist,opts.sil)
f= open(opts.output,'w') if opts.output!=None else sys.stdout
lextri.write(f)
