#!/bin/bash

# Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <tmpdir> 

  <tmpdir>        Temporal directory of the estimation
  <splits>        Number of splits in previous align

  Options:

  -h:             Print this help
  -e <path>       Path to tLbin2txt
  -r <list>       Rename sample names from this list
EOF
}

NUMARGS=2
BINTEXTEX=tLbin2txt
ADDITIONAL_COUNTS=""
RENAME_SAMPLES=""
while getopts "he:r:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) BINTEXTEX=$OPTARG;;
        r) RENAME_SAMPLES=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
TMP=$1
SPLITS=$2
shift 1

set -e

COMMAND="$BINTEXTEX amodel -i $TMP/model_0 -o $TMP/model_0"
echo $COMMAND
$COMMAND

ALIGNS=""
for (( i=1;i<=$SPLITS;i++ )); do
  if [ -s $TMP/align_$i ]; then
    ALIGNS="$ALIGNS $TMP/align_$i"
  else
    echo "[EE] Missing align file $TMP/align_$i" >&2
    exit 1
  fi
done

cat $ALIGNS > $TMP/align_all

COMMAND="phos-wrapper.py $TMP/model_0 $TMP/align_all"
echo "$COMMAND"
$COMMAND > $TMP/align.labels

tail -n +2 $TMP/align.labels | awk '{print $NF}' | uniq > $TMP/align.labels.lst

if [ "$RENAME_SAMPLES" != "" ]; then

  mv $TMP/align.labels $TMP/align.labels.old
  mv $TMP/align.labels.lst $TMP/align.labels.lst.old

  cp "$RENAME_SAMPLES" $TMP/align.labels.lst

  paste -d '|' $TMP/align.labels.lst.old "$RENAME_SAMPLES" |
    awk -v f=$TMP/align.labels.old 'BEGIN { printf("sed") } { printf(" -e \"s|%s|g\"", $0) } END { printf(" %s\n",f) }' |
    bash > $TMP/align.labels
fi

