#!/bin/bash 
#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"

. `dirname $0`/mhrecognise_prolog.sh

function print_help {
  echo $"$0 [options] <tmpdir> <language-model> <acoustic-model> [options-for-tLrecognise]

  <tmpdir>              Temporal directory created by mhrecognise_init.sh
  <language-model>      Path to the language model file in TLK format
  <acoustic-model>      Path to an acoustic TLK model

Options:

  -h:                   Print this help
  -r <path>             tLrecognise PATH
  -g <command>          get_cache.sh command plus options to be used
  -w                    generate word-graphs
  -f                    forces the recognition even if there is an existing one"
}

NUMARGS=3
TLRECOGNISE_EX=tLrecognise
GET_CACHE="get_cache.sh"
WORDGRAPH="no"
FORCE="no"
CACHE_FILES="no"
while getopts "hr:tg:wf" OPT
do
    case $OPT in
        h) print_help; exit 0;;
        r) TLRECOGNISE_EX=$OPTARG;;
        t) CACHE_FILES="yes";;
        g) GET_CACHE="$OPTARG";;
        w) WORDGRAPH="yes";;
        f) FORCE="yes";;
    esac
done
[ $[$#-OPTIND+1] -ge $NUMARGS ] || { print_help >&2; echo $[$#-OPTIND+1] $NUMARGS; exit 1; }
shift $[OPTIND-1]
TMP=$1
LM=$2
AMODEL=$3
shift 3
OPTS_ORIG="$@"
OPTS=$OPTS_ORIG

SGE_TASK_ID=${SGE_TASK_ID:-1}
i=$SGE_TASK_ID

#prepare default epilog
TRAPCOMMANDS="date >> '$TMP/tmp/outs_${i}.date' "
trap "$TRAPCOMMANDS" EXIT

if [ "$WORDGRAPH" = "yes" ]; then
  OPTS="$OPTS -W $TMP/tmp/wgs_${i}"
fi

# Check input features list
if [ ! -s "$TMP/tmp/feas_${i}" ]  ; then 
    error $"sample list file '$TMP/tmp/feas_${i}' is empty"
fi

# Abort if list has been already recognised
if [ "$FORCE" == "no" ]; then
  NSAMPLES=`wc -l< "$TMP/tmp/feas_${i}" `
  if [ -s "$TMP/tmp/outs_${i}" ] ; then
      NREC=`grep '^\.$' "$TMP/tmp/outs_${i}" | wc -l`
      if [ "$NREC" -eq "$NSAMPLES" ] ; then
          log $"Skipping recognition. Found valid part '$TMP/tmp/outs_${i}'"
          exit 0
      fi
  fi
fi

# Cache large model files if necessary
if [ "$CACHE_FILES" == "yes" ]; then
  LM_CACHE=`$GET_CACHE -p $LM`
  AMODEL_CACHE=`$GET_CACHE -p $AMODEL`
  [ "$LM_CACHE" == "$LM" ] || \
    TRAPCOMMANDS="$TRAPCOMMANDS ; log 'deleting cached file $LM_CACHE' >&2 ; rm '$LM_CACHE'"
  [ "$AMODEL_CACHE" == "$AMODEL" ] || \
    TRAPCOMMANDS="$TRAPCOMMANDS ; echo 'deleting cached file $AMODEL_CACHE' >&2 ; rm '$AMODEL_CACHE'"
  trap "$TRAPCOMMANDS" EXIT
else
  LM_CACHE=`$READLINK $LM`
  AMODEL_CACHE=`$READLINK $AMODEL`
fi

hostname > "$TMP/tmp/outs_${i}.host"
date > "$TMP/tmp/outs_${i}.date"


# Continue of perform a new recognition
COMMAND="$TLRECOGNISE_EX $LM_CACHE $AMODEL_CACHE $TMP/tmp/feas_${i} $OPTS -o $TMP/tmp/outs_${i}"

# Check if there is an existing output file 
if [ "$FORCE" == "no" ] && [ -s "$TMP/tmp/outs_${i}" ] ; then
      if perl -we ' use strict; 
              my @f ; open(F, $ARGV[0]) or die ("[EE] Could not open feature list file $ARGV[0]");
              @f=<F>; chomp @f;  close(F); # Read feature lists
              my %h ; @h{@f} = (1) x @f;   # feautures list to hash
              # Read transcriptions. Drop uncompleted transcriptions (not ending with \.). 
              # If more than one for the same file, keep last.
              open(F, $ARGV[1]) or die ("[EE] Could not open previous transcription file $ARGV[1]");
              my %f_transcripted;
              my $f_current; 
              my $t="";
              while(<F>){
                  $t.=$_;
                  if (/^\.\s*$/) {
                  die ("Found Transcribed file is corrupted:  @ARGV") if $t=~ /^\s*$/;
                  $f_transcripted{$f_current}=$t;
                  $t="";
                  }
                  if (/^\"(.*)\"\s*$/) {
                  $f_current=$1; 
                  unless  (exists($h{$f_current})) {
                      print STDERR "[II] Possible mistmatch between feature list file and previous transcripted file: @ARGV. Found a transcripted fea. file not listed ($f_current)\n";
                  }; 
                  $h{$f_current}=0;
                  }
              }
              close (F);
              open (O, ">$ARGV[1].tmp") or die ("[EE] Could not create temporary file with for the validated previous transcriptions\n");
              my $abort=0;
              for (@f) {
                  if ($abort || ($h{$_}==1) ) {
                      print STDOUT "$_\n";
                      $abort=1;
                  } else {
                      print O "$f_transcripted{$_}";
                  }
              }
              close(O);
          ' "$TMP/tmp/feas_${i}" "$TMP/tmp/outs_${i}" > "$TMP/tmp/feas_${i}.cont" ; then 
        mv "$TMP/tmp/outs_${i}.tmp" "$TMP/tmp/outs_${i}"
        if [ "$WORDGRAPH" == "yes" ]; then
          cat $TMP/tmp/feas_${i}.cont | sed -e "s+.+$TMP/wordgraphs/&+" > $TMP/tmp/wgs_${i}.cont
          OPTS="$OPTS_ORIG -W $TMP/tmp/wgs_${i}.cont"
        fi

        NREC=`grep '^"' "$TMP/tmp/outs_${i}" | wc -l`
        log $"Found partial recognition  $TMP/tmp/outs_${i} ($NREC valid samples of $NSAMPLES). Continuing... "
        COMMAND="$TLRECOGNISE_EX $LM_CACHE $AMODEL_CACHE $TMP/tmp/feas_${i}.cont $OPTS"
        log $"Executing: $COMMAND >> $TMP/tmp/outs_${i}"
        $COMMAND >> $TMP/tmp/outs_${i}
      else
          # recognising from scratch
          warning $"Ignoring previous recognition of '$TMP/tmp/outs_${i}'. Possible mistmatch against $TMP/tmp/feas_${i}. "
          log $"Executing: $COMMAND"
          $COMMAND
      fi
else
# recognising from scratch
  log $"Executing: $COMMAND"
  $COMMAND
fi


#######################################
# Check resulting transcription part
NREC=0
if [ -s "$TMP/tmp/outs_$i" ] ; then 
    NREC=`grep '^\.$' "$TMP/tmp/outs_$i" | wc -l`
fi
if [ "$NSAMPLES"  -eq "$NREC" ] ; then
    log $"Recognized $NREC samples ($TMP/tmp/outs_$i)"
    touch $TMP/tmp/outs_$i.done.back; mv $TMP/tmp/outs_$i.done.back $TMP/tmp/outs_$i.done
    exit 0
else
    error $"Recognized only $NREC of $NSAMPLES samples ($TMP/tmp/outs_$i) "
fi

