#!/bin/bash 
#
#  Copyright 2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

. `dirname $0`/mhrecognise_prolog.sh

function print_help {
  echo $"$0 [options] <tmpdir> <splits> <output>

  <tmpdir>                    Temporal directory to store temporary files for the recognition 
  <split>                     Number of splits to divide the sample list
  <output>                    Path prefix name to store the recognised transcriptions

Options:

  -h:                         Print this help
  -p:                         Generate monophoneme transcription files from the transcription
  -c:                         Extract confidence measures from the wordgraphs
  -a <float>                  Weight applied in confidence measure extraction. Default 10"
}

NUMARGS=3
GENERATE_PHOS="no"
GENERATE_CM="no"
ASF=10
while getopts "hpca:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        p) GENERATE_PHOS="yes";;
        c) GENERATE_CM="yes";;
        a) ASF=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

TMP=$1
SPLITS=$2
OUTPUT=$3

#join all outputs
AUX="$TMP/tmp/outs_1"
for (( n= 2; n <= $SPLITS; ++n )) ; do
  AUX="$AUX $TMP/tmp/outs_$n"
  echo `cat "$TMP/tmp/outs_$n.date"` " " `cat "$TMP/tmp/outs_$n.host"`
done > $OUTPUT.timelog
cat $AUX > "$OUTPUT.align"

#ouput raw txt transcriptions
cat "$OUTPUT.align" | \
    perl -ne 'next if /^\"/ ;  
            if (/^\./) 
            { 
              print "\n"; 
              next 
            } ;  
            @_=split ; 
            print "$_[4] " if $_[3]==0 ' > "$OUTPUT"

#generate phonetic transcription for later alignment
if [ "$GENERATE_PHOS" == "yes" ]; then
  #create directories
    cat "$OUTPUT.align" |
    grep -e '^".*"$' | 
    sed -e 's+^"++' -e 's+"$++' |
    gawk -vTMP="$TMP" '{print gensub("(.*)/.*",TMP"/tmp/results_phos/\\1",1)}' |
    sort -u |
    while read l
    do
	mkdir -p $l
    done

  gawk -v phos_dir=$TMP/tmp/results_phos '  $0 ~ /^".*"$/ { 
            trans = "";
            file = gensub(/^"(.*)"$/, phos_dir"/\\1", "g", $0);
          }
          $0 !~ /^".*"$/ && $0 !~ /^\.$/ {
            pho = $3;
            if(trans == "")
              trans = pho;
            else
              trans = trans" "pho
          }
          $0 ~ /^\.$/ {
            print trans > file;
            print file
          }' "$OUTPUT.align" > "$TMP/tmp/phos.lst"
  for i in $(cat "$TMP/tmp/phos.lst"); do
    cat "$i" | sed -e 's|.-||g' -e 's|+.||g' > "$i.monopho"
    echo "$i.monopho"
  done > "$TMP/tmp/monophos.lst"
fi

#extract CMs
if [ "$GENERATE_CM" == "yes" ]; then
  awk -v tmp=$TMP '{ n=$1; sub(tmp"/wordgraphs/",tmp"/cms/",n); print n }' $TMP/tmp/wgs.lst > $TMP/tmp/cms.lst
#cat $TMP/tmp/wgs.lst | sed -e "s+^$TMP/wordgraphs/+$TMP/cms/+" > $TMP/tmp/cms.lst
  mkdir -p `cat $TMP/tmp/cms.lst | sed -e 's+^\(.*\)/.*$+\1+' | sort -u`
  while read WG CM; do
    if [ -f $WG ]; then
      wgraph.py -c -a $ASF < $WG > $CM
    else
      warning $"Could not find wordgraph $WG confidence measure will not be extracted"
    fi
  done < <( paste -d ' ' $TMP/tmp/wgs.lst $TMP/tmp/cms.lst )

  cat $TMP/tmp/wgs.lst | while read WG; do
    if [ -f $WG ]; then
      echo "\"$WG\""
      wgraph.py -c --tLalign < "$WG"
    fi
  done > "$TMP/tmp/wg.tLalign"
  cat $TMP/tmp/cms.lst | while read CM; do
    if [ -f $CM ]; then
     awk '{print $1,$2}' $CM
    fi
  done > "$TMP/tmp/tx_cms.txt"

fi

#output reference in sclite and srt format
cat "$OUTPUT.align" | tLoutalign2ctm.sh > "$OUTPUT.ctm"
if [ "$GENERATE_CM" == "no" ]; then
    tLoutalign2dfxp.py "$OUTPUT.align" > "$OUTPUT.dfxp"
else
    tLoutalign2dfxp.py "$TMP/tmp/wg.tLalign" -S -c "$TMP/tmp/tx_cms.txt" > "$OUTPUT.dfxp"
fi
dfxp2srt.py "$OUTPUT.dfxp" > "$OUTPUT.srt"

#add cms if necessary
if [ "$GENERATE_CM" == "yes" ]; then
  cat "$OUTPUT.ctm" | awk -v cms_dir=$TMP/cms '{
    if ( system( "[ -f "cms_dir"/"$1" ] " ) == 0 ) 
    { 
      chars=(getline line < (cms_dir"/"$1)); 
      split(line,a," "); 
      print $0,a[2] 
    } 
    else 
      print $0 }' > "$OUTPUT.ctm2"
    mv "$OUTPUT.ctm2" "$OUTPUT.ctm"
fi

#delete CMLLR samples prefix names
for i in "$OUTPUT.align" "$OUTPUT.ctm"; do
    cat $i | 
    gawk -vTMP="$TMP/tmp/fea.cmllr/" -vTMP2="$TMP/tmp/pre-segment/" -vTMP3="$TMP/tmp/senones/"  'BEGIN{
      ltmp=length(TMP);
      ltmp2=length(TMP2);
      ltmp3=length(TMP3);
    }
    { 
      s = $0; 
      while((i = index(s,TMP)) != 0) 
      {
        s = substr(s,0,i-1)""substr(s,i+ltmp,length(s))
      } 
      while((i = index(s,TMP2)) != 0) 
      {
        s = substr(s,0,i-1)""substr(s,i+ltmp2,length(s))
      } 
      while((i = index(s,TMP3)) != 0) 
      {
        s = substr(s,0,i-1)""substr(s,i+ltmp3,length(s))
      } 
      print s
    }' > $i.2
  mv $i.2 $i
done

