#!/usr/bin/python
# -*- coding: utf-8 -*-

# segment_pre-cmllr.py: shorten samples heuristically
# Copyright 2013,2014 transLectures-UPV Team
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os,sys, math
import argparse
import tempfile
import subprocess
import shlex
import re

import gettext
_ = gettext.gettext

from itertools import izip

THRESHOLD = 15.00

def execute_proc_str(command):
  cmd = shlex.split(command)
  try:
    ret = subprocess.call(cmd)
    if ret != 0:
      fatal_err( _( "Failed when executing: '%s' with exit code: %d" ) %(string, ret))
  except Exception as e:
    fatal_err( _( "Could not execute command: \"%s\" . Exception: %s" %(string, e) ) )

def readfile(filename):
  data = {}
  order = []
  i = ""
  j = 0
  tx = open(filename)
  for l in tx:
    line = l.strip()
    if line[0]=='"':
      i = line[1:-1]
      j = 0
      data[i]={}
      order.append(i)
    elif line[0]=='.':
      i=""
    else:
      elements=line.split(" ")
      #if elements[2]=="SP" and int(elements[3])!=-1:
      j=j+1
      data[i][j]=elements
  tx.close()
  return data, order

def read_samples_lists_samples(samples_lst, samples_clusters_samples_lst):
  d_samples_clusters = {}
  with open(samples_lst, "r") as f1, open(samples_clusters_samples_lst, "r") as f2:
    for sample, cluster in izip(f1, f2):
      sample = sample.strip()
      cluster = cluster.strip()
      d_samples_clusters[sample] = cluster

  return d_samples_clusters

def duration(elements):
  last = 0
  for p in elements:
    last = int(elements[p][0])+int(elements[p][1])
  return (last*10.0)/1000.0

def cuts_each(duration):
  global THRESHOLD
  num = math.ceil(duration/THRESHOLD)
  return duration/num

def cut_words(elements, each):
  i = 0
  last = 0
  cut = round(each * 100)
  allw = {}
  mark = {}
  for p in elements:
    if (cut*(i+1)) < int(elements[p][0]) and elements[p][2]=="SP":
      allw[i] = allw[i] + "SP"
      mark[i] = int(elements[p][0])+int(elements[p][1])-1
      i = i+1
    if i not in allw: allw[i]=""
    allw[i] = allw[i] + elements[p][2] + " "
    last = int(elements[p][0])+int(elements[p][1])
  mark[i] = int(last)
  return allw, mark

def rename(sample_name, marks): 
  name = sample_name.split("/")[-1]
  init = float(name.split("_")[0])
  fina = float((name.split("_")[1]).split("-")[0])
  allm = name.split("-")[1:]
  next = init
  last = max(marks.keys())
  names = {}
  syncs = []
  i = 0
  for e in marks:
    end = init+(marks[e]/100.0)
    #if e == last: nombres[i] = str(round(next,3))+"_"+str(round(float(fina),3))+"-"+str('-'.join(allm))
    #else: nombres[i] = str(round(next,3))+"_"+str(round(end,3))+"-"+str('-'.join(allm))
    if e == last: 
      syncs.append((next, fina))
      names[i] = "{0:.3f}".format(next).zfill(9)+"_"+"{0:.3f}".format(fina).zfill(9)+"-"+str('-'.join(allm))
    else: 
      syncs.append((next, end))
      names[i] = "{0:.3f}".format(next).zfill(9)+"_"+"{0:.3f}".format(end).zfill(9)+"-"+str('-'.join(allm))
    i = i + 1
    next = end + 0.001
  return names, syncs

def fatal_err(message):
  stderr.write("[EE] "+ message + "\n")
  sys.exit(1)

def split_tlk_sample(segment, names, marks, parts, output_dir, tLbin2txt_ex, clusters_samples,
    new_samples_lst, new_phos_lst, new_monophos_lst, new_samples_clusters_samples_lst, feature_vector_size):
  #Generate a txt file with the sample
  file_d, file_name = tempfile.mkstemp()
  os.close(file_d)

  s = "%s realfea -i %s -o %s" %(tLbin2txt_ex, segment, file_name)
  execute_proc_str(s)

  tx = open(file_name,"r")
  f = [ i.strip() for i in tx ]
  tx.close()
  dirb = output_dir + "/" + ('/'.join(segment.split("/")[:-1]))
  #create dir if necessary
  if not os.path.exists(dirb):
    os.makedirs(dirb)

  feature_vector_size = int(f[1].split()[0])
  f = f[2:]
  i = 0
  for e in marks:
    new_filename = dirb+"/"+names[e]
    g = open(new_filename + ".seg","w")
    h = open(new_filename + ".seg.pho","w")
    hm = open(new_filename + ".seg.monopho","w")
    fin=int(marks[e])
    g.write("AKREALTF\n"+str(feature_vector_size)+" "+(str(fin-i))+"\n")
    h.write(parts[e])
    monophos = re.sub("\+.", '', re.sub(".-", '', parts[e]))
    hm.write(monophos)
    #print("--->"+str(i)+"->"+str(fin)+" ->"+str(len(f)))
    for j in range(i,fin):
      g.write(f[j]+"\n")
    i = fin
    g.close()
    h.close()
    hm.close()

    #convert the file to binary
    s = "%s realfea -r -i %s -o %s" %(tLbin2txt_ex, new_filename+".seg", new_filename+".seg")
    execute_proc_str(s)

    #put new files in the lists
    new_samples_lst.write("%s\n" %(dirb+"/"+names[e]+".seg"))
    new_phos_lst.write("%s\n" %(dirb+"/"+names[e]+".seg.pho"))
    new_monophos_lst.write("%s\n" %(dirb+"/"+names[e]+".seg.monopho"))
    new_samples_clusters_samples_lst.write(
        "%s\n" %(clusters_samples[segment]))
  pass

  os.remove(file_name)

def main():

  global THRESHOLD

  parser = argparse.ArgumentParser(
      prog="segment_pre-cmllr.py",
      description=_('Shorten samples (generating more if necessary)'),
      epilog=_("""segment_pre-cmllr.py shortens samples from a standard recognition
splitting the recognised samples in segments. It generates a new samples, phoneme
and cluster related files and lists."""))

  parser.add_argument(dest='align_file',
      help=_("Align file generated by tLrecognise"))

  parser.add_argument(dest='samples_lst',
      help=_("Samples recognised in align file"))

  parser.add_argument(dest='samples_clusters_samples_lst',
      help=_("Correspondance of cluster for each sample in samples_lst"))

  parser.add_argument(dest='output_directory',
      help=_("Output directory to write new samples and lists"))

  parser.add_argument('--threshold',
    help=_('Time threshold to divide the samples'), type=int)

  parser.add_argument('--fulla-comb-CNN-DNN',
    help=_('Trick to make CNN+DNN system combination work'), action='store_true', dest='fulla', default=False)

  parser.add_argument('--tLbin2txt',
    help=_('Path to tLbin2txt tool'), type=str, default='tLbin2txt')

  parser.add_argument('--feature-vector-size',
    help=_('Dimension of the feature vector'), type=int, default=48)


  args = parser.parse_args(sys.argv[1:])

  if args.threshold not in ["", None, False]:
    THRESHOLD = args.threshold

  #read the correspondance between samples and clusters
  samples_clusters = \
      read_samples_lists_samples(args.samples_lst, args.samples_clusters_samples_lst)

  #Open files for the new samples phoneme and cluster lists
  if not os.path.exists(args.output_directory):
    os.makedirs(args.output_directory)

  new_samples_lst = open(args.output_directory+"/samples.lst", "w")
  new_phos_lst = open(args.output_directory+"/samples-phos.lst", "w")
  new_monophos_lst = open(args.output_directory+"/samples-monophos.lst", "w")
  new_samples_clusters_samples_lst = open(args.output_directory+"/samples-clusters-samples.lst", "w")

  #read align file
  syncs = []
  data, order = readfile(args.align_file)
  for segment in order:
    elements = data[segment]
    time = duration(elements)
    if time > THRESHOLD: each = cuts_each(time)+0.1
    else: each = time+0.1
    parts, marks = cut_words(elements, each)
    names, s = rename(segment, marks)
    syncs += s
    if len(parts)==0:
      #void samples
      print(segment)
      if args.fulla:
        new_samples_clusters_samples_lst.write("%s\n" %(samples_clusters[segment])) ### FIX for CNN+DNN system combination
    else:
      split_tlk_sample(segment, names, marks, parts, \
          args.output_directory, args.tLbin2txt, samples_clusters, \
          new_samples_lst, new_phos_lst, new_monophos_lst, new_samples_clusters_samples_lst, args.feature_vector_size)

  

  new_samples_lst.close()
  new_phos_lst.close()
  new_monophos_lst.close()
  new_samples_clusters_samples_lst.close()

  segmentation_trs = open(args.output_directory+"/segmentation.trs", "w")
  generate_trs(syncs, segmentation_trs)
  segmentation_trs.close()


def generate_trs(syncs, trs):
  header= ('<?xml version="1.0" encoding="ISO-8859-1"?>\n'+
    '<!DOCTYPE Trans SYSTEM "trans-14.dtd">\n'+
    '<Trans scribe="aktoolkit" audio_filename="unknown.wav" version="2" version_date="120116">\n'+
    '<Speakers>\n'+
    '<Speaker id="unk" name="Unknown" check="no" type="" dialect="" accent="" scope="local"/>\n'+
    '</Speakers>\n'+
    '<Episode>\n'+
    '<Section type="report" startTime="0" endTime="'+str(syncs[len(syncs)-1][1])+'">\n'+
    '<Turn startTime="0" endTime="'+str(syncs[len(syncs)-1][1])+'" speaker="unk">\n')
  trs.write(header)

  if syncs[0][0] > 0:
    sync= '<Sync time="0"/>\nB\n'
    trs.write(sync) 

  for s in syncs:
     b = s[0]
     e = s[1]
     sync= '<Sync time="'+str(b)+'"/>\nS\n'
     trs.write(sync)
     sync= '<Sync time="'+str(e)+'"/>\nB\n'
     trs.write(sync)
  end= ('</Turn>\n'+
        '</Section>\n'+
        '</Episode>\n'+
        '</Trans>\n')
  trs.write(end)
 
####################################################################################################
if __name__ == "__main__":
  main()

