#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

# Defaults
PRGNAME=`basename ${BASH_SOURCE[0]}`
VERBOSE=${VERBOSE:-1}
SUB_NAME=""
WAIT_SUB=""
OUT=""
STRJOBS=""
JOBS=""
TIME=""
MEM=""
PARALLEL_MAXPROC=`perl -e 'my $cpu=2;  if (open(C,"/proc/cpuinfo")) {  $cpu=scalar(grep {/^processor/} <C>); $cpu=($cpu>0)?$cpu:2      } ;     print STDOUT $cpu ' `

#################
# HELPER FUNCTIONS
function log {
    (( "$VERBOSE" )) && echo -e "[II] $PRGNAME : $@" >&2 || return 0 
}
parallel () {
        gawk 'BEGIN{
              print "all: ALL_TARGETS\n"
             }
             {
               print "TARGET_"NR":\n\t@"$0"\n"
             }
             END{printf "ALL_TARGETS:";
                   for(i=1;i<=NR;i++){
                     printf " TARGET_%d",i};
                     print"\n"
              }' | make -j $@ -f - all
}
function print_help {
    cat <<EOF
$0 [options] cmd_to_submit ...
  
  cmd_to_submit     command line execution
  
  Options:
  
    -h            : Print this help
    -n <name>     : Submission name
    -w <name>     : (not used, only present for compatibility)
    -t <time>     : (not used, only present for compatibility)
    -m <memory>   : (not used, only present for compatibility)
    -g            : (not used, only present for compatibility)
    -o <dir/file> : Redirect all outputs to a file
    -j <array>    : Launch an array of jobs, each with a different SGE_TASK_ID variable
    -x <integer>  : Maximum number of running processes in parallel of an array (def. $PARALLEL_MAXPROC)

EOF
}

################
while getopts "hn:w:t:m:o:j:g" OPT
do
    case $OPT in
      x) PARALLEL_MAXPROC=$OPTARG;;
      h) print_help; exit 0;;
      n) SUB_NAME=$OPTARG;;
      w) WAIT_SUB=$OPTARG;;
      t) TIME=$OPTARG;;
      m) MEM=$OPTARG;;
      g) ;;
      o) OUT=$OPTARG;;
      j) STRJOBS=$OPTARG;
         JOBS=`echo $OPTARG | gawk \
          '{  L=split($0,a,","); 
          for(i=1;i<=L;i++) 
          { 
            lj=split(a[i],j,"-"); 
            if(lj == 1) 
              printf("%d ",j[1]);                                                   
            else
              for(k=j[1];k<=j[2];k++)                                               
                printf("%d ",k);                                                    
          }
        }'`;;
    esac
done
shift $[OPTIND-1]

if [ -z $SUB_NAME ]; then
  SUB_NAME=`echo $1 | cut -d ' ' -f 1` #work around to solve "ls -l; sleep 10" like submits
fi

if [ -z $OUT ]; then
  OUT="$SUB_NAME.o"
elif [ -d $OUT ]; then
  OUT="$OUT/$SUB_NAME.o"
fi

#############
#Launch Job
WAIT=""
COMMAND="$*"
TMPFILE=`mktemp -t tmp.XXXXXX`

echo "$COMMAND" > "$TMPFILE"

if [ "$JOBS" ]; then
  log "Job Name: $SUB_NAME Log File(s): $OUT.* Tasks: $STRJOBS Max Parallel: $PARALLEL_MAXPROC"
  for i in $JOBS ; do 
    echo  "SGE_TASK_ID=$i bash $TMPFILE > $OUT.$i 2>&1" 
  done | parallel $PARALLEL_MAXPROC
else
  log "Job Name: $SUB_NAME Log File(s): $OUT"
  bash "$TMPFILE" &>"$OUT" &
  wait $!
fi

rm $TMPFILE
