#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
from libdfxp import *
from datetime import datetime
from optparse import OptionParser
from math import log, exp

class Seg:
    def __init__(self,b,d,p,pp,w):
        self.begin= b
        self.duration= d
        self.phoneme= p
        self.pho_pos= pp
        self.word= w
    def __repr__(self):
        return '<%d,%d,%s,%d,%s>'%(self.begin,self.duration,self.phoneme,self.pho_pos,self.word.encode('utf-8'))

class CSeg:
    def __init__(self,b,d,w):
        self.begin= b
        self.duration= d
        self.word= w
    def __repr__(self):
        return '<%d,%d,%s>'%(self.begin,self.duration,self.word)

class Word:
    def __init__(self,b,e,c,w):
        self.begin= b
        self.end= e
        self.conf= c
        self.word= w
        self.nwus = -1 # number words until next sil
    def __repr__(self):
        if self.conf != None:
          return '<%.2f,%.2f,%.2f,%d,%s>'%(self.begin,self.end,self.conf,self.nwus,self.word.encode('utf-8'))
        else:
          return '<%.2f,%.2f,None,%d,%s>'%(self.begin,self.end,self.nwus,self.word.encode('utf-8'))

def compact_sample(sample,sil_tok):
    if sample == [] : return []
    ret= []
    ini = 0
    if sample[ini].phoneme == sil_tok:
        if sample[ini] > 0:
            ret.append(CSeg(sample[ini].begin,sample[ini].duration,sil_tok))
        ini+=1
    cw = None; cbeg = 0; cdur = 0
    for s in sample[ini:]:
        if cw == None:
            cw = s.word; cbeg = s.begin
        if s.phoneme != sil_tok:
            cdur+= s.duration
        else:
            ret.append(CSeg(cbeg,cdur,cw))       
            cw = None; cbeg = 0; cdur = 0
            if s.duration > 0:
                ret.append(CSeg(s.begin,s.duration,sil_tok))

    return ret

def read_sample(f,nf):
    # Descarta capçalera
    sn = ""
    l= f.readline()
    while l != '':
        if l[0] == '"':
            sn = os.path.basename(l.split('"')[1])
        line= l.strip()
        l= f.readline()
        if line != '' : break
    # Aliniaments
    aux= []
    while l != '' and l.strip()!='.':
        line= l.strip()
        l= f.readline()
        if line == '' : continue
        if nf == 6: 
            b,d,p,pp,w,cm = line.split()
        elif nf == 5: 
            b,d,p,pp,w = line.split()
        elif nf == 3:
            b,d,w = line.split(); pp = 0
            if w == "SP": p = w
            else: p = ""
        else:
            sys.stderr.write("[EE] Unexpected number of fields.\n")
            sys.exit(1)
        aux.append(Seg(int(b),int(d),p,int(pp),w.decode('utf-8')))
    if l == '' and len(aux) == 0:
        return None,None
    elif l == '' and len(aux) > 0:
        sys.stderr.write("[EE] Wrong align file format: unexpected EOF.\n")
        sys.exit(1)
    else:
        return aux, sn

def read_aklign(fn,nf):
    f= open(fn)
    ret= []
    sn=[]; snd={}; i=0
    s,n = read_sample(f,nf)
    while s != None:
        if n not in sn:
          ret.append(s)
          sn.append(n); snd[n] = i; i += 1
        else:
          if s != []:
            ret[snd[n]] = s
        s,n = read_sample(f,nf)
    return ret, sn, snd

def read_cm(fn):
    f = open(fn)
    ret = []
    for l in f:
        fields = l.strip().split()
        w = fields[0].decode('utf-8')
        cm = float(fields[1])
        ret.append((w,cm))
    return ret

def gen_sorted_word_list(sn,snd,cakligns,cms,cm_fn):
    sorted_sn = sorted(snd.keys())
    wi = 0
    swords = []

    for s1 in sorted_sn:
        v = snd[s1]
        try:
            st = float(sn[v].split("-")[0].split("_")[0])
            et = float(sn[v].split("-")[0].split("_")[1])
        except: 
            st = 0.0
            et = 0.0
        for s in range(len(cakligns[v])):
            b = st + cakligns[v][s].begin/100.0
            e = st + (cakligns[v][s].begin + cakligns[v][s].duration - 1)/100.0
            w = cakligns[v][s].word.strip()
            if w != sil_tok: 
              if cm_fn != None:
                try:
                  cms_w = cms[wi][0].strip()
                  if cms_w == w:
                    c = cms[wi][1]
                  else:
                    sys.stderr.write("[WW] CM-align Non-matching words: [align %d] %s -> [cm %d] %s\n"%(s, w.encode('utf-8'), wi, cms_w.encode('utf-8')))
                    c = None
                except IndexError:
                  sys.stderr.write("[WW] CM word list exhausted before ending align word list. [align %d] %s -> [cm %d] None\n"%(s, w.encode('utf-8'),wi))
                  c = None
                wi += 1
              else:
                c = None
            else:
              c = None
            word = Word(b,e,c,w)
            swords.append(word)

    return swords

def fill_silences(swords, sil_tok):
    ret = []
    w = swords[0]
    ret.append(w)
    for i in xrange(1,len(swords)):
        nw = swords[i]
        if nw.begin - w.end >= 0.02:
            if w.word == sil_tok:
                w.end = nw.begin-0.01
            else:
                silW = Word(w.end+0.01,nw.begin-0.01,None,sil_tok)
                ret.append(silW)
        ret.append(nw)
        w = nw
    return ret

def add_info_sils(swords, sil_tok):
    i = len(swords)-1
    w = swords[i]
    nwus = 1
    w.nwus = nwus
    i -= 1
    while i>= 0:
        w = swords[i]
        if w.word != sil_tok:
            nwus += 1 # number words until next sil
            w.nwus = nwus
        else:
            nwus = 0
            w.nwus = nwus
        i -= 1
    return swords 

#------- Main

usage = "usage: %prog [options] <aklign_file>"
parser = OptionParser(usage)
parser.add_option("-i", "--video_id", dest="video_id", type="string", default=None, help="Video ID.")
parser.add_option("-a", "--authorId", dest="authorId", type="string", default="UPV", help="Author Id. Default 'UPV'.")
parser.add_option("-C", "--authorConf", dest="authorConf", type="float", default="0.5", help="Author Confidence. Default 0.5.")
parser.add_option("-c", "--conf_measures", dest="cm_fn", type="string", default=None, help="Confidence measures file in HTK format.")
parser.add_option("-L", "--max_seg_len", dest="max_seg_len", type="float", default="20.0", help="Segment length threshold (in seconds) used to cut segments. Default 20.")
parser.add_option("-l", "--max_sil_len", dest="max_sil_len", type="float", default="0.6", help="Silence length threshold (in seconds) used to cut segments. Default 0.6.")
parser.add_option("-S", "--heuristic_sil_len", action="store_true", dest="heur_sil", default=False, help="Heuristically determine the silence length threshold (in seconds) used to cut segments.")
parser.add_option("-M", "--max_words", dest="max_words", type="int", default="15", help="Maximum number of words allowed on a single segment. Default 15.")
parser.add_option("-m", "--min_words", dest="min_words", type="int", default="1", help="Minimum number of words allowed on a single segment. Default 1.")
parser.add_option("-t", "--tradeoff-words", dest="perextrawords", type="float", default="0.4", help="Define trade-off, in percentatge of words (computed over the maximum number of words allowed in a single segment, see -M option), in which a segment could be enlarged if a silence would be found afterwards. Default 0.4.")
parser.add_option("-s", "--sil_token", dest="sil_tok", type="string", default="SP", help="Silence Token. Default 'SP'.")
parser.add_option("-e", "--hes_token", dest="hes_tok", type="string", default="[hesitation]", help="Hesitation Token. Default '[hesitation]'.")
parser.add_option("-u", "--unk_token", dest="unk_tok", type="string", default="<unk>", help="Unknown Word Token. Default '<unk>'.")

(options, args) = parser.parse_args()

if len(args) != 1:
	parser.error("Incorrect number of arguments.")

aklign_fn = args[0]

f = open(aklign_fn,'r'); 
l=f.readline().strip(); 
while l == '.' or l == '' or l[0] == '"': l=f.readline().strip(); 
nf = len(l.split()); f.close()

sil_tok = options.sil_tok; hes_tok = options.hes_tok; unk_tok = options.unk_tok
cm_fn = None
cms = None
if options.cm_fn != None:
  cm_fn = options.cm_fn
  cms = read_cm(cm_fn)

akligns,sn,snd = read_aklign(aklign_fn,nf)
if nf == 5 or nf == 6:
    cakligns = []
    for a in akligns:
        ca = compact_sample(a,sil_tok)
        cakligns.append(ca)
elif nf == 3:
    cakligns = akligns
else: 
    sys.stderr.write("[EE] Unexpected number of fields.\n")
    sys.exit(1)

swords = add_info_sils(fill_silences(gen_sorted_word_list(sn,snd,cakligns,cms,options.cm_fn), options.sil_tok), options.sil_tok)

sil_len_thres = options.max_sil_len
if options.heur_sil:
    acc = 0; nsil = 0
    for w in swords:
        if w.word == options.sil_tok:
            acc += (w.end - w.begin)
            nsil += 1
    avg_sil_len = acc/float(nsil)
    sil_len_thres = avg_sil_len - avg_sil_len*0.3

dfxp = TlDfxp()
dfxp.videoId = options.video_id
dfxp.authorType = "automatic"
dfxp.authorId = options.authorId
dfxp.timeStamp = datetime.now()
dfxp.authorConf = options.authorConf
dfxp.begin = swords[0].begin
dfxp.end = swords[len(swords)-1].end
dfxp.dfxp = dfxp

vc = 0; vnw = 0
segdata = []; segc = 0; segnw = 0; segnrw = 0; segid = 1; segdur = 0; segbeg = None; segend = None; cut = False; cut_next_sil = False

for w in swords:
    #print segnrw, options.max_words, segdur, options.max_seg_len, w.nwus, int(options.max_words*options.perextrawords)
    if w.word != options.sil_tok:
        segdur += (w.end - w.begin)
        if segbeg == None: segbeg = w.begin
        segend = w.end
        word = TlWord(dfxp)
        word.begin = w.begin
        word.end = w.end
        word.txt = w.word.lower().replace(".","")
        if options.cm_fn != None: 
            word.confMeasure = w.conf
            if w.conf != None and segc != None and vc != None:
              segc += log(w.conf); vc += log(w.conf)
            else:
              segc = None; vc = None
        segnw += 1; vnw += 1
        if w.word != hes_tok and w.word != unk_tok: segnrw += 1
        segdata.append(word)
        if (segnrw > options.max_words or segdur > options.max_seg_len):
            if w.nwus > (int(options.max_words*options.perextrawords)):
              cut = True
            else:
              cut_next_sil = True
    else:
        if cut_next_sil:
            cut = True
        else:
            sil_len = (w.end - w.begin)
            if sil_len > sil_len_thres and segnrw >= options.min_words:
              cut = True
    if cut:
        seg = TlSegment(dfxp,str(segid),segbeg,segend)
        seg.data = segdata
        if options.cm_fn != None:
            if segc != None:
                segc = exp(segc/float(segnw))
                seg.confMeasure = segc
            else:
                seg.confMeasure = None
        dfxp.data.append(seg)
        segdata = []; segid += 1; segc = 0; segnw = 0; segnrw = 0; segdur = 0; segbeg = None; segend = None; cut = False; cut_next_sil = False

if len(segdata) > 0:
    seg = TlSegment(dfxp,str(segid),segbeg,segend)
    seg.data = segdata
    if options.cm_fn != None:
        if segc != None:
            segc = exp(segc/float(segnw))
            seg.confMeasure = segc
        else:
            seg.confMeasure = None
    dfxp.data.append(seg)
   
if options.cm_fn:
    if vc != None:
        vc = exp(vc/float(vnw))
        dfxp.confMeasure = vc
    else:
        dfxp.confMeasure = None

dfxp.status = "fully_automatic"

print dfxp.toXml()

