#!/usr/bin/env python
# -*- coding: utf-8 -*-

# tLprogressbar: simple progress bar to be used in the recognition
# Copyright 2013,2014 transLectures-UPV Team
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import time


def _get_terminal_size_linux():
  # credit to https://gist.github.com/jtriley/1108174
  def ioctl_GWINSZ(fd):
    try:
      import fcntl
      import termios
      cr = struct.unpack('hh',
         fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
      return cr
    except:
      pass
  cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
  if not cr:
    try:
      fd = os.open(os.ctermid(), os.O_RDONLY)
      cr = ioctl_GWINSZ(fd)
      os.close(fd)
    except:
      pass
  if not cr:
    try:
      cr = (os.environ['LINES'], os.environ['COLUMNS'])
    except:
      return None
  return int(cr[1]), int(cr[0])


class tLProgressBar():
  def __init__(self, val=None, maxval=None, outputfd=sys.stdout):
    self.val = val if val != None else 0
    self.maxval = maxval if maxval != None else 10
    self.outputfd = outputfd

  def update(self, val):
    self.val = val
    self.print_bar()

  def print_bar(self):
    ret = _get_terminal_size_linux()
    if ret == None:
      width = 50
    else:
      (width, height) = ret

    bar_msg = "\r[II] %3d of %3d -- " %(self.val, self.maxval)
    remaining = width - ( len(bar_msg) + 2 )
    progress = int(self.val*remaining/self.maxval+0.5)
    progress_rem = int(self.maxval*remaining/self.maxval+0.5)

    bar_msg += "[" + "="*progress + " "*(progress_rem-progress) + "]"

    self.outputfd.write(bar_msg)

  def finish(self):
    self.outputfd.write("\n")

####################################################################################################
if __name__ == "__main__":
  t = tLProgressBar()
  t.maxval=10
  for i in range(11):
    t.update(i)
    time.sleep(1)
