#!/bin/bash
#
#  Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

set -eu

function print_help {
    cat <<EOF
$0 <samples-list> <tmpdir>

  Segments speech and non speech from a list of samples.
  

  <samples-list>    List with samples in TLK format
  <tmpdir>          Temporary dir to store samples and wavs

Options:

  -r <path>         Path to tLrecognise executable
  -b <beam>         Beam threshold for prunning (def. 300)
  -g <gsf>          Grammar Scale Factor (def. 20)
  -m <max-states>   Maximum number of states in trellis (def. 10000)

EOF
}

NUMARGS=4
TLRECOGNISEEX="tLrecognise"
SEG_beam=300
SEG_gsf=20
SEG_maxas=10000
while getopts "hr:b:g:m:" OPT
do
    case $OPT in
        h) print_help; exit 0;;
        r) TLRECOGNISEEX=$OPTARG;;
        b) SEG_beam=$OPTARG;;
        g) SEG_gsf=$OPTARG;;
        m) SEG_maxas=$OPTARG;;
    esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

LST=$1
TMPDIR=$2
SEG_LM=$3
SEG_AM=$4

function log {
    echo -e "$0: [II] $@" >&2
}

log $"Recognising speech and non speech segments"
COMMAND="$TLRECOGNISEEX $SEG_LM $SEG_AM $LST -o $TMPDIR/frametags.align \
      -v --word-align \
      -m $SEG_maxas -b $SEG_beam -g $SEG_gsf"
log $"Executing $COMMAND"
$COMMAND

log $"Generating trs files"
COMMAND="tLoutalign2trs.py $TMPDIR/frametags.align $TMPDIR/segm/trs"
$COMMAND

find $TMPDIR/segm/trs/ -name '*.trs' | sort > $TMPDIR/trs.lst

log $"Resegmenting trs files by threshold"

for t in `<$TMPDIR/trs.lst`; do 
  resegment_speech_trs.py -t 40 "$t" > ${t%.trs}.resegmented.trs
done

find $TMPDIR/segm/trs/ -name '*.resegmented.trs' | sort > $TMPDIR/trs.lst
