#!/bin/bash
#
#  Copyright 2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

CONVERTER=`which avconv`
CONVERTER=${CONVERTER:-ffmpeg}

set -eu

if [ -z $2 ]; then
  echo "USAGE: $0 <LIST_VIDEOS> <TMPDIR>"
  exit 1
fi

function print_help {
    cat <<EOF
$0 <videos-list> <tmpdir>

  Generates feature vectors from a list of videos <list-videos> and return the list
  of created samples in <tmpdir>/tLfeas.lst

  <videos-list>     List with videos
  <tmpdir>          Temporary dir to store samples and wavs

Options:

  -e <path>         Path to tLextract executable
  -o <str>          tLextract options

EOF
}

NUMARGS=2
TLEXTRACTEX="tLextract"
TLEXTRACTOPTS=""
while getopts "he:o:" OPT
do
    case $OPT in
        h) print_help; exit 0;;
        e) TLEXTRACTEX=$OPTARG;;
        o) TLEXTRACTOPTS="$OPTARG";;
    esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

LST=$1
TMPDIR=$2

function log {
    echo -e "$0: [II] $@" >&2
}

mkdir -p $TMPDIR
for i in `< $LST`; do
  DIRNAME=`dirname $i`

  mkdir -p $TMPDIR/$DIRNAME

  log $"Converting video $i"
  COMMAND="$CONVERTER -i $i -y $TMPDIR/$i.wav"
  log $"Executing: $COMMAND"
  $COMMAND

  echo $TMPDIR/$i.wav
done | sort > $TMPDIR/wavs.lst

cat $TMPDIR/wavs.lst | sed -e 's+\.wav$+.tLfea+' > $TMPDIR/tLfeas.lst

COMMAND="$TLEXTRACTEX $TLEXTRACTOPTS $TMPDIR/wavs.lst $TMPDIR/tLfeas.lst"
log $"Executing: $COMMAND"
$COMMAND

