#!/usr/bin/python 
from trs import *
from math import ceil
import sys
from optparse import OptionParser

def resegment_trs(trs, th, stag, btag):
  ctl = trs.episodes[0].sections[0].turns[0].content
  # We assume that pair positions contains always a TransSync object
  # and impair positions contains a speech/background tag
  if not(isinstance(ctl[0],TransSync)):
    sys.stderr.write("[EE] Unexpected trs format. It was expecting a <TransSync> element.\n")
    sys.exit(1)
  st = ctl[0].time
  if ctl[1] not in (stag,btag):
    sys.stderr.write("[EE] Unexpected trs format. It was expecting a Speech (%s) or Background (%s) tag.\n" % (stag, btag))
    sys.exit(1)
  t = ctl[1]
  i = 2
  while i < len(ctl):
    if not(isinstance(ctl[i],TransSync)):
      sys.stderr.write("[EE] Unexpected trs format. It was expecting a <TransSync> element.\n")
      sys.exit(1)
    ct = ctl[i].time
    seglen = ct - st
    if t == stag and seglen > th:
      nsplits = int(ceil(seglen/th))
      splen = int(seglen/nsplits)
      sys.stderr.write("[WW] Speech segment length exceeded threshold (%d > %d). It will be splitted into %d new segments of %ds.\n" % (int(seglen), th, nsplits, splen))
      i -= 2
      ctl.pop(i) # delete sync
      ctl.pop(i) # delete tag
      j = 0
      spst = st
      while j < nsplits:
        ctl.insert(i, TransSync(spst))
        ctl.insert(i+1, t)
        spst = spst + splen
        j += 1
        i += 2
    st = ct
    if ctl[i+1] not in (stag,btag):
      sys.stderr.write("[EE] Unexpected trs format. It was expecting a Speech (%s) or Background (%s) tag.\n" % (stag, btag))
      sys.exit(1)
    t = ctl[i+1]
    i += 2

if __name__ == "__main__":

  usage = """Usage: resegment_speech_trs.py <TRS>\nResegments speech segments longer than a threshold. Type -h for help.\n"""
  parser = OptionParser(usage)
  parser.add_option("-t", "--threshold", dest="thres", type="int", default=60, help="Segment length threshold. Default 60 seconds.")
  parser.add_option("-S", "--speech-tag", dest="sp_tag", type="string", default="S", help="Speech tag. Default 'S'.")
  parser.add_option("-B", "--background-tag", dest="bg_tag", type="string", default="B", help="Background tag. Default 'B'.")

  (opts, args) = parser.parse_args()

  if len(args) < 1:
    sys.stderr.write(usage)
    sys.exit(1)

  trs_fp = args[0]

  trs = load_trans_file(file(trs_fp))
  resegment_trs(trs, opts.thres, opts.sp_tag, opts.bg_tag)
  trs.write(sys.stdout)

