#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

function print_help {
    cat <<EOF
$0 [options] <cluster-lists> <target-model> <tmpdir>

  <cluster-lists>       List of clusters
  <target-model>        Path to the model used in the transformation
  <tmpdir>              Temporal directory of the estimation

  Options:

  -h:               Print this help
  -e <tLcmllr>      Path to the tLcmllr executable
  -g <tLcmllrfeas>  Path to the tLcmllrfea executable
EOF
}

NUMARGS=3
CMLLREX="tLcmllr"
CMLLRFEAEX="tLcmllrfeas"
while getopts "he:g:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) CMLLREX=$OPTARG;;
        g) CMLLRFEAEX=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

CLUSTER_LST=$1
TARGET_MODEL=$2
TMPDIR=$3
shift 3

if [ -z $SGE_TASK_ID ]; then
  echo $"SGE_TASK_ID variable is not defined, this script should be called by an array job"
  exit 1
fi

CLUSTER="`awk -v line=$SGE_TASK_ID 'NR == line { print $0 }' $CLUSTER_LST`"

cmllr-features.sh -e $CMLLREX -g $CMLLRFEAEX $CLUSTER $TARGET_MODEL \
    $TMPDIR/clusters/$CLUSTER-phos_tied.lst \
    $TMPDIR/clusters/$CLUSTER.lst \
    $TMPDIR \
    $TMPDIR/clusters/$CLUSTER-cmllrfeas.lst

