#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

function print_help {
    cat <<EOF
$0 [options] <cluster-name> <target-model> <transcriptions-list> <samples-list> <tmpdir> <cmllr-samples-list>

  <cluster-name>        Name of the cluster
  <target-model>        Path to the model used in the transformation
  <transcription-list>  List of transcriptions
  <samples-list>        List of samples
  <tmpdir>              Temporal directory of the estimation
  <cmllr-samples>       List of samples to be generated

  Options:

  -h:               Print this help
  -e <tLcmllr>      Path to the tLcmllr executable
  -g <tLcmllrfeas>  Path to the tLcmllrfea executable
EOF
}

NUMARGS=6
CMLLREX="tLcmllr"
CMLLRFEAEX="tLcmllrfeas"
while getopts "he:g:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) CMLLREX=$OPTARG;;
        g) CMLLRFEAEX=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

CLUSTER=$1
TARGET_MODEL=$2
TRANSCRIPTIONS_LST=$3
SAMPLES_LST=$4
TMPDIR=$5
CMLLRSAMPLES_LST=$6
shift 6

if [ ! -d "$TMPDIR/matrices" ]; then
  mkdir -p "$TMPDIR/matrices"
fi

function log {
    echo -e "$0: [II] $@" >&2
}

CMLLROPTS="-M 200" #transformation matrix full
COMMAND="$CMLLREX $CMLLROPTS $TARGET_MODEL $TRANSCRIPTIONS_LST $SAMPLES_LST-in -o $TMPDIR/matrices/$CLUSTER.matrix -v"
log $COMMAND
$COMMAND

COMMAND="$CMLLRFEAEX $TARGET_MODEL $SAMPLES_LST-out $TMPDIR/matrices/$CLUSTER.matrix $CMLLRSAMPLES_LST -v"
log $COMMAND
$COMMAND

