#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

#include i18n
import gettext
_ = gettext.gettext

import optparse
from sys import argv, exit, stdin, stdout, stderr
import os
import subprocess

def main():
  parser = optparse.OptionParser(usage='%prog [options] destination_dir feature_vector_size states phonemes_list samples_list',
      description=_('Creates an initial model for a list of phonemes using the mean and variance of a list of samples'))

#  parser.add_argument('destination_dir', help=_('Directory to store the initial model'))
#  parser.add_argument('feature_vector_size', help=_('Size of each feature vector in samples'), type=int)
#  parser.add_argument('states', help=_('Number of states per HMM'), type=int)
#  parser.add_argument('phonemes_list', help=_('List of phonemes'))
#  parser.add_argument('samples_list', help=_('List of samples'))

  parser.add_option('--tLmkproto', help=_('Path to the tLmkproto executable'),
    type="string", dest="tLmkproto_ex", default="tLmkproto")
  parser.add_option('--initial-model-name', help=_('Change the name of the initial model. By default, model_0'),
    type=str, dest="model_name", default='model_0')
  parser.add_option('--var-floor', help=_('Minimum variance at each component'),
    type=float, dest="var_floor", default=0.001)
  parser.add_option('--transition-prob', help=_('Transition log-probability'),
    type=float, dest="trans_prob", default=-0.916291)
  parser.add_option('--add-sp', help=_('Adds an additional one-state model dealing with silence (SP)'),
    action='store_true', dest='add_sp')
  parser.add_option('-v', help=_('Outputs additional verbose information'),
    action='store_true')

  (opts, args) = parser.parse_args()

  if len(args) < 5:
    parser.print_help()
    exit(1)

  destination_dir = args[0]
  feature_vector_size = int(args[1])
  states = int(args[2])
  phonemes_list = args[3]
  samples_list = args[4]

  proto_name = destination_dir+'/'+opts.model_name+"_proto"
  command = [opts.tLmkproto_ex, 'DGaussian', samples_list, str(feature_vector_size),
      '-o', proto_name]
  if opts.v:
    stderr.write(_("[II] Calculating mean and variance\n"))
    stderr.write("%s\n" %" ".join(command))
  subprocess.call(command)

  if opts.v:
    stderr.write(_("[II] Copying model from %s to each phonemes\n") %(proto_name))

  proto_f = open(proto_name,'r')
  vrs = []
  body = ""
  n=1
  #get the variance and body of the prototype model
  for l in proto_f:
    w = l.split()
    if w[0] == 'VAR':
      for i in w[1:]:
        vrs.append(str(float(i)*opts.var_floor))
    if n >= 10:
      body += l
    n+=1
  proto_f.close()

  #read phonemes
  phonemes = []
  phonemes_f = open(phonemes_list,"r")
  for p in phonemes_f:
    if p[:-1] == "SP":
      continue
    phonemes.append(p[:-1])
  phonemes_f.close()

  #write header
  model_name = destination_dir+'/'+opts.model_name
  model_f = open(model_name,'w')
  model_f.write("CMODEL\n")
  model_f.write("DGaussian\n")
  model_f.write("D %d\n" %feature_vector_size)
  model_f.write("SMOOTH\n")
  model_f.write("%s\n" %(" ".join(vrs)))
  model_f.write("N %d\n" %(len(phonemes)+ (1 if opts.add_sp else 0)))
  #write mean and variance for each state and phoneme
  for i in phonemes:
    model_f.write("'%s'\n" %i)
    model_f.write("Q %d\n" %states)
    model_f.write("Trans %s\n" %(" ".join([str(opts.trans_prob)]*states)))
    for j in xrange(states):
      model_f.write("%s" %body)
  if opts.add_sp:
    model_f.write("""'SP'
Q 1
TransL
I
1 -0.510826
F -0.916291
.
1
1 -0.510826
F -0.916291
.
""")
  model_f.write("%s" %body)
  model_f.close()

if __name__ == "__main__":
  main()
