#!/bin/bash -x

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

function print_help {
    cat <<EOF
$0 <samples-list> <transcriptions-list> <clusters-samples-list> <clusters-lists> <tmpdir> <new-samples-list>

  <samples-list>                List with samples
  <transcription-list>          List with transcriptions
  <clusters-samples-list>       List which indicates the cluster of each sample
  <clusters-lists>              List of all clusters
  <tmpdir>                      Temporal directory of the estimation
  <new-samples-list>            List of all new samples to be generated

EOF
}

if [[ $# != 6 ]]
then
    print_help
    exit 1
fi

SAMPLES_LST=$1
TRANSCRIPTION_LST=$2
CLUSTERS_SAMPLES_LST=$3
CLUSTERS_LST=$4
TMPDIR=$5
CMLLR_LST=$6

#Check file consistency
if [ ! -s $CLUSTERS_LST ]; then
  echo "[EE] Empty cluster list $CLUSTERS_LST" >&2
  exit 1
fi

N_SAMPLES=`wc -l < $SAMPLES_LST`
N_CLUSTERS_SAMPLES=`wc -l < $CLUSTERS_SAMPLES_LST`
if [ "$N_SAMPLES" -ne "$N_CLUSTERS_SAMPLES" ]; then
  echo "[EE] Samples list and cluster-samples list differ in size" >&2
  echo "[EE] $SAMPLES_LST -> $N_SAMPLES | $CLUSTERS_SAMPLES_LST -> $N_CLUSTERS_SAMPLES" >&2
  exit 1
fi

# Make dir and clean
DEST_DIR=$TMPDIR/clusters
FEADESTDIR=$TMPDIR/fea.cmllr

#Create dir to generate CMLLR features
mkdir -p $DEST_DIR
cat $SAMPLES_LST |
sed -e 's+\(.*\)/.*+\1+' |
sort -u |
gawk -vDIR="$FEADESTDIR" '{print gensub(".",DIR"/&",1)}' |
while read l
do
    mkdir -p $l
done

for cl in `< $CLUSTERS_LST`; do
  rm -f ${DEST_DIR}/$cl.lst ${DEST_DIR}/$cl-phos_tied.lst ${DEST_DIR}/$cl-cmllrfeas.lst
done

#Generate the clusters lists
paste -d ' ' $CLUSTERS_SAMPLES_LST $SAMPLES_LST $TRANSCRIPTION_LST |
while read cl fea pho
do
    NAME=$fea
    tfea="$FEADESTDIR/$NAME"
    if [ -s $pho ] && [ `wc -w < $pho` -gt 0 ]; then #avoid empty phos
      echo $fea >> ${DEST_DIR}/$cl.lst-in
      echo $pho >> ${DEST_DIR}/${cl}-phos_tied.lst
    else
      echo "[WW] There is a void phoneme transcription: $pho" >&2
    fi
    echo $fea >> ${DEST_DIR}/$cl.lst-out
    echo $tfea >> ${DEST_DIR}/${cl}-cmllrfeas.lst
    echo $tfea
done > $CMLLR_LST

