#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

function print_help {
    cat <<EOF
$0 [options] <trimodel> <state-occs> <tiedmodel> <tlist> <freephos> <queries>

  <trimodel>          Path to the triphoneme model
  <state-occs>        Path to the state occupation from triphoneme model
  <tied-model>        Path to the output tied model
  <tlist>             Path to the output list of tied phonemes generated
  <freephos>          Free context phonems separated by commas
  <queries>           Path to the python script containing the CART queries   

Options:

  -h:                 Print this help
  -e  <path>          Path to the tLbin2txt executable
  -m  <path>          Path to the tLmumix executable
  -c  <path>          Path to the make_tied.py script
  -O  <float>         minimum occupation in CART
  -I  <float>         minumum required lilelyhood increment in CART

EOF
}

READLINK=`which greadlink`; READLINK=${READLINK:-readlink}; READLINK="$READLINK -f"
BPATH=`$READLINK $0`
BPATH=`dirname $BPATH`

NUMARGS=6
BIN2TXT=tLbin2txt
MUMIX=tLmumix
TOMIX=tLtomix
MAKETIED=make_tied.py
MINOCC=400
MININC=750
while getopts "he:c:m:O:I:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) BIN2TXT=$OPTARG;;
        c) MAKETIED=$OPTARG;;
        m) TOMIX=$OPTARG;;
        O) MINOCC=$OPTARG;;
	I) MININC=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

TRIMODEL=$1
OCCS=$2
TIEDMODEL=$3
TLIST=$4
FREEPHOS=$5
QUERIES=$6

# helper functions
function error {
    echo -e "$0: [EE] $@" >&2
    exit 1
}
function log {
    echo -e "$0: [II] $@" >&2
}

# CHECK FILES :
for i in 1 2 ;  do 
	[ -s "${!i}" ]      || error "Argument $i: file '${!i}' is empty or does not exist"
done

log "Cluster/tie from all possible tri-phonemes..."
set -e 

log "(1/3) Convert acoustic model to ASCII format ... $BIN2TXT amodel -i $TRIMODEL -o $TRIMODEL"
$BIN2TXT amodel -i $TRIMODEL -o $TRIMODEL

[ ${QUERIES:0:1} != "/" ] && [ ! -e $QUERIES ] && QUERIES="$BPATH/$QUERIES"
FREEPHOS=`echo "$FREEPHOS" | sed 's|[[:space:]]||g'`

log "(2/3) Generate all possible triphonemes and tied them ...  $MAKETIED $TRIMODEL $TIEDMODEL $OCCS $TLIST $FREEPHOS $MINOCC $MININC $QUERIES"
$MAKETIED $TRIMODEL $TIEDMODEL $OCCS $TLIST "$FREEPHOS" $MINOCC $MININC "$QUERIES"
[ -s "$TIEDMODEL" ]     || error "Tied triphonemes models: $TIEDMODEL is empty"
[ -s "$TLIST" ]         || error "Tied triphonemes list  : $TLIST is empty"

log "(3/3) Convert generated model to Gaussian-mixture format ... $TOMIX -i $TIEDMODEL -o $TIEDMODEL"
$TOMIX -i $TIEDMODEL -o $TIEDMODEL
[ -s "$TIEDMODEL" ]     || error "Tied triphonemes 1-mix models: $TIEDMODEL is empty (conversion failed)"

