#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

function print_help {
    cat <<EOF
$0 [options] <monophone-model> <triphos-list> <triphoneme-model>

  <monophone-model>     Path to monophoneme model
  <triphos-list>        List of all triphos
  <triphoneme-model>    Path to store the triphoneme model

  Options:

  -h:                   Print this help
  -e <tLbin2txt>        Path to tLbin2txt executable

EOF
}

NUMARGS=3
BIN2TXT="tLbin2txt"
while getopts "hcoms:e:d" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) BIN2TXT="$OPTARG";;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

MONOPHONEME_MODEL=$1
TRIPHOS=$2
TRIPHONEME_MODEL=$3

$BIN2TXT amodel -i $MONOPHONEME_MODEL -o $MONOPHONEME_MODEL

mono2tri.py $MONOPHONEME_MODEL $TRIPHOS > $TRIPHONEME_MODEL

