#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

if [ -z $4 ]; then
  cat <<EOF
$0 <list-of-phonemes> <list-of-triphonemes> <tmpdir> [OPTIONS]

  <list-of-phonemes>       List of files containing the phoneme annotation
  <list-of-triphonemes>    List of files containing the triphoneme annotation
  <tmpdir>                 Temporal directory of the estimation
  <freephos>               list of context free phoenems separated by comma

OPTIONS
  <1|0>       (def. 0): Do or do not generate crossword triphonemes. 
  If 1:
  <Real>      (def. 0): Generate crossword triphonemes only if the duration of
                        the shortpause between words is longer than <Real>
                        frames (def. 0.0)
                        #  Note: if <Real>=0 the crossword will always be
                        #  generate when possible.
                        #  Otherwise, an additional translitertion
                        #  files are needed to tell which is the frame
                        #  duration of the short pauses.

  <extension> (def. '.SPlength') : Additional extension to the .pho
                                   supplied files.
                                   #  the .pho.SPlength files should
                                   #  contain the same exact phonetic
                                   #  transliteration as in their .pho
                                   #  counterpart, except for the
                                   #  context free phonemes, which has
                                   #  been changed by 'FREEPHO=duration' ,
                                   #  with duration a real number.
EOF
  exit 1
fi

PHOS=$1
TRIPHOS=$2
TMPDIR=$3
FREEPHOS=${4//[[:space:]]/}

CROSSWORDS=${5:-"no"}
SPthreshold=${6:-0}
PHOSPlength_ext='.SPlength'   


if [[ "$CROSSWORDS" == "yes" ]]
then
    while read l ;   do
	[ -s "$l$PHOSPlength_ext"  ] ||
	error "Expected non-emtpy phonetic transliteration for file '$l$PHOSPlength_ext'"
	j=`dirname "$l"`
	k=${l##*/};
	DIR=$TMPDIR/triphos/$j
	mkdir -p "$DIR" || error "Cannot create/access temporary path '$DIR'"
	echo "$l$PHOSPlength_ext" "$DIR/$k"
    done < "$PHOS"  |
    gawk -v SPthreshold="$SPthreshold"  -v FREEPHOS="$FREEPHOS" '
BEGIN {
  N= split(FREEPHOS,aux,",")
  for ( n= 1; n <= N; ++n ) freephos[aux[n]]= 1
}
{
  input= $1
  output= $2
  N= 0
  while ( getline < input )
    for ( i= 1; i <= NF; ++i )
    {
      N+= 1
      phos[N]= $i
      if ( $i in freephos ) lengths[N]= SPthreshold+1
      else if ( $i ~ /.=./ )
      {
        split($i,aux,"=")
        if ( aux[1] in freephos )
        {
           phos[N]= aux[1]
           lengths[N]= aux[2]
        }
        else lengths[N]= -1
      }
      else lengths[N]= -1
    }
  close(input)
  ret= ""
  for ( i= 1; i <= N; ++i )
  {
    if ( phos[i] in freephos ) tri= phos[i]
    else
    {
      if ( i > 1 && !(phos[i-1] in freephos) )
         tri= phos[i-1]"-"
      else if ( (i>2) && lengths[i-1]<=SPthreshold && (phos[i-1] in freephos) )
         tri= phos[i-2]"-"
      else
         tri= ""
      tri= tri $i
      if ( i < N && !(phos[i+1] in freephos) )
         tri= tri"+"phos[i+1]
      else if ( (i<(N-1)) && lengths[i+1]<=SPthreshold \
                 && (phos[i+1] in freephos) )
         tri= tri"+"phos[i+2]
    }
    ret= ret" "tri
  }
  print ret > output
  close ( output )
}
'

# Word internal
else 

while read l ; do 
    j=`dirname $l`
    k=${l##*/};
    DIR=$TMPDIR/triphos/$j
    mkdir -p $DIR
    echo $l $DIR/$k
done < "$PHOS" |  gawk -v crosswords="$CROSSWORDS" -v FREEPHOS="$FREEPHOS" '
BEGIN {
  N= split(FREEPHOS,aux,",")
  for ( n= 1; n <= N; ++n ) freephos[aux[n]]= 1
}
{
    input= $1
    output= $2
    ret= ""
    while ( getline < input )
	for ( i= 1; i <= NF; ++i )
	{
	    if ( $i in freephos ) tri= $i
	    else
	    {
              if ( i > 1 && !($(i-1) in freephos) )
                 tri= $(i-1)"-"
              else if ( (crosswords==1) && (i>2) && ($(i-1) in freephos) )
                 tri=$(i-2)"-"
              else
                 tri= ""
              tri= tri $i
              if ( i < NF && !($(i+1) in freephos) )
                 tri= tri"+"$(i+1)
              else if ( (crosswords==1) && (i<(NF-1)) && ($(i+1) in freephos) )
                 tri= tri"+"$(i+2)
	    }
	    ret= ret" "tri
	}
    print ret > output
    close ( output )
    close ( input )
  }'
fi

while read l
do
    j=`dirname $l`
    k=${l##*/};
    DIR=$TMPDIR/triphos/$j
    echo $DIR/$k
done < $PHOS > $TRIPHOS

while read l
do
    cat $l
done < $TRIPHOS |
gawk '{for(n=1;n<=NF;++n)syms[$n]=1}
  END {for(s in syms)print s|"sort"}' > $TMPDIR/triphos.lst
