#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

function warning_msg {
  local msg=$1
  echo "[WW] generic_mutex.sh `date`: $msg" >> /dev/stderr
} 

function error_msg {
  local msg=$1
  echo "[EE] generic_mutex.sh `date`: $msg" >> /dev/stderr
} 

function debug_msg {
  local msg=$1
  echo "[II] generic_mutex.sh `date`: $msg" >> /dev/stderr
}

function mutex_generic_in {
  local lock=$1
  debug_msg "Trying to lock $lock"
  lockfile -5 -r 720 $lock.lock
  if [ $? -ne 0 ]; then #retry attemps reached
    warning_msg "File $lock could not be locked. Aborting cache"
    echo 0
  else
    debug_msg "Locked $lock.lock"
    echo $HOSTNAME > $lock
    if [ "$JOB_ID" ]; then
      echo "JOB ID: $JOB_ID" >> $lock
    fi
    if [ "$SGE_TASK_ID" ]; then
      echo "SGE_TASK_ID: $SGE_TASK_ID" >> $lock
    fi
    echo 1
  fi
}

function mutex_generic_out {
  local lock=$1
  if [ ! -f $lock ]; then
    warning_msg "Missing file $lock"
  fi
  rm -rf $lock $lock.lock
}

function is_local_filesystem {
  local dir=$1

  IS_ORIG_NFS=`df -l "$1" &> /dev/null; echo $?`
  if [ "$IS_ORIG_NFS" -eq "0" ]; then
    echo 1
  else
    echo 0
  fi
}

function print_help {
    cat <<EOF
$0 <mutex_name> [<command_to_be_mutexed>]

  <mutex_file>              Name of the mutex file
  <command_to_be_mutexed>   Command to be perfomed in the mutex

  Options:

  -h:             Print this help

EOF
}

NUMARGS=1
while getopts "h" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) ITEREX=$OPTARG;;
        o) OCCS="yes";;
        t) TARFILE=$OPTARG;;
        s) TARTRANSFILE=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -gt $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
MUTEX=$1
shift 1
COMMAND=$@

#Check if we are in local machine and execute command directly if so
if [ `is_local_filesystem $MUTEX` -eq 1 ]; then
  warning_msg "Local Machine. Aboring mutex"
  $COMMAND
  exit 0
fi

if [ `mutex_generic_in "$MUTEX"` -eq 0 ]; then
  warning_msg "Command is not going to be performed without mutex!!!"
  $COMMAND
else
  trap "warning_msg \"An error ocurred. Unlocking $MUTEX\"; mutex_generic_out $MUTEX" ERR TERM
  trap "debug_msg \"Exiting. Unlocking $MUTEX\"; mutex_generic_out $MUTEX" EXIT
  $COMMAND
fi

