#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <tmpdir> <iteration> <output-model>

  <tmpdir>        Temporal directory of the estimation
  <iteration>     Number of iterations 
  <out-model>     Path to store the output model (extensions will be added)

  Options:

  -h:             Print this help
  -c              Store the output model counts
  -o              Store state occupations  (needs also -s)
  -m              Store mixture occupations
  -s <split>      Number of splits
EOF
}

NUMARGS=3
COUNTS=""
OCCS=""
MOCCS=""
SPLITS=""
while getopts "hcoms:e:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        c) COUNTS="yes";;
        o) OCCS="yes";;
        m) MOCCS="yes";;
        s) SPLITS=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
TMP=$1
k=$2
OUT=$3
shift 3

#OPTS="$@"

#Store model
cat $TMP/model_$k > $OUT.model

#Store counts
if [ ! -z $COUNTS ]; then
  cat $TMP/model_${k}_counts > $OUT.counts
fi

#Store mix counts
if [ ! -z $MOCCS ]; then
  cat $TMP/model_${k}_mixcounts > $OUT.mixcounts
fi

#Store occupations
if [ ! -z $OCCS ]; then
  if [ -z $SPLITS ]; then
    echo "[EE] The number of splits are need to store the occupations"
    exit 1
  fi

  for (( i=1;i<=$SPLITS;i++ )); do
    if [ ! -s $TMP/occs_$i ]; then
      echo "[EE] Empty occs file $TMP/occs_$i"
      exit 1
    fi
    echo $TMP/occs_$i
  done | \
    gawk '
    NR == 1 {
      f= $0
      while ( getline < f )
      {
         if ( NF == 0 || $1 == "#" ) continue
         syms[$1]= $2
         for ( q= 1; q <= $2; ++q )
           occs[$1 SUBSEP q]= $(q+2)
      }
      close(f)
    }

    NR > 1 {
      f= $0
      while ( getline < f )
      {
         if ( NF == 0 || $1 == "#" ) continue
         syms[$1]= $2
         for ( q= 1; q <= $2; ++q )
           occs[$1 SUBSEP q]+= $(q+2)
      }
      close(f)
    }

    END {
      for ( s in syms )
      {
        ret= s"\t\t"syms[s]
        for ( q= 1; q <= syms[s]; ++q )
          ret= ret"\t\t"occs[s SUBSEP q]
        print ret
      }
    }
    ' > $OUT.occs
fi
      

