#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <tmpdir> <iteration> [tLtrain-ops]

  <tmpdir>        Temporal directory of the estimation
  <iteration>     Iteration number
  [tLtrain-ops]   Options to pass to tLtrain

  Options:

  -h:             Print this help
  -e <path>:      tLtrain PATH
  -o:             Store ocupations for CART
  -t <path>:      Load all samples from tar file
  -f:             Tar features file is splitted
  -s <path>:      Load all transcriptions from a tar file
  -g <command>:   get_cache.sh command plus options to be used
  -m:             Mutex on write
  -T <path>:      TMP folder used by get_cache and generic_mutex

EOF
}

NUMARGS=2
ITEREX=tLtrain
OCCS=""
TARFILE=""
TARTRANSFILE=""
TARSPLIT=""
MUTEXONWRITE=""
GETCACHE="get_cache.sh"
GETCACHETMP="/tmp"
while getopts "he:ot:fs:g:mT:" OPT
do
    case $OPT in
        h) print_help; exit 0;;
        e) ITEREX=$OPTARG;;
        o) OCCS="yes";;
        t) TARFILE=$OPTARG;;
        s) TARTRANSFILE=$OPTARG;;
        f) TARSPLIT="yes";;
        g) GETCACHE="$OPTARG";;
	m) MUTEXONWRITE="yes";;
	T) GETCACHETMP="$OPTARG";;
    esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
TMP=$1
k=$2
shift 2
OPTS=$@

if [ -z $SGE_TASK_ID ]; then
  echo "[EE] External variable \$SGE_TASK_ID needed"
  exit 1
fi
i=${SGE_TASK_ID}

if [ ! -z $OCCS  ]; then
  OPTS="$OPTS --print-occs $TMP/occs_$i"
fi

if [ ! -z $TARFILE ]; then
  if [ ! -z $TARSPLIT  ]; then
    TARFILE=${TARFILE}_${SGE_TASK_ID}
  fi

  echo "#Caching samples tar if necessary"
  CACHED_TAR_FILE=`$GETCACHE -p -t $GETCACHETMP $TARFILE`
  #deleting the duplicate file when finished
  if [ "$CACHED_TAR_FILE" != "$TARFILE" ]; then
    TRAPCOMMANDS="rm $CACHED_TAR_FILE"
    trap "$TRAPCOMMANDS" ERR EXIT
  fi 

  #TLK OPTION
  OPTS="$OPTS --tar $CACHED_TAR_FILE"
fi

if [ ! -z $TARTRANSFILE ]; then
  echo "#Caching transcription tar if necessary"
  CACHED_TARTRANS_FILE=`$GETCACHE -p -t $GETCACHETMP $TARTRANSFILE`
  #deleting the duplicate file when finished
  if [ "$CACHED_TARTRANS_FILE" != "$TARTRANSFILE" ]; then
    TRAPCOMMANDS="rm $CACHED_TARTRANS_FILE; $TRAPCOMMANDS"
    trap "$TRAPCOMMANDS" ERR EXIT
  fi

  #TLK OPTION
  OPTS="$OPTS --tar-trans $CACHED_TARTRANS_FILE"
fi

if [ ! -z $MUTEXONWRITE ]; then
    TEMP=`mktemp $GETCACHETMP/tmp.XXXXXX`
else
    TEMP=$TMP/counts_$i
fi

COMMAND="$ITEREX -b `wc -l < $TMP/feas_$i` \
    $TMP/model_$[k-1] $TMP/words_$i $TMP/feas_$i \
    $OPTS --print-counts -B -o $TEMP -v"

echo $COMMAND
$COMMAND

#move and delete temporals
if [ ! -z $MUTEXONWRITE ]; then
    generic_mutex.sh $HOME/.get-cache-write mv $TEMP $TMP/counts_$i
fi
