#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

. mhtrain_em_prolog.sh

function print_help {
    cat <<EOF
$0 [options] <tmpdir> <splits> <iteration> [tLupdate-ops]

  <tmpdir>        Temporal directory of the estimation
  <splits>        Number of counts
  <iteration>     Iteration number
  [tLupdate-ops]   Options to pass to tLtrain

  Options:

  -h:             Print this help
  -e <path>       Path to tLupdate
  -c              Store counts on
  -m              Store mixture counts
  -A <path>       Additional count files to add
EOF
}

NUMARGS=3
UPEX=tLupdate
COUNTS=""
MOCCS=""
ADDITIONAL_COUNTS=""
while getopts "he:cmA:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        e) UPEX=$OPTARG;;
        c) COUNTS="yes";;
        m) MOCCS="yes";;
        A) ADDITIONAL_COUNTS=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]
TMP=$1
N=$2
k=$3
shift 3

OPTS="$@"

if [ ! -z $COUNTS ]; then
  OPTS="$OPTS --print-counts  $TMP/model_${k}_counts"
fi

if [ ! -z $MOCCS ]; then
  OPTS="$OPTS --print-mixcounts  $TMP/model_${k}_mixcounts"
fi

AUX="$ADDITIONAL_COUNTS"
for (( n= 1; n <= N; ++n )) ; do 
  if [ ! -s "$TMP/counts_$n" ]; then
    echo "[EE] $TMP/counts_$n has not been correctly estimated"
    exit 1
  fi
  AUX="${AUX} $TMP/counts_$n"
done

COMMAND="$UPEX $TMP/model_$[k-1] em -o $TMP/model_$k $OPTS -B $AUX"
echo $COMMAND
$COMMAND

for (( n= 1; n <= N; ++n )) ; do 
  rm $TMP/counts_$n
done
