#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

TMPSUBMITDIR=/tmp/submit
if [ ! -d $TMPSUBMITDIR ]; then
  mkdir -p $TMPSUBMITDIR
fi

function print_help {
    cat <<EOF
$0 [options] cmd_to_submit ...
  
  cmd_to_submit     command line execution
  
  Options:
  
    -h            : Print this help
    -n <name>     : Submission name
    -w <name>     : Wait a submission
    -o <file/dir> : Redirect all outputs to a file
    -j <array>    : Launch an array of jobs, each with a different SGE_TASK_ID variable

EOF
}

SUB_NAME=""
WAIT_SUB=""
OUT=""
JOBS=""
while getopts "hn:w:o:j:" OPT
do
    case $OPT in
      h) print_help; exit 0;;
      n) SUB_NAME=$OPTARG;;
      w) WAIT_SUB=$OPTARG;;
      o) OUT=$OPTARG;;
      j) JOBS=`echo $OPTARG | awk \
          '{  split($0,a,","); 
          for(i=1;i<=length(a);i++) 
          { 
            split(a[i],j,"-"); 
            if(length(j) == 1) 
              printf("%d ",j[1]);                                                   
            else
              for(k=j[1];k<=j[2];k++)                                               
                printf("%d ",k);                                                    
          }
        }'`;;
    esac
done
shift $[OPTIND-1]

if [ -z $SUB_NAME ]; then
  SUB_NAME=`echo $1 | cut -d ' ' -f 1` #work around to solve "ls -l; sleep 10" like submits
fi

if [ -z $OUT ]; then
  OUT="$SUB_NAME.o"
elif [ -d $OUT ]; then
  OUT="$OUT/$SUB_NAME.o"
fi

if [ -f $TMPSUBMITDIR/$SUB_NAME ]; then
  echo "[WW] There is already another submission launched with the same name"
  echo "[WW] This could cause unexpected behaviour"
fi

echo "#!/bin/bash" > $TMPSUBMITDIR/$SUB_NAME
#Wait Dependency
if [ ! -z $WAIT_SUB ]; then
#include wait code
  echo '
anywait(){
for pid in "$@"; do
  while kill -0 "$pid"; do
    sleep 5
  done
done &> /dev/null
}' >> $TMPSUBMITDIR/$SUB_NAME

  echo "anywait `pgrep -x -f \"bash $TMPSUBMITDIR/$WAIT_SUB\" | tr '\n' ' '`" >> $TMPSUBMITDIR/$SUB_NAME
fi

#prolog
echo "echo \"#Started at \`date\`\" &> \$OUT" >> $TMPSUBMITDIR/$SUB_NAME

COMMAND="($*) &>> \$OUT" 
echo "$COMMAND" >> $TMPSUBMITDIR/$SUB_NAME

#Epilog
echo "echo \"#Ended at \`date\`\" &>> \$OUT" >> $TMPSUBMITDIR/$SUB_NAME
echo "rm $TMPSUBMITDIR/$SUB_NAME &> /dev/null" >> $TMPSUBMITDIR/$SUB_NAME

#Launch Job
if [ ! -z "$JOBS" ]; then
  for i in $JOBS; do
    SGE_TASK_ID=$i OUT=$OUT.$i bash $TMPSUBMITDIR/$SUB_NAME &
  done
else
  OUT=$OUT bash $TMPSUBMITDIR/$SUB_NAME &
fi
