#!/bin/bash

# Copyright 2012,2013,2014 transLectures-UPV Team
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

set -e

function print_help {
    cat <<EOF
USAGE: $0 [options] <output_tar_name> <list_to_tar>"
  
  <output_tar_name>   Path to the tar file to be created
  <list_to_tar>       Path to the list of samples to be created

  Options:
  -s <splits>:        Number of tar to be created, splitting the list
EOF
}

NUMARGS=2
SPLITS=""
while getopts "hs:" OPT
do
  case $OPT in
        h) print_help; exit 0;;
        s) SPLITS=$OPTARG;;
  esac
done
[ $[$#-OPTIND+1] -eq $NUMARGS ] || { print_help >&2; exit 1; }
shift $[OPTIND-1]

OUT_TAR=$1
LST=$2

if [ "$SPLITS" == "" ]; then
  if [ ! -f $1 ]; then
    tar hcf $OUT_TAR -T $LST
  fi
else
  echo $SPLITS >> /dev/stderr
  TMPDIR=`mktemp -d`
  split_list.sh $SPLITS $TMPDIR/lst $LST
  for (( i=1 ; i<=$SPLITS ; i++ )); do
    tar hcf ${OUT_TAR}_$i -T $TMPDIR/lst_$i
  done
  rm -rf $TMPDIR
fi

