/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  cmdl.c - Implementació de 'cmdl.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "cmdl.h"
#include "error.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define COUNTER 0
#define VALUE 1

#define add_short(CMDL,SNAME,VAR) \
        if ( (SNAME) != CMDL_NOSHORT ) \
          (CMDL)->sopts[(unsigned char) (SNAME)]= (VAR)

#define add_long(CMDL,LNAME,VAR) \
        if ( (LNAME) != CMDL_NOSHORT ) \
          add_long_aux ( (CMDL), (LNAME), (VAR) )

#define IS_VALUE(VAR) ((VAR)->type == VALUE)




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
add_long_aux (
	      cmdl_t           cmdl,
	      const char      *lname,
	      struct cmdl_var *var
	      )
{
  
  struct cmdl_long *new;
  
  
  new= mem_alloc ( struct cmdl_long, 1 );
  new->var= var;
  new->name= lname;
  new->next= NULL;
  if ( cmdl->lopts_last != NULL )
    {
      cmdl->lopts_last->next= new;
      cmdl->lopts_last= new;
    }
  else cmdl->lopts_first= cmdl->lopts_last= new;
  
} /* add_long_aux */


static struct cmdl_var *
get_var (
	 cmdl_t cmdl
	 )
{
  
  struct cmdl_var *new;
  
  
  new= mem_alloc ( struct cmdl_var, 1 );
  new->next= cmdl->vars;
  cmdl->vars= new;
  
  return new;
  
} /* end get_var */


static struct cmdl_var *
search_long (
	     cmdl_t      cmdl,
	     const char *name
	     )
{
  
  struct cmdl_long *prev, *p;
  
  
  for ( prev= NULL, p= cmdl->lopts_first;
	p != NULL && strcmp ( p->name, name );
	prev= p, p= p->next );
  
  if ( p == NULL ) return NULL;
  if ( IS_VALUE ( p->var ) && cmdl->lopts_last != p )
    {
      if ( prev == NULL )
	{
	  cmdl->lopts_first= p->next;
	  cmdl->lopts_last->next= p;
	  cmdl->lopts_last= p;
	}
      else
	{
	  prev->next= p->next;
	  cmdl->lopts_last->next= p;
	  cmdl->lopts_last= p;
	}
      p->next= NULL;
    }
  
  return p->var;
  
} /* end search_long */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

/* NOTA: explota el fet de que
 * normalment no vaig a afegir opcions
 * sense opció llarga i curta.
 */

void
add_cmdl_counter (
		  cmdl_t      cmdl,
		  char        sname,
		  const char *lname,
		  int        *var
		  )
{
  
  struct cmdl_var *nvar;
  
  
  nvar= get_var ( cmdl );
  nvar->type= COUNTER;
  nvar->ptr.i= var;
  *var= 0;
  add_short ( cmdl, sname, nvar );
  add_long ( cmdl, lname, nvar );
  
} /* end add_cmdl_counter */


void
add_cmdl_value (
		cmdl_t       cmdl,
		char         sname,
		const char  *lname,
		const char **var
		)
{
  
  struct cmdl_var *nvar;
  
  
  nvar= get_var ( cmdl );
  nvar->type= VALUE;
  nvar->ptr.s= var;
  *var= NULL;
  add_short ( cmdl, sname, nvar );
  add_long ( cmdl, lname, nvar );
  
} /* end add_cmdl_value */


cmdl_t
create_cmdl ()
{
  
  cmdl_t new;
  int i;
  
  
  new= mem_alloc ( struct cmdl, 1 );
  new->vars= NULL;
  new->lopts_first= NULL;
  new->lopts_last= NULL;
  for ( i= 0; i < 256; ++i )
    new->sopts[i]= NULL;
  
  return new;
  
} /* end create_cmdl */


void
free_cmdl (
	   cmdl_t cmdl
	   )
{
  
  struct cmdl_long *p_long, *fem_long;
  struct cmdl_var *p_var, *fem_var;
  
  
  p_long= cmdl->lopts_first;
  while ( p_long != NULL )
    {
      fem_long= p_long;
      p_long= p_long->next;
      free ( fem_long );
    }
  p_var= cmdl->vars;
  while ( p_var != NULL )
    {
      fem_var= p_var;
      p_var= p_var->next;
      free ( fem_var );
    }
  free ( cmdl );
  
} /* end free_cmdl */


int
parse_cmdl (
	   cmdl_t       cmdl,
	   int          argc,
	   const char  *argv[],
	   const char **prog
	   )
{
  
  int nargs, i;
  const char *p, *q;
  struct cmdl_var *var;
  
  
  if ( prog != NULL )
    *prog= argv[0];
  
  i= 1;
  nargs= 0;
  while ( i < argc )
    {
      p= argv[i++];
      if ( *p != '-' ) argv[nargs++]= p;
      else
	{
	  if ( p[1] == '-' )
	    {
	      if ( p[2] == '\0' )
		for ( ; i < argc; ++i )
		  argv[nargs++]= argv[i];
	      else
		{
		  var= search_long ( cmdl, &(p[2]) );
		  if ( var == NULL )
		    argv[nargs++]= p;
		  else if ( IS_VALUE ( var ) )
		    {
		      if ( i == argc )
			argv[nargs++]= p;
		      else
			*(var->ptr.s)= argv[i++];
		    }
		  else ++(*(var->ptr.i));
		}
	    }
	  else
	    {
	      var= cmdl->sopts[ (unsigned char) p[1] ];
	      if ( var == NULL )
		argv[nargs++]= p;
	      else if ( IS_VALUE ( var ) )
		{
		  if ( p[2] != '\0' )
		    *(var->ptr.s)= &(p[2]);
		  else if ( i == argc )
		    argv[nargs++]= p;
		  else
		    *(var->ptr.s)= argv[i++];
		}
	      else
		{
		  for ( q= &(p[2]); *q != '\0'; ++q )
		    {
		      var= cmdl->sopts[(unsigned char) *q];
		      if ( var == NULL || IS_VALUE ( var ) ) break;
		    }
		  if ( *q != '\0' )
		    argv[nargs++]= p;
		  else
		    for ( ++p; *p != '\0'; ++p )
		      {
			var= cmdl->sopts[(unsigned char) *p];
			++(*(var->ptr.i));
		      }
		}
	    }
	}
    }
  
  return nargs;
  
} /* end parse_cmdl */
