/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor, Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  cmdl.h - Per a processar la línia de comandaments.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */

#ifndef __CMDL_H__
#define __CMDL_H__


/* MACROS */

#define CMDL_NOSHORT '\0'
#define CMDL_NOLONG NULL


/* TIPUS */

struct cmdl_var
{
  
  unsigned char    type;
  union
  {
    
    int         *i;
    const char **s;
    
  }                ptr;
  struct cmdl_var *next;
  
};

struct cmdl_long
{
  
  const char       *name;
  struct cmdl_var  *var;
  struct cmdl_long *next;
  
};

struct cmdl
{
  
  struct cmdl_var  *vars;
  struct cmdl_long *lopts_first;
  struct cmdl_long *lopts_last;
  struct cmdl_var  *sopts[256];
  
};

typedef struct cmdl * cmdl_t;


/* FUNCIONS */

/* Quan s'afegeix una opció, 'sname'
 * és el caràcter per a l'opció curta i
 * 'lname' el nom de l'opció llarga.
 * L'opció llarga ha de ser una cadena de
 * caràcters que ha d'estar en memòria sense
 * modificar-se. CMDL_NOSHORT i CMDL_NOLONG
 * indiquen que no hi ha opció curta ni opció
 * llarga respectivament.  
 */

/* Afegeix una opció sense paràmetre
 * que incrementa la variable 'var'.
 */
void
add_cmdl_counter (
		  cmdl_t      cmdl,
		  char        sname,
		  const char *lname,
		  int        *var
		  );

/* Afegeix una opció amb paràmetre
 * i el emmagatzema en 'var'.
 */
void
add_cmdl_value (
		cmdl_t       cmdl,
		char         sname,
		const char  *lname,
		const char **var
		);

cmdl_t
create_cmdl ();

void
free_cmdl (
	   cmdl_t cmdl
	   );

/* Parseja la línia de comandaments. Les opcions
 * s'actualitzen i s'eliminen de 'argv', els arguments
 * es deixen en 'argv' en l'ordre d'aparició. Torna
 * el nombre d'arguments. En 'prog' es guarda la cridada al
 * programa o res si 'prog' és NULL. La sintaxi de les opcions
 * és la següent:
 *
 * --: S'elimina de 'argv' i la resta de tokens es consideren
 *     arguments.
 * -c [token]: Si existeix l'opció curta 'c' i no és un comptador,
 *             elimina els dos tokens de 'argv'. Si no es consideren
 *             dos arguments.
 * --name token: Si 'name' és una opció llarga i no és comptador
 *               s'eliminen de 'argv'. Si no es consideren arguments.
 * --name: Si 'name' és una opció llarga i és comptador s'elimina
 *         de 'argv'. Si no es considera argument.
 * -token: Si el primer caràcter és una opció curta i no és comptador
 *         elimina el token de 'argv'. Si tots el caràcters son
 *         comptadors elimina el token de 'argv'. Si no es considera
 *         un argument.
 */
int
parse_cmdl (
	   cmdl_t       cmdl,
	   int          argc,
	   const char  *argv[],
	   const char **prog
	   );


#endif /* __CMDL_H__ */
