/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *  Copyright 2007,2008,2009,2010,2011,2012 Jesús Andrés Ferrer
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#include "options.h"

#include <cassert>
#include <cstdlib>
#include <cstring>
#include<iostream>
#include <sstream>
#include "../../intl.h"

using namespace std;

namespace JAF{
//------------------------------------------------------------
void  Options::print_type(option &opt,ostream &out)
{
  //  streamsize old= out.getwidth();
  //  out.setwidth(20);
    if(opt._type==option::OPT_NULL){ out<<"(null)";}
    else if(opt._type==option::OPT_CLASS) { out<<"["<<opt._value<<"]";}
    else if(opt._type==option::OPT_UINT) { out<<"<unsigned integer>";}
    else if(opt._type==option::OPT_INT) { out<<"<signed integer>";}
    else if(opt._type==option::OPT_ULONG) { out<<"<unsigned long>";}
    else if(opt._type==option::OPT_LONG) { out<<"<signed long>";}
    else if(opt._type==option::OPT_CHAR) { out<<"<char>";}
    else if(opt._type==option::OPT_STRCHAR) { out<<"<string>";}
    else if(opt._type==option::OPT_STRING) { out<<"<std::string>";}
    else if(opt._type==option::OPT_FLOAT) { out<<"<float>";}
    else if(opt._type==option::OPT_DOUBLE) { out<<"<double>";}
    else if(opt._type==option::OPT_LDOUBLE) { out<<"<long double>";}
    else if(opt._type==option::OPT_TRUE){ out<<" true if set ";;}
    else if(opt._type==option::OPT_FALSE){ out<<" false if set ";}
    else if(opt._type==option::OPT_READ){ out<<"<string>";}
    else if(opt._type==option::OPT_WRITE){ out<<"<string>";}
//    else if(opt._type==option::OPT_) { out<<"<>";}
    else if( (opt._type>0)){ out<<"<int "<<opt._type<<" >";}
    else{ out<<"error";}
    //  out.setwidth(old);
}
//------------------------------------------------------------
option * Options::find(const string val)
{
  unsigned long n;
  for(n=0;n<_opts.size();++n)
    {
      if(_opts[n]._value==val)
	{return &_opts[n];}
    }
  return 0;
}
//------------------------------------------------------------
void Options::syntax(ostream &out)
{
  //  streamsize old= out.getwidth();
  //  out.setf(ios_base::left,ios_base::adjust_field);
  unsigned long n;
  out<<_("syntax:")<<_name<<" "<<_("options")<<" "<<endl;
  out<<OPT_SEPARATOR<<endl;
  out<<_("options:")<<endl<<endl;
  for(n=0;n<_opts.size();++n)
    {
      if(_opts[n]._type==option::OPT_NULL) continue;
      else if(_opts[n]._type==option::OPT_CLASS)
	{
	  out<<endl<<"["<<_opts[n]._value<<"] "<<"\t"<<_opts[n]._desc<<endl; 
	  out<<OPT_SEPARATOR<<endl;
	}else{
	  char required=(_opts[n]._req)?'*':' ';
	  // out.width(15);
	  out<<'-'<<_opts[n]._value<<required<<"\t";
	  print_type(_opts[n],out);
	  // out.width(50);
	  cout<<"\t"<<_opts[n]._desc<<endl;
	}
    }
  cout<<endl<<"*"<<"\t "<<_("stands for mandatory options") << "\n";
  //out.setwidth(old);
}
//------------------------------------------------------------
int Options::read (int argc, char * argv[])
{
 int argi=1; 
    unsigned long n;
    bool processed;
    while(argi<argc)
      {  
	if(strlen(argv[argi])==0){++argi; continue;} 
	else if(argv[argi][0]!='-') break;
        else{// Look for the option
            string arg(argv[argi]+1); // create the arg string without the '-'
	    processed=false;
            for(n=0;n<_opts.size();++n)
            {
	      if(_opts[n]._value==arg){//match
		_opts[n]._count++; //increase the number of times it has been seen
		if(_opts[n]._type==option::OPT_CLASS){
		  cerr<<"[EE] "<<_("Reading options OPT_CLASS:") << " "
		     <<_opts[n]._value<<" "<<_("seen as an option") <<"\n";
		}
		else  if(_opts[n]._type==option::OPT_NULL) {	    processed=true; break;}
		else if(_opts[n]._type==option::OPT_READ){
		  istringstream sin(argv[++argi]);
		  sin>>_read;
		  if(sin.bad()) 
		    { cerr<<"[EE] "<< _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") <<" ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  ifstream f(_read.c_str()); // open the _read file
		  if(f.fail())
		    { cerr<<"[EE] "<< _("Unable to open read file:")<<_read<<endl; 
		    return -1;}
		  if(!this->read(f))
		    {cerr<<"[EE] "<< _("Error reading parameters from file:")<<_read<<endl;
		    return -1;}
		  //erase from the list of parameters
		  strcpy(argv[argi],"");
		  strcpy(argv[argi-1],"");
		  //argv[argi]=(const char*)"";  argv[argi-1]=(const char *)"";
		  //start again to process the argument list
		  argi=0; 	   
		  processed=true; 
		  break; 
		}else if(_opts[n]._type==option::OPT_WRITE){
		  istringstream sin(argv[++argi]);
		  sin>>_write;
		  if(sin.bad()) 
		    { cerr<<"[EE] "<< _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  processed=true;
		}else if(_opts[n]._type==option::OPT_UINT){
		  unsigned int tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] "<< _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting")<<" ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] "<<_("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<unsigned int *>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_INT){
		  int tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] "<<_("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<int *>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_ULONG){		    
		  unsigned long tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" " << _("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<unsigned long  *>(_opts[n]._ptrval)=tmp;
		  processed=true; 
		    }
		else if(_opts[n]._type==option::OPT_LONG){		    
		  long tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the paramer of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<long *>(_opts[n]._ptrval)=tmp;
		  	    processed=true;    }
		else if(_opts[n]._type==option::OPT_FLOAT){		    
		  float tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<float*>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_DOUBLE){
		  double tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<double *>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_LDOUBLE){
		  long double tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<long double *>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_CHAR){		    
		  char tmp;      istringstream sin(argv[++argi]);
		  sin>>tmp;
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<char*>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_STRING){
		  string tmp,tmp2;      istringstream sin(argv[++argi]);
		  while(sin>>tmp2){
		    if(tmp.size()==0) tmp=tmp+tmp2; 
		    else  tmp=tmp+' '+tmp2; 
		  }
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<string*>(_opts[n]._ptrval)=tmp;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_STRCHAR){
		  string tmp,tmp2;      istringstream sin(argv[++argi]);
		  while(sin>>tmp2)
		    {
		      if(tmp.size()==0) tmp=tmp+tmp2; 
		      else  tmp=tmp+' '+tmp2; 
		    }
		  if(sin.bad()) 
		    { cerr<<"[EE] " << _("Error something wrong reading the param of")
			  <<"-"<<_opts[n]._value<<" "<<_("expecting") << " ";
		    print_type(_opts[n],cerr);
		    return -1;
		    }
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<char**>(_opts[n]._ptrval)=strdup(tmp.c_str());
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_TRUE){
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<bool*>(_opts[n]._ptrval)=1;
		  processed=true;
		}
		else if(_opts[n]._type==option::OPT_FALSE){
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<bool*>(_opts[n]._ptrval)=0;
		  processed=true;
		  break;
		} else if(_opts[n]._type>0){  //Integer
		  if(_opts[n]._ptrval==0)
		    {cerr<<"[EE] " << _("invalid pointer in options") << "\n"; return -1;}
		  *static_cast<unsigned int *>(_opts[n]._ptrval)=_opts[n]._type;
		  processed=true;
		}else{
		  cerr<<"[EE] " << _("Error unknown option._type:")<<
		    _opts[n]._type<<endl; 
		  return -1;}
	      }
            }
	    if(!processed){
	      cerr<<"[EE] " << _("Error unknown option: -")<<arg<<endl;;
	      return -1;
	    }
	}
	++argi;
      }
    n=0;
    bool error=false;
    for(n=0;n<_opts.size();++n)
      {if(_opts[n]._req && (_opts[n]._count==0))
	{cerr<<"[EE] " << _("Error option:") << " -" <<
	   _opts[n]._value<<_(" is mandatory and is not provided") << "\n";
	error=true; break;}
      }
    if(error) return -1;
    //If we had  No problem at all, and we had to write options, 
    // we  write  them
    if(_write.size()>0){
      ofstream of(_write.c_str());
      if(of.fail()){cerr<<"[EE] " << _("Opening file")<<_write<<endl; return 0;}
      this->write(of);
    }
    if(argi<argc) return argc-argi;
    return 0;
}

//------------------------------------------------------------
bool Options::write(ostream &out){
unsigned long int n;
 for(n=0;n<_opts.size();++n)
   {
     if(!print_option(_opts[n],out)) return false;
   }
 return true;
}
//------------------------------------------------------------
bool Options::read(istream &in){
  const int mem_buff=1024*1024;
  char buff[mem_buff];
  string token;
  vector<string> tokens;
  while(1)
    {
      in.getline(buff,mem_buff);
      if(in.eof()) { break; }
      { 
	istringstream tok_stream(buff);
	unsigned int NF=0;
	string value;
	tokens.clear();
	while(tok_stream>>token)
	  {	   
	    ++NF;
	    if(token[0]=='#') break; // the rest of the line is a comment
	    tokens.push_back(token);
	  }
	read_option(tokens);
      }
      if( in.fail()) { return false; break;}
    }
  return true;
}

//---------------------------------------------------------------------------
bool  Options::read_option(vector<string> & tokens)
{
  string value;
  option *opt;
  if(tokens.size()==0) return true; // Blank line, ignore it
  // test if it is a Class
  if ( (tokens[0][0]=='[') && (tokens[0][tokens[0].size()-1]==']') )
    { 
      value=tokens[0].substr(1,tokens[0].size()-2); // Erase '[' and ']'
      opt=find(value);
      if( (opt==0) || (opt->_type!=option::OPT_CLASS)) 
	{cerr<<"[EE] " << _("class option ")<<value<<_(" not found or was not CLASS ")<<endl;
	return false;}
    }
  else if ( (tokens.size()<2) || (tokens[1][0]!='='))
    { cerr <<"[EE] " << _("Reading parameters file, expecting ") << "\"=\" \n";
    return false;} // error  either size==1 or size >=2 and tok[1] == "="
  else{ // Ok, everything is ok
    value=tokens[0];
    opt=find(value);
    if(opt==0)
      {cerr<<"[EE] " << _("Option") << " -"<<value<<_("not found in application options")<<endl;
      return false;}
    //[DDD
    //cout<<"[GG] read option :"<< value<< tokens[1]<< tokens[2]<<endl;
    //DDD]
    switch(opt->_type)
      {
      case option::OPT_NULL:
      case option::OPT_WRITE:
      case option::OPT_READ:
	cerr<<"[WW] " << _("Neither Read nor Write options can be read from a file")<<endl;
	break;
      case option::OPT_CLASS:
	cerr<<"[EE] " << _("Option ")<<opt->_value<<_(" is a Class option but is stored as")
	    <<_(" as a normal option")<<endl;
	return false;
	break;
      case option::OPT_UINT:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<unsigned int *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_INT:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<int *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_ULONG:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<unsigned long *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_LONG:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<long *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_FLOAT:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<float *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_DOUBLE:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<double *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_LDOUBLE:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<long double *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_CHAR:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<char *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_TRUE:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<bool *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_FALSE:
	if  (tokens.size()<3) 
	  { cerr <<"[EE] " << _("Reading parameters file, expecting a value ") << "\n";return false;} 
	{istringstream sspar(tokens[2]);
	sspar>>*static_cast<bool *>(opt->_ptrval);
	++opt->_count;
	}break;
      case option::OPT_STRING:
	{
	  if(tokens.size()>=3){
	    string strval(tokens[2]);
	    for(unsigned long int i=3;i<tokens.size();++i)
	      strval=strval+' '+tokens[i];
	    *static_cast<string *>(opt->_ptrval)=strval;
	    ++opt->_count;
	    }else{
	    *static_cast<string *>(opt->_ptrval)=string();
	    }
	}break;
      case option::OPT_STRCHAR:
	{
	  if(tokens.size()>=3){
	    string strval(tokens[2]);
	    for(unsigned long int i=3;i<tokens.size();++i)
	      strval=strval+' '+tokens[i];
	    *static_cast<char * *>(opt->_ptrval)=strdup(strval.c_str());
	    ++opt->_count;
	    }else{
	      *static_cast<char * *>(opt->_ptrval)=0;
	    }
	}break;
      default:
	if(opt->_type>0){  //Integer
	  *static_cast<unsigned int *>(opt->_ptrval)=opt->_type;
	}
	else{
	  cerr<<"[EE] " << _("Unrecognised option in Options") << "\n";
	  return false;}
      }
  }
  return true;
}

//------------------------------------------------------------
bool Options::print_option(option &opt,ostream &out)
{
  switch(opt._type)
       { 
       case option::OPT_NULL:
       case option::OPT_WRITE:
       case option::OPT_READ:
	 break;
       case option::OPT_CLASS:
	 out<<endl<<"["<<opt._value<<"]"<<endl;
	 break;
       case option::OPT_UINT:
	 {assert(opt._ptrval!=0);
	 unsigned int  tmp=*static_cast<unsigned int *>(opt._ptrval);
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_INT:
	 {assert(opt._ptrval!=0);
	 int tmp=*static_cast<int*>(opt._ptrval);
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_ULONG:
	 {assert(opt._ptrval!=0);
	 unsigned long tmp=*static_cast<unsigned long *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_LONG:
	 {assert(opt._ptrval!=0);
	 long tmp=*static_cast<long *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_FLOAT:
	 {assert(opt._ptrval!=0);
	 float tmp=*static_cast<float *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_DOUBLE:
	 {assert(opt._ptrval!=0);
	 double tmp=*static_cast<double *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_LDOUBLE:
	 {assert(opt._ptrval!=0);
	 long double  tmp=*static_cast<long double *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_CHAR:
	 {assert(opt._ptrval!=0);
	 char tmp=*static_cast<char *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_STRCHAR:
	 {assert(opt._ptrval!=0);
	 char *tmp=*static_cast<char * *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_STRING:
         {assert(opt._ptrval!=0);
	 string tmp=*static_cast<string *>(opt._ptrval);
	 //out<<opt._value<<" = "<<tmp<<endl;
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_TRUE:
	 {bool tmp=*static_cast<bool *>(opt._ptrval);
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       case option::OPT_FALSE:
	 {bool tmp=*static_cast<bool *>(opt._ptrval);
	 if(_extended) out<<opt._value<<" = "<<tmp<<" # "<<opt._desc<<endl;
	 else out<<opt._value<<" = "<<tmp<<endl;
	 break;}
       default:
	if (opt._type>0){  //Integer
	  if(opt._type==*static_cast<int*>(opt._ptrval))
	    out<<opt._value<<" = "<<opt._type<<" # "<<opt._desc<<endl;
	}else{
	 cerr<<"[EE] " << _("Unrecognised option in Options") << "\n";
	 return false;
	}
       }
  return true;
}

};//nm
