/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *  Copyright 2007,2008,2009,2010,2011,2012 Jesús Andrés Ferrer
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

#ifndef JAF_OPTIONS_H__
#define JAF_OPTIONS_H__

#include <fstream>
#include <vector>
#include <string>
#include <cstdlib>

using namespace std;
namespace JAF{
// This class is a structure which stores one option
struct option{
  string _value;   ///< The value of the option, the string after '-'
  string _desc;	   ///< The option description
  int _type;	   ///< which type of data is stored 
  void * _ptrval;  ///< The pointer where the value must be stored
  int    _count;   ///< count of times that
  bool   _req;	   ///< Whether the option is required or not

public:
  enum{OPT_NULL=0,
	 OPT_UINT=-1,OPT_INT=-2, OPT_ULONG=-3, OPT_LONG=-4, 
	 OPT_TRUE=-5,OPT_FALSE=-6,
	 OPT_CHAR=-10,OPT_STRCHAR=-11,OPT_STRING=-12, 
	 OPT_FLOAT=-20, OPT_DOUBLE=-21,OPT_LDOUBLE=-22, 
	 OPT_READ=-30, OPT_WRITE=-31,
	 OPT_CLASS=-99};
public:
  option():_type(OPT_NULL),_ptrval(0),_count(0),_req(false){}
  option(const string& value,const string desc,
	 int type=OPT_NULL,void *ptr=0, bool req=false)
    :_value(value),_desc(desc),_type(type),
     _ptrval(ptr),_count(0), _req(req) {}
  //    if ((_type==OPT_STRCHAR) &&
  //    (_ptrval!=0)
  //    &&(*_ptrval!=0)) 
  // { free(*_ptrval);}

};

// This class stores all the possible options of the aplication
class Options{
protected:
  vector<option> _opts; 	///< vector of options
  string _name ;		///< Name of the program
  ///The files where the defaultparameters are stored 
  //and are going to be stored
  string _read,_write;         
  bool _extended; ///< if description should be output in files

  /// for adding to the output
  void  print_type(option &opt,ostream &out); 
  /// Print an option to the stream out
  bool print_option(option &opt,ostream &out);
  bool read_option(vector<string> & tokens);
public:
  inline Options(const char *progname)
    :_name(progname),_extended(0){};
  inline Options(const string &progname):_name(progname){};
  inline ~Options()
  {
    for(unsigned long int n=0;n<_opts.size();++n)
      {
	if ((_opts[n]._type==option::OPT_STRCHAR) 
	    && 
	    (_opts[n]._ptrval!=0)
	    &&
	    (*static_cast<char**>(_opts[n]._ptrval)!=0))
	  {free(*static_cast<char**>(_opts[n]._ptrval)); 
	  *static_cast<char**>(_opts[n]._ptrval)=0;}
      }
  }
  
  // Find an option
  option * find(const string val);

  /// extended output
  void setExtended(){ _extended=true;}
  void clearExtended(){ _extended=false;}

  // Add an option to the set of possible options
  inline bool add(const string value,const string desc,
		  int type=option::OPT_NULL,
		  void *ptr=0, bool req=false)
  {if(find(value)==0) 
    _opts.push_back(option(value,desc,type,ptr,req));
  else return false;
  return true;}

  // Add an option to the set of possible options
  template <typename tt>  
  inline bool add(const string value,const string desc,tt &ptr, bool req=false)
  { return this->add(value,desc,option::OPT_NULL,&ptr,req);}
  
  inline bool addFalse(const string value,const string desc,bool &ptr, bool req=false)
  {return this->add(value,desc,option::OPT_FALSE,&ptr,req);}
  
  inline bool addTrue(const string value,const string desc,bool &ptr, bool req=false)
  {return this->add(value,desc,option::OPT_TRUE,&ptr,req);}
 


  inline bool addInt(const string value,const string desc,
		     unsigned int val_int,void *ptr, bool req=false)
  {  if(find(value)==0) 
    _opts.push_back(option(value,desc,val_int,ptr,req));
  else return false;
  return true;}
  // Add an option to the set of possible options
  inline bool add(const option opt) 
  {if(find(opt._value)==0) _opts.push_back(opt);
  else return false;
  return true;}
  // Add a Class option
  inline bool addClass(const string cat,const string desc)
  {if(find(cat)==0)  
    _opts.push_back(option(cat,desc,option::OPT_CLASS));
   else return false;
  return true;}
  // Add a Write option
  inline bool addWrite(const string name,
		       const string desc="File to which the\
 options will be written")
  {if(find(name)==0)  
    _opts.push_back(option(name,desc,option::OPT_WRITE));
   else return false;
  return true;}
  
  inline bool addRead(const string name,
		const string desc="File from where the\
 default options will be read")
  {if(find(name)==0)
    _opts.push_back(option(name,desc,option::OPT_READ));
   else return false;
  return true;}
  
  // Write the program syntax
  void syntax(ostream &out);
  // Write the current options to a file
  bool write(ostream &out);
  // Read the current options to a file
  bool read (istream &in);
  // Read the options from command line
  int read (int argc,char * argv[]);
};



template <>  
inline bool Options::add(const string value,const string desc,int &ptr, bool req)
{return this->add(value,desc,option::OPT_INT,&ptr,req);}
template <>
inline bool Options::add(const string value,const string desc,unsigned int &ptr, bool req)
{return this->add(value,desc,option::OPT_UINT,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,unsigned long &ptr, bool req)
{return this->add(value,desc,option::OPT_ULONG,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,signed long &ptr, bool req)
{return this->add(value,desc,option::OPT_LONG,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,bool &ptr, bool req)
{return this->add(value,desc,option::OPT_TRUE,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,float &ptr, bool req)
{return this->add(value,desc,option::OPT_FLOAT,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,double &ptr, bool req)
  {return this->add(value,desc,option::OPT_DOUBLE,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,long double &ptr, bool req)
{return this->add(value,desc,option::OPT_LDOUBLE,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,char &ptr, bool req)
{return this->add(value,desc,option::OPT_CHAR,&ptr,req);}
template <>  
inline bool Options::add(const string value,const string desc,char *&ptr, bool req)
{return this->add(value,desc,option::OPT_STRCHAR,ptr,req);}
template <>
inline bool Options::add(const string value,const string desc,std::string &ptr, bool req)
{return this->add(value,desc,option::OPT_STRING,&ptr,req);}




#define OPT_SEPARATOR "-----------------------------------------------------"
}; //nm
#endif
