/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  Miguel Ángel del Agua Teba - mdelagua@dsic.upv.es, madelagua@gmail.com
 *  Jesús Andrés Ferrer - jandres@dsic.upv.es
 *  Adrià Giménez Pastor - agimenez@dsic.upv.es
 */

#ifndef CUDAUTILS_H_
#define CUDAUTILS_H_

#include <cuda.h>
#include <cuda_runtime.h>
#include <curand_kernel.h>
#include <curand.h>
#include <cublas_v2.h>
#include "../intl.h"


#define IDX2C(i, j, ld) (((j)*(ld))+(i))
/* Positionning of a 3D Matrix: lr=|ROWS|, sMax=|MaxRows*MaxCols| */
#define IDX3C(i, j, k, lr, sMax) (((k)*(sMax))+((j)*(lr))+(i))
#define CUDA_CALL(x) do { if ((x) != cudaSuccess) {\
  printf(_("Error at %s:%d : %s, %d\n"), __FILE__, __LINE__, cudaGetErrorString(x), x); \
  return EXIT_FAILURE;}} while (0)
#define CUBLAS_CALL(x) do { if ((x) != CUBLAS_STATUS_SUCCESS) {\
  printf(_("Error at %s:%d : %s, %d\n"), __FILE__, __LINE__, cublasGetStatusString(x), x); \
  return EXIT_FAILURE;}} while (0)
#define CURAND_CALL(x) do { if ((x) != CURAND_STATUS_SUCCESS) { \
  printf(_("Error at %s:%d\n"), __FILE__, __LINE__);\
  return EXIT_FAILURE;}} while (0)

inline static const char * cublasGetStatusString(cublasStatus_t error) {
  switch (error) {
  case CUBLAS_STATUS_SUCCESS:
    return "CUBLAS_STATUS_SUCCESS";

  case CUBLAS_STATUS_NOT_INITIALIZED:
    return "CUBLAS_STATUS_NOT_INITIALIZED";

  case CUBLAS_STATUS_ALLOC_FAILED:
    return "CUBLAS_STATUS_ALLOC_FAILED";

  case CUBLAS_STATUS_INVALID_VALUE:
    return "CUBLAS_STATUS_INVALID_VALUE";

  case CUBLAS_STATUS_ARCH_MISMATCH:
    return "CUBLAS_STATUS_ARCH_MISMATCH";

  case CUBLAS_STATUS_MAPPING_ERROR:
    return "CUBLAS_STATUS_MAPPING_ERROR";

  case CUBLAS_STATUS_EXECUTION_FAILED:
    return "CUBLAS_STATUS_EXECUTION_FAILED";

  case CUBLAS_STATUS_INTERNAL_ERROR:
    return "CUBLAS_STATUS_INTERNAL_ERROR";
  }

  return "<unknown>";
}

cublasStatus_t cublasgemm(cublasHandle_t handle,
                          cublasOperation_t transa,
                          cublasOperation_t transb, int m, int n,
                          int k, const double * kAlpha,
                          const double * A_matrix, int lda,
                          const double * B,
                          int ldb, const double * kBeta,
                          double * C, int ldc) {
  return cublasDgemm(handle, transa, transb, m, n, k, kAlpha,
                     A_matrix, lda, B, ldb, kBeta, C, ldc);
}

cublasStatus_t cublasgemm(cublasHandle_t handle,
                          cublasOperation_t transa,
                          cublasOperation_t transb, int m, int n,
                          int k, const float * kAlpha,
                          const float * A_matrix, int lda,
                          const float * B, int ldb,
                          const float * kBeta, float * C, int ldc) {
  return cublasSgemm(handle, transa, transb, m, n, k, kAlpha,
                     A_matrix, lda, B, ldb, kBeta, C, ldc);
}

cublasStatus_t cublascopy(cublasHandle_t handle, int n,
                          const double * x, int incx, double * y,
                          int incy) {
  return cublasDcopy(handle, n, x, incx, y, incy);
}

cublasStatus_t cublascopy(cublasHandle_t handle, int n,
                          const float * x, int incx, float * y,
                          int incy) {
  return cublasScopy(handle, n, x, incx, y, incy);
}

cublasStatus_t cublasaxpy(cublasHandle_t handle, int n,
                          const double          *alpha,
                          const double          *x, int incx,
                          double                *y, int incy) {
  return cublasDaxpy(handle, n, alpha, x, incx, y, incy);
}
cublasStatus_t cublasaxpy(cublasHandle_t handle, int n,
                          const float          *alpha,
                          const float          *x, int incx,
                          float                *y, int incy) {
  return cublasSaxpy(handle, n, alpha, x, incx, y, incy);
}

#endif
