/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  Miguel Ángel del Agua Teba - mdelagua@dsic.upv.es, madelagua@gmail.com
 *  Jesús Andrés Ferrer - jandres@dsic.upv.es
 *  Adrià Giménez Pastor - agimenez@dsic.upv.es
 */

#include <cuda.h>
#include <cuda_runtime.h>
#include "./cudautils.h"

#ifndef DEVICEMEMORY_H_
#define DEVICEMEMORY_H_

template<class T>
struct DeviceElement {
  T ** variable; ///< Pointer to device variable
  int length; ///< Its length
  bool init; ///< Sets whether it should be initialised or not
  DeviceElement(T ** variable, int length, bool init): variable(variable), length(length), init(init){}
};

template <class T>
class DeviceMemory {
 public:
 DeviceMemory() {};
 ~DeviceMemory() {
   for (int i = 0; i < mem_.size(); ++i)
     DeallocVariable(mem_[i].variable);
   mem_.clear();
 };
 void Alloc();
 void Dealloc();
 void Add(const DeviceElement<T> & device_element);

 private:
 vector<DeviceElement<T>> mem_;
 ///Allocates in device memory a given variable and optionally initialises it
 int AllocVariable(T ** d_variable, int size, bool initialize);
 ///Deallocates in device memory a given variable
 int DeallocVariable(T ** d_variable);
};

template <class T>
void DeviceMemory<T>::Alloc () {
  for (int i = 0; i < mem_.size(); ++i)
    AllocVariable(mem_[i].variable, mem_[i].length, mem_[i].init);
}

template <class T>
void DeviceMemory<T>::Dealloc () {
  for (int i = 0; i < mem_.size(); ++i)
    DeallocVariable(mem_[i].variable);
  mem_.clear();
}

template <class T>
void DeviceMemory<T>::Add (const DeviceElement<T> & device_element) {
  mem_.push_back(device_element);
}

template <class T>
inline int DeviceMemory<T>::AllocVariable (T ** d_variable, int size, bool initialize) {
  CUDA_CALL(cudaMalloc(d_variable, size * sizeof(T)));
  if (initialize)
    CUDA_CALL(cudaMemset(*d_variable, 0, size * sizeof(T)));
}

template <class T>
inline int DeviceMemory<T>::DeallocVariable (T ** d_variable) {
  CUDA_CALL(cudaFree(*d_variable));
}

#endif
