/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  Miguel Ángel del Agua Teba - mdelagua@dsic.upv.es, madelagua@gmail.com
 *  Jesús Andrés Ferrer - jandres@dsic.upv.es
 *  Adrià Giménez Pastor - agimenez@dsic.upv.es
 */

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <unistd.h>
#include <ctime>
#include "Options/options.h"
#include "msg.h"
#include "dnn.h"
#include <random>
#include "../intl.h"

#define IDX2C(i,j,ld) (((j)*(ld))+(i))

#ifdef USE_DOUBLE
typedef double floatingpoint;
#else
typedef float floatingpoint;
#endif

using namespace std;
int main(int argc, char *argv[]) {
  vector<string> file_names;
  vector<string> validation_file_names;
  vector<string> tokens;
  vector<int> output_units_per_lang;
  vector<int> validation_output_units_per_lang;
  stringstream ss;
  string f_adaptation_list, f_validation_list, f_validation_labels, f_proto, line,
         f_adaptation_labels, f_output, f_net, f_load_state, f_save_state;
  int width = 0, height = 0;
  int count = 0, seed = -1;
  int epochs, batch_size, window_size = 1;
  int num_adaptation_samples = 0;
  vector<int> num_samples_per_lang;
  vector<int> validation_num_samples_per_lang;
  int num_validation_samples = 0;
  unsigned int aux;
  floatingpoint decreasing_lr_constant = 0;
  floatingpoint decreasing_constant_epoch = 10;
  bool verbose = 0;
  bool only_update_last_layer = false;
  bool load_state = false;
  kFileType adaptation_file_type;
  kFileType validation_file_type;
  bool likelihood_per_batch = false, likelihood_per_epoch = false,
      error_per_batch = false, error_per_epoch = false, validate = false;
  floatingpoint * adaptation_content = NULL;
  floatingpoint * validation_content = NULL;
  floatingpoint learning_rate, momentum;
  string ac_fun_str;
  kActivationFunction ac_fun = UNKNOWN;
  clock_t dnn_adaptation_time;
  clock_t data_loading_time;
  clock_t total_time;
  total_time = clock();
  data_loading_time = clock();
  DNN<floatingpoint> * d1;

  MESSAGE msg;

  /* i18n functions */
  setlocale ( LC_MESSAGES, "" );
  setlocale ( LC_CTYPE, "" );
  bindtextdomain ( PACKAGE, LOCALEDIR );
  textdomain ( PACKAGE );
  

  f_validation_list = "";
  f_validation_labels = "";
  f_proto = "";
  f_net = "";

  JAF::Options prog(argv[0]);

  prog.addClass(_("Initialization"), _("Options concerning shuffling of the input samples"));
  prog.add<int>("seed", _("Seed used for shuffling"), seed, true);
  prog.addClass(_("Adaptation"), _("Options concerning the files needed for adaptation"));
  prog.add<string>("tsamples", _("Path to the file with the adaptation samples list"), f_adaptation_list, true);
  prog.add<string>("tlabels", _("Path to the file with adaptation labels"), f_adaptation_labels, true);
  prog.add<string>("net", _("Path to the file with initalised network weights"), f_net);
  prog.add<string>("out", _("Location where the resulting neural network will be stored"), f_output, true);
  prog.addClass(_("Validation"), _("Files needed for validation"));
  prog.add<string>("vsamples", _("Path to the file with the validation samples list"), f_validation_list);
  prog.add<string>("vlabels", _("Path to the file with validation labels"), f_validation_labels);
  prog.addClass(_("Adaptation Algorithm"), _("Options concerning the adaptation algorithm"));
  prog.add<int>("epochs", _("Epochs"), epochs, true);
  prog.add<int>("batch", _("Batch size used for adaptation and/or validation"), batch_size, true);
  prog.add <floatingpoint> ("lr", _("Learning Rate"), learning_rate, true);
  prog.add <floatingpoint> ("mom", _("Momentum"), momentum, true);
  prog.addClass(_("Verbosity"), _("Options for tracking adaptation progress"));
  prog.addTrue("llb", _("Show likelihood per batch"), likelihood_per_batch);
  prog.addTrue("lle", _("Show likelihood per epoch"), likelihood_per_epoch);
  prog.addTrue("errb", _("Show classification error per batch on validation"), error_per_batch);
  prog.addTrue("erre", _("Show classification error per epoch on validation"), error_per_epoch);
  prog.addClass("Configuration", _("Options concerning load/write options"));
  prog.addWrite("write");
  prog.addRead("read");

  if (prog.read(argc, argv) < 0) {
    prog.syntax(cerr);
    return 1;
  }

  if (f_net.compare("") == 0) {
    msg << _("An initialised net must be provided") << MESSAGE::error;
    return 1;
  }

  if (f_load_state.compare("") != 0)
    load_state = true;

  if (f_net.compare("") == 0 and f_load_state.compare("") != 0){
    msg << _("The specified state will not be loaded because a net should also be provided") << MESSAGE:: warning;
    load_state = false;
  }

  if (f_validation_list.compare("") == 0
      and f_validation_labels.compare("") != 0) {
    msg << _("If Validation labels are specified, validation sample list must be") << MESSAGE::error;
    prog.syntax(cerr);
    return 1;
  } else if (f_validation_list.compare("") != 0
      and f_validation_labels.compare("") == 0) {
    msg << _("If Validation list is specified, validation labels must be") << MESSAGE::error;
    prog.syntax(cerr);
    return 1;
  } else if (f_validation_list.compare("") != 0
      and f_validation_labels.compare("") != 0)
    validate = true;

  if ((error_per_batch or error_per_epoch) and (validate == false)) {
    msg << _("Classification will not take place due to the absence of validation files") << MESSAGE::error;
  }

  if (!validate) {
    error_per_epoch = false;
    error_per_batch = false;
  }

  msg << _("Reading file lists...") << MESSAGE::info;

  /* Reading adaptation file list */
  if (!utils::ReadFileList(f_adaptation_list, file_names)) {
    msg << _("Unable to load adaptation file list") << MESSAGE::error;
    return 1;
  }

  /* Check all adaptation files exists */
  if (!utils::CheckFileExists(file_names))
    return 1;

  if (!utils::ReadFileType(file_names[0], adaptation_file_type, height, width)) {
    msg << _("Unable to check adaptation file type") << MESSAGE::error;
    return 1;
  }

  if (validate) {
    /* Reading validation file list */
    if (!utils::ReadFileList(f_validation_list, validation_file_names)) {
      msg << _("Unable to load validation file list") << MESSAGE::error;
      return 1;
    }

    /* Check all adaptation files exists */
    if (!utils::CheckFileExists(validation_file_names))
      return 1;

    if (!utils::ReadFileType(validation_file_names[0], validation_file_type,
        height, width)) {
      msg << _("Unable to check validation file type") << MESSAGE::error;
      return 1;
    }
  }

  msg << _("Reading labels...") << MESSAGE::info;

  vector<sample<floatingpoint>> adaptation_frames;
  vector<sample<floatingpoint>> validation_frames;

  if (!utils::ReadLabels(f_adaptation_labels, adaptation_frames, output_units_per_lang,
                         num_samples_per_lang)) {
    msg << _("Unable to load adaptation labels") << MESSAGE::error;
    return 1;
  }

  num_adaptation_samples = adaptation_frames.size();

  msg << _("Reading samples...") << MESSAGE::info;

  if (validate) {
    if (!utils::ReadLabels(f_validation_labels, validation_frames,
                           validation_output_units_per_lang, validation_num_samples_per_lang)) {
      msg << _("Unable to load validations labels") << MESSAGE::error;
      return 1;
    }
    num_validation_samples = validation_frames.size();
    validation_content = new floatingpoint[validation_file_names.size()*height*width];

    /* VALIDATION FEA READING */
    if (validation_file_type == AKREALTF or validation_file_type == AKREALBF) {
      //READ AKREALTF
      ifstream file;
      vector<floatingpoint> aux_content;
      int numSamplesAnt = 0, numSamplesAux = 0;
      int sampleSizeAux = 0;
      delete[] validation_content;
      validation_content = new floatingpoint[(ul)num_validation_samples * (ul)height];

      numSamplesAnt = 0;
      for (vector<string>::iterator it = validation_file_names.begin();
          it != validation_file_names.end(); ++it) {
        file.open(*it);
        if (!utils::LoadRealFea(file, aux_content, numSamplesAux,
            sampleSizeAux)) {
          msg << _("Could not load tlRealFea from validation samples") << MESSAGE::error;
          return 1;
        }

        for (int i = 0; i < sampleSizeAux; ++i)
          for (int j = numSamplesAnt; j < numSamplesAnt + numSamplesAux; ++j)
            validation_content[IDX2C((ul)i,(ul)j,(ul)sampleSizeAux)] =
                aux_content[IDX2R(j-numSamplesAnt,i,sampleSizeAux)];

        for (int i = numSamplesAnt; i < numSamplesAnt + numSamplesAux; ++i) {
          validation_frames[i].segmentLength = numSamplesAux;
          validation_frames[i].position = i - numSamplesAnt;
          validation_frames[i].dim = sampleSizeAux;
          validation_frames[i].content =
              &validation_content[IDX2C(0L, (ul)i, (ul)sampleSizeAux)];
          validation_frames[i].segmentPosition = numSamplesAnt;
        }

        file.close();
        aux_content.clear();
        numSamplesAnt += numSamplesAux;
      }
    } else {
      msg << _("Unknown sample file type on validation: ") << validation_file_type
          << MESSAGE::error;
      return 1;
    }
  }

  /* TRAINING FEA READING */
  adaptation_content = new floatingpoint[file_names.size() * height * width];

  if (adaptation_file_type == AKREALTF or adaptation_file_type == AKREALBF) {
    //READ AKREALTF
    ifstream file;
    vector<floatingpoint> aux_content;
    int numSamplesAnt = 0;
    int numSamplesAux = 0;
    int sampleSizeAux = 0;
    delete[] adaptation_content;

    adaptation_content = new floatingpoint[(ul)num_adaptation_samples * (ul)height];

    for (vector<string>::iterator it = file_names.begin(); it != file_names.end(); ++it) {
      file.open(*it);
      if (!utils::LoadRealFea(file, aux_content, numSamplesAux, sampleSizeAux)) {
        msg << _("Could not read tlRealFea from adaptation samples") << MESSAGE::error;
        return 1;
      }

      for (int i = 0; i < sampleSizeAux; ++i)
        for (int j = numSamplesAnt; j < numSamplesAnt + numSamplesAux; ++j)
          adaptation_content[IDX2C((ul)i,(ul)j,(ul)sampleSizeAux)] =
              aux_content[IDX2R(j-numSamplesAnt,i,sampleSizeAux)];

      for (unsigned long i = numSamplesAnt; i < numSamplesAnt + numSamplesAux; ++i) {
        adaptation_frames[i].segmentLength = numSamplesAux;
        adaptation_frames[i].position = i - numSamplesAnt;
        adaptation_frames[i].dim = sampleSizeAux;
        adaptation_frames[i].content =
            &adaptation_content[IDX2C(0L, (ul)i, (ul)sampleSizeAux )];
        adaptation_frames[i].segmentPosition = numSamplesAnt;
      }

      file.close();
      aux_content.clear();
      numSamplesAnt += numSamplesAux;
    }
    height = 1;
    width = sampleSizeAux;
  } else {
    msg << _("Incorrect sample file type for adaptation") << MESSAGE::error;
    return 1;
  }


  try {
    d1 = new DNN<floatingpoint>(f_net, seed);
    window_size = d1->units_per_layer_[0] / adaptation_frames[0].dim;
    if (d1->units_per_layer_[0] % adaptation_frames[0].dim != 0) {
      msg << _("The dimension of the input data does not fit the number of input units")
          << "\n" << d1->units_per_layer_[0] << _(" vs ") << adaptation_frames[0].dim * window_size
          << MESSAGE::error;
      return 1;
    }
    if (validate)
      if (d1->units_per_layer_[0] % validation_frames[0].dim != 0) {
        msg << _("The dimension of the validation data does not fit the number of input units")
            << MESSAGE::error;
        return 1;
      }
  } catch (DNN<floatingpoint>::DNNError ) {
    msg << _("The DNN could not be correctly loaded") << MESSAGE::error;
    return 1;
  }

  switch(d1->af_){
    case SIGMOID: ac_fun_str = "SIGMOID";
                  break;
    case TANH: ac_fun_str = "TANH";
               break;
    case RELU: ac_fun_str = "RELU";
               break;
    case SOFTRELU: ac_fun_str = "SOFTRELU";
                   break;
    default: ac_fun_str = "SIGMOID";
  }

  cerr << endl;
  cerr << _("Adaptation parameters") << endl;
  cerr << "-------------------" << endl;
  cerr << "-> " << _("Network configuration:");
  cerr << "\t" << d1->units_per_layer_[0];
  for (unsigned i = 1; i < (d1->units_per_layer_.size()-1); ++i)
    cerr << " - " << d1->units_per_layer_[i];
  cerr << " - [ " << output_units_per_lang[0]; 
  for (int i = 1; i < output_units_per_lang.size(); ++i)
    cerr << " - " << output_units_per_lang[i];
  cerr << " ]" << endl;
  cerr << "-> " << _("Epochs:") << " \t\t\t" << epochs << endl;
  cerr << "-> " << _("Batch size:") << " \t\t\t" << batch_size << endl;
  cerr << "-> " << _("Learning rate:") << " \t\t" << learning_rate << endl;
  cerr << "-> " << _("Momentum:") << " \t\t\t" << momentum << endl;
  cerr << "-> " << _("Window size:") << " \t\t" << window_size << endl;

  data_loading_time = clock() - data_loading_time;
  cerr << _("Data loading time: ")
      << (floatingpoint) data_loading_time / CLOCKS_PER_SEC << endl;
  dnn_adaptation_time = clock();
  d1->Adapt(adaptation_content, adaptation_frames, num_adaptation_samples,
            learning_rate, momentum, epochs, batch_size, window_size,
            validation_content, validation_frames, num_validation_samples,
            likelihood_per_batch, likelihood_per_epoch, error_per_batch,
            error_per_epoch, verbose);
  dnn_adaptation_time = clock() - dnn_adaptation_time;
  ofstream fout(f_output);
  d1->WriteToFile(fout);

  for(int lang = 0; lang < output_units_per_lang.size(); ++lang){
    fout << "LPriori " << lang << endl;
    for (int i = 0; i < output_units_per_lang[lang]; ++i)
      fout << d1->priori_log_probs_[lang][i] << " ";
    fout << endl;
  }

  fout << ac_fun_str << endl;

  if (f_save_state.compare("") != 0)
    d1->WriteStateToFile(f_save_state);

  file_names.clear();

  cerr << endl << "Main " << endl;
  cerr << _("Data loading: ") << (floatingpoint) data_loading_time / CLOCKS_PER_SEC
      << " s" << endl;
  cerr << _("DNN adaptation call: ")
      << (floatingpoint) dnn_adaptation_time / CLOCKS_PER_SEC << " s" << endl;
  cerr << _("Total time: ")
      << (floatingpoint) (clock() - total_time) / CLOCKS_PER_SEC << " s"
      << endl;

  adaptation_frames.clear();
  validation_frames.clear();
  delete[] adaptation_content;
  delete[] validation_content;
  delete d1;

  return 0;
}
