/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  Miguel Ángel del Agua Teba - mdelagua@dsic.upv.es, madelagua@gmail.com
 *  Jesús Andrés Ferrer - jandres@dsic.upv.es
 *  Adrià Giménez Pastor - agimenez@dsic.upv.es
 */

#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>
#include <unistd.h>
#include "utils.h"
#include "Options/options.h"
#include "msg.h"
#include "../intl.h"

using namespace std;

int MaxIndex(vector<vector<float>> &prob, vector<float> &weight, int k){
  int index = 0;
  float val = prob[0][k]+weight[0];
  for(uint i = 1; i < prob.size(); ++i) 
    if (prob[i][k]+weight[i] > val) { val = prob[i][k]+weight[i]; index = i; }
  return index;
}

int main(int argc, char *argv[]){
  vector<string> file_list;
  vector<float> weight_per_classifier;
  vector<vector<string>> file_names;
  vector<vector<float>> content;
  vector<float> output_content;
  int num_samples;
  int sample_size;
  string file_output_list;
  vector<string> file_output_names;
  ifstream file;
  ofstream out_file;
  string header = "AKREALBF";
  stringstream ss;
  MESSAGE msg;
  JAF::Options prog(argv[0]);
  bool perform_sum = false;
  bool perform_prod = false;

  string c1="",c2="",c3="",c4="";
  float w1=0,w2=0,w3=0,w4=0;

  float exp_sum;
  uint index;
  uint n_classifiers;

  prog.addClass(_("Combination type"), _("Options concerning the combination type to be performed"));
  prog.addTrue("sum", _("Applies the sum rule to the classifiers"), perform_sum);
  prog.addTrue("prod", _("Applies the product rule to the classifiers"), perform_prod);
  prog.addClass(_("Classifiers to be combined"), _("Options to set the classifiers to combine"));
  prog.add<string>("c1", _("List containing the resulting features of classifier 1"), c1, true);
  prog.add<string>("c2", _("List containing the resulting features of classifier 2"), c2);
  prog.add<string>("c3", _("List containing the resulting features of classifier 3"), c3);
  prog.add<string>("c4", _("List containing the resulting features of classifier 4"), c4);
  prog.addClass(_("Weights assigned to each classifier"), _("Options to set the weights in the combination"));
  prog.add<float>("w1", _("Weight applied to classifier c1"), w1, true);
  prog.add<float>("w2", _("Weight applied to classifier c2"), w2);
  prog.add<float>("w3", _("Weight applied to classifier c3"), w3);
  prog.add<float>("w4", _("Weight applied to classifier c4"), w4);
  prog.addClass(_("Output"), _("Options to specify the location of the resulting combination"));
  prog.add<string>("out", _("Output file list"), file_output_list, true);

  if (prog.read(argc, argv) < 0){ prog.syntax(cerr); return EXIT_FAILURE; }

  if (!perform_sum)
    if (!perform_prod){
      msg << _("-sum or -product must be specified") << MESSAGE::error;
      return EXIT_FAILURE;
    }

  file_list.push_back(c1); weight_per_classifier.push_back(w1); n_classifiers=1;
  if (c2.compare("") != 0 && w2 > 0){ file_list.push_back(c2); weight_per_classifier.push_back(w2); n_classifiers++; }
  if (c3.compare("") != 0 && w3 > 0){ file_list.push_back(c3); weight_per_classifier.push_back(w3); n_classifiers++; }
  if (c4.compare("") != 0 && w4 > 0){ file_list.push_back(c4); weight_per_classifier.push_back(w4); n_classifiers++; }

  file_names.resize(n_classifiers);
  content.resize(n_classifiers);

  msg << _("Combining") << MESSAGE::info;
  msg << "----------" << MESSAGE::info;
  for (uint i = 0; i < n_classifiers; ++i)
    msg << "-> " << _("Classifier: \"") << file_list[i] << _("\" Weight: ") << weight_per_classifier[i] << MESSAGE::info;
  msg << "-> " << _("Output file: \"") << file_output_list << "\"" << MESSAGE::info;

  //msg << _("Reading input file lists") << MESSAGE::info;
  for (uint i = 0; i < n_classifiers; ++i ){
    if (!utils::ReadFileList(file_list[i], file_names[i])){
      msg << _("File names could not be read from ") << file_list[i] << MESSAGE::error;
      return EXIT_FAILURE;
    }
    if (i > 1)
      if (file_names[i].size() != file_names[0].size()){
        msg << _("The number of features to combine does not match between classifier c") << i+1
             << _(" and classifier c1") << MESSAGE::error;
        return EXIT_FAILURE;
      }
  }

  //msg << _("Reading output file list") << MESSAGE::info;
  if (!utils::ReadFileList(file_output_list, file_output_names)) {
    msg << _("Output file names list could not be read") << MESSAGE::error;
    return EXIT_FAILURE;
  }

  if (file_output_names.size() != file_names[0].size()){
    msg << _("The number of features to combine does not match with the output file list") << MESSAGE::error;
    return EXIT_FAILURE;
  }

  if (perform_sum) {

    for(uint i = 0; i < n_classifiers; ++i){
      weight_per_classifier[i] = log(weight_per_classifier[i]);
    }

    msg << _("Generating output with the sum rule") << MESSAGE::info;
    for (uint j = 0; j < file_names[0].size(); ++j){
      for (uint i = 0; i < n_classifiers; ++i ) {
        content[i].clear();
        file.open(file_names[i][j]);
        if(!utils::LoadRealFea(file, content[i], num_samples, sample_size)){
          msg << _("Could not load real fea") << MESSAGE::error;
          return EXIT_SUCCESS;
        }
        if( i == 0){
          output_content.clear();
          output_content.resize(num_samples*sample_size);
        }
        file.close();
      }

      for (int k = 0; k < num_samples*sample_size; ++k){
        exp_sum = 0;
        index = MaxIndex(content, weight_per_classifier, k);
        for (uint i = 0; i < n_classifiers; ++i){
          exp_sum += exp((content[i][k]+weight_per_classifier[i])-(content[index][k]+weight_per_classifier[index]));
        }
        output_content[k] = log(exp_sum) + (content[index][k]+weight_per_classifier[index]);
      }

      out_file.open(file_output_names[j],ios::out | ios::binary);
      if(!out_file.is_open()){
        msg << _("Could not open file ") << file_output_names[j] << _(" for writing") << MESSAGE::error;
        return EXIT_SUCCESS;
      }
      out_file.write(header.c_str(), header.length());
      out_file.write(reinterpret_cast<char*>(&sample_size), sizeof(int));
      out_file.write(reinterpret_cast<char*>(&num_samples), sizeof(int));
      out_file.write(reinterpret_cast<char*>(&output_content[0]), sizeof(float)*num_samples*sample_size);
      out_file.close();
    } 
  } else {
    msg << _("Generating output with the product rule") << MESSAGE::info;
    for (uint j = 0; j < file_names[0].size(); ++j){
      for (uint i = 0; i < n_classifiers; ++i ) {
        content[i].clear();
        file.open(file_names[i][j]);
        if(!utils::LoadRealFea(file, content[i], num_samples, sample_size)){
          msg << _("Could not load real fea") << MESSAGE::error;
          return EXIT_FAILURE;
        }
        if( i == 0){
          output_content.clear();
          output_content.resize(num_samples*sample_size);
        }
        file.close();
      }

      for(uint i = 0; i < n_classifiers; ++i){
        for(int k = 0; k < num_samples*sample_size; ++k)
          output_content[k] += content[i][k] * weight_per_classifier[i];
      }

      out_file.open(file_output_names[j],ios::out | ios::binary);
      if(!out_file.is_open()){
        msg << _("Could not open file ") << file_output_names[j] << _(" for writing") << MESSAGE::error;
        return EXIT_FAILURE;
      }
      out_file.write(header.c_str(), header.length());
      out_file.write(reinterpret_cast<char*>(&sample_size), sizeof(int));
      out_file.write(reinterpret_cast<char*>(&num_samples), sizeof(int));
      out_file.write(reinterpret_cast<char*>(&output_content[0]), sizeof(float)*num_samples*sample_size);
      out_file.close();
    }
  }
  return EXIT_SUCCESS;
}



