/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  get_amodel.c - Implementació de 'get_amodel.h'.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2013.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "get_amodel.h"




tLAModel *
get_amodel (
            const char *fn,
	    tLDict     *dict,
            int         verbose
            )
{
  
  tLAModel *amodel;
  gzFile f;
  char *err;
  
  
  if ( fn != NULL )
    {
      f= gzopen ( fn, "r" );
      if ( f == NULL )
	tL_error ( _("Could not open file '%s'"), fn );
    }
  else f= gzdopen(fileno(stdin), "r");
  amodel= tL_amodel_new_from_file ( f, dict, &err );
  if ( amodel == NULL ) tL_error ( "AMODEL: %s.", err );
  if ( fn != NULL ) gzclose ( f );
  
  if ( verbose )
    {
      fprintf ( stderr, _("The acoustic model has been read\n") );
      if ( verbose > 2 )
        tL_amodel_print ( amodel, stderr, dict, FALSE );
    }
  
  return amodel;
  
} /* end get_amodel */
