/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  get_lexicon.c - Implementació de 'get_lexicon.h'.
 *
 *  Adrià Giménez Pastor, 2009,2011,2012,2013.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "get_lexicon.h"




tLLexicon *
get_lexicon (
	     const char *fn,
	     tLDict     *syms,
	     int         verbose
	     )
{
  
  tLLexicon *lexicon;
  gzFile f;
  char *err;
  
  
  if ( fn == NULL ) return tL_lexicon_new ( TL_LEXICON_CONV );
  
  f= gzopen ( fn, "r" );
  if ( f == NULL )
    tL_error ( _("Could not open file '%s'"), fn );
  lexicon= tL_lexicon_new_from_file ( f, syms, FALSE, &err );
  if ( lexicon == NULL ) tL_error ( "LEXICON: %s.", err );
  gzclose ( f );
  
  if ( verbose )
    {
      fprintf ( stderr, _("The lexicon has been read\n") );
      if ( verbose > 2 )
	tL_lexicon_print ( lexicon, stderr, syms );
    }
  
  return lexicon;
  
} /* end get_lexicon */
