/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  get_priors.c - Implementació de 'get_priors.h'.
 *
 *  Adrià Giménez Pastor, 2010,2011,2012,2013.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "get_priors.h"




tLPriors *
get_priors (
	    const char *fn,
	    tLLexicon  *lexicon,
	    tLBool      emptylex,
	    tLDict     *syms,
	    int         verbose
	    )
{
  
  tLPriors *priors;
  gzFile f;
  char *err;
  
  
  f= gzopen ( fn, "r" );
  if ( f == NULL )
    tL_error ( _("Could not open file '%s'"), fn );
  priors= tL_priors_new_from_file ( f, lexicon, emptylex, syms, &err );
  if ( priors == NULL ) tL_error ( "PRIORS: %s.", err );
  gzclose ( f );
  
  if ( verbose )
    {
      fprintf ( stderr, _("Priors have been read\n") );
      if ( verbose > 2 )
	tL_priors_print ( priors, stderr, lexicon->words );
      if ( emptylex )
	{
	  fprintf ( stderr, _("The lexicon has been created\n") );
	  if ( verbose > 2 )
	    tL_lexicon_print ( lexicon, stderr, syms );
	}
    }
  
  return priors;
  
} /* end get_priors */
