/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  get_words.c - Implementació de 'get_words.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011,2013.
 *
 */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "get_words.h"
#include "macros.h"




tLSeqs *
get_words (
	   const char   *name,
	   const tLDict *syms,
	   const tLBool  nowords,
           const char   *tar_filename,
	   const int     verbose
	   )
{
  
  tLFileList *fl;
  tLSeqs *words;
  size_t n, i, c;
  const tLSeq *cs;
  const char *aux, *aux2;
  char sep;
  char *err;
  tLTar *tar=NULL;
  
  
  fl= tL_filelist_new_from_file ( name );
  if ( fl == NULL ) tL_error ( _("Could not open '%s'"), name );
  if ( tar_filename != NULL) 
    {
        tar = tL_tar_new ( tar_filename, &err );
        if ( tar == NULL ) tL_error ( _("Error while reading '%s': %s"), tar_filename, err );
    }

  words=
    nowords ?
    tL_seqs_load ( fl, syms, &err, tar ):
    tL_seqs_load_words ( fl, syms, &err, tar );
  if ( words == NULL ) tL_error ( err );
  if ( verbose )
    {
      if ( nowords )
	{
	  aux= _("Sequence table");
	  aux2= _("Symbol table");
	  sep= ' ';
	  fprintf ( stderr, _("The sequences have been read,"
			      " and the tables have been created\n") );
	}
      else
	{
	  aux= _("Word table");
	  aux2= _("Character table");
	  sep= '\0';
	  fprintf ( stderr, _("The words have been read,"
			      " and the tables have been created\n") );
	}
      if ( verbose > 1 )
	{
	  
	  fprintf ( stderr, "%s:\n", aux2 );
	  for ( c= 0; c < syms->N; ++c )
	    fprintf ( stderr, "  - %lu: '%s'\n",
		      _PS(c), tL_dict_get_token ( syms, c ) );
	  
	  fprintf ( stderr, "%s:\n", aux );
	  for ( n= 0; n < tL_seqs_get_num_seqs ( words ); ++n )
	    {
	      cs= tL_seqs_get_seq ( words, n );
	      fprintf ( stderr, " - %lu: '", _PS(n) );
	      tL_seq_print ( cs, stderr, syms, sep );
	      fprintf ( stderr, "' =>" );
	      for ( i= 0; i < cs->length; ++i )
		fprintf ( stderr, " %lu", _PS(cs->v[i]) );
	      fputc ( '\n', stderr );
	    }
	  
	}
      if ( verbose > 2 )
	{
	  if ( nowords )
	    {
	      fprintf ( stderr, _("List of sequences:\n") );
	    }
	  else
	    {
	      fprintf ( stderr, _("List of words:\n") );
	    }
	  for ( n= 0; n < fl->N; ++n )
	    {
	      tL_seq_print ( tL_seqs_get_seq ( words, words->seqs[n] ),
			     stderr, syms, sep );
	      fprintf ( stderr, "'\n" );
	    }
	}
    }
  tL_filelist_free ( fl );
  if ( tar != NULL) tL_tar_free ( tar );
  
  return words;
  
} /* end get_words */
