/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  itertrain_viterbi.c - Implementació de 'itertrain_viterbi.h'.
 *
 *  Adrià Giménez Pastor, 2010,2011,2012.
 *
 */

#include <math.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "itertrain_viterbi.h"
#include "macros.h"




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static tLProb
itertrain_sample (
		  tLVTrainer *vtrainer,
		  size_t      seq,
                  const char *pbm,
                  size_t     *N,
		  tLFea      *fea,
                  int         verbose
                  )
{
  
  char *err;
  tLProb prob;
  
  
  /* Prepara. */
  if ( tL_fea_load ( fea, pbm, &err ) == -1 )
    tL_error ( _("Error while reading '%s': %s"), pbm, err );
  if ( !tL_amodel_test ( vtrainer->amodel, fea, &err ) == -1 )
    {
      tL_warning ( _("The sample '%s' will not be used for training: %s"),
		   pbm, err );
      free ( err );
      return 0.0;
    }
  if ( verbose > 1 )
    fprintf ( stderr,
              _("Processing '%s'. Rows:%d, columns:%d"),
              pbm, fea->rows, fea->cols );
  
  /* Acumula. */
  prob= tL_vtrain ( vtrainer, fea, seq );
  if ( prob == -HUGE_VAL )
    {
      tL_warning ( _("'%s': too short, will be ignored"), pbm );
      return 0.0;
    }
  ++(*N);
  
  if ( verbose > 1 )
    fprintf ( stderr, "  lprob:%g\n", _DB(prob) );
  
  return prob;
  
} /* end itertrain_sample */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

tLProb
itertrain_viterbi (
		   tLVTrainer *vtrainer,
		   tLFea      *fea,
		   tLFileList *pbm_lst,
		   int         verbose
		   )
{
  
  tLProb lv;
  int n;
  size_t N_samples;
  
  
  /* Inicialitza. */
  tL_amicounts_init ( vtrainer->counts, vtrainer->amodel );
  lv= 0.0;
  
  N_samples= 0;
  for ( n= 0; n < vtrainer->seqs->N; ++n )
    lv+= itertrain_sample ( vtrainer, vtrainer->seqs->seqs[n],
                            pbm_lst->files[n], &N_samples,
                            fea, verbose );
  
  return (N_samples > 0) ? (lv / vtrainer->seqs->N) : -HUGE_VAL;
  
} /* end itertrain_viterbi */
