/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  TLK.h - The library of the transLectures-UPV-tools.
 *
 *  Adrià Giménez Pastor, 2011,2012,2013.
 *
 */

/*! \mainpage libTLK Documentation
 *
 *  libTLK implements the core functionality of the <A
 *  HREF="http://www.translectures.eu/tlk/">transLectures-UPV toolkit (TLK)</A>. This
 *  includes:
 *
 *      - Feature extraction.
 *      - Reading and writing of acoustic models.
 *      - Initialization of acoustic models.
 *      - Parameter estimation of acoustic models: EM and Viterbi algorithms.
 *      - Adaptation of acoustic models: MLLR and CMLLR features.
 *      - Recognition of samples using acoustic models and language models.
 *
 */

#ifndef __LIBTLK_H__
#define __LIBTLK_H__
#ifdef __cplusplus
extern "C" {
#endif

#include <math.h>

#include "error_public.h"
#include "common_public.h"
#include "text_public.h"
#include "samples_public.h"
#include "filelist.h"
#include "amodel_public.h"
#include "amcounts_public.h"
#include "lexicon_public.h"
#include "lm_public.h"
#include "arpa.h"
#include "wordnet.h"
#include "priors.h"
#include "init_uniform_ci.h"
#include "init_hmm.h"
#include "train_viterbi_hmm.h"
#include "align.h"
#include "classify.h"
#include "init_amodel.h"
#include "create_amodels.h"
#include "recognise.h"
#include "itertrain.h"
#include "itertrain_viterbi.h"
#include "wordgraph_public.h"
#include "mixtcompmult.h"
#include "cmllr.h"
#include "mllr.h"
#include "extract.h"
#include "mustd.h"

#ifdef __cplusplus
}
#endif
#endif /* __LIBTLK_H__ */
