/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  abmat.h - Estructura que representa una matriu per a
 *            Forward-Backward per a qualsevol tipus de model. Està
 *            pensada per a quan es sap a priori el número màxim de
 *            caràcters que té el model. Està ací perquè s'utilitza en
 *            diferents llocs.
 *
 *  Adrià Giménez Pastor, 2011,2012
 *
 */

#ifndef __TL_PRV_ABMAT_H__
#define __TL_PRV_ABMAT_H__

#include "amodel.h"
#include "common.h"

typedef struct
{
  
  tLFloat *v;     /* Estats normals. */
  tLFloat  IF;    /* Estat inicial final. */
  
} abmat_sym_t;

typedef struct
{
  
  tLFloat      *aI;      /* |S|. */
  abmat_sym_t **a;       /* |T|*|S|. */
  abmat_sym_t **b;       /* 2*|S|. */
  int           max_width;
  tLFloat      *calc;    /* |Q|+1. */
  
} abmat_t;

void
tL_abmat_free (
	       abmat_t *m
	       );

abmat_t *
tL_abmat_new (
	      const tLAModel *amodel,
	      unsigned int    max_length
	      );

void
tL_abmat_resize (
		 abmat_t      *m,
		 unsigned int  max_length,
		 unsigned int  max_Q,
		 int           width
		 );

/* FORWARD-BACKWARD */
typedef struct
{
  
  /* Torna el número de símbols de la seqüència. */
  size_t (*get_length) (void *udata);
  
  /* Fixa el símbol actual i torna un punter al model associat. */
  const tLAModelHMM * (*a_set_sym) (const size_t  sym,
                                     void         *udata);
  
  /* Torna la probabilitat d'emissió. */
  tLFloat (*a_get_prob) (const tLFea        *fea,
                         const int           t,
                         void               *states,
                         const unsigned int  q,
                         void               *udata);
  
  /* Com a_set_sym pero en beta. Fixa també comptadors. */
  const tLAModelHMM * (*b_set_sym) (const size_t  sym,
                                     void         *udata);
  
  /* Torna a fixar els comptadors. */
  void (*b_reset_counts) (const unsigned int  c,
			  void               *udata);
  
  /* Probabilitat en un instant donat. */
  tLFloat (*b_get_prob) (const unsigned int  q,
                         const int           t,
			 void               *udata);
  
  /* Acumula pes. */
  void (*b_accum_weight) (const unsigned int  q,
                          const int           t,
                          const tLFloat       weight,
                          void               *udata);
  
  /* Acumula transició lineal p0. */
  void (*b_accum_trans_p0) (const tLFloat       weight,
			    const unsigned int  q,
			    void               *udata);
  
  /* Acumula transició lineal p1. */
  void (*b_accum_trans_p1) (const tLFloat       weight,
			    const unsigned int  q,
			    void               *udata);
  
  /* Acumula transició final mode llista. La q és l'índex en el vector
     de fp. */
  void (*b_accum_trans_fp) (const tLFloat       weight,
			    const unsigned int  q,
			    void               *udata);
  
  /* Acumula transició normal mode llista. pos és la posició. */
  void (*b_accum_trans_edge) (const tLFloat  weight,
			      ptrdiff_t      pos,
			      void          *udata);
  
} fb_meths_t;

tLFloat
tL_calc_alpha (
	       abmat_t          *m,
	       const fb_meths_t *fb,
	       const tLFea      *fea,
	       const int        *dist, /* |Q SEQ|. Número d'estast que
					  hi ha que votar per a
					  aplegar a l'estat final. */
	       void             *udata
	       );

void
tL_calc_beta (
	      abmat_t          *m,
	      const fb_meths_t *fb_meths,
	      const int         T,
	      void             *udata
	      );

#endif /* __TL_PRV_ABMAT_H__ */
