/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor, Jesús Andrés
 *  Ferrer and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  amcounts.h - Per a mantindre els comptes associats als
 *               paràmetres d'un model de caràcters.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2012,2013.
 *
 */
/*
 *  Necessita que es defineixen les següents macros:
 *    AMC_T: El tipus
 *    AMC_0: Valor inicial dels comptadors.
 *    AMC_P(NAME): Nom dels identificadors.
 *    AMC_H: Per a obtindre la definició
 *    AMC_C: Per a obtindre el codi.
 *    AMC_LL: Si es defineix en el mode C inclou els tipus
 *            log-lineals si no la resta.
 *    AMC_RA: Mode rprop_aux.
 *
 */

#include "cat.h"
#include "port.h"

#ifdef AMC_H

#include <stdio.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "amodel.h"
#include "amcounts_aux.h"
#include "mem.h"
#include "text.h"


/* DEFINICIÓ */

typedef union
{
  
  struct
  {
    AMC_T *edges;    /* Comptes de les probabilitats de cada
			arista. */
    AMC_T *fp;       /* Comptes per a les probabilitats finals. El 0
			és l'inicial, per tant |Q|+1. */
  } list;
  struct
  {
    CAT(AMC_P(count),_t) *p;    /* Comptes de transicions entre
                                   columnes. */
  } lineal;
  
} CAT(AMC_P(amcount),_trans_t);

typedef struct
{
  
  void                         *states;    /* Comptes dels estats. */
  CAT(AMC_P(amcount),_trans_t)  trans;
  
} CAT(AMC_P(amcount),_hmm_t);

#ifndef __AMCOUNTS_MLLRINFO
#define __AMCOUNTS_MLLRINFO
enum {
  NONE,
  CMLLR,
  MLLR
};
#endif

typedef struct
{
  
  /* Crea global. */
  void * (*create_global) (void *global,const int mllrinfo);
  
  /* Reserva espai per a un caràcter. */
  void * (*alloc_states) (void               *global,
			  void               *cglobal,
			  void               *states,
			  const unsigned int  Q);
  
  /* Allibera el global. */
  void   (*free_global) (void *cglobal,
			 void *global);
  
  /* Allibera els estats d'un caràcter. */
  void   (*free_states) (void               *global,
			 void               *cglobal,
			 void               *states,
			 void               *cstates,
			 const unsigned int  Q);
  
  /* Inicialitza els comptadors globals. Pot ser NULL. */
  void   (*init_global) (void        *global,
			 void        *cglobal,
			 const AMC_T  val);
  
  /* Inicialitza els estats indicats. */
  void   (*init_states) (void               *global,
			 void               *cglobal,
			 void               *states,
			 void               *cstates,
			 const unsigned int  Q,
			 const AMC_T         val);
  
  /* Imprimeix els comptadors globals. Pot ser NULL. */
  void   (*print_global) (
			  FILE         *to,
			  void         *global,
			  void         *cglobal,
			  const tLBool  binary
			  );
  
  /* Imprimeix els comptadors dels estats indicats. */
  void   (*print) (
		   FILE               *to,
		   void               *global,
		   void               *cglobal,
		   void               *states,
		   void               *cstates,
		   const unsigned int  Q,
		   const tLBool        binary
		   );

#ifndef AMC_RA
  /* Llig i acumula els comptadors globals. Torna 0 si tot ha anat bé,
     -1 en cas contrari. Pot ser NULL. */
  int   (*accum_global) (
			 gzFile         from,
			 tLBuffer      *buffer,
			 void          *global,
			 void          *cglobal,
			 const tLBool   binary,
			 unsigned long *counter
			 );
  
  /* Llig i acumula els comptadors dels estats indicats. Torna 0 si
     tot ha anat bé, -1 en cas contrari.  */
  int   (*accum) (
		   gzFile              from,
		   tLBuffer           *buffer,
		   void               *global,
		   void               *cglobal,
		   void               *states,
		   void               *cstates,
		   const unsigned int  Q,
		   const tLBool        binary,
		   unsigned long      *counter
		   );

#ifdef AMC_LOG
  /* Llig i acumula els comptadors globals interpolant. Torna 0 si tot
     ha anat bé, -1 en cas contrari. Pot ser NULL. */
  int   (*accum_global_eps) (
                             gzFile         from,
                             tLBuffer      *buffer,
                             void          *global,
                             void          *cglobal,
                             const tLFloat  eps,
                             const tLFloat  logeps,
                             const tLFloat  logeps_1,
                             const tLBool   binary,
                             unsigned long *counter
                             );
  
  /* Llig i acumula els comptadors interpolant dels estats
     indicats. Torna 0 si tot ha anat bé, -1 en cas contrari.  */
  int   (*accum_eps) (
                      gzFile              from,
                      tLBuffer           *buffer,
                      void               *global,
                      void               *cglobal,
                      void               *states,
                      void               *cstates,
                      const unsigned int  Q,
                      const tLFloat       eps,
                      const tLFloat       logeps,
                      const tLFloat       logeps_1,
                      const tLBool        binary,
                      unsigned long      *counter
                      );
#endif
#endif
  
  /* Llig els comptadors globals. Torna 0 si tot ha anat bé, -1 en cas
     contrari. Pot ser NULL. */
  int   (*read_global) (
			 gzFile         from,
			 tLBuffer      *buffer,
			 void          *global,
			 void          *cglobal,
			 const tLBool   binary,
			 unsigned long *counter
			 );
  
  /* Llig els comptadors dels estats indicats. Torna 0 si tot ha anat
     bé, -1 en cas contrari.  */
  int   (*read) (
		 gzFile              from,
		 tLBuffer           *buffer,
		 void               *global,
		 void               *cglobal,
		 void               *states,
		 void               *cstates,
		 const unsigned int  Q,
		 const tLBool        binary,
		 unsigned long      *counter
		 );

#if !(defined AMC_LL || defined AMC_RA)
  /* Updateja els paràmetres globals. */
  void (*update_global) (void *global,
			 void *cglobal);
  
  /* Updateja els paràmetres dels estats. */
  void (*update_states) (void               *global,
			 void               *cglobal,
			 void               *states,
			 void               *cstates,
			 const unsigned int  Q);
#endif

} CAT(AMC_P(amcount),_methods_t);

typedef struct
{
  
  CAT(AMC_P(amcount),_hmm_t)          *hmms;    /* Comptadors caràcters. */
  const CAT(AMC_P(amcount),_methods_t) *meths;    /* Mètodes. */
  void                                 *global;    /* Comptadors globals o
						      atributs necessaris. */
  AMC_T                                *calc;      /* Vector auxiliar
						      per a
						      calculs. */
  tLProb                               *res;       /* Vector auxiliar
						      per a
						      calculs. */
  
} CAT(tLAM,AMC_P(Counts));


/* FUNCIONS */

TL_API_FUNC(CAT(tLAM,AMC_P(Counts)) *)
CAT(tL_am,AMC_P(counts_new__)) ( 
				const tLAModel *amodel,
				const int       mllrinfo
				 );

TL_API_FUNC(CAT(tLAM,AMC_P(Counts)) *)
CAT(tL_am,AMC_P(counts_new)) ( 
			      const tLAModel *amodel
			       );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_free)) (
			       CAT(tLAM,AMC_P(Counts))  *amcounts,
			       const tLAModel           *amodel
			       );

const CAT(AMC_P(amcount),_methods_t) *
CAT(tL_get_methods_,AMC_P(amcount)) (
				     const char *name
				     );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_hmm_val)) (
				       CAT(tLAM,AMC_P(Counts)) *amcounts,
				       const tLAModel          *amodel,
				       const unsigned int       c,
				       const AMC_T              val
				       );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_hmm)) (
				   CAT(tLAM,AMC_P(Counts)) *amcounts,
				   const tLAModel          *amodel,
				   const unsigned int       c
				   );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_val)) (
				   CAT(tLAM,AMC_P(Counts)) *counts,
				   const tLAModel          *amodel,
				   const AMC_T              val
				   );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init)) (
			       CAT(tLAM,AMC_P(Counts)) *amcounts,
			       const tLAModel          *amodel
			       );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_print)) (
				const CAT(tLAM,AMC_P(Counts)) *amcounts,
				const tLAModel                *amodel,
				FILE                          *to,
				const tLBool                   binary
				);

#ifndef AMC_RA
TL_API_FUNC(int)
CAT(tL_am,AMC_P(counts_accum)) (
				const CAT(tLAM,AMC_P(Counts))  *amcounts,
				const tLAModel                 *amodel,
				gzFile                          from,
				const tLBool                    binary,
				char                          **err
				);
#endif

TL_API_FUNC(int)
CAT(tL_am,AMC_P(counts_read)) (
			       const CAT(tLAM,AMC_P(Counts))  *amcounts,
			       const tLAModel                 *amodel,
			       gzFile                          from,
			       const tLBool                    binary,
			       char                          **err
			       );

#if !(defined AMC_LL || defined AMC_RA)
TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_update_hmm)) (
				     const CAT(tLAM,AMC_P(Counts)) *counts,
				     tLAModel                      *amodel,
				     const unsigned int             c
				     );

TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_update_amodel)) (
					const CAT(tLAM,AMC_P(Counts)) *counts,
					tLAModel                      *amodel
					);
#endif

#endif /* AMC_H */


#ifdef AMC_C

#include <stddef.h>
#include <stdlib.h>

#include "amcounts_aux.h"
#include "error.h"
#include "mem.h"




/**********************/
/* TIPUS ENREGISTRATS */
/**********************/

typedef struct
{
  
  const char *name;                                      /* Nom del tipus. */
  const CAT(AMC_P(amcount),_methods_t) * const meths;    /* Mètodes
							    del
							    tipus. */
  
} type_t;


extern const CAT(AMC_P(amcount),_methods_t)
     CAT(AMC_P(tL_amcounts_methods),bernoulli);
extern const CAT(AMC_P(amcount),_methods_t)
     CAT(AMC_P(tL_amcounts_methods),dgaussian);
extern const CAT(AMC_P(amcount),_methods_t)
     CAT(AMC_P(tL_amcounts_methods),mixture);
extern const CAT(AMC_P(amcount),_methods_t)
     CAT(AMC_P(tL_amcounts_methods),tiedstates);

static const type_t types[]=
  {
    { "Bernoulli", &CAT(AMC_P(tL_amcounts_methods),bernoulli) },
    { "DGaussian", &CAT(AMC_P(tL_amcounts_methods),dgaussian) },
    { "Mixture", &CAT(AMC_P(tL_amcounts_methods),mixture) },
    { "TiedStates", &CAT(AMC_P(tL_amcounts_methods),tiedstates) },
    { NULL, NULL }
  };




/*********************/
/* FUNCIONS PRIVADES */
/*********************/
#if !(defined AMC_LL || defined AMC_RA)
static void
update_trans_list (
		   tLAModelTransListState *state,
		   tLAModelTransListEdge  *edges,
		   AMC_T                   fpcount,
		   AMC_T                  *counts,
		   AMC_T                  *calc,
		   tLProb                 *res
		   )
{
  
  ptrdiff_t begin, end, i;
  unsigned int N;
  int ret;
  
  
  begin= state->begin - edges;
  end= state->end - edges;
  
  /* Copia en un vector. */
  N= 0;
  for ( i= begin; i < end; ++i )
    calc[N++]= counts[i];
  if ( state->fp != -HUGE_VAL )
    calc[N++]= fpcount;
  
  /* Normalitza i recupera. */
  ret= CAT(tL_amcounts_,AMC_P(norm_probs)) ( res, calc, N );
  if ( ret == -1 ) return;
  N= 0;
  for ( i= begin; i < end; ++i )
    edges[i].p= res[N++];
  if ( state->fp != -HUGE_VAL )
    state->fp= res[N];
  
} /* end update_trans_list */
#endif




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(CAT(tLAM,AMC_P(Counts)) *)
CAT(tL_am,AMC_P(counts_new__)) ( 
				const tLAModel *amodel,
				const int       mllrinfo
				 )
{
  
  CAT(tLAM,AMC_P(Counts)) *new;
  unsigned int n;
  CAT(AMC_P(amcount),_hmm_t) *ccount;
  tLAModelHMM cchar;
  unsigned long maxNE;
  
  
  mt ( new= mem_alloc ( CAT(tLAM,AMC_P(Counts)), 1 ) );
  mt ( new->hmms= mem_alloc ( CAT(AMC_P(amcount),_hmm_t),
			       amodel->N ) );
  new->meths= CAT(tL_get_methods_,AMC_P(amcount)) ( amodel->type );
  new->global= new->meths->create_global ( amodel->global, mllrinfo );
  maxNE= 0;
  for ( n= 0; n < amodel->N; ++n )
    {
      
      ccount= &(new->hmms[n]);
      cchar= amodel->hmms[n];
      
      /* Estats. */
      ccount->states= new->meths->alloc_states ( amodel->global, new->global,
						 cchar.states, cchar.Q );
      
      /* Comptes de transició. */
      switch ( cchar.trans.type )
        {
        case TLLINEAL:
          mt ( ccount->trans.lineal.p= mem_alloc ( CAT(AMC_P(count),_t),
                                                   cchar.Q ) );
          break;
        case TLLIST:
	  if ( cchar.trans.list.NE > maxNE ) maxNE= cchar.trans.list.NE;
	  mt ( ccount->trans.list.fp= mem_alloc ( AMC_T, cchar.Q+1 ) );
          mt ( ccount->trans.list.edges=
	       mem_alloc ( AMC_T, cchar.trans.list.NE ) );
          break;
	case TLPOINTER: break;
        default: break;
        }
      
    }
  
  /* Reserva vector de càlcul auxiliar si hi han transicions
     llista. Reserve una de més per a la probabilitat final. */
  if ( amodel->tflags&FLIST )
    {
      mt ( new->calc= mem_alloc ( AMC_T, maxNE+1 ) );
      mt ( new->res= mem_alloc ( tLProb, maxNE+1 ) );
    }
  else { new->calc= NULL; new->res= NULL; }
  
  return new;
  
} /* end tL_amcounts_new__ */


TL_API_FUNC(CAT(tLAM,AMC_P(Counts)) *)
CAT(tL_am,AMC_P(counts_new)) ( 
			      const tLAModel *amodel
			       )
{
  return CAT(tL_am,AMC_P(counts_new__)) ( amodel, NONE );
} /* end tL_amcounts_new */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_free))(
			      CAT(tLAM,AMC_P(Counts)) *amcounts,
			      const tLAModel          *amodel
			      )
{
  
  unsigned int n;
  
  
  if ( amcounts->res != NULL ) free ( amcounts->res );
  if ( amcounts->calc != NULL ) free ( amcounts->calc );
  for ( n= 0; n < amodel->N; ++n )
    {
      amcounts->meths->free_states ( amodel->global,
				     amcounts->global,
				     amodel->hmms[n].states,
				     amcounts->hmms[n].states,
				     amodel->hmms[n].Q );
      switch ( amodel->hmms[n].trans.type )
        {
        case TLLINEAL: free ( amcounts->hmms[n].trans.lineal.p ); break;
        case TLLIST:
	  free ( amcounts->hmms[n].trans.list.fp );
	  free ( amcounts->hmms[n].trans.list.edges );
	  break;
        case TLPOINTER: break;
	default: break;
        }
    }
  free ( amcounts->hmms );
  amcounts->meths->free_global ( amcounts->global, amodel->global );
  free ( amcounts );
  
} /* end tL_amcounts_free */


const CAT(AMC_P(amcount),_methods_t) *
CAT(tL_get_methods_,AMC_P(amcount)) (
				     const char *name
				     )
{
  
  int i;
  
  
  for ( i= 0; types[i].name != NULL &&
	  strcmp ( name, types[i].name );
	++i );
  if ( types[i].name == NULL )
    tL_error ( _( "Type '%s': not implemented '" /* FIXME possible error in intl18n */
	       TOSTR(AMC_P(amcounts))
	       "'" ), name );
  
  return types[i].meths;
  
} /* end tL_get_methods_amcounts */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_hmm_val)) (
				       CAT(tLAM,AMC_P(Counts)) *amcounts,
				       const tLAModel          *amodel,
				       const unsigned int       c,
				       const AMC_T              val
				       )
{
  
  unsigned int q;
  CAT(AMC_P(amcount),_hmm_t) ccount;
  tLAModelHMM cchar;
  unsigned long ed;
  
  
  cchar= amodel->hmms[c];
  ccount= amcounts->hmms[c];
  
  /* Comptes dels estats. */
  amcounts->meths->init_states ( amodel->global,
				 amcounts->global,
				 cchar.states,
				 ccount.states,
				 cchar.Q,
				 val );
  
  /* Comptes de transició. */
  switch ( cchar.trans.type )
    {
    case TLLINEAL:
      for ( q= 0; q < cchar.Q; ++q )
        ccount.trans.lineal.p[q].p1= ccount.trans.lineal.p[q].p0= val;
      break;
    case TLLIST:
      for ( q= 0; q <= cchar.Q; ++q )
	ccount.trans.list.fp[q]= val;
      for ( ed= 0; ed < cchar.trans.list.NE; ++ed )
        ccount.trans.list.edges[ed]= val;
      break;
    case TLPOINTER: break;
    default: break;
    }
  
} /* end tL_amcounts_init_hmm_val */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_hmm)) (
				   CAT(tLAM,AMC_P(Counts)) *amcounts,
				   const tLAModel          *amodel,
				   const unsigned int       c
				   )
{
  CAT(tL_am,AMC_P(counts_init_hmm_val)) ( amcounts, amodel, c, AMC_0 );
} /* end tL_init_amcount_hmm */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init_val)) (
				   CAT(tLAM,AMC_P(Counts)) *amcounts,
				   const tLAModel          *amodel,
				   const AMC_T              val
				   )
{
  
  unsigned int n;
  
  
  if ( amcounts->meths->init_global != NULL )
    amcounts->meths->init_global ( amodel->global, amcounts->global, val );
  for ( n= 0; n < amodel->N; ++n )
    CAT(tL_am,AMC_P(counts_init_hmm_val)) ( amcounts, amodel, n, val );
  
} /* end tL_amcounts_init_val */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_init)) (
			       CAT(tLAM,AMC_P(Counts)) *amcounts,
			       const tLAModel          *amodel
			       )
{
  CAT(tL_am,AMC_P(counts_init_val)) ( amcounts, amodel, AMC_0 );
} /* end tL_amcounts_init */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_print)) (
				const CAT(tLAM,AMC_P(Counts)) *amcounts,
				const tLAModel                *amodel,
				FILE                          *to,
				const tLBool                   binary
				)
{
  
  CAT(AMC_P(amcount),_hmm_t) ccount;
  tLAModelHMM cchar;
  unsigned int n, q;
  unsigned long ed;
  
  if ( amcounts->meths->print_global != NULL )
    amcounts->meths->print_global ( to, amodel->global,
				    amcounts->global, binary );
  for ( n= 0; n < amodel->N; ++n )
    {
      cchar= amodel->hmms[n];
      ccount= amcounts->hmms[n];
      switch ( cchar.trans.type )
        {
        case TLLINEAL:
          for ( q= 0; q < cchar.Q; ++q )
            {
              CAT(tL_amcounts_,AMC_P(print)) ( to, ccount.trans.lineal.p[q].p0,
					       binary );
              if ( !binary ) putc ( ' ', to );
              CAT(tL_amcounts_,AMC_P(print)) ( to, ccount.trans.lineal.p[q].p1,
					       binary );
              if ( !binary ) putc ( '\n', to );
            }
          break;
        case TLLIST:
	  for ( q= 0; q <= cchar.Q; ++q )
            {
              CAT(tL_amcounts_,AMC_P(print)) ( to, ccount.trans.list.fp[q],
					       binary );
              if ( !binary ) putc ( '\n', to );
            }
          for ( ed= 0; ed < cchar.trans.list.NE; ++ed )
            {
              CAT(tL_amcounts_,AMC_P(print)) ( to, ccount.trans.list.edges[ed],
					       binary );
              if ( !binary ) putc ( '\n', to );
            }
          break;
        case TLPOINTER: break;
	default: break;
        }
      amcounts->meths->print ( to, amodel->global, amcounts->global,
			       cchar.states, ccount.states, cchar.Q,
			       binary );
    }
  
} /* end tL_amcounts_print */


#ifndef AMC_RA
TL_API_FUNC(int)
CAT(tL_am,AMC_P(counts_accum)) (
				const CAT(tLAM,AMC_P(Counts))  *amcounts,
				const tLAModel                 *amodel,
				gzFile                          from,
				const tLBool                    binary,
				char                          **err
				)
{
  
  CAT(AMC_P(amcount),_hmm_t) ccount;
  tLAModelHMM cchar;
  unsigned int n, q;
  tLBuffer *buffer;
  unsigned long counter, ed;
  
  
  counter= 0;
  buffer= !binary ? tL_buffer_new () : NULL;
  if ( amcounts->meths->accum_global != NULL )
    {
      if ( amcounts->meths->accum_global ( from, buffer, amodel->global,
					   amcounts->global, binary,
					   &counter ) == -1 )
	goto error;
    }
  for ( n= 0; n < amodel->N; ++n )
    {
      cchar= amodel->hmms[n];
      ccount= amcounts->hmms[n];
      switch ( cchar.trans.type )
        {
        case TLLINEAL:
	  for ( q= 0; q < cchar.Q; ++q )
	    {
	      if ( !CAT(tL_amcounts_,AMC_P(accum))
		   ( from, buffer,
		     &(ccount.trans.lineal.p[q].p0),
		     binary ) )
		goto error;
	      ++counter;
	      if ( !CAT(tL_amcounts_,AMC_P(accum))
		   ( from, buffer,
		     &(ccount.trans.lineal.p[q].p1),
		     binary ) )
		goto error;
	      ++counter;
	    }
	  
	  break;
	case TLLIST:
	  for ( q= 0; q <= cchar.Q; ++q )
	    {
	      if ( !CAT(tL_amcounts_,AMC_P(accum))
		   ( from, buffer,
		     &(ccount.trans.list.fp[q]),
		     binary ) )
		goto error;
	      ++counter;
	    }
	  for ( ed= 0; ed < cchar.trans.list.NE; ++ed )
            {
	      if ( !CAT(tL_amcounts_,AMC_P(accum))
		   ( from, buffer,
		     &(ccount.trans.list.edges[ed]),
		     binary ) )
		goto error;
	      ++counter;
            }
	  break;
	case TLPOINTER: break;
	default: break;
	}
      if ( amcounts->meths->accum ( from, buffer, amodel->global,
				    amcounts->global, cchar.states,
				    ccount.states, cchar.Q, binary,
				    &counter ) == -1 )
	goto error;
    }
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  
  return 0;
  
 error:
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  tL_msgerror ( err, _( "Unable to read all counts, only %ul were read"
		" and accumulated" ) , counter );
  return -1;
  
} /* end tL_amcounts_accum */
#endif


TL_API_FUNC(int)
CAT(tL_am,AMC_P(counts_read)) (
			       const CAT(tLAM,AMC_P(Counts))  *amcounts,
			       const tLAModel                 *amodel,
			       gzFile                          from,
			       const tLBool                    binary,
			       char                          **err
			       )
{
  
  CAT(AMC_P(amcount),_hmm_t) ccount;
  tLAModelHMM cchar;
  unsigned int n, q;
  tLBuffer *buffer;
  unsigned long counter, ed;
  
  
  counter= 0;
  buffer= !binary ? tL_buffer_new () : NULL;
  if ( amcounts->meths->read_global != NULL )
    {
      if ( amcounts->meths->read_global ( from, buffer, amodel->global,
					  amcounts->global, binary,
					  &counter ) == -1 )
	goto error;
    }
  for ( n= 0; n < amodel->N; ++n )
    {
      cchar= amodel->hmms[n];
      ccount= amcounts->hmms[n];
      switch ( cchar.trans.type )
        {
        case TLLINEAL:
	  for ( q= 0; q < cchar.Q; ++q )
	    {
	      if ( !CAT(tL_amcounts_,AMC_P(read))
		   ( from, buffer,
		     &(ccount.trans.lineal.p[q].p0),
		     binary ) )
		goto error;
	      ++counter;
	      if ( !CAT(tL_amcounts_,AMC_P(read))
		   ( from, buffer,
		     &(ccount.trans.lineal.p[q].p1),
		     binary ) )
		goto error;
	      ++counter;
	    }
	  
	  break;
	case TLLIST:
	  for ( q= 0; q <= cchar.Q; ++q )
	    {
	      if ( !CAT(tL_amcounts_,AMC_P(read))
		   ( from, buffer,
		     &(ccount.trans.list.fp[q]),
		     binary ) )
		goto error;
	      ++counter;
	    }
	  for ( ed= 0; ed < cchar.trans.list.NE; ++ed )
            {
	      if ( !CAT(tL_amcounts_,AMC_P(read))
		   ( from, buffer,
		     &(ccount.trans.list.edges[ed]),
		     binary ) )
		goto error;
	      ++counter;
            }
	  break;
	case TLPOINTER: break;
	default: break;
	}
      if ( amcounts->meths->read ( from, buffer, amodel->global,
				   amcounts->global, cchar.states,
				   ccount.states, cchar.Q, binary,
				   &counter ) == -1 )
	goto error;
    }
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  
  return 0;
  
 error:
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  tL_msgerror ( err, _( "Unable to read all counts, only %ul were read" ) , counter );
  return -1;
  
} /* end tL_amcounts_read */


#if !(defined AMC_LL || defined AMC_RA)
TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_update_hmm)) (
				     const CAT(tLAM,AMC_P(Counts)) *counts,
				     tLAModel                      *amodel,
				     const unsigned int             c
				     )
{
  
  unsigned int q;
  tLAModelHMM *pcchar, cchar;
  CAT(AMC_P(amcount),_hmm_t) ccount;
  tLAModelTransListEdge *edges;
  
  
  pcchar= &(amodel->hmms[c]); cchar= *pcchar;
  ccount= counts->hmms[c];
  counts->meths->update_states ( amodel->global, counts->global,
				 cchar.states, ccount.states, cchar.Q );
  switch ( cchar.trans.type )
    {
    case TLLINEAL:
      for ( q= 0; q < cchar.Q; ++q )
	CAT(tL_amcounts_,AMC_P(norm_pair)) ( ccount.trans.lineal.p[q],
					     &(cchar.trans.lineal.p[q]) );
      pcchar->trans.lineal.ll= FALSE;
      break;
    case TLLIST:
      edges= cchar.trans.list.edges;
      update_trans_list ( &(amodel->hmms[c].trans.list.init), edges,
			  ccount.trans.list.fp[0], ccount.trans.list.edges,
			  counts->calc, counts->res );
      for ( q= 0; q < cchar.Q; ++q )
	update_trans_list ( &(cchar.trans.list.states[q]), edges,
			    ccount.trans.list.fp[q+1], ccount.trans.list.edges,
			    counts->calc, counts->res );
      break;
    case TLPOINTER: break;
    default: break;
    }
  
} /* end tL_amocounts_update_hmm */


TL_API_FUNC(void)
CAT(tL_am,AMC_P(counts_update_amodel)) (
					const CAT(tLAM,AMC_P(Counts)) *counts,
					tLAModel                      *amodel
					)
{
  
  unsigned int c;
  
  
  counts->meths->update_global ( amodel->global, counts->global );
  for ( c= 0; c < amodel->N; ++c )
    CAT(tL_am,AMC_P(counts_update_hmm)) ( counts, amodel, c );
  
} /* end tL_amounts_update_amodel */
#endif

#endif /* AMC_C */
