/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  bernoulli.h - Bernoulli.
 *
 *  Adrià Giménez Pastor, 2008,2011,2012,2013.
 *
 */
/*
 *  Necessita que es defineixen les següents macros:
 *    AMC_T: El tipus
 *    AMC_0: Valor inicial dels comptadors.
 *    AMC_P(NAME): Nom dels identificadors.
 *    AMC_H: Per a obtenir la definició.
 *    AMC_C: Per a obtenir el codi.
 *
 */

#include "cat.h"



#ifdef AMC_H

typedef struct
{
  CAT(AMC_P(count),_t) *p;
} CAT(AMC_P(amcount),_bernoulli_state_t);

#endif /* AMC_H */




#ifdef AMC_C

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>

#include "amcounts_aux.h"
#include "amodel/bernoulli.h"
#include "error.h"
#include "global.h"
#include "mem.h"
#include "text.h"



/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) BERNOULLI_GLOBAL(ptr)
#define CSTATE(ptr) ((CAT(AMC_P(amcount),_bernoulli_state_t) *) (ptr))
#define STATE(ptr) BERNOULLI_STATE(ptr)




/***********/
/* MÈTODES */
/***********/

static void *
alloc_states (
	      void               *global,
	      void               *cglobal,
	      void               *states,
	      const unsigned int  Q
	      )
{
  
  CAT(AMC_P(amcount),_bernoulli_state_t) *new;
  CAT(AMC_P(count),_t) *aux;
  unsigned int q, D;
  
  
  D= GLOBAL(global)->D;
  mt ( new= mem_alloc ( CAT(AMC_P(amcount),_bernoulli_state_t),
			Q ) );
  mt ( new[0].p= mem_alloc ( CAT(AMC_P(count),_t),
			     Q * D ) );
  for ( q= 1, aux= new[0].p + D;
	q < Q; ++q, aux+= D )
    new[q].p= aux;
  
  return (void *) new;
  
} /* end alloc_states */


static void
free_states (
	     void               *global,
	     void               *cglobal,
	     void               *states,
	     void               *cstates,
	     const unsigned int  Q
	     )
{
  
  free ( CSTATE(cstates)[0].p );
  free ( cstates );
  
} /* end free_states */
  

static void
init_states (
	     void               *global,
	     void               *cglobal,
	     void               *states,
	     void               *cstates,
	     const unsigned int  Q,
	     const AMC_T         val
	     )
{
  
  unsigned int q, d, i, D;
  CAT(AMC_P(count),_t) *p;
  
  
  D= GLOBAL(global)->D;
  p= CSTATE(cstates)[0].p;
  for ( q= 0, i= 0; q < Q; ++q )
    for ( d= 0; d < D; ++d, ++i )
      p[i].p1= p[i].p0= val;
  
} /* end init_states */


static void
print (
       FILE               *to,
       void               *global,
       void               *cglobal,
       void               *states,
       void               *cstates,
       const unsigned int  Q,
       const tLBool        binary
       )
{
  
  unsigned int d, q, D, i;
  const CAT(AMC_P(count),_t) *p;
  
  
  D= GLOBAL(global)->D;
  p= CSTATE(cstates)[0].p;
  for ( q= 0, i= 0; q < Q; ++q )
    for ( d= 0; d < D; ++d, ++i )
      {
	CAT(tL_amcounts_,AMC_P(print)) ( to, p[i].p0, binary );
	if ( !binary ) putc ( ' ', to );
	CAT(tL_amcounts_,AMC_P(print)) ( to, p[i].p1, binary );
	if ( !binary ) putc ( '\n', to );
      }
  
} /* end print */


static int
accum (
       gzFile              from,
       tLBuffer           *buffer,
       void               *global,
       void               *cglobal,
       void               *states,
       void               *cstates,
       const unsigned int  Q,
       const tLBool        binary,
       unsigned long      *counter
       )
{
  
  unsigned int d, q, D, i;
  CAT(AMC_P(count),_t) *p;
  
  
  D= GLOBAL(global)->D;
  p= CSTATE(cstates)[0].p;
  for ( q= 0, i= 0; q < Q; ++q )
    for ( d= 0; d < D; ++d, ++i )
      {
	if ( !CAT(tL_amcounts_,AMC_P(accum)) ( from, buffer,
					       &(p[i].p0), binary ) )
	  return -1;
	++(*counter);
	if ( !CAT(tL_amcounts_,AMC_P(accum)) ( from, buffer,
					       &(p[i].p1), binary ) )
	  return -1;
	++(*counter);
      }
  
  return 0;
  
} /* end accum */


#ifdef AMC_LOG
static int
accum_eps (
           gzFile              from,
           tLBuffer           *buffer,
           void               *global,
           void               *cglobal,
           void               *states,
           void               *cstates,
           const unsigned int  Q,
           const tLFloat       eps,
           const tLFloat       logeps,
           const tLFloat       logeps_1,
           const tLBool        binary,
           unsigned long      *counter
           )
{
  
  unsigned int d, q, D, i;
  logcount_t *p;
  
  
  D= GLOBAL(global)->D;
  p= CSTATE(cstates)[0].p;
  for ( q= 0, i= 0; q < Q; ++q )
    for ( d= 0; d < D; ++d, ++i )
      {
        if ( !tL_amcounts_logaccum_eps ( from, buffer, &(p[i].p0),
                                        logeps, logeps_1, binary ) )
          return -1;
        ++(*counter);
        if ( !tL_amcounts_logaccum_eps ( from, buffer, &(p[i].p1),
                                         logeps, logeps_1, binary ) )
          return -1;
        ++(*counter);
      }
  
  return 0;
  
} /* end accum_eps */
#endif


static int
_read (
      gzFile              from,
      tLBuffer           *buffer,
      void               *global,
      void               *cglobal,
      void               *states,
      void               *cstates,
      const unsigned int  Q,
      const tLBool        binary,
      unsigned long      *counter
      )
{
  
  unsigned int d, q, D, i;
  CAT(AMC_P(count),_t) *p;
  
  
  D= GLOBAL(global)->D;
  p= CSTATE(cstates)[0].p;
  for ( q= 0, i= 0; q < Q; ++q )
    for ( d= 0; d < D; ++d, ++i )
      {
	if ( !CAT(tL_amcounts_,AMC_P(read)) ( from, buffer,
					      &(p[i].p0), binary ) )
	  return -1;
	++(*counter);
	if ( !CAT(tL_amcounts_,AMC_P(read)) ( from, buffer,
					      &(p[i].p1), binary ) )
	  return -1;
	++(*counter);
      }
  
  return 0;
  
} /* end read */


static void
update_states (
	       void               *global,
	       void               *cglobal,
	       void               *states,
	       void               *cstates,
	       const unsigned int  Q
	       )
{
  
  unsigned int q, d, D;
  bernoulli_state_t state;
  CAT(AMC_P(amcount),_bernoulli_state_t) scount;
  
  
  D= GLOBAL(global)->D;
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      scount= CSTATE(cstates)[q];
      for ( d= 0; d < D; ++d )
	CAT(tL_amcounts_,AMC_P(norm_pair)) ( scount.p[d], &(state.p[d]) );
      tL_Bernoulli_update_hiddendata ( D, &(STATE(states)[q]) );
    }
  
} /* end update_states */


CAT(AMC_P(amcount),_methods_t) CAT(AMC_P(tL_amcounts_methods),bernoulli)=
{
  tL_amcounts_create_global_empty,
  alloc_states,
  tL_amcounts_free_global_empty,
  free_states,
  NULL,
  init_states,
  NULL,
  print,
  NULL,
  accum,
#ifdef AMC_LOG
  NULL,
  accum_eps,
#endif
  NULL,
  _read,
  tL_update_global_empty,
  update_states
};


#endif /* AMC_C */
