/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  mixture.h - Mixtura.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2012,2013.
 *
 */
/*
 *  Necessita que es defineixen les següents macros:
 *    AMC_T: El tipus
 *    AMC_0: Valor inicial dels comptadors.
 *    AMC_P(NAME): Nom dels identificadors.
 *    AMC_H: Per a obtenir la definició.
 *    AMC_C: Per a obtenir el codi.
 *    AMC_LL: Per al mode LL.
 *
 */

#include "cat.h"


#ifdef AMC_H

typedef struct
{
  
  const CAT(AMC_P(amcount),_methods_t) *meths;
  void                                 *global;
  
} CAT(AMC_P(amcount),_mixture_global_t);

typedef struct
{
  
  AMC_T *p;
  void  *members;
  
} CAT(AMC_P(amcount),_mixture_state_t);

#endif /* AMC_H */




#ifdef AMC_C

#include <stddef.h>
#include <stdlib.h>

#include "amcounts_aux.h"
#include "amodel/mixture.h"
#include "error.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define CGLOBAL(ptr) ((CAT(AMC_P(amcount),_mixture_global_t) *) (ptr))
#define GLOBAL(ptr) MIXTURE_GLOBAL(ptr)
#define RAGLOBAL(ptr) ((raamcount_mixture_global_t *) (ptr))
#define STATE(ptr) MIXTURE_STATE(ptr)
#define CSTATE(ptr) ((CAT(AMC_P(amcount),_mixture_state_t) *) (ptr))
#define RSTATE(ptr) ((raamcount_mixture_state_t *) (ptr))




/***********/
/* MÈTODES */
/***********/

static void *
create_global (
	       void      *global,
	       const int  mllrinfo
	       )
{
  
  CAT(AMC_P(amcount),_mixture_global_t) *new;
  
  
  mt ( new= mem_alloc ( CAT(AMC_P(amcount),_mixture_global_t), 1 ) );
  new->meths= CAT(tL_get_methods_,AMC_P(amcount)) ( GLOBAL(global)->type );
  new->global= new->meths->create_global ( GLOBAL(global)->global, mllrinfo );
  
  return new;
  
} /* end create_global */


static void *
alloc_states (
	      void               *global,
	      void               *cglobal,
	      void               *states,
	      const unsigned int  Q
	      )
{
  
  CAT(AMC_P(amcount),_mixture_state_t) *new;
  size_t total_I;
  unsigned int q;
  mixture_state_t state;
  AMC_T *aux;
  
  
  mt ( new= mem_alloc ( CAT(AMC_P(amcount),_mixture_state_t), Q ) );
  total_I= 0;
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      total_I+= state.I;
      new[q].members=
	CGLOBAL(cglobal)->meths->alloc_states ( GLOBAL(global)->global,
						CGLOBAL(cglobal)->global,
						state.members, state.I );
    }
  mt ( new[0].p= mem_alloc ( AMC_T, total_I ) );
  for ( q= 1, aux= new[0].p + STATE(states)[0].I;
	q < Q; aux+= STATE(states)[q++].I )
    new[q].p= aux;
  
  return (void *) new;
  
} /* end alloc_states */


static void
free_global (
	     void *cglobal,
	     void *global
	     )
{
  
  CGLOBAL(cglobal)->meths->free_global ( CGLOBAL(cglobal)->global,
					 GLOBAL(global)->global );
  free ( cglobal );
  
} /* end free_global */


static void
free_states (
	     void               *global,
	     void               *cglobal,
	     void               *states,
	     void               *cstates,
	     const unsigned int  Q
	     )
{
  
  unsigned int q;
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      CGLOBAL(cglobal)->meths->free_states ( GLOBAL(global)->global,
					     CGLOBAL(cglobal)->global,
					     state.members,
					     cstate.members,
					     state.I );
    }
  free ( CSTATE(cstates)[0].p );
  free ( cstates );
  
} /* end free_states */


static void
init_global (
	     void        *global,
	     void        *cglobal,
	     const AMC_T  val
	     )
{
  
  if ( CGLOBAL(cglobal)->meths->init_global != NULL )
    CGLOBAL(cglobal)->meths->init_global ( GLOBAL(global)->global,
					   CGLOBAL(cglobal)->global,
					   val );
  
} /* end init_global */


static void
init_states (
	     void               *global,
	     void               *cglobal,
	     void               *states,
	     void               *cstates,
	     const unsigned int  Q,
	     const AMC_T         val
	     )
{
  
  unsigned int q, i;
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      CGLOBAL(cglobal)->meths->init_states ( GLOBAL(global)->global,
					     CGLOBAL(cglobal)->global,
					     state.members,
					     cstate.members,
					     state.I, val );
      for ( i= 0; i < state.I; ++i )
	cstate.p[i]= val;
    }
  
} /* end init_states */


static void
print_global (
	      FILE         *to,
	      void         *global,
	      void         *cglobal,
	      const tLBool  binary
	      )
{
  if ( CGLOBAL(cglobal)->meths->print_global != NULL )
    CGLOBAL(cglobal)->meths->print_global ( to,
					    GLOBAL(global)->global,
					    CGLOBAL(cglobal)->global,
					    binary );
} /* end print_global */


static void
print (
       FILE               *to,
       void               *global,
       void               *cglobal,
       void               *states,
       void               *cstates,
       const unsigned int  Q,
       const tLBool        binary
       )
{
  
  unsigned int q, i;
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      CAT(tL_amcounts_,AMC_P(print)) ( to, cstate.p[0], binary );
      for ( i= 1; i < state.I; ++i )
	{
	  if ( !binary ) putc ( ' ', to );
	  CAT(tL_amcounts_,AMC_P(print)) ( to, cstate.p[i], binary );
	}
      if ( !binary ) putc ( '\n', to );
      CGLOBAL(cglobal)->meths->print ( to,
				       GLOBAL(global)->global,
				       CGLOBAL(cglobal)->global,
				       state.members,
				       cstate.members,
				       state.I, binary );
    }
  
} /* end print */


#ifndef AMC_RA
static int
accum_global (
	      gzFile         from,
	      tLBuffer      *buffer,
	      void          *global,
	      void          *cglobal,
	      const tLBool   binary,
	      unsigned long *counter
	      )
{
  
  if ( CGLOBAL(cglobal)->meths->accum_global != NULL )
    {
      if ( CGLOBAL(cglobal)->meths->accum_global ( from, buffer,
						   GLOBAL(global)->global,
						   CGLOBAL(cglobal)->global,
						   binary, counter ) == -1 )
	return -1;
    }
  
  return 0;
  
} /* end accum_global */


#ifdef AMC_LOG
static int
accum_global_eps (
                  gzFile         from,
                  tLBuffer      *buffer,
                  void          *global,
                  void          *cglobal,
                  const tLFloat  eps,
                  const tLFloat  logeps,
                  const tLFloat  logeps_1,
                  const tLBool   binary,
                  unsigned long *counter
                  )
{
  
  if ( CGLOBAL(cglobal)->meths->accum_global_eps != NULL )
    {
      if ( CGLOBAL(cglobal)->meths->accum_global_eps ( from, buffer,
                                                       GLOBAL(global)->global,
                                                       CGLOBAL(cglobal)->global,
                                                       eps, logeps, logeps_1,
                                                       binary, counter ) == -1 )
        return -1;
    }
  
  return 0;
  
} /* end accum_global_eps */
#endif


static int
accum (
       gzFile              from,
       tLBuffer           *buffer,
       void               *global,
       void               *cglobal,
       void               *states,
       void               *cstates,
       const unsigned int  Q,
       const tLBool        binary,
       unsigned long      *counter
       )
{
  
  unsigned int q, i;
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      for ( i= 0; i < state.I; ++i )
	{
	  if ( !CAT(tL_amcounts_,AMC_P(accum)) ( from, buffer,
						 &(cstate.p[i]),
						 binary ) )
	    return -1;
	  ++(*counter);
	}
      if ( CGLOBAL(cglobal)->meths->accum ( from, buffer,
					    GLOBAL(global)->global,
					    CGLOBAL(cglobal)->global,
					    state.members, cstate.members,
					    state.I, binary, counter ) == -1 )
	return -1;
    }
  
  return 0;
  
} /* end accum */


#ifdef AMC_LOG
static int
accum_eps (
           gzFile              from,
           tLBuffer           *buffer,
           void               *global,
           void               *cglobal,
           void               *states,
           void               *cstates,
           const unsigned int  Q,
           const tLFloat       eps,
           const tLFloat       logeps,
           const tLFloat       logeps_1,
           const tLBool        binary,
           unsigned long      *counter
           )
{
  
  unsigned int q, i;
  mixture_state_t state;
  logamcount_mixture_state_t cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      for ( i= 0; i < state.I; ++i )
        {
          if ( !tL_amcounts_logaccum_eps ( from, buffer, &(cstate.p[i]),
                                           logeps, logeps_1, binary ) )
            return -1;
          ++(*counter);
        }
      if ( CGLOBAL(cglobal)->meths->accum_eps ( from, buffer,
                                                GLOBAL(global)->global,
                                                CGLOBAL(cglobal)->global,
                                                state.members, cstate.members,
                                                state.I, eps, logeps, logeps_1,
                                                binary, counter ) == -1 )
        return -1;
    }
  
  return 0;
  
} /* end accum_eps */
#endif
#endif


static int
read_global (
	     gzFile         from,
	     tLBuffer      *buffer,
	     void          *global,
	     void          *cglobal,
	     const tLBool   binary,
	     unsigned long *counter
	     )
{
  
  if ( CGLOBAL(cglobal)->meths->read_global != NULL )
    {
      if ( CGLOBAL(cglobal)->meths->read_global ( from, buffer,
						  GLOBAL(global)->global,
						  CGLOBAL(cglobal)->global,
						  binary, counter ) == -1 )
	return -1;
    }
  
  return 0;
  
} /* end read_global */


static int
_read (
      gzFile              from,
      tLBuffer           *buffer,
      void               *global,
      void               *cglobal,
      void               *states,
      void               *cstates,
      const unsigned int  Q,
      const tLBool        binary,
      unsigned long      *counter
      )
{
  
  unsigned int q, i;
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      for ( i= 0; i < state.I; ++i )
	{
	  if ( !CAT(tL_amcounts_,AMC_P(read)) ( from, buffer,
						&(cstate.p[i]),
						binary ) )
	    return -1;
	  ++(*counter);
	}
      if ( CGLOBAL(cglobal)->meths->read ( from, buffer,
					   GLOBAL(global)->global,
					   CGLOBAL(cglobal)->global,
					   state.members, cstate.members,
					   state.I, binary, counter ) == -1 )
	return -1;
    }
  
  return 0;
  
} /* end read */

#if !(defined AMC_LL || defined AMC_RA)
static void
update_global (
	       void *global,
	       void *cglobal
	       )
{
  
  CGLOBAL(cglobal)->meths->update_global ( GLOBAL(global)->global,
					   CGLOBAL(cglobal)->global );
  
} /* end update_global */


static void
update_states (
	       void               *global,
	       void               *cglobal,
	       void               *states,
	       void               *cstates,
	       const unsigned int  Q
	       )
{
  
  mixture_state_t state;
  CAT(AMC_P(amcount),_mixture_state_t) scount;
  unsigned int q;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      scount= CSTATE(cstates)[q];
      CAT(tL_amcounts_,AMC_P(norm_probs)) ( state.p, scount.p, state.I );
      CGLOBAL(cglobal)->meths->update_states ( GLOBAL(global)->global,
					       CGLOBAL(cglobal)->global,
					       state.members,
					       scount.members,
					       state.I );
    }
  
} /* end update_states */
#endif /* AMC_LL */
#ifdef AMC_LL
static void
update_global_ggis (
		    void          *global,
		    void          *cglobal,
		    const tLFloat  fmax
		    )
{
  
  if ( CGLOBAL(cglobal)->meths->update_global_ggis != NULL )
    CGLOBAL(cglobal)->meths->update_global_ggis ( GLOBAL(global)->global,
						  CGLOBAL(cglobal)->global,
						  fmax );
  
} /* end update_global_ggis */


static void
update_states_ggis (
		    void               *global,
		    void               *cglobal,
		    void               *states,
		    void               *cstates,
		    const unsigned int  Q,
		    const tLFloat       fmax
		    )
{
  
  mixture_state_t state;
  llamcount_mixture_state_t scount;
  unsigned int q, i;
  tLFloat aux, f;
  
  
  f= 1.0 / fmax;
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      scount= CSTATE(cstates)[q];
      for ( i= 0; i < state.I; ++i )
	UPDATE_GGIS ( state.p[i], scount.p[i], aux, f );
      CGLOBAL(cglobal)->meths->update_states_ggis ( GLOBAL(global)->global,
						    CGLOBAL(cglobal)->global,
						    state.members,
						    scount.members,
						    state.I, fmax );
    }
  
} /* end update_states_ggis */


static void
update_global_ggis_init_rprop (
			       void          *global,
			       void          *cglobal,
			       void          *raglobal,
			       const tLFloat  fmax,
			       const tLFloat  dmin,
			       const tLFloat  dmax
			       )
{
  
  if ( CGLOBAL(cglobal)->meths->update_global_ggis_init_rprop != NULL )
    CGLOBAL(cglobal)->meths->update_global_ggis_init_rprop
      ( GLOBAL(global)->global,
	CGLOBAL(cglobal)->global,
	RAGLOBAL(raglobal)->global,
	fmax, dmin, dmax );
  
} /* end update_global_ggis_init_rprop */


static void
update_states_ggis_init_rprop (
			       void               *global,
			       void               *cglobal,
			       void               *raglobal,
			       void               *states,
			       void               *cstates,
			       void               *rastates,
			       const unsigned int  Q,
			       const tLFloat       fmax,
			       const tLFloat       dmin,
			       const tLFloat       dmax
			       )
{
  
  mixture_state_t state;
  llamcount_mixture_state_t scount;
  raamcount_mixture_state_t rachar;
  unsigned int q, i;
  tLFloat f;
  
  
  f= 1.0 / fmax;
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      scount= CSTATE(cstates)[q];
      rachar= RSTATE(rastates)[q];
      for ( i= 0; i < state.I; ++i )
	tL_update_ggis_init_rprop ( &(state.p[i]), scount.p[i], &(rachar.p[i]),
				    f, dmin, dmax );
      CGLOBAL(cglobal)->meths->update_states_ggis_init_rprop
	( GLOBAL(global)->global,
	  CGLOBAL(cglobal)->global,
	  RAGLOBAL(raglobal)->global,
	  state.members, scount.members, rachar.members,
	  state.I, fmax, dmin, dmax );
    }
  
} /* end update_states_ggis_init_rprop */


static void
update_global_rprop (
		     void              *global,
		     void              *cglobal,
		     void              *raglobal,
		     const tLRpropArgs *args
		     )
{
  
  if ( CGLOBAL(cglobal)->meths->update_global_rprop != NULL )
    CGLOBAL(cglobal)->meths->update_global_rprop ( GLOBAL(global)->global,
						   CGLOBAL(cglobal)->global,
						   RAGLOBAL(raglobal)->global,
						   args );
  
} /* end update_global_rprop */


static void
update_states_rprop (
		     void               *global,
		     void               *cglobal,
		     void               *raglobal,
		     void               *states,
		     void               *cstates,
		     void               *rastates,
		     const unsigned int  Q,
		     const tLRpropArgs  *args
		     )
{
  
  mixture_state_t state;
  llamcount_mixture_state_t scount;
  raamcount_mixture_state_t rachar;
  unsigned int q, i;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      scount= CSTATE(cstates)[q];
      rachar= RSTATE(rastates)[q];
      for ( i= 0; i < state.I; ++i )
	tL_update_rprop ( &(state.p[i]), scount.p[i], &(rachar.p[i]), args );
      CGLOBAL(cglobal)->meths->update_states_rprop ( GLOBAL(global)->global,
						     CGLOBAL(cglobal)->global,
						     RAGLOBAL(raglobal)->global,
						     state.members,
						     scount.members,
						     rachar.members,
						     state.I, args );
    }
  
} /* end update_states_rprop */
#endif /* AMC_LL */

#ifdef AMC_RA
static void
set_l0_global (
	       void *global,
	       void *cglobal
	       )
{
  if ( CGLOBAL(cglobal)->meths->set_l0_global != NULL )
    CGLOBAL(cglobal)->meths->set_l0_global ( GLOBAL(global)->global,
					     CGLOBAL(cglobal)->global );
} /* end set_l0_global */


static void
set_l0_states (
	       void               *global,
	       void               *cglobal,
	       void               *states,
	       void               *cstates,
	       const unsigned int  Q
	       )
{
  
  unsigned int q, i;
  mixture_state_t state;
  raamcount_mixture_state_t cstate;
  
  
  for ( q= 0; q < Q; ++q )
    {
      state= STATE(states)[q];
      cstate= CSTATE(cstates)[q];
      CGLOBAL(cglobal)->meths->set_l0_states ( GLOBAL(global)->global,
					       CGLOBAL(cglobal)->global,
					       state.members, cstate.members,
					       state.I );
      for ( i= 0; i < state.I; ++i )
	cstate.p[i].l0= state.p[i];
    }
  
} /* end set_l0_states */
#endif


CAT(AMC_P(amcount),_methods_t) CAT(AMC_P(tL_amcounts_methods),mixture)=
{
  create_global,
  alloc_states,
  free_global,
  free_states,
  init_global,
  init_states,
  print_global,
  print,
#ifndef AMC_RA
  accum_global,
  accum,
#ifdef AMC_LOG
  accum_global_eps,
  accum_eps,
#endif
#endif
  read_global,
  _read
#if !(defined AMC_LL || defined AMC_RA)
  ,
  update_global,
  update_states
#endif
#ifdef AMC_LL
  ,
  update_global_ggis,
  update_states_ggis,
  update_global_ggis_init_rprop,
  update_states_ggis_init_rprop,
  update_global_rprop,
  update_states_rprop
#endif
#ifdef AMC_RA
  ,
  set_l0_global,
  set_l0_states
#endif
};


#endif /* AMC_C */
