/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  amcounts_aux.h - Funcions concretes per a cada tipus de instància.
 *
 *  Adrià Giménez Pastor, 2010,2011,2012,2013.
 *
 */

#ifndef __TL_PRV_AMCOUNTS_AUX_H__
#define __TL_PRV_AMCOUNTS_AUX_H__

#include <stdio.h>
#include <zlib.h>

#include "cat.h"
#include "common.h"
#include "text.h"

#ifdef __TL_PRV
typedef struct
{
  
  int        type;
  tLFloat ***counts;
  
} tLMixCounts;
#endif

typedef struct
{
  
  tLFloat inc;     /* Increment. */
  tLFloat grad;    /* Gradient. */
  tLFloat l0;      /* Valor inicial. */
  
} rprop_aux_node_t;

extern const rprop_aux_node_t ra_zero;

typedef struct
{
  tLFloat a;
  tLFloat b;
} llcount_type;

extern const llcount_type llcount_type_zero;

#define AMCOUNT(TYPE,PREF)			\
  typedef struct				\
  {						\
  						\
    TYPE p1;    /* Acumulat si és. */		\
    TYPE p0;    /* Acumulat si no és. */	\
						\
  } CAT(PREF,count_t);
AMCOUNT(unsigned long,i)
AMCOUNT(tLFloat,log)
AMCOUNT(llcount_type,ll)
AMCOUNT(rprop_aux_node_t,ra)

void
tL_amcounts_inorm_pair (
			const icount_t  count,
			tLProbPair     *var
			);

void
tL_amcounts_lognorm_pair (
			  const logcount_t  count,
			  tLProbPair       *var
			  );

/* -1 si no es pot normalitzar */
int
tL_amcounts_inorm_probs (
			 tLProb              *probs,
			 const unsigned long *vals,
			 const unsigned int   N
			 );

/* -1 si no es pot normalitzar */
int
tL_amcounts_lognorm_probs (
			   tLProb             *probs,
			   const tLFloat      *vals,
			   const unsigned int  N
			   );

void
tL_amcounts_iprint (
		    FILE                *f,
		    const unsigned long  value,
		    const tLBool         binary
		    );

void
tL_amcounts_logprint (
		      FILE          *f,
		      const tLFloat  value,
		      const tLBool   binary
		      );

void
tL_amcounts_llprint (
		     FILE               *f,
		     const llcount_type  value,
		     const tLBool        binary
		     );

void
tL_amcounts_raprint (
		     FILE                   *f,
		     const rprop_aux_node_t  value,
		     const tLBool            binary
		     );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_iaccum (
		    gzFile         f,
		    tLBuffer      *buffer,
		    unsigned long *var,
		    const tLBool   binary
		    );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_logaccum (
		      gzFile        f,
		      tLBuffer     *buffer,
		      tLFloat      *var,
		      const tLBool  binary
		      );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_logaccum_eps (
                          gzFile         f,
                          tLBuffer      *buffer,
                          tLFloat       *var,
                          const tLFloat  logeps,
                          const tLFloat  logeps_1, /*log(1-eps)*/
                          const tLBool   binary
                          );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_llaccum (
		     gzFile        f,
		     tLBuffer     *buffer,
		     llcount_type *var,
		     const tLBool  binary
		     );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_iread (
		   gzFile         f,
		   tLBuffer      *buffer,
		   unsigned long *var,
		   const tLBool   binary
		   );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_logread (
		     gzFile        f,
		     tLBuffer     *buffer,
		     tLFloat      *var,
		     const tLBool  binary
		     );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_llread (
		    gzFile        f,
		    tLBuffer     *buffer,
		    llcount_type *var,
		    const tLBool  binary
		    );

/* Torna 0 si no s'ha pogut llegir res. */
int
tL_amcounts_raread (
		    gzFile            f,
		    tLBuffer         *buffer,
		    rprop_aux_node_t *var,
		    const tLBool      binary
		    );

#endif /* __TL_PRV_AMCOUNTS_AUX_H__ */
