/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  bernoulli.h - Bernoulli.
 *
 *  Adrià Giménez Pastor, 2008,2009,2010,2011,2012.
 *
 */

#ifndef __TL_PRV_AMODEL__BERNOULLI_H__
#define __TL_PRV_AMODEL__BERNOULLI_H__

#include <stdio.h>

#include "../amodel.h"
#include "../common.h"
#include "../samples.h"
#include "../text.h"


/* MACROS */

#define BERNOULLI_STATE(ptr) ((bernoulli_state_t *) (ptr))
#define BERNOULLI_GLOBAL(ptr) ((bernoulli_global_t *) (ptr))

#define BERNOULLI_STATE_CONTENT					\
  tLProbPair *p;        /* |D|. Probabilitat d'emissió. */
  

#define BERNOULLI_GLOBAL_CONTENT			\
   unsigned int D;    /* Dimensió dels vectors. */


/* TIPUS */

typedef struct
{
  
  BERNOULLI_STATE_CONTENT
  tLProb     *l;        /* |D|. Conté p1-p0. */
  tLProb      logc;     /* Constant amb la logprobabilitat d'un vector
			   en blanc. */
  tLBool      zeros;    /* A cert si té probabilitats a cero en p0. */
  
} bernoulli_state_t;

/* Global. */
typedef struct
{
  BERNOULLI_GLOBAL_CONTENT
} bernoulli_global_t;


/* MÈTODES */

extern const tLAModelMethods tL_bernoulli_methods;


/* FUNCIONS */

/* A= B. */
void
tL_Bernoulli_assign_global (
			    bernoulli_global_t       *a,
			    const bernoulli_global_t *b
			    );

void *
tL_Bernoulli_copy_global (
			  void *global
			  );

void
tL_Bernoulli_free_global (
			  void *global
			  );

void
tL_Bernoulli_print_global (
			   void         *global,
			   FILE         *to,
			   const tLBool  binary
			   );

void
tL_Bernoulli_print_state (
			  const unsigned int       D,
			  FILE                    *to,
			  const bernoulli_state_t *state
			  );

/* Torna 0 si tot va be, -1 en cas d'error. */
int
tL_Bernoulli_read_global (
			  bernoulli_global_t  *global,
			  gzFile               from,
			  tLBuffer            *buffer,
			  const tLBool         binary,
			  char               **err
			  );

void *
tL_Bernoulli_read_global_meth (
			       gzFile         from,
			       tLBuffer      *buffer,
			       const tLBool   binary,
			       char         **err
			       );

/* Torna -1 en cas d'error, 0 si tot ha anat bé. */
int
tL_Bernoulli_read_state (
			 const unsigned int   D,
			 gzFile               from,
			 tLBuffer            *buffer,
			 bernoulli_state_t   *state,
			 const unsigned int   q,
			 const char          *sym,
			 char               **err
			 );

void
tL_Bernoulli_smooth_state (
			   const unsigned int  D,
			   bernoulli_state_t  *state,
			   const tLFloat       _f, /* 1.0-f*/
			   const tLFloat       f05 /* f*0.5 */
			   );

tLBool
tL_Bernoulli_test (
		   void         *global,
		   const tLFea  *fea,
		   char        **err
		   );

void
tL_Bernoulli_update_hiddendata (
				const unsigned int  D,
				bernoulli_state_t  *state
				);

#endif /* __TL_PRV_AMODEL__BERNOULLI_H__ */
