/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  dgaussian.h - Gaussiana diagonal.
 *
 *  Adrià Giménez Pastor, 2009,2011,2012.
 *
 */

#ifndef __TL_PRV_AMODEL__DGAUSSIAN_H__
#define __TL_PRV_AMODEL__DGAUSSIAN_H__

#include "../amodel.h"
#include "../common.h"


/* MACROS */

#define DGAUSSIAN_STATE(ptr) ((dgaussian_state_t *) (ptr))
#define DGAUSSIAN_GLOBAL(ptr) ((dgaussian_global_t *) (ptr))


/* TIPUS */

/* Estat */
typedef struct
{
  
  tLFloat *mu;      /* |D|. Mitja. */
  tLFloat *var;     /* |D|. Variança. */
  tLFloat *ivar;    /* |D|. 1/Variança. */
  tLFloat  logc;    /* Logarisme de la constant de normalització. */
  
} dgaussian_state_t;

/* Global */
typedef struct
{
  
  unsigned int  D;         /* Dimensió dels vectors. */
  tLFloat      *smooth;    /* |D|. Varianes per a suavitzar. */
  
} dgaussian_global_t;


/* MÈTODES */

extern const tLAModelMethods tL_dgaussian_methods;


/* FUNCIONS */

void
tL_DGaussian_update_logc (
			  const unsigned int  D,
			  dgaussian_state_t  *state
			  );


#endif /* __TL_PRV_AMODEL__DGAUSSIAN_H__ */
