/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  hybrid.c - 'hybrid.h' implementation.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bernoulli.h"
#include "amodel.h"
#include "common.h"
#include "error.h"
#include "global.h"
#include "hybrid.h"
#include "mem.h"
#include "samples.h"
#include "text.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) HYBRID_GLOBAL(ptr)
#define STATE(ptr) HYBRID_STATE(ptr)




/***********/
/* METHODS */
/***********/

static void *
alloc_states (
              void               *global,
              const unsigned int  Q
              )
{
  
  unsigned int *ret;
  
  
  ret= mem_alloc ( unsigned int, Q );
  
  return ret;
  
} /* end alloc_states */


static void
copy_states (
             void               *global,
             void               *src_states,
             void               *dst_states,
             const unsigned int  src_begin,
             const unsigned int  dst_begin,
             const unsigned int  Q
             )
{
  
  unsigned int q, q2, q3;
  
  
  for ( q= 0, q2= src_begin, q3= dst_begin; q < Q; ++q, ++q2, ++q3 )
    HYBRID_STATE(dst_states)[q3]= HYBRID_STATE(src_states)[q2];
  
} /* end copy_states */


static void
free_states (
             void               *global,
             void               *states,
             const unsigned int  Q
             )
{
  free ( states );
} /* end free_states */


static void
print_states (
              void               *global,
              FILE               *to,
              void               *states,
              const unsigned int  first,
              const unsigned int  Q,
              const tLBool        binary
              )
{
  
  unsigned int q, end;
  
  
  if ( binary )
    fwritee ( &(STATE(states)[first]), sizeof(unsigned int), Q, to );
  else
    {
      end= first+Q;
      fprintf ( to, "%u", STATE(states)[first] );
      for ( q= first+1; q < end; ++q )
        fprintf ( to, " %u", STATE(states)[q] );
      putc ( '\n', to );
    }
  
} /* end print_states */


static int
read_states (
             void                *global,
             gzFile               from,
             tLBuffer            *buffer,
             void                *states,
             const unsigned int   first,
             const unsigned int   Q,
             const char          *sym,
             const tLBool         binary,
             char               **err
             )
{
  
  unsigned int q, end, state;
  
  
  if ( binary )
    {
      if ( gzread ( from, &(STATE(states)[first]),
                   sizeof(unsigned int)*Q ) != Q*sizeof(unsigned int) )
        {
          MSGERR_DESCS ( err, "the state references", sym );
          return -1;
        }
    }
  else
    {
      end= first+Q;
      for ( q= first; q < end; ++q )
        {
          if ( !tL_gtoken ( from, buffer ) )
            {
              MSGERR_DESC ( err, "state reference", q, sym );
              return -1;
            }
	  state= atoui ( buffer->v );
          if ( state >= GLOBAL(global)->D )
            {
              tL_msgerror ( err, "In the %uth state of symbol '%s',"
                            " index '%u' is out of range", q+1, sym, state );
              return -1;
            }
          STATE(states)[q]= state;
        }
    }
  
  return 0;
  
} /* end read_states */


const tLAModelMethods tL_hybrid_methods=
  {
    alloc_states,
    NULL,
    tL_Bernoulli_copy_global,
    copy_states,
    tL_Bernoulli_free_global,
    free_states,
    tL_get_fea_type_real,
    tL_Bernoulli_print_global,
    print_states,
    tL_Bernoulli_read_global_meth,
    read_states,
    tL_smooth_global_empty,
    tL_smooth_states_empty,
    tL_Bernoulli_test
  };
