/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  tiedstates.c - Implementació de 'tiedstates.h'.
 *
 *  Adrià Giménez Pastor, 2011,2012.
 *
 */


#include <math.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../../intl.h"

#include "amodel.h"
#include "common.h"
#include "error.h"
#include "global.h"
#include "mem.h"
#include "samples.h"
#include "text.h"
#include "tiedstates.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) TIEDSTATES_GLOBAL(ptr)
#define STATE(ptr) TIEDSTATES_STATE(ptr)

#define REPEATED_NAME(ERR,NAME)						\
  tL_msgerror ( ERR, _( "There is already a state with name '%s'" ) , NAME )




/***********/
/* MÈTODES */
/***********/

static void *
alloc_states (
	      void               *global,
              const unsigned int  Q
	      )
{
  
  unsigned int *ret;
  
  
  mt ( ret= mem_alloc ( unsigned int, Q ) );
  
  return ret;
  
} /* end alloc_states */


static void *
copy_global (
	     void *global
	     )
{
  
  tiedstates_global_t *new;
  size_t n, id;
  
  
  mt ( new= mem_alloc ( tiedstates_global_t, 1 ) );
  new->type= TIEDSTATES_GLOBAL(global)->type;
  new->meths= TIEDSTATES_GLOBAL(global)->meths;
  new->global= new->meths->copy_global ( TIEDSTATES_GLOBAL(global)->global );
  new->N= TIEDSTATES_GLOBAL(global)->N;
  new->states= new->meths->alloc_states ( new->global, new->N );
  new->meths->copy_states ( new->global,
			    TIEDSTATES_GLOBAL(global)->states,
			    new->states, 0, 0, new->N );
  new->names= tL_dict_new ();
  for ( n= 0; n < TIEDSTATES_GLOBAL(global)->names->N; ++n )
    tL_dict_insert ( new->names,
		     tL_dict_get_token ( TIEDSTATES_GLOBAL(global)->names, n ),
		     &id );
  
  return new;
  
} /* end copy_global */


static void
copy_states (
	     void               *global,
	     void               *src_states,
	     void               *dst_states,
	     const unsigned int  src_begin,
	     const unsigned int  dst_begin,
	     const unsigned int  Q
	     )
{
  
  unsigned int q, q2, q3;
  
  
  for ( q= 0, q2= src_begin, q3= dst_begin; q < Q; ++q, ++q2, ++q3 )
    TIEDSTATES_STATE(dst_states)[q3]= TIEDSTATES_STATE(src_states)[q2];
  
} /* end copy_states */


static void
free_global (
	     void *global
	     )
{
  
  if ( GLOBAL(global)->names != NULL )
    tL_dict_free ( GLOBAL(global)->names );
  if ( GLOBAL(global)->states != NULL )
    GLOBAL(global)->meths->free_states ( GLOBAL(global)->global,
					 GLOBAL(global)->states,
					 GLOBAL(global)->N );
  if ( GLOBAL(global)->global != NULL )
    GLOBAL(global)->meths->free_global ( GLOBAL(global)->global );
  free ( global );
  
} /* end free_global */


static void
free_states (
             void               *global,
             void               *states,
             const unsigned int  Q
             )
{
  free ( states );
} /* end free_states */


static tLFeaType
get_fea_type (
              void *global
              )
{
  return GLOBAL(global)->meths->get_fea_type ( GLOBAL(global)->global );
} /* end get_fea_type */


static void
print_global (
              void         *global,
              FILE         *to,
              const tLBool  binary
              )
{
  
  unsigned int n;
  
  
  if ( binary ) tL_print_string_binary ( to, GLOBAL(global)->type );
  else          fprintf ( to, "%s\n", GLOBAL(global)->type );
  if ( GLOBAL(global)->global != NULL )
    GLOBAL(global)->meths->print_global ( GLOBAL(global)->global,
					  to, binary );
  if ( binary )
    fwritee ( &(GLOBAL(global)->N), sizeof(GLOBAL(global)->N), 1, to );
  else fprintf ( to, "N %u\n", GLOBAL(global)->N );
  if ( binary )
    {
      for ( n= 0; n < GLOBAL(global)->N; ++n )
	tL_print_string_binary ( to, tL_dict_get_token ( GLOBAL(global)->names,
							 n ) );
      GLOBAL(global)->meths->print_states ( GLOBAL(global)->global,
					    to, GLOBAL(global)->states,
					    0, GLOBAL(global)->N, TRUE );
    }
  else
    {
      fprintf ( to, "States\n" );
      for ( n= 0; n < GLOBAL(global)->N; ++n )
	{
	  fprintf ( to, "%s\n",
		    tL_dict_get_token ( GLOBAL(global)->names, n ) );
	  GLOBAL(global)->meths->print_states ( GLOBAL(global)->global,
						to, GLOBAL(global)->states,
						n, 1, FALSE );
	  
	}
    }
  
} /* end print_global */


static void
print_states (
              void               *global,
              FILE               *to,
              void               *states,
	      const unsigned int  first,
              const unsigned int  Q,
              const tLBool        binary
              )
{
  
  unsigned int q, end;
  
  
  if ( binary )
    fwritee ( &(STATE(states)[first]), sizeof(unsigned int), Q, to );
  else
    {
      end= first+Q;
      fprintf ( to, "%s", tL_dict_get_token ( GLOBAL(global)->names,
					      STATE(states)[first] ) );
      for ( q= first+1; q < end; ++q )
	fprintf ( to, " %s", tL_dict_get_token ( GLOBAL(global)->names,
						 STATE(states)[q] ) );
      putc ( '\n', to );
    }
  
} /* end print_states */


static void *
read_global (
             gzFile         from,
             tLBuffer      *buffer,
             const tLBool   binary,
	     char         **err
             )
{
  
  tiedstates_global_t *global;
  amodel_type_t type;
  char *token;
  unsigned int n;
  size_t id;
  
  
  mt ( global= mem_alloc ( tiedstates_global_t, 1 ) );
  global->global= NULL;
  global->states= NULL;
  global->names= NULL;
  if ( binary )
    {
      token= tL_read_string_binary ( from );
      if ( token == NULL ) goto errorgt;
      type= tL_search_amodel_type ( token );
      if ( type.name == NULL )
	{
	  MSGERR_UT ( err, token );
          free ( token );
          goto error;
	}
      free ( token );
    }
  else
    {
      if ( !tL_gtoken ( from, buffer ) ) goto errorgt;
      type= tL_search_amodel_type ( buffer->v );
      if ( type.name == NULL )
	{
	  MSGERR_UT ( err, buffer->v );
          goto error;
	}
    }
  global->type= type.name;
  global->meths= type.meths;
  if ( type.meths->read_global != NULL )
    {
      global->global=
        type.meths->read_global ( from, buffer, binary, err );
      if ( global->global == NULL ) goto error;
    }
  else global->global= NULL;
  if ( binary )
    {
      if ( gzread ( from, &(global->N),
		    sizeof(global->N) ) != sizeof(global->N) )
	{
	  MSGERR_GDESC ( err, _( "the number of states N" ) );
	  goto error;
	}
    }
  else
    {
      if ( !tL_gtoken ( from, buffer ) ||
	   strcmp ( "N", buffer->v ) )
	{
	  MSGERR_GTOK ( err, "N" );
	  goto error;
	}
      if ( !tL_gtoken ( from, buffer ) )
	{
	  MSGERR_GDESC ( err, _( "the number of states N" ) );
	  goto error;
	}
      global->N= atoui ( buffer->v );
    }
  if ( global->N == 0 )
    {
      MSGERR_GDESC0 ( err, _( "The number of states N" ) );
      goto error;
    }
  global->states= global->meths->alloc_states ( global->global, global->N );
  global->names= tL_dict_new ();
  if ( binary )
    {
      for ( n= 0; n < global->N; ++n )
	{
	  token= tL_read_string_binary ( from );
	  if ( token == NULL ) goto errorname;
	  if ( !tL_dict_insert ( global->names, token, &id ) )
	    {
	      REPEATED_NAME ( err, token );
	      free ( token );
	      goto error;
	    }
	  free ( token );
	}
      if ( global->meths->read_states ( global->global, from, buffer,
					global->states, 0, global->N,
					"[GLOBAL]", TRUE, err ) == -1 )
	goto error;
    }
  else
    {
      if ( !tL_gtoken ( from, buffer ) || strcmp ( "States", buffer->v ) )
	{
	  MSGERR_GTOK ( err, "States" );
	  goto error;
	}
      for ( n= 0; n < global->N; ++n )
	{
	  if ( !tL_gtoken ( from, buffer ) ) goto errorname;
	  if ( !tL_dict_insert ( global->names, buffer->v, &id ) )
	    {
	      REPEATED_NAME ( err, buffer->v );
	      goto error;
	    }
	  if ( global->meths->read_states ( global->global, from, buffer,
					    global->states, n, 1,
					    "[GLOBAL]", FALSE, err ) == -1 )
	    goto error;
	}
    }
  
  return (void *) global;
  
 errorname:
  tL_msgerror ( err, _( "Unable to read the name of the %uth state" ) , n+1 );
  goto error;
 errorgt:
  MSGERR_GDESC ( err, _( "the emission probability functions type" ) );
 error:
  free_global ( global );
  return NULL;
  
} /* end read_global */


static int
read_states (
             void                *global,
             gzFile               from,
             tLBuffer            *buffer,
             void                *states,
	     const unsigned int   first,
             const unsigned int   Q,
             const char          *sym,
             const tLBool         binary,
	     char               **err
             )
{
  
  unsigned int q, end;
  size_t id;
  
  
  if ( binary )
    {
      if ( gzread ( from, &(STATE(states)[first]), Q*sizeof(unsigned int) ) 
          != Q*sizeof(unsigned int) )
	{
	  MSGERR_DESCS ( err, _( "the state references" ) , sym );
	  return -1;
	}
    }
  else
    {
      end= first+Q;
      for ( q= first; q < end; ++q )
	{
	  if ( !tL_gtoken ( from, buffer ) )
	    {
	      MSGERR_DESC ( err, _( "state reference" ) , q, sym );
	      return -1;
	    }
	  id= tL_dict_find ( GLOBAL(global)->names, buffer->v );
	  if ( id == GLOBAL(global)->names->N )
	    {
	      tL_msgerror ( err, _( "In the %uth state of symbol '%s',"
			    " '%s' is not a state" ) , q+1, sym, buffer->v );
	      return -1;
	    }
	  STATE(states)[q]= id;
	}
    }
  
  return 0;
  
} /* end read_states */


static void
smooth_global (
               void          *global,
               const tLFloat  f
               )
{
  
  if ( GLOBAL(global)->global != NULL )
    GLOBAL(global)->meths->smooth_global ( GLOBAL(global)->global, f );
  GLOBAL(global)->meths->smooth_states ( GLOBAL(global)->global,
					 GLOBAL(global)->states,
					 GLOBAL(global)->N, f );
  
} /* end smooth_global */


static tLBool
test (
      void         *global,
      const tLFea  *fea,
      char        **err
      )
{
  return GLOBAL(global)->meths->test ( GLOBAL(global)->global, fea, err );
} /* end test */


const tLAModelMethods tL_tiedstates_methods=
  {
    alloc_states,
    NULL,
    copy_global,
    copy_states,
    free_global,
    free_states,
    get_fea_type,
    print_global,
    print_states,
    read_global,
    read_states,
    smooth_global,
    tL_smooth_states_empty,
    test
  };
