/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  amodel_itrans.h - Informació extra per a les transicions en format
 *                    llista. Conté les transicions en direcció
 *                    contraria i la distància de cada estat a l'estat
 *                    final.
 *
 *  Adrià Giménez Pastor, 2011.
 *
 */

#ifndef __LIBTL_AMODEL_ITRANS_H__
#define __LIBTL_AMODEL_ITRANS_H__

#include "amodel.h"
#include "common.h"

/* TIPUS */

/* Aresta inversa. */
typedef struct
{
  
  unsigned int  q;    /* Estat origen. */
  const tLProb *p;    /* Referència al paràmetre. */
  
} itrans_edge_t;

typedef struct
{
  
  const itrans_edge_t *begin;
  const itrans_edge_t *end;
  const tLProb        *ip;
  
} itrans_state_t;

/* Transicions inverses. */
struct tLAModelITrans
{
  
  itrans_state_t  final;
  itrans_state_t *states;
  itrans_edge_t  *edges;
  int            *dist;
  int             init_dist;
  
};

/* FUNCIONS */

/* Crea tots els itrans, o no fa res si ja estan creats. */
void
tL_amodel_alloc_all_itrans (
			    tLAModel *amodel
			    );

/* Allibera un itrans. */
void
tL_amodel_free_itrans (
		       tLAModelITrans *itrans
		       );

/* Funció auxiliar per a calcular les distàncies d'un estat a partir
   d'un offseti unes transicions. Requereix que estiguen creades les
   itrans. Torna el nou offset. */
int
tL_init_state_dists (
		     int                *dists,
		     const tLAModelHMM *cchar,
		     int                 offset
		     );

#endif /* __LIBTL_AMODEL_ITRANS_H__ */
