/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  auxfstats.c - Implementació de 'auxfstats.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2010,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "auxfstats.h"
#include "error.h"
#include "mem.h"




/**********************/
/* TIPUS ENREGISTRATS */
/**********************/

typedef auxfstats_mng_t * (create_auxfstats_mng_t) (void *global,char **err);

typedef struct
{

  const char             *name;
  create_auxfstats_mng_t *create;

} type_t;


#include "auxfstats/bernoulli.h"
#include "auxfstats/dgaussian.h"
#include "auxfstats/mixture.h"
#include "auxfstats/tiedstates.h"

static const type_t types[]=
  {
    { "Bernoulli", tL_create_afmng_bernoulli },
    { "DGaussian", tL_create_afmng_dgaussian },
    { "Mixture", tL_create_afmng_mixture },
    { "TiedStates", tL_create_afmng_tiedstates },
    { NULL, NULL }
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

auxfstats_mng_t *
tL_auxfstats_mng_new (
		      const char  *type,
		      void        *global,
		      char       **err
		      )
{
  
  int i;

  
  for ( i= 0; types[i].name != NULL &&
          strcmp ( type, types[i].name );
        ++i );
  if ( types[i].name == NULL )
    {
      TLMSG_TYPE ( err, type );
      return NULL;
    }
  
  return types[i].create ( global, err );
  
} /* end tL_auxfstats_mng_new */
