/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  calcelprob.c - Implementació de 'calcelprob.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2010,2011.
 *
 */


#include <math.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "calc.h"
#include "calcelprob.h"
#include "error.h"
#include "mem.h"




/**********************/
/* TIPUS ENREGISTRATS */
/**********************/

typedef calcelprob_t * (alloc_t) (void *,const int,char **err);

typedef struct
{
  
  const char *name;     /* Nom del tipus. */
  alloc_t    *alloc;    /* Reserva un model d'eixe tipus
			   sense incialitzar. */
  
} type_t;


#include "calcelprob/bernoulli.h"
#include "calcelprob/dgaussian.h"
#include "calcelprob/hybrid.h"
#include "calcelprob/loglinear.h"
#include "calcelprob/loglinearb.h"
#include "calcelprob/mixture.h"
#include "calcelprob/tiedstates.h"

static const type_t types[]=
  {
    { "Bernoulli", tL_alloc_bernoulli_calcelprob },
    { "DGaussian", tL_alloc_dgaussian_calcelprob },
    { "Hybrid", tL_alloc_hybrid_calcelprob },
    { "LogLinear", tL_alloc_loglinear_calcelprob },
    { "LogLinearB", tL_alloc_loglinearb_calcelprob },
    { "Mixture", tL_alloc_mixture_calcelprob },
    { "TiedStates", tL_alloc_tiedstates_calcelprob },
    { NULL, NULL }
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

calcelprob_t *
tL_create_calcelprob_type (
			   const char  *type,
			   void        *global,
			   const int    flags,
			   char       **err
			   )
{
  
  int i;
  
  
  for ( i= 0; types[i].name != NULL && strcmp ( type, types[i].name ); ++i );
  if ( types[i].name == NULL )
    {
      TLMSG_TYPE ( err, type );
      return NULL;
    }
  
  return types[i].alloc ( global, flags, err );
  
} /* end tL_create_calcelprob_type */
