/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  calcelprob.h - Càlcul de la probabilitat d'emisió en un estat.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */

#ifndef __TL_PRV_CALCELPROB_H__
#define __TL_PRV_CALCELPROB_H__

#include "amodel.h"
#include "common.h"
#include "samples.h"


/* TIPUS */

typedef struct calcelprob calcelprob_t;

typedef struct
{
  
  /* Calcula la probabilitat del frame indicat en l'estat indicat. Es
   * supossa que els frames han passat el test global. S'indica la
   * posició del frame que es vol calcular. L'estat s'indica com la
   * tupla punter a estats i posició de l'estat.
   */
  tLProb (*calc) (calcelprob_t       *cep,
		  void               *global,
		  void               *states,
		  const unsigned int  q,
		  const tLFea        *fea,
		  const int           t);
  
  /* Allibera memòria. */
  void   (*free) (calcelprob_t *cep);
  
  /* Prepara el calculador de probabilitats per a una nova seqüència
   * de frames. La seqüència pot ser una subseqüència d'una seqüència
   * Es supossa que l'imatge ha passat el test global.
   */
  void   (*init) (calcelprob_t  *cep,
		  void          *global,
		  const tLFea   *fea,
		  const int      begin,
		  const int      nframes);
  
  /* Torna cert si els valors calculats són densitats de probabilitats
   * en comptes de probabilitat.
   */
  tLBool (*isdensity) (calcelprob_t *cep);
  
} cep_methods_t;

#define CALCELPROB            \
                              \
  const cep_methods_t *meths;

struct calcelprob
{
  CALCELPROB
};


/* FUNCIONS */

#define CEP_MAX 0x1 /* Sustitueix sumatoris per màxims. */
#define CEP_FRAMECACHE 0x2 /* Indica que totes les operacions de un
			      frame es van a fer de manera
			      consecutiva. */

#define create_calcelprob(AMODEL,FLAGS,ERR)				\
  tL_create_calcelprob_type ( (AMODEL)->type, (AMODEL)->global, (FLAGS), ERR )

calcelprob_t *
tL_create_calcelprob_type (
			   const char  *type,
			   void        *global,
			   const int    flags,
			   char       **err
			   );

#define calc_elprob(CEP,AMODEL,STATES,Q,FEA,T)		       \
    ((CEP)->meths->calc ( (CEP), (AMODEL)->global, (STATES),   \
                          (Q), (FEA), (T) ))

#define free_calcelprob(CEP) ((CEP)->meths->free ( (CEP) ))

#define init_calcelprob(CEP,AMODEL,FEA,BEGIN,NFRAMES)			\
  ((CEP)->meths->init ( (CEP), (AMODEL)->global, (FEA), (BEGIN), (NFRAMES) ))

#define isdensity_calcelprob(CEP) (CEP)->meths->isdensity ( (CEP) )


#endif /* __TL_PRV_CALCELPROB_H__ */
