/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  bernoulli.c - Implementació de 'bernoulli.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */


#include <math.h>
#include <stddef.h>
#include <stdlib.h>

#include "calcelprob.h"
#include "amodel/bernoulli.h"
#include "common.h"
#include "error.h"
#include "global.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) BERNOULLI_GLOBAL(ptr)
#define STATE(ptr) BERNOULLI_STATE(ptr)
#define CEP(ptr) ((bernoulli_calcelprob_t *) (ptr))




/*********/
/* TIPUS */
/*********/

typedef struct
{
  
  CALCELPROB
  
} bernoulli_calcelprob_t;




/***********/
/* MÈTODES */
/***********/

static tLProb
calc (
      calcelprob_t        *cep,
      void                *global,
      void                *states,
      const unsigned int   q,
      const tLFea         *fea,
      const int            t
      )
{
  
  int d, D;
  tLProb ret;
  const bit *frame;
  const int *i;
  bernoulli_state_t state;
  
  
  state= STATE(states)[q];
  ret= 0.0;
  if ( state.zeros )
    {
      frame= B2cb(fea->v[t]);
      D= GLOBAL(global)->D;
      for ( d= 0; d < D; ++d )
	ret+= frame[d] ? state.p[d].p1 : state.p[d].p0;
    }
  else
    {
      /* NOTA: És molt important que primer es sumen els 1 i després
	 la massa de 0. En cas contrari al estar en rangs molt
	 diferents, en casos extrems poden eixir resultats absurds. */
      for ( i= tL_fea_get_sparse ( fea )[t]; *i != -1; ++i )
	ret+= state.l[*i];
      ret+= state.logc;
    }
  
  return ret;
  
} /* end calc */


static void
_free (
       calcelprob_t *cep
       )
{
  
  free ( cep );
  
} /* end _free */


static const cep_methods_t methods=
  {
    calc,
    _free,
    tL_calcelprob_init_empty,
    tL_calcelprob_notisdensity
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

calcelprob_t *
tL_alloc_bernoulli_calcelprob (
			       void       *global,
			       const int   flags,
			       char      **err
			       )
{
  
  bernoulli_calcelprob_t *new;
  
  
  mt ( new= mem_alloc ( bernoulli_calcelprob_t, 1 ) );
  new->meths= &methods;
  
  return (calcelprob_t *) new;
  
} /* tL_alloc_bernoulli_calcelprob */
