/*
 *  Copyright 2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  hybrid.h - 'hybrid.h' implementation.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "calcelprob.h"
#include "amodel/hybrid.h"
#include "common.h"
#include "error.h"
#include "global.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define STATE(ptr) HYBRID_STATE(ptr)




/*********/
/* TYPES */
/*********/

typedef struct
{
  
  CALCELPROB
  
} hybrid_calcelprob_t;




/***********/
/* METHODS */
/***********/

static tLProb
calc (
      calcelprob_t        *cep,
      void                *global,
      void                *states,
      const unsigned int   q,
      const tLFea         *fea,
      const int            t
      )
{
  return (tLProb) B2cR(fea->v[t])[STATE(states)[q]];
} /* end calc */


static void
_free (
       calcelprob_t *cep
       )
{
  free ( cep );
} /* end _free */


static const cep_methods_t methods=
  {
    calc,
    _free,
    tL_calcelprob_init_empty,
    tL_calcelprob_isdensity
  };




/********************/
/* PUBLIC FUNCTIONS */
/********************/

calcelprob_t *
tL_alloc_hybrid_calcelprob (
			    void       *global,
			    const int   flags,
			    char      **err
			    )
{
  
  hybrid_calcelprob_t *new;
  
  
  mt ( new= mem_alloc ( hybrid_calcelprob_t, 1 ) );
  new->meths= &methods;
  
  return (calcelprob_t *) new;
  
} /* tL_alloc_hybrid_calcelprob */
