/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  loglinearb.c - Implementació de 'loglinearb.h'.
 *
 *  Adrià Giménez Pastor, 2012.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "calcelprob.h"
#include "amodel/loglinear.h"
#include "common.h"
#include "error.h"
#include "global.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) LOGLINEAR_GLOBAL(ptr)
#define STATE(ptr) LOGLINEAR_STATE(ptr)
#define CEP(ptr) ((loglinearb_calcelprob_t *) (ptr))




/*********/
/* TIPUS */
/*********/

typedef struct
{
  
  CALCELPROB
  
} loglinearb_calcelprob_t;




/***********/
/* MÈTODES */
/***********/

static tLProb
calc (
      calcelprob_t       *cep,
      void               *global,
      void               *states,
      const unsigned int  q,
      const tLFea        *fea,
      const int           t
      )
{
  
  const int *sframe, *p;
  loglinear_state_t state;
  tLFloat ret;
  
  
  sframe= tL_fea_get_sparse ( fea )[t];
  state= STATE(states)[q];
  ret= 0.0;
  for ( p= sframe; *p != -1; ++p )
    ret+= state.l[*p];
  
  return (tLProb) ret;
  
} /* end calc */


static void
_free (
       calcelprob_t *cep
       )
{
  free ( cep );
} /* end _free */


static const cep_methods_t methods=
  {
    calc,
    _free,
    tL_calcelprob_init_empty,
    tL_calcelprob_isdensity
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

calcelprob_t *
tL_alloc_loglinearb_calcelprob (
				void       *global,
				const int   flags,
				char      **err
				)
{
  
  loglinearb_calcelprob_t *new;
  
  
  mt ( new= mem_alloc ( loglinearb_calcelprob_t, 1 ) );
  new->meths= &methods;
  
  return (calcelprob_t *) new;
  
} /* tL_alloc_loglinearb_calcelprob */
