/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  tiedstates.c - Implementació de 'tiedstates.h'.
 *
 *  Adrià Giménez Pastor, 2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../../intl.h"

#include "calcelprob.h"
#include "amodel/tiedstates.h"
#include "common.h"
#include "error.h"
#include "mem.h"
#include "tiedstates.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) TIEDSTATES_GLOBAL(ptr)
#define STATE(ptr) TIEDSTATES_STATE(ptr)
#define CEP(ptr) ((tiedstates_calcelprob_t *) (ptr))




/*********/
/* TIPUS */
/*********/

typedef struct
{
  
  tLProb p;
  tLBool a;
  
} node_t;


typedef struct
{
  
  CALCELPROB
  calcelprob_t  *cep;
  node_t        *v;
  unsigned int   N;
  node_t       **stack;
  int            last_frame;
  
} tiedstates_calcelprob_t;




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
clean (
       tiedstates_calcelprob_t *cep
       )
{
  
  unsigned int n;
  node_t **stack;
  
  stack= cep->stack;
  for ( n= 0; n < cep->N; ++n )
    stack[n]->a= FALSE;
  cep->N= 0;
  
} /* end clean */




/***********/
/* MÈTODES */
/***********/

static tLProb
calc (
      calcelprob_t        *cep,
      void                *global,
      void                *states,
      const unsigned int   q,
      const tLFea         *fea,
      const int            t
      )
{
  
  unsigned int state;
  
  
  if ( t != CEP(cep)->last_frame )
    {
      clean ( CEP(cep) );
      CEP(cep)->last_frame= t;
    }
  state= STATE(states)[q];
  if ( CEP(cep)->v[state].a ) return CEP(cep)->v[state].p;
  CEP(cep)->v[state].a= TRUE;
  CEP(cep)->stack[CEP(cep)->N++]= &(CEP(cep)->v[state]);
  return CEP(cep)->v[state].p=
    CEP(cep)->cep->meths->calc ( CEP(cep)->cep, GLOBAL(global)->global,
				 GLOBAL(global)->states, state, fea, t );
  
} /* end calc */


static void
_free (
       calcelprob_t *cep
       )
{
  
  free ( CEP(cep)->stack );
  free ( CEP(cep)->v );
  CEP(cep)->cep->meths->free ( CEP(cep)->cep );
  free ( cep );
  
} /* end _free */


static void
init (
      calcelprob_t  *cep,
      void          *global,
      const tLFea   *fea,
      const int      begin,
      const int      nframes
      )
{
  
  CEP(cep)->cep->meths->init ( CEP(cep)->cep, GLOBAL(global)->global,
			       fea, begin, nframes );
  clean ( CEP(cep) );
  CEP(cep)->last_frame= -1;
  
} /* end init */


static tLBool
isdensity (
           calcelprob_t *cep
           )
{
  return CEP(cep)->cep->meths->isdensity ( CEP(cep)->cep );
} /* end isdensity */


static const cep_methods_t methods=
  {
    calc,
    _free,
    init,
    isdensity
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

calcelprob_t *
tL_alloc_tiedstates_calcelprob (
				void       *global,
				const int   flags,
				char      **err
				)
{
  
  tiedstates_calcelprob_t *new;
  unsigned int N, n;
  
  
  if ( !(flags&CEP_FRAMECACHE) )
    tL_error ( _( "TiedStates without CEP_FRAMECACHE support is not implemented" ) );
  mt ( new= mem_alloc ( tiedstates_calcelprob_t, 1 ) );
  new->meths= &methods;
  new->cep= tL_create_calcelprob_type ( GLOBAL(global)->type,
					GLOBAL(global)->global,
					flags, err );
  if ( new->cep == NULL )
    {
      free ( new );
      return NULL;
    }
  N= GLOBAL(global)->N;
  mt ( new->v= mem_alloc ( node_t, N ) );
  for ( n= 0; n < N; ++n ) new->v[n].a= FALSE;
  mt ( new->stack= mem_alloc ( node_t *, N ) );
  new->N= 0;
  new->last_frame= -1;
  
  return (calcelprob_t *) new;
  
} /* end tL_alloc_tiedstates_calcelprob */
