/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  cmllr_alg.h - Algorisme CMLLR.
 *
 *  Miguel Ángel del Agua Teba.
 */

#ifndef __CMLLR_ALG_H__
#define __CMLLR_ALG_H__

#include "amcounts/dgaussian.h"
#include "amodel/dgaussian.h"
#include "common.h"

typedef struct {
  
  double      *newmu;
  double      *newvar;
  unsigned int  size;
  
} cmllr_alg_data_t ;

typedef struct
{
  
  unsigned int                I;
  dgaussian_state_t         **params;
  amcount_dgaussian_state_t **counts;
  
} cmllr_alg_paramscounts_t;

typedef int matrix_t;
typedef unsigned int uint;

void
tL_cmllr_alg_data_alloc (
			 cmllr_alg_data_t *mem_vars,
			 uint              D
			 );

void
tL_cmllr_alg_data_free (
			cmllr_alg_data_t *mem_vars
			);

int
tL_cmllr_alg (
	      const cmllr_alg_paramscounts_t    *pcs,
	      const dgaussian_global_t          *global,
	      double                           **W,
	      cmllr_alg_data_t                  *mem_vars,
	      char                             **err
	      );

#endif /* __CMLLR_ALG_H__ */
