/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 * =====================================================================================
 *
 *       Filename:  cmllr_alg_full.c
 *
 *    Description:  
 *
 *        Version:  1.0
 *        Created:  24/07/12 11:29:17
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Miguel Ángel del Agua Teba (), mdelagua@dsic.upv.es
 *        Company:  UPV
 *
 * =====================================================================================
 */


#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#define __TL_PRV

#include "amcounts/dgaussian.h"
#include "amodel/dgaussian.h"
#include "common.h"
#include "error.h"
#include "mem.h"
#include "cmllr_alg_full.h"
#include <math.h>
#include <cblas.h>
#include <clapack.h>

#define ABS(a) (((a) < 0) ? -(a) : (a))


void tL_cmllr_alg_full_data_alloc(
				  cmllr_alg_full_data_t *mem_vars,
				  uint                   D
				  )
{
  double *mem;
  double **mem2;
  uint d, i, j;

  /* Reserva de Memoria para kd*/
  mt(mem_vars->k= mem_alloc(double *, (D)));
  mt(mem_vars->k[0]= mem= mem_alloc (double, D*(D+1)));
  for (d= 0; d < D; d++, mem+=(D+1) )mem_vars->k[d]= mem;
  /* ****************** */

  /* Reserva de Memoria para G */
  mt(mem_vars->G= mem_alloc(double **, (D))); /* D Matrices de tamaño D+1*D+1 */
  mt(mem2= mem_alloc(double *, D*(D+1)));
  mt(mem= mem_alloc(double, D*(D+1)*(D+1)));
  for ( i= 0; i < D; ++i, mem2+=D+1){
    mem_vars->G[i]= mem2;
    for ( j= 0; j < D+1; ++j, mem+=D+1 )
      mem2[j]= mem;
  }
  /******************************/

  /* Reserva de Memoria para p */
  mt(mem_vars->p= mem_alloc(double, D+1));
  /******************************/

  /* Reserva para la matrix A  */
  mt(mem_vars->A= mem_alloc(double *, D));
  mt(mem_vars->A[0]= mem= mem_alloc(double, D*D));
  for ( d= 0; d < D; ++d, mem+= D )mem_vars->A[d]= mem;
  /******************************/

  /* Reserva para el vector b 
   * el vector de permutacion ipiv
   * y el termino intermedio e12 (vector) */
  mt(mem_vars->b= mem_alloc(double, D));
  mt(mem_vars->ipiv= mem_alloc(int, D+1));
  mt(mem_vars->e12= mem_alloc(double, D+1));
  mt(mem_vars->waux= mem_alloc(double, D+1));
  mt(mem_vars->waux2= mem_alloc(double, D+1));
  /******************************/
}

void
tL_cmllr_alg_full_data_free (
			     cmllr_alg_full_data_t *mem_vars
			     )
{
  free(mem_vars->k[0]); free(mem_vars->k);
  free(mem_vars->G[0][0]); free(mem_vars->G[0]); free(mem_vars->G);
  free(mem_vars->p);
  free(mem_vars->A[0]); free(mem_vars->A);
  free(mem_vars->b); free(mem_vars->ipiv); free(mem_vars->e12); 
  free(mem_vars->waux); free(mem_vars->waux2);
}



/*********************/
/* FUNCIONS PRIVADES */
/*********************/

/* Estimte vector k for a given row */
static void 
getKd ( 
       dgaussian_state_t         **dgstate,   /* Componentes de mix de
						 Gaussianas del
						 estado */
       amcount_dgaussian_state_t **dgcounts,  /* Cuentas de la
						 gaussiana */
       const uint                  d,         /* Dimensión solicitada */
       const uint                  D,         /* Dimension max*/
       const double               I,         /* Numero de componentes*/
       double                    *k          /* Vector k a devolver*/
      )
{
  uint i,j; 
  double kaux;

  /* Ojo que simulo la extensión de x */
  for ( i= 0; i< I; ++i )
  {
    kaux = (dgstate[i]->mu[d])/(dgstate[i]->var[d]);
    for ( j= 0; j< D ; ++j ) 
      if ( j== 0 )
        k[j]+= kaux * dgcounts[i]->norm;
      else
        k[j]+= kaux * dgcounts[i]->x[j-1];
  }
}

/* Estimate matrix G for a given row */
static void 
getGd ( 
       dgaussian_state_t         **dgstate,   /* Componentes de mix de
						 Gaussianas del
						 estado */
       amcount_dgaussian_state_t **dgcounts,  /* Cuentas de la
						 gaussiana */
       uint                        d,         /* Dimension solicitada */
       uint                        D,         /* Dimension*/
       uint                        I,         /* Numero de componentes*/
       double                   **G          /* Matriz G a devolver */
      )
{
  uint i,j,t; 

  for ( i= 0; i< I; ++i )
    for ( j= 0; j< D+1 ; ++j )
      for ( t= 0; t< D+1 ; ++t )
        G[j][t]+=dgstate[i]->ivar[d] * dgcounts[i]->xx[j][t];
}


/* Obtains a minor of a matrix given its indices */
static void 
getMinorMatrix(
          double         **A,
          uint              D, /* Max rows or cols (square matrix)*/
          uint              i,
          uint              j,
          double         **minor
        )
{
  uint r,c;

  for ( c= 0; c < D; ++c )
    for ( r= 0; r < D; ++r )
    { 
      if ( c == i || r == j )
        continue;
      if ( c < i && r < j )
        minor[c][r] = A[c][r];
      else if ( c > i && r > j )
        minor[c-1][r-1] = A[c][r];
      else if ( c < i && r > j )
        minor[c][r-1] = A[c][r];
      else if ( c > i && r < j )
        minor[c-1][r] = A[c][r];
    }
}

/* Estimate the cofactor matrix */
static int
getP (
       double        **A,
       uint             d,
       uint             D,
       double         *p
       )
{
  uint i,j,k; 
  double *mem, *q;
  double **Aaux;
  int *ipiv;
  int result;
  uint nperms= 0;
  int sign= 0;
  double det= 1;

  mt(Aaux= mem_alloc(double*, (D-1)));
  mt(Aaux[0]= mem= mem_alloc(double,(D-1)*(D-1)));
  for ( i= 0; i < (D-1); ++i, mem+=(D-1))
    Aaux[i]= mem;

  mt(ipiv= mem_alloc(int,(D-1)));

  p[0]= 0;
  for ( j= 0; j < D; ++j )
  {
    det= 1;
    nperms= 0;
    getMinorMatrix(A,D,j,d,Aaux);

    result= clapack_dgetrf(CblasColMajor, D-1, D-1, Aaux[0], D-1, ipiv);
    
    if(result < 0)
      return -1;

    nperms=0;
    for( k= 0, q= Aaux[0]; k < D-1; ++k, q+=D )
    {
      det *= *q;

      if( ipiv[k] != k )
        nperms++;
    }
    det= (nperms & 1) == 1? det*-1 : det;
    sign= ((d+j) & 1) == 0? 1 : -1; 
    p[j+1]= sign * det;
  }

  free(ipiv);
  free(Aaux[0]);
  free(Aaux);
  return 0;
}

static void
getAB (
        double **W,
        uint D,
        double **A,
        double *b
      )
{
  uint i,j;

  for ( i= 0; i < D; ++i )
  {
    b[i]= W[0][i];
    for ( j= 0; j < D; ++j )
      A[i][j]= W[i+1][j]; 
  }
}


/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

int
tL_cmllr_alg_full (
		   const cmllr_alg_paramscounts_t    *pcs,
		   const dgaussian_global_t          *global,
		   const uint                         niters,
		   double                          **W,
		   cmllr_alg_full_data_t             *mem_vars,
		   char                             **err
		   )
{

  uint D,i,d,ite,j;
  double beta;
  double ***G;
  double *p;
  double **A;
  double **k;
  double *b;
  double e1,e2;
  double *e12;
  double alpha, alpha1, alpha2;
  double q1, q2;
  double *waux, *waux2;
  int *ipiv;
  int result;
  double aux;

  G= mem_vars->G;
  p= mem_vars->p;
  A= mem_vars->A;
  k= mem_vars->k;
  b= mem_vars->b;
  ipiv= mem_vars->ipiv;
  e12= mem_vars->e12;
  waux= mem_vars->waux;
  waux2= mem_vars->waux2;

  assert ( pcs->counts[0]->xx != NULL );
  D= global->D; /* Dimension (39) */
    
  /* BETA************************** */
  beta= 0;
  for ( i= 0; i < pcs->I; ++i ) beta+=pcs->counts[i]->norm; /*Recorremos todas las componentes de la mix*/

  /* Estimate D matrices (G) and a matrix k with D rows. */
  for ( d= 0; d < D; ++d ) 
  {
    /* Inicializacion de G y k a 0 */
    for( i= 0; i < D+1; ++i )
    {
      k[d][i]= 0;
      for( j= 0; j < D+1; ++j )
        G[d][i][j]= 0;

      ipiv[i]=0;
    }

    getGd(pcs->params, pcs->counts, d, D, pcs->I, G[d]); 
    getKd(pcs->params, pcs->counts, d, D+1, pcs->I, k[d]);

    /* Obtenemos las inversas de las matrices G */

    /* Inversa de G */
    result= clapack_dgetrf(CblasColMajor, D+1, D+1, G[d][0], D+1, ipiv);
    if ( result != 0 )
    {
      tL_msgerror ( err, _( "error in LU factorization" ) );
      return -1;
    }
    
    /* En G dejo la G invertida */
    result= clapack_dgetri(CblasColMajor, D+1,  G[d][0] , D+1, ipiv);
    if ( result != 0 )
    {
	    tL_msgerror ( err, _( "error calculating the inverse of G" ) );
	    return -1;
    }
  }

  /* Tras cada iteración lo que obtenemos es una nueva estimación de W */
  for ( ite= 0; ite < niters ; ++ite )
  {
    d= ite % D;
    getAB(W, D, A, b); 
    getP(A, d, D, p); 

    alpha1= alpha2= 0;
    cblas_dgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, /* m */ 1, /* n */ D+1, /* k */ D+1, 1, p, 1, G[d][0], D+1, 0, e12, 1); 

    /* e1 */
    
    e1= cblas_ddot(D+1, e12, 1, p, 1);

    /* e2 */
    e2= cblas_ddot(D+1, e12, 1, k[d], 1);

    aux = e2*e2 + 4 * e1 * beta; 

    if ( e1 == 0 || aux < 0 )
    {
      tL_msgerror ( err, _( "e1 is 0 or root solution is a complex number" ) );
      return -1;
    }

    aux = sqrt( aux );


    alpha1= ((-e2 + aux) / ( 2 * e1 ));
    alpha2= ((-e2 - aux) / ( 2 * e1 )); 

    /* Y ahora me quedo con el alpha que maximice Q */
    aux= alpha1 * e1 + e2;
    aux= ABS ( aux );
    q1= beta * log( aux ) - ( alpha1*alpha1*e1/2 );
    aux= alpha2 * e1 + e2;
    aux= ABS ( aux );
    q2= beta * log( aux ) - ( alpha2*alpha2*e1/2 );

    alpha= q1 > q2? alpha1: alpha2;

    /* Nos guardamos k[d] para no perderlo */
    cblas_dcopy( D+1, k[d], 1, waux, 1 );

    /* Por tanto la fila d de W será: */
    cblas_daxpy( D+1, alpha, p, 1 , waux, 1 ); /* El resultado se
                 queda en
                 waux */
    cblas_dgemm(CblasColMajor, CblasNoTrans, CblasNoTrans,
    1, D+1, D+1, 1, waux, 1, G[d][0], D+1, 0, waux2, 1); 
    for( i= 0; i < D+1; ++i )
      W[i][d]= waux2[i];
  }


  /* XULLAS */
  /* dgstates= DGAUSSIAN_STATE(state->members);
  D= DGAUSSIAN_GLOBAL(global->global)->D;*/
  /* dgcounts[m].norm Esto es gamma_m para todo t*/
  /* mt ( kk= mem_alloc ( int, 5 ) ); */
  /* mt ( kk= mem_realloc ( int, kk, 5 ) ); */

  return 0;
  
} /* end tL_cmllr_alg_full */
