/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  common.h - Definicions comuns.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2012.
 *
 */

#ifndef __TL_PRV_COMMON_H__
#define __TL_PRV_COMMON_H__

#include <math.h>
#include <stdio.h>
#include <zlib.h>

#include "common_public.h"
#include "calc.h"


/* MACROS */

#define _PS(val) ((unsigned long) (val))

#define _DB(val) ((double) (val))

#define atoui(STR) ((unsigned int) strtol ( (STR), (char **) NULL, 10 ))

#define atoul(STR) ((unsigned long) strtol ( (STR), (char **) NULL, 10 ))

#define atos(STR) ((size_t) strtol ( STR, (char **) NULL, 10 ))

#define p1top0(PROB) ((tLProb) log ( 1.0 - exp ( (double) (PROB) ) ))

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#define printprobsep(TO,PROB)			\
  do {						\
    fputc ( ' ', TO );				\
    tL_prob_print ( PROB, TO );			\
  } while(0)

#define smooth(PROB,AUX,AUX2)                                                 \
          do {                                                                \
            (PROB).p1= (tLProb) log ( ((AUX) * exp ( (PROB).p1 )) + (AUX2) ); \
	    (PROB).p0= p1top0 ( (PROB).p1 );                                  \
          } while ( 0 )


/* TIPUS */

struct ab
{
  
  tLProb alpha;
  tLProb beta;
  
};


/* FUNCIONS */

/* Escriu en un fitxer una cadena de caràcters amb un format binari
 * totalment depenent de la màquina. En cas d'tL_error produeix un tL_error
 * fatal. STRING es supossa no NULL.
 */
void
tL_print_string_binary (
			FILE       *to,
			const char *string
			);

/* Llig d'un fitxer una cadena de caràcters amb un format binari
 * totalment depenent de la màquina. Torna NULL si la longitut és
 * cero.
 */
char *
tL_read_string_binary (
		       gzFile from
		       );


#endif /* __TL_PRV_COMMON_H__ */
