/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  common_public.h - Part pública de 'common.h'.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2012.
 *
 */

#ifndef __LIBTL_COMMON_H__
#define __LIBTL_COMMON_H__

#include <stdio.h>

#include "port.h"

/** \defgroup Basic Basic types and utilities
 */

/** \addtogroup Basic
 *  @{
 */

/* MACROS */

#ifndef FALSE
/** The False value */
#define FALSE 0
/** The True value */
#define TRUE  1
#endif


/* TIPUS */

/** Type float. */
typedef double tLFloat;

/** An alias of the type tLFloat used to represent log-probabilities */
typedef tLFloat tLProb;

/** A pair of log-probabilities. This structure is used to store two
 *  log-probabilities P1 and P0; usually these two probabilities are
 *  constrained to sum 1.
 */
typedef struct
{
  
  tLProb p1;    /**< Log-Probability P1. */
  tLProb p0;    /**< Log-Probability P0. */
  
} tLProbPair;

/** Boolean type */
typedef unsigned char tLBool;


/* FUNCIONS */

/** From ASCII to Prob. Converts a string to tLProb.
 *
 *  @param str A pointer to string. Cannot be NULL.
 *  @return The converted value.
 */
TL_API_FUNC(tLProb)
tL_atop (
	 const char *str
	 );

/** Prints a probability. Prints a probability to the specified
 *  file. The format depends on the current locale.
 *
 *  @param prob Probability.
 *  @param to File to which the probability will be printed.
 */
TL_API_FUNC(void)
tL_prob_print (
	       const tLProb  prob,
	       FILE         *to
	       );


/** @} */

#endif /* __LIBTL_COMMON_H__ */
