/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  create_amodels.h - Funcions per a crear nous models de caràcters.
 *
 *  Adrià Giménez Pastor, 2011,2012,2013.
 *
 */

#ifndef __LIBTL_CREATE_AMODELS_H__
#define __LIBTL_CREATE_AMODELS_H__

#include "amodel_public.h"
#include "port.h"
#include "text_public.h"

/** \addtogroup Models
 *  @{
 */
/** \addtogroup AModel
 *  @{
 */

/** Creates a new set of Bernoulli HMMs. This functions
 *  creates a new set of Bernoulli HMMs. For each symbol
 *  of the provided dictionary, a new Bernoulli HMM with Q
 *  states and D-dimensional feature vectors is created.
 *
 *  @param syms Non-empty dictionary in which HMM labels are
 *  registered.
 *  @param Q The number of states for the new Bernoulli HMM.
 *  @param D The dimension of feature vector for the new Bernoulli
 *  HMMs.
 *  @return The Bernoulli HMMs.
 */
TL_API_FUNC(tLAModel *)
tL_amodel_bernoulli_new (
			 const tLDict       *syms,
			 const unsigned int  Q,
			 const unsigned int  D
			 );

/** Creates a new set of Bernoulli HMMs with variable states. This
 *  functions creates a new set of Bernoulli HMMs. For each symbol of
 *  the provided dictionary, a new Bernoulli HMM with D-dimensional
 *  feature vector is created. An array of size syms->N is provided in
 *  order to indicate the number of states for each HMM.
 *
 *  @param syms Non-empty dictionary in which HMM labels are registered.
 *  @param Qs An array with the number of states for each label of
 *  the dictionary.
 *  @param D The dimension of feature vector for the new Bernoulli
 *  HMMs.
 *  @return The Bernoulli HMMs.
 */
TL_API_FUNC(tLAModel *)
tL_amodel_bernoulli_new_vs (
			    const tLDict       *syms,
			    const unsigned int *Qs,
			    const unsigned int  D
			    );

/** Creates a new set of diagonal Gaussian HMMs. This functions
 *  creates a new set of diagonal Gaussian HMMs. For each symbol of
 *  the provided dictionary, a new diagonal Gaussian HMM with Q states
 *  and D-dimensional feature vectors is created.
 *
 *  @param syms Non-empty dictionary in which HMM labels are
 *  registered.
 *  @param Q The number of states for the new diagonal Gaussian
 *  HMMs.
 *  @param D The dimension of feature vector for the new diagonal
 *  Gaussian HMMs.
 *  @return The diagonal Gaussian HMMs.
 */
TL_API_FUNC(tLAModel *)
tL_amodel_dgaussian_new (
			 const tLDict       *syms,
			 const unsigned int  Q,
			 const unsigned int  D
			 );

/** Creates a new set of diagonal Gaussian HMMs with variable
 *  states. This functions creates a new set of diagonal Gaussian
 *  HMMs. For each symbol of the provided dictionary, a new diagonal
 *  Gaussian HMM with D-dimensional feature vector is created. An
 *  array of size syms->N is provided in order to indicate the number
 *  of states for each HMM.
 *
 *  @param syms Non-empty dictionary in which HMM labels are registered.
 *  @param Qs An array with the number of states for each label of
 *  the dictionary.
 *  @param D The dimension of feature vector for the new diagonal
 *  Gaussian HMMs.
 *  @return The diagonal Gaussian HMMs.
 */
TL_API_FUNC(tLAModel *)
tL_amodel_dgaussian_new_vs (
			    const tLDict       *syms,
			    const unsigned int *Qs,
			    const unsigned int  D
			    );

/** Gets a pointer to the smooth array. This function gets a pointer
 *  to the smooth array of a diagonal Gaussian HMMs. The
 *  content of the array can be modified, but the pointer cannot be
 *  freed.
 *
 *  @param amodel The diagonal Gaussian HMMs.
 */
TL_API_FUNC(tLFloat *)
tL_amodel_dgaussian_get_smooth (
				tLAModel *amodel
				);

/** @} */
/** @} */

#endif /* __LIBTL_CREATE_AMODELS_H__ */
