/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  eprobs.c - Implementació de 'eprobs.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "eprobs.h"
#include "error.h"
#include "mem.h"




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

eprobs_t *
tL_create_eprobs (
		  const tLAModel  *amodel,
		  char           **err
		  )
{
  
  eprobs_t *new;
  struct eprob **aux_prob, *aux_prob2;
  size_t aux_size;
  unsigned int q, c;
  
  
  mt ( new= mem_alloc ( eprobs_t, 1 ) );
  
  new->cep= create_calcelprob ( amodel, 0, err );
  if ( new->cep == NULL )
    {
      free ( new );
      return NULL;
    }
  
  mt ( new->p= mem_alloc ( struct eprob **,
			   amodel->N ) );
  
  aux_size= amodel->total_Q;
  mt ( new->p[0]= mem_alloc ( struct eprob *,
			      aux_size ) );
  aux_prob= new->p[0];
  for ( c= 0; c < amodel->N; ++c )
    {
      new->p[c]= aux_prob;
      aux_prob+= amodel->hmms[c].Q;
    }
  
  mt ( new->p[0][0]= mem_alloc ( struct eprob,
				 aux_size ) );
  for ( c= 0, aux_prob2= new->p[0][0];
	c < amodel->N; ++c )
    for ( q= 0; q < amodel->hmms[c].Q;
	  ++q, ++aux_prob2 )
      {
	new->p[c][q]= aux_prob2;
	aux_prob2->active= FALSE;
      }
  
  new->length= 1;
  
  return new;
  
} /* end tL_create_eprobs */


tLProb
tL_eprobs_lprob (
		 eprobs_t           *eprobs,
		 const tLAModel     *amodel,
		 const tLFea        *fea,
		 const unsigned int  c,
		 const unsigned int  q,
		 const int           t
	      )
{
  
  if ( !(eprobs->p[c][q][t].active) )
    {
      eprobs->p[c][q][t].prob=
	calc_elprob ( eprobs->cep, amodel, amodel->hmms[c].states, q, fea, t );
      eprobs->p[c][q][t].active= TRUE;
    }
  
  return eprobs->p[c][q][t].prob;
  
} /* end tL_eprobs_lprob */


void
tL_free_eprobs (
		eprobs_t *eprobs
		)
{
  
  free_calcelprob ( eprobs->cep );
  free ( eprobs->p[0][0] );
  free ( eprobs->p[0] );
  free ( eprobs->p );
  free ( eprobs );
  
} /* end tL_free_eprobs */


void
tL_reset_eprobs (
		 eprobs_t       *eprobs,
		 const tLAModel *amodel,
		 const int       length
		 )
{
  
  size_t c;
  unsigned int q;
  int t;
  
  
  for ( c= 0; c < amodel->N; ++c )
    for ( q= 0; q < amodel->hmms[c].Q; ++q )
      for ( t= 0; t < length; ++t )
        eprobs->p[c][q][t].active= FALSE;
  
} /* end tL_reset_eprobs */


/* Sols si és necessari. */
void
tL_resize_eprobs (
		  eprobs_t       *eprobs,
		  const tLAModel *amodel,
		  const int       length
		  )
{
  
  size_t new_size, old_size, i;
  unsigned int q, c;
  struct eprob *aux;
  
  
  new_size= amodel->total_Q * length;
  old_size= amodel->total_Q * eprobs->length;
  
  mt ( eprobs->p[0][0]= mem_realloc ( struct eprob,
				      eprobs->p[0][0],
				      new_size ) );
  for ( i= old_size, aux= eprobs->p[0][0];
	i < new_size; ++i )
    aux[i].active= FALSE;
  
  for ( c= 0, aux= eprobs->p[0][0];
	c < amodel->N; ++c )
    for ( q= 0; q < amodel->hmms[c].Q;
	  ++q, aux+= length )
      eprobs->p[c][q]= aux;
  
  eprobs->length= length;
  
} /* end tL_resize_eprobs */
