/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  eprobst.h - Implementació de 'eprobst.h'.
 *
 *  Adrià Giménez Pastor, 2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "eprobst.h"
#include "error.h"
#include "mem.h"




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

eprobst_t *
tL_create_eprobst (
		   const tLAModel  *amodel,
		   const tLBool     viterbi_states,
		   char           **err
		   )
{
  
  eprobst_t *new;
  unsigned int n, N, q, Q;
  eprobst_node_t **nodes, *mem;
  const tLAModelHMM *chars;
  
  
  mt ( new= mem_alloc ( eprobst_t, 1 ) );
  new->cep= create_calcelprob ( amodel,
				CEP_FRAMECACHE | (viterbi_states?CEP_MAX:0),
				err );
  if ( new->cep == NULL )
    {
      free ( new );
      return NULL;
    }
  N= amodel->N;
  mt ( new->nodes= nodes= mem_alloc ( eprobst_node_t *, N ) );
  mt ( nodes[0]= mem= mem_alloc ( eprobst_node_t, amodel->total_Q ) );
  mt ( new->stack= mem_alloc ( eprobst_node_t *, amodel->total_Q ) );
  chars= amodel->hmms;
  for ( n= 0; n < N; ++n )
    {
      nodes[n]= mem;
      Q= chars[n].Q;
      for ( q= 0; q < Q; ++q ) mem[q].a= FALSE;
      mem+= Q;
    }
  new->N= 0;
  
  return new;
  
} /* end tL_create_eprobst */


tLProb
tL_eprobst_lprob (
		  eprobst_t          *eprobst,
		  const tLAModel     *amodel,
		  const tLFea        *fea,
		  const unsigned int  c,
		  const unsigned int  q,
		  const int           t
		  )
{
  
  eprobst_node_t *nodes_c, *node;
  
  
  nodes_c= eprobst->nodes[c];
  if ( nodes_c[q].a ) return nodes_c[q].p;
  node= &(nodes_c[q]);
  eprobst->stack[eprobst->N++]= node;
  node->a= TRUE;
  
  return (node->p= calc_elprob ( eprobst->cep, amodel,
				 amodel->hmms[c].states,
				 q, fea, t ));
  
} /* end tL_eprobst_lprob */


void
tL_free_eprobst (
		 eprobst_t *eprobst
		 )
{
  
  free ( eprobst->nodes[0] );
  free ( eprobst->nodes );
  free ( eprobst->stack );
  free_calcelprob ( eprobst->cep );
  free ( eprobst );
  
} /* end tL_free_eprobst */


void
tL_reset_eprobst (
		  eprobst_t *eprobst
		  )
{
  
  eprobst_node_t **stack;
  size_t n, N;
  
  
  stack= eprobst->stack;
  N= eprobst->N;
  for ( n= 0; n < N; ++n )
    stack[n]->a= FALSE;
  eprobst->N= 0;
  
} /* end tL_reset_eprobst */
