/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  error.c - Implementació de 'error.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */


#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include "error.h"
#include "mem.h"




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
print_msg (
	   const char *type,
	   const char *format,
	   va_list     ap
	   )
{
  
  fprintf ( stderr, "%s: ", type );
  vfprintf ( stderr, format, ap );
  fprintf ( stderr, "\n" );
  
} /* end verror */


/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(void)
tL_error (
	  const char *format,
	  ...
	  )
{

  va_list ap;


  va_start ( ap, format );
  print_msg ( "[EE] ", format, ap );
  va_end ( ap );

  exit ( EXIT_FAILURE );

} /* end tL_error */


void
tL_msgerror (
	     char       **err,
	     const char  *format,
	     ...
	     )
{
  
  va_list ap;
  static char buffer[1000];
  
  
  if  ( err == NULL ) return;
  va_start ( ap, format );
  vsnprintf ( buffer, 1000, format, ap );
  va_end ( ap );
  mt ( *err= mem_alloc ( char, strlen(buffer)+1 ) );
  strcpy ( *err, buffer );
  
} /* end tL_msgerror */


TL_API_FUNC(void)
tL_warning (
	    const char *format,
	    ...
	    )
{

  va_list ap;


  va_start ( ap, format );
  print_msg ( "[WW] ", format, ap );
  va_end ( ap );

} /* end tL_warning */
