/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  error.h - Per a gestionar els errors i avisos.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */

#ifndef __TL_PRV_ERROR_H__
#define __TL_PRV_ERROR_H__

#include <errno.h>
#include <stdarg.h>
#include <stddef.h>
#include <string.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "error_public.h"


/* MACROS */

/* Error de la llibreria C. */
#define cerror()                     \
  tL_error ( (errno != 0) ?	     \
	     strerror ( errno ) :    \
	     "'errno' is 0" )

#define tLassert(FUNC,COND) \
  if ( COND ) tL_error ( #FUNC " : " #COND )

/* Test de memòria. */
#ifdef TL_MT_DETAIL
#define mt(ptr) (ptr)
#define fwritee(ptr,size,nmemb,fd)					\
  do {									\
    if ( fwrite(ptr,size,nmemb,fd) != nmemb )				\
      tL_error ( "'" __FILE__						\
		 ":%d: " # ptr _( "' - unexpected error during writing" ) ,	\
		 __LINE__ );						\
  } while(0)
#else
#define mt(ptr) (ptr)
#define fwritee(ptr,size,nmemb,fd)			\
  do {							\
    if ( fwrite(ptr,size,nmemb,fd) != nmemb )		\
      tL_error ( _( "unexpected error during writing" ) );	\
  } while(0)
#endif


#define TLMSG_TOKID(ERR) tL_msgerror ( ERR, _( "Token identifier cannot be read" ) )
#define TLMSG_UNKTOKID(ERR,ID)					\
  tL_msgerror ( ERR, _( "Unknown token identifier '%s'" ) , ID )
#define TLMSG_ORFILE(ERR,NAME)					\
  tL_msgerror ( ERR, _( "File '%s' cannot be opened" ) , (NAME) )
#define TLMSG_TYPE(ERR,TYPE)					\
  tL_msgerror ( ERR, _( "Type %s is not supported" ) , (TYPE) )
#define TLMSG_GTOK(ERR,TOK)					\
  tL_msgerror ( ERR, _( "Unable to read the token %s") , TOK )
#define TLMSG_GDESC(ERR,DESC)				\
  tL_msgerror ( ERR, _( "Unable to read %s" ) , DESC )


/* FUNCIONS */

/* Crea un missatge d'error si la variable d'error no és NULL, si és
   NULL no fa res.*/
void
tL_msgerror (
	     char       **err,
	     const char  *format,
	     ...
	     );

#endif /* __TL_PRV_ERROR_H__ */
