/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2003,2004,2005,2006,2007,2008,2009,2010,2011 Jesús Andrés Ferrer
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/***************************************************************************/
/*  FFTStockham: Calcula x=fft(x) según el alg Stocham Transpuesto         */
/*    Calcula los pesos sin usar un wlong j y  necesita espacio            */
/* adicional.                                                              */
/*    Tiene un stride de 1.                                                */
/*                                                                         */
/*  AUTOR: Jesús Andrés Ferrer                                             */
/*  Fecha Creación: 11-11-2003                                             */
/*  Fecha de Ultima modificacion:29-04-2013                                */
/* Modificaciones:                                                         */
/*  <dd-mm-yyyy> ->                                                        */
/***************************************************************************/

#define M_PI 3.14159265359

#include <math.h>
#include <string.h>

void tL_FFTStockham(double *xr,double *xi,unsigned long n)
{
  unsigned long L,r,r2,L2,i;
  unsigned long j,k;
  double tr,ti,wr,wi,theta;
  double factor;
  double yr[n],yi[n];
  
  for(L=2;L<=n;L=L<<1)
    {
      //we compute x=(I_r\kron B_L)*y
      // y=(\Pi_{r*}\kron I_{L*})*x^(q-1)
      L2=L/2; r=n/L; r2=2*r;
      // y<- x
      memcpy(yr,xr,(n)*sizeof(double));
      memcpy(yi,xi,(n)*sizeof(double));
      factor=(2*M_PI)/L;
      for(j=0;j<L2;++j)
	{
	  theta=j*factor;
	  wr=cos(theta);    wi=-sin(theta);
	  
	  for(k=0;k<r;++k)
	    {
	      
	      //t=w*(x(k*L+L_+j));
	      tr=wr*yr[j*r2+r+k]-wi*yi[j*r2+r+k];
	      ti=wr*yi[j*r2+r+k]+wi*yr[j*r2+r+k];
	      
	      xr[(j+L2)*r+k]=yr[j*r2+k]-tr;
	      xi[(j+L2)*r+k]=yi[j*r2+k]-ti;
	      
	      xr[j*r+k]=yr[j*r2+k]+tr;
	      xi[j*r+k]=yi[j*r2+k]+ti;
	    }
	}
    }
} 
