/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  filelist.c - Implementació de 'filelist.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

#include "error.h"
#include "filelist.h"
#include "mem.h"
#include "text.h"




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(tLFileList *)
tL_filelist_new_from_file (
			   const char *file_name
			   )
{
  
  tLFileList *new;
  gzFile f;
  tLBuffer *buffer;
  size_t size, n;
  
  
  /* Inicialitza. */
  mt ( new= mem_alloc ( tLFileList, 1 ) );
  mt ( new->files= mem_alloc ( char *, 1 ) );
  size= 1;
  n= 0;
  
  
  /* Llig. */
  f= gzopen ( file_name, "r" );
  if ( f == NULL )
    {
      free ( new->files );
      free ( new );
      return NULL;
    }
  
  buffer= tL_buffer_new ();
  while ( tL_gline ( f, buffer ) )
    {
      if ( n == size )
	{
	  size<<= 1;
	  mt ( new->files= mem_realloc ( char *, new->files,
					 size ) );
	}
      mt ( new->files[n]= mem_alloc ( char,
				      strlen ( buffer->v ) + 1 ) );
      strcpy ( new->files[n], buffer->v );
      ++n;
    }
  tL_buffer_free ( buffer );
  
  gzclose ( f );
  
  /* Reajusta. */
  new->N= n;
  if ( size > n && n > 0 )
    mt ( new->files= mem_realloc ( char *, new->files, n ) );
  
  return new;
  
} /* end tL_filelist_new_from_file */


TL_API_FUNC(void)
tL_filelist_free (
		  tLFileList *filelist
		  )
{
  
  size_t n;
  
  
  for ( n= 0; n < filelist->N; ++n )
    free ( filelist->files[n] );
  free ( filelist->files );
  free ( filelist );
  
} /* ebd tL_filelist_free */
