/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  filelist.h - Definició de una estructura per a tractar
 *               amb fitxers de llistes de fitxers.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */

#ifndef __LIBTL_FILELIST_H__
#define __LIBTL_FILELIST_H__

#include <stddef.h>

#include "port.h"

/** \addtogroup Basic
 *  @{
 */

/* DEFINCIÓ */

/** List of file names. This structure is used to store a list of file
 *  names, usually containing samples or transcriptions.
 */
typedef struct
{
  
  char   **files;    /**< Array containing the filenames. */
  size_t   N;        /**< Number of filenames. */
  
} tLFileList;


/* FUNCIONS */

/** Creates a new file list from a text file. This function reads the
 *  content of a text file, where each line is the name of a file, and
 *  loads it into a new file list.
 *
 *  @param file_name Name of the file conatining the file list.
 *  @return The new file list, or NULL if the file cannot be
 *  opened. In this case, errno is set by the lib C.
 */
TL_API_FUNC(tLFileList *)
tL_filelist_new_from_file (
			   const char *file_name
			   );

/** Frees memory. Frees the memory allocated for the file list.
 */
TL_API_FUNC(void)
tL_filelist_free (
		  tLFileList *filelist
		  );

/** @} */

#endif /* __LIBTL_FILELIST_H__ */
