/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 1996,2008,2009,2010,2011 Bob Jenkins, Alfons Juan
 *  Císcar, Jorge Civera Saiz and Adrià Giménez Pastor.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 * hash_jc is based on the lookup3.c.
 * lookup3.c, by Bob Jenkins, May 2006, Public Domain.
 * http://burtleburtle.net/bob/c/lookup3.c
 */
/*
 *  hash.c - Implementació de 'hash.h'.
 *
 *  Adrià Giménez Pastor, 2009,2011.
 *
 */


#include "hash.h"




/**********/
/* MACROS */
/**********/

#define NPRIMES 24


#ifdef __TL_HASH_JC_64
#define mix_jc(a,b,c)          \
{                              \
  a=a-b;  a=a-c;  a=a^(c>>43); \
  b=b-c;  b=b-a;  b=b^(a<<9);  \
  c=c-a;  c=c-b;  c=c^(b>>8);  \
  a=a-b;  a=a-c;  a=a^(c>>38); \
  b=b-c;  b=b-a;  b=b^(a<<23); \
  c=c-a;  c=c-b;  c=c^(b>>5);  \
  a=a-b;  a=a-c;  a=a^(c>>35); \
  b=b-c;  b=b-a;  b=b^(a<<49); \
  c=c-a;  c=c-b;  c=c^(b>>11); \
  a=a-b;  a=a-c;  a=a^(c>>12); \
  b=b-c;  b=b-a;  b=b^(a<<18); \
  c=c-a;  c=c-b;  c=c^(b>>22); \
}
#endif

#ifdef __TL_HASH_JC_32
#define mix_jc(a,b,c)          \
{                              \
  a=a-b;  a=a-c;  a=a^(c>>13); \
  b=b-c;  b=b-a;  b=b^(a<<8);  \
  c=c-a;  c=c-b;  c=c^(b>>13); \
  a=a-b;  a=a-c;  a=a^(c>>12); \
  b=b-c;  b=b-a;  b=b^(a<<16); \
  c=c-a;  c=c-b;  c=c^(b>>5);  \
  a=a-b;  a=a-c;  a=a^(c>>3);  \
  b=b-c;  b=b-a;  b=b^(a<<10); \
  c=c-a;  c=c-b;  c=c^(b>>15); \
}
#endif




/*********/
/* TIPUS */
/*********/

typedef unsigned char u1;
typedef unsigned long ui;




/*************/
/* CONSTANTS */
/*************/

static const unsigned long _primes[NPRIMES]=
  {
    3UL,
    5UL,
    11UL,
    17UL,
    37UL,
    67UL,
    131UL,
    257UL,
    521UL,
    1031UL,
    2053UL,
    4099UL,
    8209UL,
    16411UL,
    32771UL,
    65537UL,
    131101UL,
    262147UL,
    524309UL,
    1048583UL,
    2097169UL,
    4194319UL,
    8388617UL,
    16777259UL
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

unsigned long
tL_hash_primesize (
		   const unsigned long size
		   )
{
  
  int l, r, m;
  
  
  if ( size <= _primes[0] ) return _primes[0];
  if ( size >= _primes[NPRIMES-1] ) return _primes[NPRIMES-1];
  
  /* NOTA: En el cas r=l+1, es a dir m=r, el número buscat o és l o és
     m, però és segur un d'eixos dos. */
  l= 1; r= NPRIMES-1;
  while ( l != r )
    {
      m= l+((r-l+1)>>1);
      if ( _primes[m] >= size )
	{
	  if ( _primes[m-1] < size )
	    return _primes[m];
	  else r= m-1;
	}
      else l= m+1;
    }
  
  return _primes[l];
  
} /* tL_hash_primesize */


unsigned long
tL_hash_aj (
	    const void          *key,
	    const unsigned long  length
	    )
{
  
  unsigned long ret, i;
  const char *p;
  
  
  for ( ret= 0, p= (char *) key, i= 0;
	i < length; ++i, ++p )
    ret= (ret<<2) + *p;
  
  return ret;
  
} /* end tL_hash_aj */


unsigned long
tL_hash_str_aj (
		const char *key
		)
{
  
  unsigned long ret;
  
  
  for ( ret= 0; *key; ++key )
    ret= (ret<<2) + *key;
  
  return ret;
  
} /* end tL_hash_str_aj */


unsigned long
tL_hash_gnu_cxx (
		 const void          *key,
		 const unsigned long  length
		 )
{
 
  unsigned long ret, i;
  const char *p;
  
  
  for ( ret= 0, p= (char *) key, i= 0;
	i < length; ++i, ++p )
    ret= 5*ret + *p;
  
  return ret;
  
} /* end tL_hash_gnu_cxx */


unsigned long
tL_hash_str_gnu_cxx (
		     const char *key
		     )
{
    
  unsigned long ret;
  
  
  for ( ret= 0; *key; ++key )
    ret= 5*ret + *key;
  
  return ret;
  
} /* end tL_hash_str_gnu_cxx */


unsigned long
tL_hash_jc (
	    const void          *key,
	    const unsigned long  length,
	    const unsigned long  level
	    )
{
  
  register ui a,b,c;  /* the internal state */
  register const u1 *k;
  ui          len;    /* how many key bytes still need mixing */
  
  
  k= (const u1 *) key;
  
  /* Set up the internal state */
   len = length;
   a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
   c = level;           /* variable initialization of internal state */

   /*---------------------------------------- handle most of the key */
   while (len >= 12)
     {
       a=a+(k[0]+((ui)k[1]<<8)+((ui)k[2]<<16) +((ui)k[3]<<24));
       b=b+(k[4]+((ui)k[5]<<8)+((ui)k[6]<<16) +((ui)k[7]<<24));
       c=c+(k[8]+((ui)k[9]<<8)+((ui)k[10]<<16)+((ui)k[11]<<24));
       mix_jc(a,b,c);
       k = k+12; len = len-12;
     }
   
   /*------------------------------------- handle the last 11 bytes */
   c = c+length;
   switch(len)              /* all the case statements fall through */
   {
   case 11: c=c+((ui)k[10]<<24);
   case 10: c=c+((ui)k[9]<<16);
   case 9 : c=c+((ui)k[8]<<8);
     /* the first byte of c is reserved for the length */
   case 8 : b=b+((ui)k[7]<<24);
   case 7 : b=b+((ui)k[6]<<16);
   case 6 : b=b+((ui)k[5]<<8);
   case 5 : b=b+k[4];
   case 4 : a=a+((ui)k[3]<<24);
   case 3 : a=a+((ui)k[2]<<16);
   case 2 : a=a+((ui)k[1]<<8);
   case 1 : a=a+k[0];
     /* case 0: nothing left to add */
   }
   mix_jc(a,b,c);
   /*-------------------------------------------- report the result */
   return c;
   
} /* end tL_has_jc */
