/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Alfons Juan Císcar, Jorge
 *  Civera Saiz and Adrià Giménez Pastor
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 * hash_jc is based on the lookup3.c.
 * lookup3.c, by Bob Jenkins, May 2006, Public Domain.
 * http://burtleburtle.net/bob/c/lookup3.c
 */
/*
 *  hash.h - Col·lecció de funcions 'hash'.
 *
 *  Adrià Giménez Pastor, 2009,2011.
 *
 */

#ifndef __TL_PRV_HASH_H__
#define __TL_PRV_HASH_H__

#include <limits.h>

#if ULONG_MAX == 18446744073709551615UL
#define __TL_HASH_JC_64
#elif ULONG_MAX == 4294967295UL
#define __TL_HASH_JC_32
#else
#error unsigned long size not supported
#endif

/* Aquesta funció torna de manera eficient el número prim més xicotet
 * que és superior a SIZE. Si no és possible torna el més
 * gran. Actualment el valor més xicotet és 3 i el més gran
 * 16777259. Els possibles números creixen amb la tendència de les
 * potències de 2.
 */
unsigned long
tL_hash_primesize (
		   const unsigned long size
		   );

/* Totes les funcions presentades tornen una clau en un 'unsigned
 * long' sense preocupar-se del valor màxim. Quan se treballa amb
 * punters 'void' la llargària fa referència al nombre de bytes.
 */

/* Funció basada en la funció 'hash' del codi d'Alfons Juan
 * Císcar. Les grandàries que sol emprar són 101U 1001U 32363U. Per a
 * traure la clau aplica l'operador mòdul.
 */
unsigned long
tL_hash_aj (
	    const void          *key,
	    const unsigned long  length
	    );

unsigned long
tL_hash_str_aj (
		const char *key
		);

/* Funció basada en la funció 'hash' de les extensions GNU per a
 * C++.
 */
unsigned long
tL_hash_gnu_cxx (
		 const void          *key,
		 const unsigned long  length
		 );

unsigned long
tL_hash_str_gnu_cxx (
		     const char *key
		     );

/* Funció que utilitza Jorge Civera Saiz i que es pot trobar en
 * http://burtleburtle.net/bob/hash/evahash.html. Jorge sol gastar
 * aquesta funció amb grandàries que són potència de 2 i aplicant una
 * màscara binària.
 */
unsigned long
tL_hash_jc (
	    const void          *key,
	    const unsigned long  length,
	    const unsigned long  level      /* Un valor arbitrari, o l'anterior
					       clau. Per exemple 23312819. */
	    );

#endif /* __TL_PRV_HASH_H__ */
