/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  hypmng.c - Implementació de 'hypmng.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "error.h"
#include "hypmng.h"
#include "mem.h"




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static size_t
length_hyp (
	    hypmng_t *hypmng,
	    size_t    hyp
	    )
{
  
  size_t length, p;
  
  
  for ( p= hyp, length= 0; p != 0;
	p= hypmng->v[p].ptr, ++length );
  
  return length;
  
} /* end length_hyp */


static void
resize_hypmng (
	       hypmng_t *hypmng
	       )
{
  
  size_t olds, i;
  
  
  olds= hypmng->s;
  hypmng->s<<= 1;
  if ( hypmng->s < olds )
    tL_error ( _( "Overflow of 'hypmng->s'" ) );
  mt ( hypmng->v= mem_realloc ( struct hypmng_node,
				hypmng->v, hypmng->s ) );
  for ( i= olds; i < hypmng->s; ++i )
    {
      hypmng->v[i].counter= 0;
      hypmng->v[i].ptr= i+1;
    }
  
} /* end resize_hypmng */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

hypmng_t *
tL_create_hypmng ()
{
  
  hypmng_t *new;
  
  
  mt ( new= mem_alloc ( hypmng_t, 1 ) );
  mt ( new->v= mem_alloc ( struct hypmng_node, 2 ) );
  new->v[0].ptr= 0;
  new->v[0].counter= 0;
  new->v[1].ptr= 2;
  new->v[1].counter= 0;
  new->s= 2;
  new->stack= 1;
  
  return new;
  
} /* end tL_create_hypmng */


void
tL_free_hypmng (
		hypmng_t *hypmng
		)
{
  
  free ( hypmng->v );
  free ( hypmng );
  
} /* end tL_free_hypmng */


void
tL_free_hyp (
	     hypmng_t *hypmng,
	     size_t    hyp
	     )
{
  
  size_t p, parent;
  
  
  p= hyp;
  while ( p != 0 && hypmng->v[p].counter == 0 )
    {
      parent= hypmng->v[p].ptr;
      hypmng->v[p].ptr= hypmng->stack;
      hypmng->stack= p;
      --(hypmng->v[parent].counter);
      p= parent;
    }
  
} /* end tL_free_hyp */


size_t
tL_get_hyp (
	    hypmng_t          *hypmng,
	    size_t             hyp,
	    unsigned long int *buffer
	    )
{
  
  size_t length, i, p;
  
  
  length= length_hyp ( hypmng, hyp );
  for ( i= length, p= hyp; i > 0;
	p= hypmng->v[p].ptr )
    buffer[--i]= hypmng->v[p].value;
  
  return length;
  
} /* end tL_get_hyp */


size_t
tL_hypmng_add (
	       hypmng_t          *hypmng,
	       size_t             parent,
	       unsigned long int  value
	       )
{
  
  struct hypmng_node node;
  size_t new;
  
  
  node.value= value;
  node.ptr= parent;
  node.counter= 1;
  ++(hypmng->v[parent].counter);
  
  if ( hypmng->stack == hypmng->s )
    resize_hypmng ( hypmng );
  new= hypmng->stack;
  hypmng->stack= hypmng->v[new].ptr;
  hypmng->v[new]= node;
  
  return new;
  
} /* end tL_hypmng_add */
