/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  hypmanager.h - Un gestor d'hipòtesis de sencers. És eficient
 *                 si a partir d'una hipòtesis no anem a crear dos
 *                 iguals.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */

#ifndef __TL_PRV_HYPMANAGER_H__
#define __TL_PRV_HYPMANAGER_H__

#include <stddef.h>


/* DEFINICIÓ */

#define NULLHYP 0

struct hypmng_node
{
  
  unsigned long int value;
  size_t            ptr;
  size_t            counter;
  
};

typedef struct
{
  
  struct hypmng_node *v;
  size_t              s;
  size_t              stack;

} hypmng_t;


/* FUNCIONS */

hypmng_t *
tL_create_hypmng ();

void
tL_free_hypmng (
		hypmng_t *hypmng
		);

/* Elimina una hipòtesi, independentment de si està referenciada o no.
 */
void
tL_free_hyp (
	     hypmng_t *hypmng,
	     size_t    hyp
	     );

/* Escriu en 'buffer' l'hipòtesi 'hyp' i torna la longitud de
 * l'hipòtesi. El buffer ha de tindre espai suficient.
 */
size_t
tL_get_hyp (
	    hypmng_t          *hypmng,
	    size_t             hyp,
	    unsigned long int *buffer
	    );

/* Si no té pare el pare deu de ser NULLHYP. */
size_t
tL_hypmng_add (
	       hypmng_t          *hypmng,
	       size_t             parent,
	       unsigned long int  value
	       );

#define release_hyp(HYPMNG,HYP)					 \
  do {								 \
     if ( --((HYPMNG)->v[(HYP)].counter) == 0 && (HYP) != 0 )	 \
       tL_free_hyp ( (HYPMNG), (HYP) );                          \
   } while(0)

#define take_hyp(HYPMNG,HYP) ++((HYPMNG)->v[(HYP)].counter)


#endif /* __TL_PRV_HYPMANAGER_H__ */
