/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  init_amodel.c - Inicialitza el model de caràcters.
 *
 *  Adrià Giménez Pastor, 2008,2009,2010,2011,2012,2013.
 *
 */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define __TL_PRV
#include "iamcounts.h"
#include "common.h"
#include "create_nstates.h"
#include "error.h"
#include "init_hmm.h"
#include "init_amodel.h"
#include "mem.h"
#include "samples.h"
#include "text.h"




/*********/
/* TIPUS */
/*********/

struct _tLInitializer
{
  
  const tLAModel   *amodel;
  const tLSeqs     *seqs;
  size_t           *nstates;
  tLHMMInitializer *ic_aux;
  
};




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(void)
tL_initializer_free (
		     tLInitializer *ini
		     )
{
  
  free ( ini->nstates );
  tL_hmm_initializer_free ( ini->ic_aux );
  free ( ini );
  
} /* end tL_initializer_free */


TL_API_FUNC(tLInitializer *)
tL_initializer_new (
		    const tLAModel  *amodel,
		    const tLSeqs    *seqs,
		    char           **err
		    )
{
  
  tLInitializer *new;
  
  
  if ( amodel->tflags != FLINEAL )
    {
      tL_msgerror ( err, "tL_initializer_new is only implemented"
                    " for acoustic models with linear topologies" );
      return NULL;
    }
  mt ( new= mem_alloc ( tLInitializer, 1 ) );
  new->amodel= amodel;
  new->seqs= seqs;
  new->ic_aux= tL_hmm_initializer_new ( amodel, err );
  if ( new->ic_aux == NULL )
    {
      free ( new );
      return NULL;
    }
  new->nstates= tL_create_nstates ( seqs->cseqs, amodel );
  
  return new;
  
} /* end tL_initializer_new */


TL_API_FUNC(int)
tL_init (
	 tLInitializer *ini,
	 tLAMiCounts   *counts,
	 const tLFea   *fea,
	 const size_t   seq
	 )
{
  
  tLFloat frac, accum;
  int i, next, prev;
  tLSeq cs;
  
  
  if ( fea->cols < ini->nstates[seq] ) return (int) ini->nstates[seq];
  cs= *(tL_seqs_get_seq ( ini->seqs, seq ));
  frac= ((tLFloat) fea->cols) / ((tLFloat) cs.length);
  accum= 0.0;
  prev= 0;
  for ( i= 0; i < cs.length; ++i )
    {
      accum+= frac;
      next= (int) (accum + 0.5);
      tL_init_hmm_seg ( ini->ic_aux, counts, cs.v[i], fea, prev, next - prev );
      prev= next;
    }
  
  return 0;
  
} /* end tL_init */
