/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  bernoulli.c - Implementació de 'bernoulli.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#define __TL_PRV
#include "amcounts/ibernoulli.h"
#include "amodel/bernoulli.h"
#include "error.h"
#include "global.h"
#include "init_hmm.h"
#include "mem.h"



/**********/
/* MACROS */
/**********/

#define CSTATE(ptr) ((iamcount_bernoulli_state_t *) (ptr))
#define GLOBAL(ptr) BERNOULLI_GLOBAL(ptr)




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
tL_init_proto_bernoulli (
			 tLHMMInitializer   *init,
			 void               *global,
			 void               *states,
			 void               *cstates,
			 const unsigned int  q,
			 const tLFea        *fea,
			 const int           t
			 )
{
  
  iamcount_bernoulli_state_t scount;
  unsigned int d, D;
  const bit *frame;
  
  
  frame= B2cb(fea->v[t]);
  D= GLOBAL(global)->D;
  scount= CSTATE(cstates)[q];
  for ( d= 0; d < D; ++d )
    if ( frame[d] ) ++(scount.p[d].p1);
    else            ++(scount.p[d].p0);
  
} /* end tL_init_proto_bernoulli */


static const tLInitHMMAuxMeths meths=
  {
    tL_init_proto_bernoulli,
    tL_init_proto_free
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

tLHMMInitializer *
tL_create_init_hmm_aux_bernoulli (
				  const tLAModel *amodel
				  )
{
  
  tLHMMInitializer *new;
  
  mt ( new= mem_alloc ( tLHMMInitializer, 1 ) );
  new->amodel= amodel;
  new->meths= &meths;
  
  return new;
  
} /* end tL_create_init_hmm_aux_bernoulli */
