/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  dgaussian.c - Implementació de 'dgaussian.h'.
 *
 *  Adrià Giménez Pastor, 2009,2011,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "samples.h"
#include "amcounts/dgaussian.h"
#include "amodel/dgaussian.h"
#include "error.h"
#include "global.h"
#include "init_hmm.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define CSTATE(ptr) ((amcount_dgaussian_state_t *) (ptr))
#define GLOBAL(ptr) DGAUSSIAN_GLOBAL(ptr)




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
tL_init_proto_dgaussian (
			 tLHMMInitializer   *init,
			 void               *global,
			 void               *states,
			 void               *cstates,
			 const unsigned int  q,
			 const tLFea        *fea,
			 const int           t
			 )
{
  
  amcount_dgaussian_state_t scount;
  const float *frame;
  float aux;
  unsigned int d, D;
  
  
  frame= B2cR(fea->v[t]);
  scount= CSTATE(cstates)[q];
  D= GLOBAL(global)->D;
  for ( d= 0; d < D; ++d )
    {
      aux= frame[d];
      scount.x[d]+= aux;
      scount.x2[d]+= aux*aux;
    }
  CSTATE(cstates)[q].norm+= 1.0;
  
} /* end tL_init_proto_dgaussian */


static const tLInitHMMAuxMeths meths=
  {
    tL_init_proto_dgaussian,
    tL_init_proto_free
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

tLHMMInitializer *
tL_create_init_hmm_aux_dgaussian (
				  const tLAModel *amodel
				  )
{
  
  tLHMMInitializer *new;
  
  mt ( new= mem_alloc ( tLHMMInitializer, 1 ) );
  new->amodel= amodel;
  new->meths= &meths;
  
  return new;
  
} /* end tL_create_init_hmm_aux_dgaussian */
