/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  init_uniform_ci.c - Implementació de 'init_uniform_ci.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>
#include <string.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#define __TL_PRV
#include "error.h"
#include "iamcounts.h"
#include "init_uniform_ci.h"




/**********************/
/* TIPUS ENREGISTRATS */
/**********************/

typedef struct
{

  const char             *name;
  tLUniHMMInitializer * (*create) (const tLAModel *amodel);

} type_t;


#include "init_uniform_ci/bernoulli.h"
#include "init_uniform_ci/dgaussian.h"

static const type_t types[]=
  {
    { "Bernoulli", tL_create_init_uniform_hmm_aux_bernoulli },
    { "DGaussian", tL_create_init_uniform_hmm_aux_dgaussian },
    { NULL, NULL }
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(tLUniHMMInitializer *)
tL_uni_hmm_initializer_new (
			    const tLAModel  *amodel,
			    char           **err
			    )
{
  
  int i;

  
  if ( amodel->tflags != FLINEAL )
    {
      tL_msgerror ( err, _( "tL_uni_hmm_initializer_new is only implemented"
		    " for acoustic models with linear topologies" ) );
      return NULL;
    }
  for ( i= 0; types[i].name != NULL &&
	  strcmp ( amodel->type, types[i].name );
        ++i );
  if ( types[i].name == NULL )
    {
      tL_msgerror ( err, _( "Unknown type %s" ) , amodel->type );
      return NULL;
    }
  
  return types[i].create ( amodel );
  
} /* end tL_uni_hmm_initializer_new */


TL_API_FUNC(void)
tL_init_uniform_hmm (
		     tLUniHMMInitializer *uci,
		     tLAMiCounts         *counts,
		     const int            c
		     )
{
  
  tLAModelHMM cchar;
  iamcount_hmm_t ccount;
  unsigned int q;
  
  
  cchar= uci->amodel->hmms[c];
  ccount= counts->hmms[c];
  uci->meths->init ( uci, uci->amodel->global, cchar.states,
		     ccount.states, cchar.Q );
  for ( q= 0; q < cchar.Q; ++q )
    {
      ccount.trans.lineal.p[q].p1= 1;
      ccount.trans.lineal.p[q].p0= 1;
    }
  
} /* end tL_init_uniform_hmm */
