/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  init_uniform_ci.h - Inicialitza els comptadors sencers del
 *                      caràcter indicat unifórmement.
 *
 *  Adrià Giménez Pastor, 2008,2011,2013.
 *
 */

#ifndef __LIBTL_INIT_UNIFORM_CI_H__
#define __LIBTL_INIT_UNIFORM_CI_H__

#include "amcounts_public.h"
#include "amodel_public.h"

/** \addtogroup Algs
 *  @{
 */
/** \defgroup InitUniformHMM Uniform initialization of HMMs
 */
/** \addtogroup InitUniformHMM
 *  @{
 */

/** Used in 'tL_init_uniform_hmm'. This structure contains auxiliar
 *  data used by 'tL_init_uniform_hmm' to initialize uniformly an HMM.
 */
typedef struct _tLUniHMMInitializer tLUniHMMInitializer;

typedef struct
{
  
  void (*init) (struct _tLUniHMMInitializer *uci,
		void                        *global,
		void                        *states,
		void                        *cstates,
		const unsigned int           Q);
  void (*free) (struct _tLUniHMMInitializer *uci);
  
} tLUniHMMInitializerMeths;

#define TL_INIT_UNIFORM_HMM_AUX						\
  const tLAModel                  *amodel;    /**< Reference to the	\
						 acoustic model used	\
						 by the uniform		\
						 HMM			\
						 initializer. */	\
  const tLUniHMMInitializerMeths *meths

struct _tLUniHMMInitializer
{
  TL_INIT_UNIFORM_HMM_AUX;
};

/** Frees memory. Frees the memory allocated for the HMM
 *  initializer.
 *
 *  @param UCI The uniform HMM initializer.
 */
#define tL_uni_hmm_initializer_free(UCI)	\
  (UCI)->meths->free ( UCI )

/* Torna NULL en cas d'error. */
/** Gets a tLUniHMMInitializer. This function creates auxiliary data
 *  used to uniformly initialize HMMs. Currently supported
 *  types are: Bernoulli and DGaussian.
 *
 *  @param amodel The acoustic model.
 *  @param err Pointer to string variable. If not NULL, an error message
 *  is allocated in the variable in case of error.
 *  @return Uniform HMM initializer data, or NULL in case of
 *  error.
 */
TL_API_FUNC(tLUniHMMInitializer *)
tL_uni_hmm_initializer_new (
			    const tLAModel  *amodel,
			    char           **err
			    );

/** Initializes uniformly an HMM. This function initializes
 *  the counts of the specified HMM in order to obtain a
 *  uniformly distributed model.
 *
 *  @param uci The uniform HMM initializer.
 *  @param counts Integer counts related to amodel.
 *  @param c The index of the HMM to be initialized.
 */
TL_API_FUNC(void)
tL_init_uniform_hmm (
		     tLUniHMMInitializer *uci,
		     tLAMiCounts         *counts,
		     const int            c
		     );

/** @} */
/** @} */

#endif /* __LIBTL_INIT_UNIFORM_CI_H__ */
