/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  bernoulli.c - Implementació de 'bernoulli.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#define __TL_PRV
#include "amcounts/ibernoulli.h"
#include "amodel/bernoulli.h"
#include "error.h"
#include "global.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) BERNOULLI_GLOBAL(ptr)
#define CSTATE(ptr) ((iamcount_bernoulli_state_t *) (ptr))




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
tL_init_uniform_states_ci_bernoulli (
				     tLUniHMMInitializer *aux,
				     void                *global,
				     void                *states,
				     void                *cstates,
				     const unsigned int   Q
				     )
{
  
  unsigned int q, d, D;
  iamcount_bernoulli_state_t scount;
  
  
  D= GLOBAL(global)->D;
  for ( q= 0; q < Q; ++q)
    {
      scount= CSTATE(cstates)[q];
      for ( d= 0; d < D; ++d )
	{
	  scount.p[d].p1= 1;
	  scount.p[d].p0= 1;
	}
    }
  
} /* end tL_init_uniform_states_ci_bernoulli */


static const tLUniHMMInitializerMeths meths=
  {
    tL_init_uniform_states_ci_bernoulli,
    tL_init_uniform_ci_free
  };



/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

tLUniHMMInitializer *
tL_create_init_uniform_hmm_aux_bernoulli (
					  const tLAModel *amodel
					  )
{
  
  tLUniHMMInitializer *new;
  
  
  mt ( new= mem_alloc ( tLUniHMMInitializer, 1 ) );
  new->amodel= amodel;
  new->meths= &meths;
  
  return new;
  
} /* end tL_create_init_uniform_hmm_aux_bernoulli */
