/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  dgaussian.c - Implementació de 'dgaussian.h'.
 *
 *  Adrià Giménez Pastor, 2009,2011,2012,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "amcounts/dgaussian.h"
#include "amodel/dgaussian.h"
#include "dgaussian.h"
#include "error.h"
#include "global.h"
#include "mem.h"




/**********/
/* MACROS */
/**********/

#define GLOBAL(ptr) DGAUSSIAN_GLOBAL(ptr)
#define CSTATE(ptr) ((amcount_dgaussian_state_t *) (ptr))




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
tL_init_uniform_states_ci_dgaussian (
				     tLUniHMMInitializer *aux,
				     void                *global,
				     void                *states,
				     void                *cstates,
				     const unsigned int   Q
				     )
{
  
  unsigned int q, d, D;
  amcount_dgaussian_state_t scount;
  const tLFloat *smooth;
  
  
  /* Fique les variances a smooth i les mitges a 0. */
  D= GLOBAL(global)->D;
  smooth= GLOBAL(global)->smooth;
  for ( q= 0; q < Q; ++q )
    {
      scount= CSTATE(cstates)[q];
      for ( d= 0; d < D; ++d )
	{
	  scount.x[d]= 0.0;
	  scount.x2[d]= smooth[d];
	}
      CSTATE(cstates)[q].norm= 1.0;
    }
  
} /* end tL_init_uniform_states_ci_dgaussian */


static const tLUniHMMInitializerMeths meths=
  {
    tL_init_uniform_states_ci_dgaussian,
    tL_init_uniform_ci_free
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

tLUniHMMInitializer *
tL_create_init_uniform_hmm_aux_dgaussian (
					  const tLAModel *amodel
					  )
{
  
  tLUniHMMInitializer *new;
  
  
  mt ( new= mem_alloc ( tLUniHMMInitializer, 1 ) );
  new->amodel= amodel;
  new->meths= &meths;
  
  return new;
  
} /* end tL_create_init_uniform_hmm_aux_dgaussian */
