/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  linabmat.h - Estructura que representa una matriu per a
 *               Forward-Backward utilitzada amb models lineals. Està
 *               pensada per a quan es sap a priori el número màxim de
 *               caràcters que té el model. Inclou també el càlcul del
 *               forward-backward. Està ací perquè s'utilitza en
 *               diferents llocs.
 *
 *  Adrià Giménez Pastor, 2010,2011,2012.
 *
 */

#ifndef __TL_PRV_LINABMAT_H__
#define __TL_PRV_LINABMAT_H__

#include <stddef.h>

#include "amodel.h"
#include "common.h"
#include "samples.h"

typedef struct
{
  
  tLFloat ***a; /* |C|*|Q|*|T|. */
  tLFloat   *b; /* |T|. */
  int        max_width;
  tLFloat   *calc0;
  tLFloat   *calc1;
  
} linabmat_t;

void
tL_linabmat_free (
		  linabmat_t *m
		  );

linabmat_t *
tL_linabmat_new (
		 const tLAModel *amodel,
		 unsigned int    max_length
		 );

/* Sempre fa el resize. Per tant fer al comprovació de si val la pena
   abans de cridar-la. */
void
tL_linabmat_resize (
		    linabmat_t   *m,
		    unsigned int  max_length,
		    unsigned int  max_Q,
		    int           width
		    );

/* FORWARD-BACKWARD */
typedef struct
{
  
  /* Torna el número de símbols de la seqüència. */
  size_t (*get_length) (void *udata);
  
  /* Fixa el símbol actual i torna un punter al model associat. */
  const tLAModelHMM * (*a_set_sym) (const size_t  sym,
				     void         *udata);
  
  /* Torna la probabilitat d'emissió. */
  tLFloat (*a_get_prob) (const tLFea        *fea,
			 const int           t,
			 void               *states,
			 const unsigned int  q,
			 void               *udata);

  /* Com a_set_sym pero en beta. */
  const tLAModelHMM * (*b_set_sym) (const size_t  sym,
				     void         *udata);
  
  /* Torna els estadístics d'un estat. */
  void * (*b_get_fstats) (const unsigned int  q,
			  void               *udata);
  
  /* Probabilitat en un instant donat. */
  tLFloat (*b_get_prob) (void      *fstats,
			 const int  t);
  
  /* Acumula pes. */
  void (*b_accum_weight) (void          *fstats,
			  const int      t,
			  const tLFloat  weight,
			  void          *udata);
  
  /* Acumula pesos transicions. */
  void (*b_accum_trans) (const tLFloat  count0,
			 const tLFloat  count1,
			 unsigned int   q,
			 void          *udata);
  
} linfb_meths_t;

tLFloat
tL_calc_alpha_lin (
		   linabmat_t          *m,
		   const linfb_meths_t *fb_meths,
		   const tLFea         *fea,
		   int                  diff, /* fea->cols - nstates + 1 */
		   const tLFloat        prior,
		   void                *udata
		   );

void
tL_calc_beta_lin (
		  linabmat_t          *m,
		  const linfb_meths_t *fb_meths,
		  const int            T,
		  const int            diff,
		  void                *udata
		  );

#endif /* __TL_PRV_LINABMAT_H__ */
