/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  lm.h - Model de llenguatge.
 *
 *  Adrià Giménez Pastor, 2008,2009,2011.
 *
 */

#ifndef __TL_PRV_LM_H__
#define __TL_PRV_LM_H__

#include <limits.h>

#include "lexicon.h"
#include "lm_public.h"
#include "text.h"

/* FUNCIONS */

/* Donat un estat i una paraula torna l'estat destí i la
 * probabilitat. Sols disponible per a models amb backoff.
 */
void
tL_lm_bo_get_trans (
		    const tLLM          *lm,
		    const unsigned long  from,
		    const unsigned long  word,
		    unsigned long       *to,
		    tLProb              *prob
		    );

/* Torna la probabilitat final d'un estat. */
tLProb
tL_lm_get_fp (
	      const tLLM    *lm,
	      unsigned long  state
	      );

/* Crea un iterador per a aquest model de llenguatge. */
tLLMIterator
tL_lm_get_iterator (
		    const tLLM *lm
		    );

/* Torna el nombre d'estats nuls del model de llenguatge. */
size_t
tL_lm_nnstates (
		const tLLM *lm
		);


/* ITERATOR */

typedef struct
{
  
  void (*free) (tLLMIterator);
  void (*set) (tLLMIterator,
	       size_t state);
  tLBool (*next) (tLLMIterator,
		  const tLLMEdge **begin,
		  const tLLMEdge **end);
  
} lm_iterator_meths_t;

#define LM_ITERATOR_CONTENT         \
  const tLLM                *lm;    \
  const lm_iterator_meths_t *meths;

/* Iterador per a recorrer les arestes no nul·les d'un model de
 * llenguatge.
 */
struct _tLLMIterator
{
  LM_ITERATOR_CONTENT
};

/* Allibera un iterador. */
#define free_lm_iterator(I) (I)->meths->free ( (I) )

/* Prepara l'iterador per a iterar sobre les arestes de l'estat
 * indicat.
 */
#define lm_iterator_set(I,STATE) (I)->meths->set ( (I), (STATE) )

/* Siguen BEGIN i END dos punter a punter d'arestes, assigna el
 * seguënt bloc d'arestes que ix de l'estat. Torna cert mentres queden
 * arestes en eixe estat. Pot ser que torne blocs buits.
 */
#define lm_iterator_next(I,BEGIN,END)        \
  (I)->meths->next ( (I), BEGIN, END )


#endif /* __TL_PRV_LM_H__ */
