/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  logamcounts.c - Implementació de 'logamcounts.h'.
 *
 *  Adrià Giménez Pastor, 2008,2011,2012,2013.
 *
 */

#include "logamcounts.h"

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#define AMC_T tLFloat
#define AMC_0 -HUGE_VAL
#define AMC_P(name) log ## name
#define AMC_C
#define AMC_LOG
#include "amcounts.h"
#undef AMC_LOG
#undef AMC_C
#undef AMC_P
#undef AMC_0
#undef AMC_T

TL_API_FUNC(tLAMlogCounts *)
tL_amlogcounts_new_cmllr ( 
			  const tLAModel *amodel
			   )
{
  return tL_amlogcounts_new__ ( amodel, CMLLR );
} /* end tL_amlogcounts_new_cmllr */


TL_API_FUNC(tLAMlogCounts *)
tL_amlogcounts_new_mllr ( 
			 const tLAModel *amodel
			  )
{
  return tL_amlogcounts_new__ ( amodel, MLLR );
} /* end tL_amlogcounts_new_mllr */


TL_API_FUNC(int)
tL_amlogcounts_accum_eps (
			  const tLAMlogCounts  *amcounts,
			  const tLAModel       *amodel,
			  gzFile                from,
			  const tLFloat         eps,
			  const tLBool          binary,
			  char                **err
			  )
{
  
  logamcount_hmm_t ccount;
  tLAModelHMM cchar;
  unsigned int n, q;
  tLBuffer *buffer;
  unsigned long counter, ed;
  tLFloat logeps, logeps_1;
  
  
  logeps= log ( eps );
  logeps_1= log ( 1.0 - eps );
  counter= 0;
  buffer= !binary ? tL_buffer_new () : NULL;
  if ( amcounts->meths->accum_global_eps != NULL )
    {
      if ( amcounts->meths->accum_global_eps ( from, buffer, amodel->global,
                                               amcounts->global,
                                               eps, logeps, logeps_1,
                                               binary, &counter ) == -1 )
        goto error;
    }
  for ( n= 0; n < amodel->N; ++n )
    {
      cchar= amodel->hmms[n];
      ccount= amcounts->hmms[n];
      switch ( cchar.trans.type )
        {
        case TLLINEAL:
          for ( q= 0; q < cchar.Q; ++q )
            {
              if ( !tL_amcounts_logaccum_eps ( from, buffer,
                                               &(ccount.trans.lineal.p[q].p0),
                                               logeps, logeps_1, binary ) )
                goto error;
              ++counter;
              if ( !tL_amcounts_logaccum_eps ( from, buffer,
                                               &(ccount.trans.lineal.p[q].p1),
                                               logeps, logeps_1, binary ) )
                goto error;
              ++counter;
            }
          
          break;
        case TLLIST:
          for ( q= 0; q <= cchar.Q; ++q )
            {
              if ( !tL_amcounts_logaccum_eps ( from, buffer,
                                               &(ccount.trans.list.fp[q]),
                                               logeps, logeps_1, binary ) )
                goto error;
              ++counter;
            }
          for ( ed= 0; ed < cchar.trans.list.NE; ++ed )
            {
              if ( !tL_amcounts_logaccum_eps ( from, buffer,
                                               &(ccount.trans.list.edges[ed]),
                                               logeps, logeps_1, binary ) )
                goto error;
              ++counter;
            }
          break;
        case TLPOINTER: break;
        default: break;
        }
      if ( amcounts->meths->accum_eps ( from, buffer, amodel->global,
                                        amcounts->global, cchar.states,
                                        ccount.states, cchar.Q,
                                        eps, logeps, logeps_1,
                                        binary, &counter ) == -1 )
        goto error;
    }
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  
  return 0;
  
 error:
  if ( buffer != NULL ) tL_buffer_free ( buffer );
  tL_msgerror ( err, _( "Unable to read all counts, only %ul were read"
                " and accumulated " ) , counter );
  return -1;
  
} /* end tL_amlogcounts_accum_eps */
