/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  lookahead.h - Implementació del look-ahead.
 *
 *  Adrià Giménez Pastor, 2012.
 *
 */

#ifndef __TL_PRV_LOOKAHEAD_H__
#define __TL_PRV_LOOKAHEAD_H__

#include "lexicon.h"
#include "lexicon_recaux.h"
#include "lm.h"

/* TIPUS */

/* Taula de lookahead */
typedef struct
{
  
  unsigned long *to;       /* Transicions. */
  tLFloat       *probs;    /* Probabilitats dels estats de l'arbre de
			      prefixes. */
  
} lookahead_table_t;

/* Node de la memòria cau. */
typedef struct lookahead_node lookahead_node_t;
struct lookahead_node
{
  
  unsigned long      state;
  lookahead_node_t  *prev;
  lookahead_node_t  *next;
  lookahead_table_t  table;
  
};

/* Memòria cau de taules. */
typedef struct
{
  
  lookahead_node_t   nodes;
  lookahead_node_t **states;
  size_t             N;        /* Número de taules actual. */
  size_t             capacity;
  
} lookahead_cache_t;

/* Estructura per al look-ahead. */
typedef struct
{
  
  unsigned long     *toporder;      /* |States|. Order topològic de
				       baix cap a dalt per a recorrer
				       l'arbre de prefixes. */
  unsigned long     *map_leaves;    /* |Entrades Lèxic|. Mapa per a
				       les fulles. */
  unsigned long     *map_states;    /* |States|. Mapa per als estats. */
  unsigned long      N;             /* Grandària d'una taula de
				       lookahead. */
  tLProb            *auxlm;         /* |Número de paraules|. Vector
				       auxilar. */
  lookahead_cache_t *cache;         /* Memòria cau de taules. */
  unsigned long     *map_hists;     /* |States|. Maps the current
                                        state with the state used for
                                        look-ahead. Can be NULL. */
  
} lookahead_t;


/* FUNCIONS */

void
tL_lookahead_change_capacity (
			      lookahead_t  *lookahead,
			      const size_t  ncapacity
			      );

#define tL_lookahead_check_state_has_table(LA,STATE)			\
  (((LA)->map_hists==NULL) || ((LA)->map_hists[(STATE)] == (STATE)))

void
tL_lookahead_free (
		   lookahead_t *lookahead
		   );

/* 'fails' és incrementat amb 1 per cada error de cache. */
/* If depth is active, this function uses 'map_hists' to obtain the
   appropriate table. */
lookahead_table_t
tL_lookahead_get_table (
			lookahead_t         *lookahead,
			const lexpref_t     *lexpref,
			const tLLM          *lm,
			tLLMIterator         i,
			const unsigned long  lms,
			unsigned long       *fails
			);

lookahead_t *
tL_lookahead_new (
		  const lexpref_t    *lexpref,
		  const tLLexicon    *lexicon,
		  const tLLM         *lm,
		  const size_t        capacity,  /* De la memòria cau. */
		  const unsigned int  depth /* 0 disables depth. */
		  );

#endif /* __TL_PRV_LOOKAHEAD_H__ */
