/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  mcounts.c - Implementació de les funcions de mixture_counts.
 *
 *  Adrià Giménez Pastor, 2012,2013.
 *
 */


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#define __TL_PRV
#include "check_is_mixture.h"
#include "amcounts/imixture.h"
#include "amcounts/itiedstates.h"
#include "amcounts/logmixture.h"
#include "amcounts/logtiedstates.h"
#include "amcounts_aux.h"
#include "amodel/mixture.h"
#include "amodel/tiedstates.h"
#include "iamcounts.h"
#include "logamcounts.h"
#include "error.h"
#include "mem.h"




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(tLMixCounts *)
tL_amicounts_get_mixcounts (
			    tLAMiCounts     *counts,
			    const tLAModel  *amodel,
			    char           **err
			    )
{
  
  tLMixCounts *new;
  int type;
  tLFloat **mem, *memi;
  unsigned int c, q, i;
  tLAModelHMM cchar;
  mixture_state_t state;
  iamcount_hmm_t ccount;
  iamcount_mixture_state_t scount;
  tiedstates_global_t *global;
  iamcount_tiedstates_global_t *cglobal;
  
  
  /* Comprovació. */
  if ( !tL_check_is_mixture ( amodel, &type, err ) ) return NULL;
  mt ( new= mem_alloc ( tLMixCounts, 1 ) );
  if ( type == MIXTURE )
    {
      mt ( new->counts= mem_alloc ( tLFloat **, amodel->N ) );
      mt ( mem= mem_alloc ( tLFloat *, amodel->total_Q ) );
      for ( c= 0; c < amodel->N; ++c )
	{
	  new->counts[c]= mem;
	  cchar= amodel->hmms[c];
	  ccount= counts->hmms[c];
	  for ( q= 0; q < cchar.Q; ++q )
	    {
	      state= MIXTURE_STATE(cchar.states)[q];
	      scount= ((iamcount_mixture_state_t *) ccount.states)[q];
	      mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	      for ( i= 0; i < state.I; ++i )
		memi[i]= (tLFloat) scount.p[i];
	    }
	  mem+= cchar.Q;
	}
    }
  else
    {
      global= TIEDSTATES_GLOBAL(amodel->global);
      cglobal= (iamcount_tiedstates_global_t *) counts->global;
      mt ( new->counts= mem_alloc ( tLFloat **, 1 ) );
      mt ( new->counts[0]= mem= mem_alloc ( tLFloat *, global->N ) );
      for ( q= 0; q < global->N; ++q )
	{
	  state= MIXTURE_STATE(global->states)[q];
	  scount= ((iamcount_mixture_state_t *) cglobal->states)[q];
	  mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	  for ( i= 0; i < state.I; ++i )
	    memi[i]= (tLFloat) scount.p[i];
	}
    }
  new->type= type;
  
  return new;
  
} /* end tL_amicounts_get_mixture_counts */


TL_API_FUNC(tLMixCounts *)
tL_amlogcounts_get_mixcounts (
			      tLAMlogCounts   *counts,
			      const tLAModel  *amodel,
			      char           **err
			      )
{
  
  tLMixCounts *new;
  int type;
  tLFloat **mem, *memi;
  unsigned int c, q, i;
  tLAModelHMM cchar;
  mixture_state_t state;
  logamcount_hmm_t ccount;
  logamcount_mixture_state_t scount;
  tiedstates_global_t *global;
  logamcount_tiedstates_global_t *cglobal;
  
  
  /* Comprovació. */
  if ( !tL_check_is_mixture ( amodel, &type, err ) ) return NULL;
  mt ( new= mem_alloc ( tLMixCounts, 1 ) );
  if ( type == MIXTURE )
    {
      mt ( new->counts= mem_alloc ( tLFloat **, amodel->N ) );
      mt ( mem= mem_alloc ( tLFloat *, amodel->total_Q ) );
      for ( c= 0; c < amodel->N; ++c )
	{
	  new->counts[c]= mem;
	  cchar= amodel->hmms[c];
	  ccount= counts->hmms[c];
	  for ( q= 0; q < cchar.Q; ++q )
	    {
	      state= MIXTURE_STATE(cchar.states)[q];
	      scount= ((logamcount_mixture_state_t *) ccount.states)[q];
	      mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	      for ( i= 0; i < state.I; ++i )
		memi[i]= exp ( scount.p[i] );
	    }
	  mem+= cchar.Q;
	}
    }
  else
    {
      global= TIEDSTATES_GLOBAL(amodel->global);
      cglobal= (logamcount_tiedstates_global_t *) counts->global;
      mt ( new->counts= mem_alloc ( tLFloat **, 1 ) );
      mt ( new->counts[0]= mem= mem_alloc ( tLFloat *, global->N ) );
      for ( q= 0; q < global->N; ++q )
	{
	  state= MIXTURE_STATE(global->states)[q];
	  scount= ((logamcount_mixture_state_t *) cglobal->states)[q];
	  mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	  for ( i= 0; i < state.I; ++i )
	    memi[i]= exp ( scount.p[i] );
	}
    }
  new->type= type;
  
  return new;
  
} /* end tL_amlogcounts_get_mixture_counts */


TL_API_FUNC(void)
tL_mixcounts_free (
		   tLMixCounts    *mcounts,
		   const tLAModel *amodel
		   )
{
  
  unsigned int Q, q;
  tLFloat **mem;
  
  
  Q= (mcounts->type==MIXTURE) ?
    amodel->total_Q :
    TIEDSTATES_GLOBAL(amodel->global)->N;
  mem= mcounts->counts[0];
  for ( q= 0; q < Q; ++q )
    if ( mem[q] != NULL ) 
      free ( mem[q] );
  free ( mcounts->counts[0] );
  free ( mcounts->counts );
  free ( mcounts );
  
} /* end tL_mixcounts_free */


TL_API_FUNC(void)
tL_mixcounts_print (
		    const tLMixCounts *mcounts,
		    const tLAModel    *amodel,
		    const tLDict      *syms,
		    FILE              *to
		    )
{
  
  unsigned int c, q, i;
  tLAModelHMM cchar;
  tLFloat **counts_c, *counts_cq;
  mixture_state_t state;
  tiedstates_global_t *global;
  
  
  if ( mcounts->type == MIXTURE )
    for ( c= 0; c < amodel->N; ++c )
      {
	counts_c= mcounts->counts[c];
	cchar= amodel->hmms[c];
	for ( q= 0; q < cchar.Q; ++q )
	  {
	    state= MIXTURE_STATE(cchar.states)[q];
	    counts_cq= counts_c[q];
	    for ( i= 0; i < state.I; ++i )
	      fprintf ( to, "%s_%u_%u %g\n",
			tL_dict_get_token ( syms, c ), q, i, counts_cq[i] );
	  }
      }
  else
    {
      global= TIEDSTATES_GLOBAL(amodel->global);
      counts_c= mcounts->counts[0];
      for ( q= 0; q < global->N; ++q )
	{
	  state= MIXTURE_STATE(global->states)[q];
	  counts_cq= counts_c[q];
	  for ( i= 0; i < state.I; ++i )
	    fprintf ( to, "%s_%u %g\n",
		      tL_dict_get_token ( global->names, q ), i, counts_cq[i] );
	}
    }
  
} /* end tL_mixcounts_print */


TL_API_FUNC(tLMixCounts *)
tL_mixcounts_read (
		   const tLAModel  *amodel,
		   gzFile           from,
		   char           **err
		   )
{
  
  tLMixCounts *new;
  int type;
  tLFloat **mem, *memi;
  unsigned int c, q, i;
  tLAModelHMM cchar;
  mixture_state_t state;
  tiedstates_global_t *global;
  tLBuffer *buffer;
  
  
  
  
  /* Comprovació. */
  if ( !tL_check_is_mixture ( amodel, &type, err ) ) return NULL;
  
  /* Llig. */
  mt ( new= mem_alloc ( tLMixCounts, 1 ) );
  buffer= tL_buffer_new ();
  if ( type == MIXTURE )
    {
      mt ( new->counts= mem_alloc ( tLFloat **, amodel->N ) );
      mt ( mem= mem_alloc ( tLFloat *, amodel->total_Q ) );
      for ( q= 0; q < amodel->total_Q; ++q ) mem[q]= NULL;
      for ( c= 0; c < amodel->N; ++c )
	{
	  new->counts[c]= mem;
	  cchar= amodel->hmms[c];
	  for ( q= 0; q < cchar.Q; ++q )
	    {
	      state= MIXTURE_STATE(cchar.states)[q];
	      mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	      for ( i= 0; i < state.I; ++i )
		{
		  if ( !tL_gtoken ( from, buffer ) )
		    goto error;
		  if ( !tL_gtoken ( from, buffer ) )
		    goto error;
		  memi[i]= atof ( buffer->v );
		}
	    }
	  mem+= cchar.Q;
	}
    }
  else
    {
      global= TIEDSTATES_GLOBAL(amodel->global);
      mt ( new->counts= mem_alloc ( tLFloat **, 1 ) );
      mt ( new->counts[0]= mem= mem_alloc ( tLFloat *, global->N ) );
      for ( q= 0; q < global->N; ++q ) mem[q]= NULL;
      for ( q= 0; q < global->N; ++q )
	{
	  state= MIXTURE_STATE(global->states)[q];
	  mt ( mem[q]= memi= mem_alloc ( tLFloat, state.I ) );
	  for ( i= 0; i < state.I; ++i )
	    {
	      if ( !tL_gtoken ( from, buffer ) )
		goto error;
	      if ( !tL_gtoken ( from, buffer ) )
		goto error;
	      memi[i]= atof ( buffer->v );
	    }
	}
    }
  new->type= type;
  tL_buffer_free ( buffer );
  
  return new;

 error:
  tL_buffer_free ( buffer );
  tL_mixcounts_free ( new, amodel );
  tL_msgerror ( err, _( "Unable to read all the mixture coefficient counts" ) );
  return NULL;
  
} /* end tL_mixcounts_read */
