/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  occs.c - Funcions de les occupancies.
 *
 *  Adrià Giménez Pastor, 2012.
 *
 */


#include <math.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "amodel.h"
#include "amodel_public.h"
#include "common.h"
#include "error.h"
#include "mem.h"
#include "text.h"




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

TL_API_FUNC(void)
tL_occupancies_init (
		     tLFloat        **occs,
		     const tLAModel  *amodel
		     )
{
  
  unsigned int q;
  tLFloat *p;
  
  
  for ( p= *occs, q= 0; q < amodel->total_Q; ++q )
    p[q]= -HUGE_VAL;
  
} /* end tL_occupancies_init */


TL_API_FUNC(tLFloat **)
tL_occupancies_new (
		    const tLAModel *amodel
		    )
{
  
  tLFloat **new, *mem;
  unsigned int c;
  
  
  mt ( new= mem_alloc ( tLFloat *, amodel->N ) );
  mt ( new[0]= mem= mem_alloc ( tLFloat, amodel->total_Q ) );
  for ( c= 0; c < amodel->N; ++c )
    {
      new[c]= mem;
      mem+= amodel->hmms[c].Q;
    }
  
  return new;
  
} /* end tL_occupancies_new */


TL_API_FUNC(void)
tL_occupancies_print (
		      tLFloat        **occs,
		      FILE            *to,
		      const tLAModel  *amodel,
		      const tLDict    *syms
		      )
{
  
  unsigned int c, q;
  
  
  fprintf ( to, "# CHAR\t\tQ\t\tOCCUPANCIES\n\n" );
  for ( c= 0; c < amodel->N; ++c )
    {
      fprintf ( to, "%s\t\t%lu",
		tL_dict_get_token ( syms, c ),
		_PS(amodel->hmms[c].Q) );
      for ( q= 0; q < amodel->hmms[c].Q; ++q )
	fprintf ( to, "\t\t%g", _DB(exp(occs[c][q])) );
      fputc ( '\n', to );
    }
  
} /* end tL_occupancies_print */
