/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  smodels.c - Implementació de 'smodels.h'.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2013.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#define __TL_PRV
#include "amodel_itrans.h"
#include "error.h"
#include "mem.h"
#include "smodels.h"




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
create_dist (
	     smodel_t *smodel
	     )
{
  
  int *dist, offset;
  size_t n, i;
  const tLAModelHMM *cchar;
  unsigned int c;
  
  
  mt ( smodel->dist= dist= mem_alloc ( int, smodel->nstates ) );
  n= smodel->length;
  i= smodel->nstates;
  offset= 0;
  do {
    c= smodel->seq[--n];
    cchar= smodel->hmms[c];
    i-= cchar->Q;
    offset= tL_init_state_dists ( &(dist[i]), cchar, offset );
  } while ( n != 0 );
  
} /* end create_sdist */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

smodels_t
tL_create_smodels (
		   const tLAModel  *amodel,
		   tLAMlogCounts   *counts,
		   tLAMiCounts     *icounts,
		   tLFloat        **occs,
		   const tLSeqs    *seqs
		   )
{
  
  smodels_t new;
  size_t n, t, i, nstates;
  tLSeq cs;
  smodel_t *smodel;
  int *map, *chars, C, c, c2;
  unsigned int Q;
  
  
  mt ( new= mem_alloc ( struct smodels, 1 ) );
  
  /* Crea models. */
  new->N= seqs->cseqs->N;
  mt ( new->models= mem_alloc ( smodel_t, new->N ) );
  mt ( map= mem_alloc ( int, amodel->N ) );
  mt ( chars= mem_alloc ( int, amodel->N ) );
  mt ( new->active= mem_alloc ( tLBool, amodel->N ) );
  for ( n= 0; n < amodel->N; ++n ) map[n]= -1;
  for ( n= 0; n < amodel->N; ++n )
    new->active[n]= FALSE;
  new->max_C= new->max_Q= 0;
  new->max_length= 0;
  for ( n= 0; n < new->N; ++n )
    {
      
      cs= *(dict_GCSEQ ( seqs->cseqs, n ));
      smodel= &(new->models[n]);
      smodel->smap= NULL;
      smodel->dist= NULL;
      
      /* Codifica caracters. */
      smodel->length= cs.length;
      if ( smodel->length > new->max_length )
	new->max_length= smodel->length;
      mt ( smodel->seq= mem_alloc ( unsigned int,
				    cs.length ) );
      nstates= 0; C= 0;
      for ( t= 0; t < cs.length; ++t )
	{
	  c= cs.v[t];
	  if ( (c2= map[c]) == -1 )
	    {
	      c2= map[c]= C;
	      chars[C++]= c;
	    }
	  smodel->seq[t]= c2;
	  Q= amodel->hmms[c].Q;
	  if ( Q > new->max_Q ) new->max_Q= Q;
	  nstates+= Q;
	}
      smodel->nstates= nstates;
      
      /* Fica models i comptadors. */
      smodel->C= C; if ( C > new->max_C ) new->max_C= C;
      mt ( smodel->hmms= mem_alloc ( tLAModelHMM *, C ) );
      if ( counts != NULL )
	{
	  mt ( smodel->counts= mem_alloc ( logamcount_hmm_t *, C ) );
	  for ( i= 0; i < C; ++i )
	    smodel->counts[i]= &(counts->hmms[chars[i]]);
	}
      else smodel->counts= NULL;
      if ( icounts != NULL )
	{
	  mt ( smodel->icounts= mem_alloc ( iamcount_hmm_t *, C ) );
	  for ( i= 0; i < C; ++i )
	    smodel->icounts[i]= &(icounts->hmms[chars[i]]);
	}
      else smodel->icounts= NULL;
      if ( occs != NULL )
	{
	  mt ( smodel->occs= mem_alloc ( tLFloat *, C ) );
	  for ( i= 0; i < C; ++i )
	    smodel->occs[i]= occs[chars[i]];
	}
      else smodel->occs= NULL;
      for ( i= 0; i < C; ++i )
	{
	  smodel->hmms[i]= &(amodel->hmms[chars[i]]);
	  new->active[chars[i]]= TRUE;
	}
      
      /* Reinicialitza. */
      for ( i= 0; i < C; ++i )
	map[chars[i]]= -1;
      
    }
  free ( map );
  free ( chars );
  
  return new;
  
} /* end tL_create_smodels */


void
tL_free_smodels (
		 smodels_t smodels
		 )
{
  
  size_t n;
  
  
  free ( smodels->active );
  for ( n= 0; n < smodels->N; ++n )
    {
      if ( smodels->models[n].dist != NULL )
	free ( smodels->models[n].dist );
      if ( smodels->models[n].smap != NULL )
	free ( smodels->models[n].smap );
      free ( smodels->models[n].seq );
      free ( smodels->models[n].hmms );
      if ( smodels->models[n].counts != NULL )
	free ( smodels->models[n].counts );
      if ( smodels->models[n].icounts != NULL )
	free ( smodels->models[n].icounts );
      if ( smodels->models[n].occs != NULL )
	free ( smodels->models[n].occs );
    }
  free ( smodels->models );
  free ( smodels );
  
} /* end tL_free_smodels */


void
tL_smodel_create_smap (
		       smodel_t *smodel
		       )
{
  
  size_t n, s;
  const tLAModelHMM *cchar;
  unsigned int q;
  
  
  mt ( smodel->smap= mem_alloc ( smodel_state_t, smodel->nstates ) );
  for ( n= s= 0; s < smodel->length; ++s )
    {
      cchar= smodel->hmms[smodel->seq[s]];
      for ( q= 0; q < cchar->Q; ++q, ++n )
	{
	  smodel->smap[n].s= s;
	  smodel->smap[n].q= q;
	}
    }
  
} /* end tL_smodel_create_smap */


const int *
tL_smodel_get_dist (
		    smodel_t *smodel
		    )
{
  
  if ( smodel->dist == NULL ) create_dist ( smodel );
  
  return smodel->dist;
  
} /* end tL_smodel_get_dist */
