/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  smodels.h - Estructura amb els models associats a cada mostra.
 *
 *  Adrià Giménez Pastor, 2008,2011,2013.
 *
 */

#ifndef __TL_PRV_SMODELS_H__
#define __TL_PRV_SMODELS_H__

#include <stddef.h>
#include <stdlib.h>

#include "common.h"
#include "amodel.h"
#include "iamcounts.h"
#include "logamcounts.h"
#include "samples.h"
#include "text.h"


/* Definició. */

/* Identificador d'estat. */
typedef struct
{
  
  size_t       s;    /* Número de 'shape' en la seqüència. */
  unsigned int q;    /* Número d'estat. */
  
} smodel_state_t;

typedef struct smodel
{
  
  size_t              length;     /* Número de caracters. */
  size_t              nstates;    /* Número d'estats. */
  unsigned int        C;          /* Números de caràcters distints. */
  unsigned int       *seq;        /* Paraula. L'alfabet és C. */
  tLAModelHMM       **hmms;      /* Caràcters. */
  logamcount_hmm_t  **counts;     /* Comptadors. */
  iamcount_hmm_t    **icounts;    /* Comptadors sencers. */
  tLFloat           **occs;       /* Occpancies associades. Pot ser
				     NULL. */
  smodel_state_t     *smap;       /* Mapa d'estat. Donat un número
				     d'estat torna l'estat
				     associat. Este camp s'ha de
				     crear, per defecte està a
				     NULL. */
  int                *dist;       /* Distància dels estats a l'estat
				     final. Este camp s'ha de crear,
				     per defecte està a NULL. */
  
} smodel_t;

struct smodels
{
 
  size_t         N;
  smodel_t      *models;
  unsigned int   max_C;
  unsigned int   max_Q;
  size_t         max_length;
  tLBool        *active; /* Indica si un caràcter està actiu. */
  
};

typedef struct smodels * smodels_t;


/* FUNCIONS */

smodels_t
tL_create_smodels (
		   const tLAModel  *amodel,
		   tLAMlogCounts   *counts,  /* Pot ser NULL. */
		   tLAMiCounts     *icounts, /* Pot ser NULL. */
		   tLFloat        **occs,    /* Pot ser NULL. */
		   const tLSeqs    *seqs
		   );

void
tL_free_smodels (
		 smodels_t smodels
		 );

/* S'enten que el mapa està a NULL. */
void
tL_smodel_create_smap (
		       smodel_t *smodel
		       );

/* Torna el vector de distàncies. Requereix que s'hagen creat
   previament les transicions inverses. */
const int *
tL_smodel_get_dist (
		    smodel_t *smodel
		    );


#endif /* __TL_PRV_SMODELS_H__ */
