/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  statestr.c - Implementació de 'statestr.h'.
 *
 *  Adrià Giménez Pastor, 2008,2009,2010,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>
#include <string.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "error.h"
#include "mem.h"
#include "statestr.h"



/**********************/
/* TIPUS ENREGISTRATS */
/**********************/

typedef str_factory_t * (create_strf_t) (void *);

typedef struct
{

  const char    *name;
  create_strf_t *create;

} type_t;

#include "statestr/bernoulli.h"
#include "statestr/dgaussian.h"
#include "statestr/mixture.h"

static const type_t types[]=
  {
    { "Bernoulli", tL_create_strf_bernoulli },
    { "DGaussian", tL_create_strf_dgaussian },
    { "Mixture", tL_create_strf_mixture },
    { NULL, NULL }
  };


typedef strv_factory_t * (create_strvf_t) (void *);

typedef struct
{

  const char     *name;
  create_strvf_t *create;

} typev_t;


static const typev_t typevs[]=
  {
    { "Bernoulli", tL_create_strvf_bernoulli },
    { "Mixture", tL_create_strvf_mixture },
    { NULL, NULL }
  };




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

str_factory_t *
tL_create_str_factory (
		       const char *type,
		       void       *global
		       )
{
  
  int i;

  
  for ( i= 0; types[i].name != NULL &&
          strcmp ( type, types[i].name );
        ++i );
  if ( types[i].name == NULL )
    tL_error ( _( "EM training not implented for type %s" ), type );
  
  return types[i].create ( global );
  
} /* end tL_create_str_factory */


statestr_t *
tL_str_factory_make (
		     str_factory_t *strf,
		     void          *global,
		     const size_t   N
		     )
{
  
  statestr_t *new;
  
  
  mt ( new= mem_alloc ( statestr_t, 1 ) );
  strf->meths->init_statestr ( strf, new,
			       global, N );
  
  return new;
  
} /* end tL_str_factory_make */


strv_factory_t *
tL_create_strv_factory (
			const char *type,
			void       *global
			)
{
  
  int i;
  
  
  for ( i= 0; typevs[i].name != NULL &&
          strcmp ( type, typevs[i].name );
        ++i );
  if ( typevs[i].name == NULL )
    tL_error ( _( "Viterbi training not implented for type %s" ), type );
  
  return typevs[i].create ( global );
  
} /* end tL_create_strv_factory */


statestrv_t *
tL_strv_factory_make (
		      strv_factory_t *strf,
		      void           *global,
		      const size_t    N
		      )
{
  
  statestrv_t *new;
  
  
  mt ( new= mem_alloc ( statestrv_t, 1 ) );
  strf->meths->init_statestrv ( strf, new,
				global, N );
  
  return new;
  
} /* end tL_strv_factory_make */
