/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2008,2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  text.c - Implementació de 'text.h'.
 *
 *  Adrià Giménez Pastor, 2008,2010,2011,2012.
 *
 */


#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "error.h"
#include "mem.h"
#include "text.h"




/**********/
/* MACROS */
/**********/

#define isreturn(c) ((c) == '\r' || (c) == '\n')

#define gbody(F,BUFFER,COND)                          \
                                                      \
  int c;                                              \
  size_t pos;                                         \
                                                      \
                                                      \
  while ( (c= gzgetc ( (F) )) != EOF && COND ( c ) );   \
  if ( c == EOF ) return 0;                           \
                                                      \
  pos= 1;                                             \
  (BUFFER)->v[0]= (char) ((unsigned char) c);         \
  while ( (c= gzgetc ( (F) )) != EOF && !COND ( c ) )   \
    {                                                 \
      if ( pos >= (BUFFER)->s )                       \
	tL_resize_buffer ( (BUFFER) );		      \
      (BUFFER)->v[pos++]= (char) ((unsigned char) c); \
    }                                                 \
  if ( pos >= (BUFFER)->s )                           \
    tL_resize_buffer ( (BUFFER) );		      \
  (BUFFER)->v[pos]= '\0';                             \
                                                      \
  return 1

#define S(ptr) ((const char *) ptr)
#define CS(ptr) ((const tLSeq *) ptr)




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static tLDict *
create_dict ()
{
  
  tLDict *new;
  size_t i;
  
  
  mt ( new= mem_alloc ( tLDict, 1 ) );
  mt ( new->data= mem_alloc ( void *, 1) );
  new->_data_size= 1;
  new->N= 0;
  for ( i= 0; i < TLDICTS; ++i )
    new->_table[i]= NULL;
  
  return new;
  
} /* end create_dict */


static int
cseq (
      const void *op1,
      const void *op2
      )
{
  
  size_t length, n, *v1, *v2;
  
  
  if ( (length= CS(op1)->length) != CS(op2)->length )
    return 0;
  
  v1= CS(op1)->v;
  v2= CS(op2)->v;
  for ( n= 0; n < length; ++n )
    if ( v1[n] != v2[n] )
      return 0;
  
  return 1;
  
} /* cseq */


static void
csfree (
	void *cs
	)
{
  tL_seq_free ( (tLSeq *) cs );
} /* end csfree */


static unsigned int
cshash (
	const void *s
	)
{
  
  unsigned int u;
  size_t n;
  tLSeq cs;
  
  
  cs= *CS(s);
  for ( u= 0, n= 0; n < cs.length; ++n )
    u= (u<<2) + (unsigned int) cs.v[n];
  return u%TLDICTS;
  
} /* end cshash */


static unsigned int
strhash (
	 const void *s
	 )
{
  
  unsigned int u;
  const char *p;
  
  p= S(s);
  for ( u= 0; *p; p++ ) u= (u<<2) + *p;
  return u%TLDICTS;
  
} /* end strhash */


static void *
strcopy (
	 const void *s
	 )
{
  
  char *ret;
  
  
  mt ( ret= mem_alloc ( char, strlen ( S(s) ) + 1 ) );
  strcpy ( ret, S(s) );
  
  return (void *) ret;
  
} /* end strcopy */


static int
streq (
       const void *op1,
       const void *op2
       )
{
  return !strcmp ( S(op1), S(op2) );
} /* end streq */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

void *
_tL_cscopy (
	    const void *s
	    )
{
  
  tLSeq *new, cs;
  size_t n;
  
  
  cs= *(CS(s));
  mt ( new= mem_alloc ( tLSeq, 1 ) );
  mt ( new->v= mem_alloc ( size_t, cs.length ) );
  for ( n= 0; n < cs.length; ++n )
    new->v[n]= cs.v[n];
  new->length= cs.length;
  
  return (void *) new;
  
} /* end _tL_cscopy */


TL_API_FUNC(tLBuffer *)
tL_buffer_new ()
{
  
  tLBuffer *new;
  
  
  mt ( new= mem_alloc ( tLBuffer, 1 ) );
  mt ( new->v= mem_alloc ( char, 2 ) );
  new->v[0]= '\0';
  new->s= 2;
  
  return new;
  
} /* end tL_buffer_new */


tLDict *
tL_dict_cseqs_new ()
{
  
  tLDict *new;
  
  
  new= create_dict ();
  new->free_h= csfree;
  new->eq= cseq;
  new->copy= _tL_cscopy;
  new->hash= cshash;
  
  return new;
  
} /* tL_dict_cseqs_new */


TL_API_FUNC(tLDict *)
tL_dict_new ()
{
  
  tLDict *new;
  
  
  new= create_dict ();
  new->free_h= free;
  new->eq= streq;
  new->copy= strcopy;
  new->hash= strhash;
  
  return new;
  
} /* end tL_dict_new */


TL_API_FUNC(int)
tL_dict_insert (
		tLDict     *dict,
		const void *value,
		size_t     *id
		)
{
  
  unsigned int u;
  struct tLDict_h *p, *pp;
  
  
  /* Busca. */
  u= dict->hash ( value );
  for ( pp= NULL, p= dict->_table[u];
	p != NULL; pp= p, p= p->p )
    if ( dict->eq ( value, p->n ) ) break;
  if ( p != NULL )
    {
      if ( id != NULL ) *id= p->i;	
      return 0;
    }
  
  /* Interta en la taula. */
  mt ( p= mem_alloc ( struct tLDict_h, 1 ) );
  p->n= dict->copy ( value  );
  p->i= dict->N;
  if ( id != NULL ) *id= p->i;
  if ( pp == NULL )
    {
      p->p= NULL;
      dict->_table[u]= p;
    }
  else
    {
      p->p= pp->p;
      pp->p= p;
    }
  
  /* Inserta en paraules. */
  if ( dict->_data_size <= dict->N )
    {
      (dict->_data_size)<<= 1;
      mt ( dict->data=
	   mem_realloc ( void *, dict->data,
			 dict->_data_size ) );
    }
  dict->data[(dict->N)++]= p->n;
  
  return 1;
  
} /* end tL_dict_insert */


TL_API_FUNC(size_t)
tL_dict_find (
	      const tLDict *dict,
	      const void   *value
	      )
{
  
  struct tLDict_h *p;
  
  
  for ( p= dict->_table[dict->hash ( value )];
	p != NULL; p= p->p )
    if ( dict->eq ( value, p->n ) ) break;
  
  return ( p == NULL ) ? dict->N : p->i;
  
} /* end tL_dict_find */


TL_API_FUNC(void)
tL_buffer_free (
		tLBuffer *buffer
		)
{
  
  free ( buffer->v );
  free ( buffer );
  
} /* end tL_buffer_free */


TL_API_FUNC(void)
tL_dict_free (
	      tLDict *dict
	      )
{
  
  size_t i;
  struct tLDict_h *p, *fem;
  
  
  for ( i= 0; i < TLDICTS; ++i )
    {
      p= dict->_table[i];
      while ( p != NULL )
	{
	  fem= p;
	  p= p->p;
	  dict->free_h ( fem->n );
	  free ( fem );
	}
    }
  free ( dict->data );
  free ( dict );
  
} /* end tL_dict_free */


TL_API_FUNC(int)
tL_gline (
	  gzFile    f,
	  tLBuffer *buffer
	  )
{
  
  gbody ( f, buffer, isreturn );
  
} /* end tL_gline */


TL_API_FUNC(int)
tL_gtoken (
	   gzFile    f,
	   tLBuffer *buffer
	   )
{
  
  gbody ( f, buffer, isspace );
  
} /* end tL_gtoken */


/* Reads from a file descriptor until the end of file or
 * once a defined number of bytes are read
 */
TL_API_FUNC(int)
tL_gntoken (
	   gzFile    f,
	   tLBuffer *buffer,
           size_t   n
	   )
{

  int c;
  size_t pos;
  size_t i=0;

  while ( (c = gzgetc(f)) != EOF && isspace(c) && i < n ) ++i;
  if ( c == EOF || i >= n ) return 0;

  pos= 1;
  buffer->v[0]= (char) ((unsigned char) c);
  ++i;
  while ( (c = gzgetc (f)) != EOF && !isspace(c) && i < n )
    {
      if ( pos >= buffer->s )
	tL_resize_buffer ( buffer );		
      buffer->v[pos++]= (char) ((unsigned char) c);
      ++i;
    }
  if ( c != EOF ) ++i;
  if ( pos >= buffer->s )
    tL_resize_buffer ( buffer );		
  buffer->v[pos]= '\0';

  return i;

} /* end tL_gntoken */




TL_API_FUNC(int)
tL_gtokenstr (
	      char  *str,
	      char **begin,
	      char **end
	      )
{
  
  char *p;
  
  
  for ( p= str; isspace ( *p ) && *p != '\0'; ++p );
  if ( *p == '\0' )
    {
      *begin= NULL;
      return 1;
    }
  for ( *begin= p; !isspace ( *p ) && *p != '\0'; ++p );
  *end= p;
  
  return (*p == '\0') ? 1 : 0;
  
} /* end tL_gtokenstr */


TL_API_FUNC(tLSeq *)
tL_seq_copy (
	      const tLSeq *cs
	      )
{
  
  tLSeq *new;
  size_t n;
  
  
  mt ( new= mem_alloc ( tLSeq, 1 ) );
  mt ( new->v= mem_alloc ( size_t, cs->length ) );
  new->length= cs->length;
  for ( n= 0; n < cs->length; ++n )
    new->v[n]= cs->v[n];
  
  return new;
  
} /* end tL_seq_copy */


TL_API_FUNC(void)
tL_seq_free (
	      tLSeq *cseq
	      )
{
  
  free ( cseq->v );
  free ( cseq );
  
} /* end tL_seq_free */


TL_API_FUNC(tLSeq *)
tL_seq_new (
	     const size_t length
	     )
{
  
  tLSeq *new;
  
  
  mt ( new= mem_alloc ( tLSeq, 1 ) );
  mt ( new->v= mem_calloc ( size_t, length ) );
  new->length= length;
  
  return new;
  
} /* end tL_seq_new */


TL_API_FUNC(tLSeq *)
tL_seq_new_from_word (
		       const char    *word,
		       const tLDict  *syms,
		       char         **err
		       )
{
  
  tLSeq *new;
  size_t cs_size;
  
  
  mt ( new= mem_alloc ( tLSeq, 1 ) );
  mt ( new->v= mem_alloc ( size_t, 1 ) );
  cs_size= 1;
  if ( tL_utf8tocseq ( word, syms, new, &cs_size, err ) == -1 )
    {
      free ( new->v );
      free ( new );
      return NULL;
    }
  if ( cs_size > new->length )
    mt ( new->v= mem_realloc ( size_t, new->v, new->length ) );
  
  return new;
  
} /* end tL_seq_new_from_word */


TL_API_FUNC(void)
tL_seq_print (
	       const tLSeq *cs,
	       FILE         *to,
	       const tLDict *syms,
	       const char    sep
	       )
{
  
  const size_t *v;
  size_t length, n;
  
  
  v= cs->v;
  length= cs->length;
  fprintf ( to, "%s", tL_dict_get_token ( syms, v[0] ) );
  for ( n= 1; n < length; ++n )
    {
      if ( sep != '\0' ) fputc ( sep, to );
      fprintf ( to, "%s", tL_dict_get_token ( syms, v[n] ) );
    }
  
} /* end tL_seq_print */


TL_API_FUNC(void)
tL_seq_print_wildcards (
			 const tLSeq *cs,
			 FILE        *to,
			 const tLDict *syms,
			 const char    sep,
			 const size_t  first,
			 const size_t  last
			 )
{
  
  const size_t *v;
  size_t length, n;
  
  
  v= cs->v;
  length= cs->length;
  fprintf ( to, "%s", tL_dict_get_token ( syms, first ) );
  for ( n= 1; n < length-1; ++n )
    {
      if ( sep != '\0' ) fputc ( sep, to );
      fprintf ( to, "%s", tL_dict_get_token ( syms, v[n] ) );
    }
  if ( length > 1 )
    {
      if ( sep != '\0' ) fputc ( sep, to );
      fprintf ( to, "%s", tL_dict_get_token ( syms, last ) );
    }
  
} /* end tL_seq_print_wildcards */


void
tL_resize_buffer (
		  tLBuffer *buffer
		  )
{
  
  buffer->s<<= 1;
  mt ( buffer->v= mem_realloc ( char, buffer->v, buffer->s ) );
  
} /* end tL_resize_buffer */


int
tL_utf8tocseq (
	       const char    *token,
	       const tLDict  *syms,
	       tLSeq        *cs,
	       size_t        *cs_size,
	       char         **err
	       )
{
  
  size_t sym;
  char tsym[5];
  int t;
  const char *p;
  
  
  cs->length= 0;
  p= token;
  while ( *p != '\0' )
    {
      
      /* Obté caràcter UTF-8.
	 NOTA: Segmenta bé els caràcters UTF-8, però pot
	 deixar passar seqüències UTF-8 no vàlides. */
      tsym[0]= *(p++);
      for ( t= 1; (*p&0xC0) == 0x80 && t < 4; ++p, ++t )
	tsym[t]= *p;
      tsym[t]= '\0';
      
      /* Reserva memòria. */
      if ( cs->length == *cs_size )
	{
	  (*cs_size)<<= 1;
	  mt ( cs->v= mem_realloc ( size_t, cs->v,
				    *cs_size ) );
	}
      
      /* Codifica. */
      sym= tL_dict_find ( syms, &(tsym[0]) );
      if ( sym == syms->N )
	{
	  tL_msgerror ( err, _( "out of vocabulary token '%s'" ) , tsym );
	  return -1;
	}
      cs->v[cs->length++]= sym;
      
    }
  
  return 0;
  
} /* end tL_utf8tocseq */
