/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  tr_lm.c - Implementació de 'tr_lm.h'.
 *
 *  Adrià Giménez Pastor, 2009,2010,2011.
 *
 */


#include <stddef.h>
#include <stdlib.h>

#include "common.h"
#include "error.h"
#include "hash.h"
#include "lm.h"
#include "mem.h"
#include "tr_lm.h"




/**********/
/* MACROS */
/**********/

/* Codi de la funció per a crear taules, la funció ha de tindre
   l'argument 'init_capacity'. */
#define MAKE_TRA(TRA_T,TRANODE_T)                                \
  TRA_T *new;                                                    \
  unsigned long p;                                               \
                                                                 \
                                                                 \
  mt ( new= mem_alloc ( TRA_T, 1 ) );                            \
                                                                 \
  mt ( new->nodes= mem_alloc ( TRANODE_T, init_capacity ) );     \
  new->snodes= init_capacity;                                    \
  new->Nnodes= 1;                                                \
                                                                 \
  new->capacity= tL_hash_primesize ( init_capacity );               \
  new->threshold= (unsigned long) ((new->capacity*0.75)+0.5);    \
  mt ( new->table= mem_alloc ( unsigned long, new->capacity ) ); \
  for ( p= 0; p < new->capacity; ++p )				 \
    new->table[p]= 0;                                   	 \
                                                                 \
  mt ( new->atentries= mem_alloc ( unsigned long, 1 ) );         \
  new->sate= 1;                                                  \
  new->Nsate= 0;                                                 \
                                                                 \
  return new


/* Codi de la funció de alliberar memòria. */
#define FREE_TRA(TRA)        \
  free ( (TRA)->atentries ); \
  free ( (TRA)->table );     \
  free ( (TRA)->nodes );     \
  free ( (TRA) )


/* Codi per a saber la posició el node en la taula hash. Els nodes han
   de tindre 3 camps obligatoris: k (Identificador), next (unsigned
   long per a una llista), pos (unsigned long denota la posició). La
   posició 0 vol dir que no està enlloc. Torna */
#define TRA_ACTIVE(TRA,ID,TRANODE_T,HASH,CMP)	                        \
  unsigned long p, ed, *table;                                          \
  TRANODE_T *nodes, node;                                               \
                                                                        \
                                                                        \
  nodes= (TRA)->nodes;                                                  \
  table= (TRA)->table;                                                  \
  p= HASH ( (ID) )%(TRA)->capacity;                                     \
  for ( ed= table[p];                                                   \
	ed != 0 && CMP ( nodes[ed], (ID) );	                        \
	ed= nodes[ed].next );                                           \
                                                                        \
  /* Inserta si no està. */                                             \
  if ( ed == 0 )                                                        \
    {                                                                   \
      if ( table[p] == 0 )                                              \
	{                                                               \
	  if ( (TRA)->sate == (TRA)->Nsate )                            \
	    {                                                           \
	      ((TRA)->sate)<<= 1;                                       \
	      mt ( (TRA)->atentries=                                    \
		   mem_realloc ( unsigned long,                         \
				 (TRA)->atentries,                      \
				 (TRA)->sate ) );                       \
	    }                                                           \
	  (TRA)->atentries[(TRA)->Nsate++]= p;                          \
	}                                                               \
      if ( (TRA)->snodes == (TRA)->Nnodes )                             \
	{                                                               \
	  ((TRA)->snodes)<<= 1;                                         \
	  mt ( nodes= (TRA)->nodes=                                     \
	       mem_realloc ( TRANODE_T,                                 \
			     nodes, (TRA)->snodes ) );                  \
	}                                                               \
      ed= ((TRA)->Nnodes)++;                                            \
      node.k= (ID);                                                     \
      node.pos= 0;                                                      \
      node.next= table[p];                                              \
      table[p]= ed;                                                     \
      nodes[ed]= node;                                                  \
                                                                        \
    }                                                                   \
                                                                        \
  return ed


#define TRA_ACTIVE_Q(TRA,ID,TRANODE_T,HASH,CMP,VAR)			\
  unsigned long p, ed, *table;                                          \
  TRANODE_T *nodes;							\
                                                                        \
                                                                        \
  nodes= (TRA)->nodes;                                                  \
  table= (TRA)->table;                                                  \
  p= HASH ( (ID) )%(TRA)->capacity;                                     \
  for ( ed= table[p];                                                   \
	ed != 0 && CMP ( nodes[ed], (ID) );	                        \
	ed= nodes[ed].next );                                           \
  *(VAR)= ed;						                \
  return (ed!=0)




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

/* TRELLIS ********************************************************************/
static tra_t *
make_tra (
	  unsigned long init_capacity
	  )
{
  MAKE_TRA ( tra_t, tra_node_t );
} /* end make_tra */


static void
free_tra (
	  tra_t *tra
	  )
{
  FREE_TRA ( tra );
} /* end free_tra */


/* Funció hash. */
static unsigned long
tra_hash (
          trid_t id
          )
{
  
  static unsigned int buffer[3];
  static const unsigned long size= sizeof(unsigned int)*3;
  
  
  buffer[0]= (unsigned int) id.lms;
  buffer[1]= (unsigned int) id.les;
  buffer[2]= (unsigned int) id.q;
  return tL_hash_gnu_cxx ( buffer, size );
  
} /* end tra_hash */


static trnodes_t *
make_trnodes ()
{
  
  trnodes_t *new;
  
  
  mt ( new= mem_alloc ( trnodes_t, 1 ) );
  mt ( new->v= mem_alloc ( trnode_t, 2 ) );
  new->size= 2;
  new->N= 0;
  
  return new;
  
} /* end make_trnodes */


static void
free_trnodes (
	      trnodes_t *trnodes
	      )
{
  
  free ( trnodes->v );
  free ( trnodes );
  
} /* end free_trnodes */


#if 0
/* Aquesta funció és per a debug. */
static tLBool
trnodes_pop (
	     trnodes_t *trnodes,
	     trnode_t  *node
	     )
{
 
  trnode_t *v;
  unsigned long N, p, p2, q;
  
  
  if ( trnodes->N == 0 ) return FALSE;
  
  v= trnodes->v;
  *node= v[1];
  N= trnodes->N--;
  for ( p= 1; (p2= (p<<1)) < N; p= p2 )
    {
      if ( (q=(p2|1)) < N && v[q].p < v[p2].p ) p2= q;
      if ( v[N].p > v[p2].p ) v[p]= v[p2];
      else break;
    }
  v[p]= v[N];
  
  return TRUE;
  
} /* end trnodes_pop */
#endif


#if 0
/* Neteja del tot el trellis. ESTA FUNCIÓ NO SE SI LA GASTARÉ, SI LA
   GASTE FINALMENT CAL QUE NETEJE LES HIPÒTESIS. */
static void
tr_clear (
	  tr_t *tr
	  )
{
  
  tL_trac_getready ( TRAC(tr->actives) );
  tr->nodes0->N= tr->nodes1->N= 0;
  
} /* end tr_clear */
#endif


/* TRELLIS LÈXIC **************************************************************/
static letra_t *
make_letra (
	    unsigned long init_capacity
	    )
{
  MAKE_TRA ( letra_t, letra_node_t );
} /* end make_letra */


static void
free_letra (
	    letra_t *letra
	    )
{
  FREE_TRA ( letra );
} /* end free_letra */


/* Funció hash. */
static unsigned long
letra_hash (
            letrid_t id
            )
{
  
  static unsigned int buffer[2];
  static const unsigned long size= sizeof(int)*2;
  
  
  buffer[0]= (unsigned int) id.lms;
  buffer[1]= (unsigned int) id.les;
  return tL_hash_gnu_cxx ( buffer, size );
  
} /* end letra_hash */


static letrnodes_t *
make_letrnodes ()
{
  
  letrnodes_t *new;
  
  
  mt ( new= mem_alloc ( letrnodes_t, 1) );
  mt ( new->v= mem_alloc ( letrnode_t, 2 ) );
  new->size= 2;
  new->N= 0;
  
  return new;
  
} /* end make_letrnodes */


static void
free_letrnodes (
		letrnodes_t *letrnodes
		)
{
  
  free ( letrnodes->v );
  free ( letrnodes );
  
} /* end free_letrnodes */


/* TRELLIS MODEL LLENGUATGE ***************************************************/
static lmtrnodes_t *
make_lmtrnodes ()
{
  
  lmtrnodes_t *new;
  
  
  mt ( new= mem_alloc ( lmtrnodes_t, 1 ) );
  mt ( new->v= mem_alloc ( lmtrnode_t, 2 ) );
  new->size= 2;
  new->N= 0;
  
  return new;
  
} /* end make_lmtrnodes */


static void
lm_freetrnodes (
		lmtrnodes_t *lmtrnodes
		)
{
  
  free ( lmtrnodes->v );
  free ( lmtrnodes );
  
} /* end lm_freetrnodes */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

/* TRA ************************************************************************/
void
tL_trac_getready (
		  trac_t *tra
		  )
{
  
  unsigned long aux, ncap, p, n, *atentries, *table;
  
  
  table= tra->table;
  atentries= tra->atentries;
  
  /* Reajusta capacitat si és necessari. */
  if ( tra->Nnodes > tra->threshold && tra->capacity < 16777259UL )
    {
      aux= tra->capacity<<1;
      while ( (ncap= tL_hash_primesize ( aux )) == tra->capacity )
	aux<<= 1;
      tra->threshold= (unsigned long) ((ncap*0.75)+0.5);
      mt ( table= tra->table= mem_realloc ( unsigned long,
					    tra->table, ncap ) );
      for ( p= tra->capacity; p < ncap; ++p )
	table[p]= 0;
      tra->capacity= ncap;
    }
  
  /* Neteja. */
  for ( n= 0; n < tra->Nsate; ++n )
    table[atentries[n]]= 0;
  tra->Nnodes= 1;
  tra->Nsate= 0;
  
} /* end tL_trac_getready */


/* TRELLIS ********************************************************************/
unsigned long
tL_tra_active (
	       tra_t *tra,
	       trid_t id
	       )
{
#define TRA_CMP(a,b) ((a).k.lms != (b).lms || \
		      (a).k.les != (b).les || \
		      (a).k.q != (b).q)
  TRA_ACTIVE ( tra, id, tra_node_t, tra_hash, TRA_CMP );
} /* end tL_tra_active */


tLBool
tL_tra_active_q (
		 tra_t         *tra,
		 trid_t         id,
		 unsigned long *ret_ed
		 )
{
#define TRA_CMP(a,b) ((a).k.lms != (b).lms || \
		      (a).k.les != (b).les || \
		      (a).k.q != (b).q)
  TRA_ACTIVE_Q ( tra, id, tra_node_t, tra_hash, TRA_CMP, ret_ed );
} /* end tL_tra_active_q */


void
tL_trnodes_add (
		trnodes_t *trnodes,
		trnode_t   node,
		tra_t     *tra
		)
{
  
  unsigned long p, p2;
  trnode_t *v;
  tra_node_t *tranodes;
  
  
  if ( (trnodes->N+1) == trnodes->size )
    {
      (trnodes->size)<<= 1;
      mt ( trnodes->v= mem_realloc ( trnode_t,
				     trnodes->v,
				     trnodes->size ) );
    }
  tranodes= tra->nodes;
  v= trnodes->v;
  for ( p= ++trnodes->N;
	p != 1 && node.p < v[(p2= p>>1)].p;
	p= p2 )
    {
      tranodes[v[p2].trapos].pos= p;
      v[p]= v[p2];
    }
  tranodes[node.trapos].pos= p;
  v[p]= node;
  
} /* end tL_trnodes_add */


trnode_t
tL_trnodes_pop_add (
		    trnodes_t *trnodes,
		    trnode_t   node,
		    tra_t     *tra
		    )
{
  
  trnode_t *v, ret;
  unsigned long N, p, p2, q;
  tra_node_t *tranodes;
  
  
  v= trnodes->v;
  tranodes= tra->nodes;
  ret= v[1];
  tranodes[ret.trapos].pos= 0;
  
  N= trnodes->N;
  for ( p= 1; (p2= (p<<1)) <= N; p= p2 )
    {
      if ( (q=(p2|1)) <= N && v[q].p < v[p2].p ) p2= q;
      if ( node.p > v[p2].p )
	{
	  tranodes[v[p2].trapos].pos= p;
	  v[p]= v[p2];
	}
      else break;
    }
  tranodes[node.trapos].pos= p;
  v[p]= node;
  
  return ret;
  
} /* end tL_trnodes_pop_add */


void
tL_trnodes_update (
		   trnodes_t     *trnodes,
		   unsigned long  pnode,
		   tra_t         *tra
		   )
{
  
  unsigned long N, p, p2, q;
  trnode_t *v, node;
  tra_node_t *tranodes;
  
  
  tranodes= tra->nodes;
  v= trnodes->v;
  node= v[pnode];
  N= trnodes->N;
  for ( p= pnode; (p2= (p<<1)) <= N; p= p2 )
    {
      if ( (q=(p2|1)) <= N && v[q].p < v[p2].p ) p2= q;
      if ( node.p > v[p2].p )
        {
          tranodes[v[p2].trapos].pos= p;
          v[p]= v[p2];
        }
      else break;
    }
  tranodes[node.trapos].pos= p;
  v[p]= node;
  
} /* end tL_trnodes_update */


tr_t *
tL_make_tr (
	    unsigned long init_capacity
	    )
{
  
  tr_t *new;
  
  
  mt ( new= mem_alloc ( tr_t, 1 ) );
  new->nodes0= make_trnodes ();
  new->nodes1= make_trnodes ();
  new->actives= make_tra ( init_capacity );
  
  return new;
  
} /* end tL_make_tr */


void
tL_free_tr (
	    tr_t *tr
	    )
{
  
  free_tra ( tr->actives );
  free_trnodes ( tr->nodes1 );
  free_trnodes ( tr->nodes0 );
  free ( tr );
  
} /* end tL_free_tr */


void
tL_tr_getready (
		tr_t *tr
		)
{
  
  trnodes_t *aux;
  
  
  tL_trac_getready ( TRAC(tr->actives) );
  aux= tr->nodes0;
  tr->nodes0= tr->nodes1;
  tr->nodes1= aux;
  tr->nodes1->N= 0;
  
} /* end tL_tr_getready */


/* TRELLIS LÈXIC **************************************************************/
unsigned long
tL_letra_active (
		 letra_t *letra,
		 letrid_t id
		 )
{
#define LETRA_CMP(a,b) ((a).k.lms != (b).lms || \
			(a).k.les != (b).les)
  TRA_ACTIVE ( letra, id, letra_node_t, letra_hash, LETRA_CMP );
} /* end tL_letra_active */


void
tL_letrnodes_add (
		  letrnodes_t   *letrnodes,
		  letrnode_t     node,
		  unsigned long  letrapos,
		  letra_t       *letra
		  )
{
  
  unsigned long pos;
  
  
  if ( ++letrnodes->N == letrnodes->size )
    {
      (letrnodes->size)<<= 1;
      mt ( letrnodes->v= mem_realloc ( letrnode_t,
				       letrnodes->v,
				       letrnodes->size ) );
    }
  letrnodes->v[(pos= letrnodes->N)]= node;
  letra->nodes[letrapos].pos= pos;
  
} /* end tL_letrnodes_add */


letr_t *
tL_make_letr (
	      unsigned long init_capacity
	      )
{
  
  letr_t *new;
  
  
  mt ( new= mem_alloc ( letr_t, 1 ) );
  new->nodes0= make_letrnodes ();
  new->nodes1= make_letrnodes ();
  new->actives= make_letra ( init_capacity );
  
  return new;
  
} /* end tL_make_letr */


void
tL_free_letr (
	      letr_t *letr
	      )
{
  
  free_letra ( letr->actives );
  free_letrnodes ( letr->nodes1 );
  free_letrnodes ( letr->nodes0 );
  free ( letr );
  
} /* end tL_free_letr */


/* TRELLIS MODEL LLENGUATGE ***************************************************/

void
tL_lmtrnodes_add (
		  lmtrnodes_t *lmtrnodes,
		  lmtrnode_t   node,
		  size_t      *actives
		  )
{
  
  if ( ++lmtrnodes->N == lmtrnodes->size )
    {
      (lmtrnodes->size)<<= 1;
      mt ( lmtrnodes->v= mem_realloc ( lmtrnode_t,
				       lmtrnodes->v,
				       lmtrnodes->size ) );
    }
  lmtrnodes->v[lmtrnodes->N]= node;
  actives[node.id]= lmtrnodes->N;
  
} /* tL_lmtrnodes_add */


lmtr_t *
tL_make_lmtr (
	      const tLLM *lm
	      )
{
  
  lmtr_t *new;
  
  
  mt ( new= mem_alloc ( lmtr_t, 1 ) );
  mt ( new->mem= new->actives0= mem_alloc ( size_t, lm->NS<<1 ) );
  new->actives1= new->actives0 + lm->NS;
  memset ( new->actives0, 0, sizeof(size_t)*(lm->NS<<1) );
  new->nodes0= make_lmtrnodes ();
  new->nodes1= make_lmtrnodes ();
  
  return new;
  
} /* end tL_make_lmtr */


void
tL_free_lmtr (
	      lmtr_t *lmtr
	      )
{
  
  free ( lmtr->mem );
  lm_freetrnodes ( lmtr->nodes0 );
  lm_freetrnodes ( lmtr->nodes1 );
  free ( lmtr );
  
} /* end tL_free_lmtr */
