/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  tr_lm.h - Estructures per a implementar un Viterbi/Forward amb
 *            model lèxic i model de llenguatge. Este codi està ací
 *            simplement per a poder reutilitzar-lo.
 *
 *  Adrià Giménez Pastor, 2009,2010,2011.
 *
 */

#ifndef __TL_PRV_TR_LM_H__
#define __TL_PRV_TR_LM_H__

#include <stddef.h>
#include <stdlib.h>


/* TRA */

#define TRAC(PTR) ((trac_t *) (PTR))

#define TRAC_CONTENT                                                  \
  unsigned long  Nnodes;       /* Nombre de nodes insertats en la     \
				  taula hash. */                      \
  unsigned long  capacity;     /* Capacitat de la taula. */           \
  unsigned long  threshold;    /* Si es supera es hora d'ampliar. */  \
  unsigned long *table;        /* Taula. */                           \
  unsigned long *atentries;    /* Entrades de la taula actives. */    \
  unsigned long  sate;         /* Grandària de 'atentries'. */        \
  unsigned long  Nsate;        /* Nombre d'entrades actives. */       \

#define TRA_CONTENT(TRANODE_T)                                        \
  TRAC_CONTENT                                                        \
  TRANODE_T     *nodes;        /* Nodes insertats. */                 \
  unsigned long  snodes;       /* Grandària dels nodes. */

/* Informació comú per a tots els *tra. */
typedef struct
{
  TRAC_CONTENT
} trac_t;

/* Prepara 'trac' per a la següent iteració. */
void
tL_trac_getready (
		  trac_t *tra
		  );


/* TRELLIS */

/* Identificador d'estat en el trelis. */
typedef struct
{
  
  unsigned long lms;    /* Estat del model de llenguatge. */
  unsigned long les;    /* Estat (global) del lèxic. */
  unsigned int  q;      /* Estat intern del HMM. */
  
} trid_t;

/* Estat en el trelis. */
typedef struct
{
  
  trid_t        id;        /* Identificador. */
  tLProb        p;         /* Probabilitat. */
  tLProb        hmmp;      /* Probabilitat HMM+LEX entre estats del
			      model de llenguatge. */
  unsigned long trapos;    /* Posició en la taula. */
  size_t        h;         /* Hipòtesi (Viterbi)/Estat en l'autómata
			      (Forward). */
  
} trnode_t;

/* Node de la taula hash 'tra_t'. */
typedef struct
{
  
  unsigned long next;
  trid_t        k;
  unsigned long pos;    /* Posició en el heap. */
  
} tra_node_t;

/* Per a saber quins estats estan en el trellis. El node 0 és
   sentinela. */
typedef struct
{
  TRA_CONTENT ( tra_node_t )
} tra_t;

/* Minheap amb els nodes del trellis ordenats per probabilitat. */
typedef struct
{
  
  trnode_t *v;
  size_t    size;
  size_t    N;
  
} trnodes_t;

/* Trellis. */
typedef struct
{
  
  trnodes_t *nodes0;     /* Nodes instant t. */
  trnodes_t *nodes1;     /* Nodes instant t+1. */
  tra_t     *actives;    /* Nodes actius. */
  
} tr_t;

#define TRNODES_EMPTY(TRN) ((TRN)->N == 0)
#define TRNODES_NSTATES(TRN) ((TRN)->N)
#define TRNODES_MINP(TRN) ((TRN)->v[1].p)

/* Torna la posició del nodre creat en la taula. */
unsigned long
tL_tra_active (
	       tra_t *tra,
	       trid_t id
	       );

/* Esta funció és com 'tL_tra_active' però sols torna la posició en
   cas d'estar. */
tLBool
tL_tra_active_q (
		 tra_t         *tra,
		 trid_t         id,
		 unsigned long *trapos
		 );

/* Inserta un node en el heap sense preoucpar-se si ja estaba i torna
   la posició en el heap. */
void
tL_trnodes_add (
		trnodes_t *trnodes,
		trnode_t   node,
		tra_t     *tra
		);

/* Borra i torna el node amb probabilitat mínima i inserta un node en
   el heap sense preocupar-se si ja estaba i torna la posició en el
   heap. S'enten que almenys hi ha un node. */
trnode_t
tL_trnodes_pop_add (
		    trnodes_t *trnodes,
		    trnode_t   node,
		    tra_t     *tra
		    );

/* Actualitza el heap tenint en compter que s'ha agumentat la
   probabilitat del node de la posició NODE. */
void
tL_trnodes_update (
		   trnodes_t     *trnodes,
		   unsigned long  pnode,
		   tra_t         *tra
		   );

/* Crea un trellis. */
tr_t *
tL_make_tr (
	    unsigned long init_capacity
	    );

/* Allibera memòria. */
void
tL_free_tr (
	    tr_t *tr
	    );

/* Prepara per a la següent iteració. */
void
tL_tr_getready (
		tr_t *tr
		);


/* TRELLIS LÈXIC */

/* Identificador d'estat en el trelis. */
typedef struct
{
  
  unsigned long lms;    /* Estat del model de llenguatge. */
  unsigned long les;    /* Estat (global) del lèxic. */
  
} letrid_t;

/* Node de la taula hash 'letra_t'. */
typedef struct
{
  
  unsigned long next;
  letrid_t      k;
  unsigned long pos;    /* Posició en el vector. */
  
} letra_node_t;

/* Per a saber quins estats estan en el trellis. El node 0 és
   sentinela. */
typedef struct
{
  TRA_CONTENT ( letra_node_t )
} letra_t;

/* Estat en el trellis. */
typedef struct
{
  
  letrid_t id;
  tLProb   p;
  tLProb   hmmp;
  size_t   h;
  
} letrnode_t;

/* Vector dinàmic amb els nodes del trellis lèxic. Comença a contar
   per u. */
typedef struct
{
  
  letrnode_t *v;
  size_t      size;
  size_t      N;
  
} letrnodes_t;

/* Trellis lèxic. */
typedef struct
{
  
  letrnodes_t *nodes0;     /* Nodes preparats per a eixir. */
  letrnodes_t *nodes1;     /* Nodes preparats per a entrar. */
  letra_t     *actives;    /* Nodes actius. */
  
} letr_t;

#define LETRNODES_EMPTY(LETRN) ((LETRN)->N == 0)

/* Torna la posició del nodre creat en la taula. */
unsigned long
tL_letra_active (
		 letra_t *letra,
		 letrid_t id
		 );

void
tL_letrnodes_add (
		  letrnodes_t   *letrnodes,
		  letrnode_t     node,
		  unsigned long  letrapos,
		  letra_t       *letra
		  );

letr_t *
tL_make_letr (
	      unsigned long init_capacity
	      );

void
tL_free_letr (
	      letr_t *letr
	      );


/* TRELLIS MODEL LLENGUATGE */

/* Estat en el trellis. */
typedef struct
{
  
  unsigned long id;    /* Identificador. */
  tLProb        p;     /* Probabilitat. */
  size_t        h;     /* Hipòtesis. */
  
} lmtrnode_t;


/* Vector dinàmic amb els nodes. Comença a contar per u. */
typedef struct
{
  
  lmtrnode_t *v;
  size_t      N;
  size_t      size;
  
} lmtrnodes_t;


/* Trellis per al model de llenguatge. */
typedef struct
{
  
  size_t      *mem;
  size_t      *actives0;    /* 0 indica no actiu. */
  size_t      *actives1;    /* 0 indica no actiu. */
  lmtrnodes_t *nodes0;
  lmtrnodes_t *nodes1;
  
} lmtr_t;

#define LMTRNODES_EMPTY(LMTRN) ((LMTRN)->N == 0)

void
tL_lmtrnodes_add (
		  lmtrnodes_t *lmtrnodes,
		  lmtrnode_t   node,
		  size_t      *actives
		  );

lmtr_t *
tL_make_lmtr (
	      const tLLM *lm
	      );

void
tL_free_lmtr (
	      lmtr_t *lmtr
	      );


#endif /* __TL_PRV_TR_LM_H__ */
