/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  wghypmng.c - Implementació de 'wghypmng.h'.
 *
 *  Adrià Giménez Pastor, 2010,2011,2012.
 *
 */


#include <assert.h>
#include <stddef.h>
#include <stdlib.h>

/* i18n includes and defines */
#include <locale.h>
#include "../intl.h"

#include "error.h"
#include "lexicon.h"
#include "mem.h"
#include "wghypmng.h"




/*********************/
/* FUNCIONS PRIVADES */
/*********************/

static void
add_stack (
	   struct wghypmng_stack *const stack,
	   size_t                       hyp
	   )
{
  
  if ( stack->N == stack->s )
    {
      stack->s*= 2;
      mt ( stack->v= mem_realloc ( size_t, stack->v, stack->s ) );
    }
  stack->v[stack->N++]= hyp;
  
} /* end add_stack */


/* Etiqueta els estats i a més conta el nombre d'estats i de
   transicions. */
static void
label_states (
	      wghypmng_t      *wghypmng,
	      size_t           hyp,
	      const tLLexicon *lexicon,
	      size_t          *nstates,
	      size_t          *nedges,
	      size_t          *nsegs,
	      const tLBool     wsil
	      )
{
  
  struct wghypmng_stack stack;
  size_t p;
  struct wghypmng_node *v;
  const struct wghhnode *v2;
  struct wghhnode **hnodes;
  unsigned int N, n;
  tLBool align;
  struct wghhnode ed;
  const tLLexiconEntry *entries;
  
  
  N= 0; /* CALLA!! */
  align= (lexicon!=NULL);
  entries= align ? lexicon->entries : NULL;
  hnodes= wghypmng->hnodes;
  v= wghypmng->v;
  stack= wghypmng->stack;
  *nstates= 1; /* la hipòtesis inicial té l'identificador 0. */
  *nedges= 0; /* l'aresta 0 --<s>--> 1. */
  *nsegs= 0;
  stack.v[0]= hyp; stack.N= 1;
  while ( stack.N != 0 )
    {
      p= stack.v[--stack.N];
      while ( p != WGNULLHYP && (N= v[p].N) == 0 ) p= v[p].ptr;
      if ( align && wsil && p == WGNULLHYP ) ++(*nsegs);
      if ( p == WGNULLHYP || v[p].flag ) continue;
      v[p].flag= 1;
      v[p].id= (*nstates)++;
      (*nedges)+= N;
      v2= hnodes[v[p].ptr];
      for ( n= 0; n < N; ++n )
	{
	  ed= v2[n];
	  if ( align && ed.e != NULL_ENTRY )
	    {
	      (*nsegs)+= entries[ed.e].cs->length;
	      if ( wsil ) ++(*nsegs);
	    }
	  add_stack ( &stack, ed.ptr );
	}
    }
  wghypmng->stack= stack;
  
} /* end label_states */


static void
fill_edges (
	    wghypmng_t      *wghypmng,
	    size_t           hyp,
	    const tLLexicon *lexicon,
	    tLWordGraph     *wg,
	    const tLBool     wsil,
	    const size_t     sil
	    )
{
  
  struct wghypmng_stack stack;
  size_t p, id, nlnodes, ptr, i;
  struct wghypmng_node *v;
  const struct wghhnode *v2;
  struct wghhnode **hnodes;
  unsigned int N, n, cwsym;
  unsigned long nedges;
  struct wghhnode ed;
  tLWGEdge *edges, *edge;
  tLWGState *wgstates, *aux, *aux_to;
  tLWGList *lnodes, *lnode;
  tLProb wip;
  tLFloat gsf;
  const tLLexiconEntry *entries;
  tLBool align, iscw;
  tLWGSegment *segs;
  const tLSeq *cs;
  int prev, end;
  
  
  align= (lexicon!=NULL);
  iscw= (lexicon!=NULL&&lexicon->type==TL_LEXICON_CW);
  cwsym= 0; segs= NULL; /* CALLA */
  if ( align )
    {
      entries= lexicon->entries;
      segs= wg->_segs;
    }
  wip= wg->wip; gsf= wg->gsf;
  wgstates= wg->states; edges= wg->edges; lnodes= wg->_nodes;
  nlnodes= 0;
  hnodes= wghypmng->hnodes;
  v= wghypmng->v;
  stack= wghypmng->stack;
  nedges= 0;
  stack.v[0]= hyp; stack.N= 1;
  while ( stack.N != 0 )
    {
      p= stack.v[--stack.N];
      if ( p == WGNULLHYP || !v[p].flag ) continue;
      v[p].flag= 0;
      id= v[p].id;
      aux_to= &(wgstates[id]);
      v2= hnodes[v[p].ptr]; N= v[p].N;
      for ( n= 0; n < N; ++n )
	{
	  
	  /* Aresta. */
	  ed= v2[n];
	  edge= &(edges[nedges++]);
	  
	  /* Segments. */
	  if ( align && ed.e != NULL_ENTRY )
	    {
	      cs= entries[ed.e].cs;
	      edge->segs= segs;
	      edge->nsegs= i= cs->length + (wsil?1:0);
	      prev= v[p].t;
	      ptr= ed.ptr;
	      if ( iscw )
		{
		  cwsym= (unsigned int) v[ptr].lms;
		  assert ( v[ptr].t == -1234 );
		  ptr= v[ptr].ptr;
		  assert ( ptr!=WGNULLHYP && v[ptr].N==0 );
		}
	      for (; !iscw || i > 1;)
		{
		  --i;
		  end= v[ptr].t;
		  if ( i == cs->length )
		    segs[i].sym= sil;
		  else
		    segs[i].sym= (unsigned int) (cs->v[i]);
		  segs[i].pos= end+1;
		  segs[i].nframes= prev-end;
		  prev= end;
		  /* Si hi ha crosswords el temps està en el següent
		     node. */
		  if ( i == 0 ) break;
		  ptr= v[ptr].ptr;
		}
	      if ( iscw )
		{
		  segs[0].sym= (unsigned int) v[ptr].lms;
		  segs[cs->length-1].sym= cwsym;
		  assert ( v[ptr].t == -1234 );
		  ptr= v[ptr].ptr;
		  end= v[ptr].t;
		  segs[0].pos= end+1;
		  segs[0].nframes= prev-end;
		  prev= end;
		}
	      if ( wsil && ptr!=WGNULLHYP && v[ptr].N==0 )
		{
		  for ( i= cs->length+1; i > 0; --i )
		    segs[i]= segs[i-1];
		  ptr= v[ptr].ptr;
		  end= v[ptr].t;
		  segs[0].sym= sil;
		  segs[0].pos= end+1;
		  segs[0].nframes= prev-end;
		  ++(edge->nsegs);
		  ++segs;
		}
	      assert ( ptr==WGNULLHYP || v[ptr].N!=0 );
	      segs+= cs->length;
	      if ( wsil ) ++segs;
	    }
	  else
	    {
	      edge->nsegs= 0;
	      for ( ptr= ed.ptr;
		    ptr!=WGNULLHYP && v[ptr].N==0;
		    ptr= v[ptr].ptr );
	    }
	  
	  /* From. */
	  edge->from= aux= &(wgstates[(ptr==WGNULLHYP) ? 0 : v[ptr].id]);
	  lnode= &(lnodes[nlnodes++]);
	  lnode->edge= edge; lnode->next= aux->oedges; aux->oedges= lnode;
	  
	  /* To. */
	  edge->to= aux_to;
	  lnode= &(lnodes[nlnodes++]);
	  lnode->edge= edge; lnode->next= aux_to->iedges; aux_to->iedges= lnode;
	  
	  /* Word i probabilitats. */
	  edge->w= ed.w;
	  edge->a= ed.hmmp;
	  edge->l= (ed.lmp-wip)/gsf;
	  
	  add_stack ( &stack, ptr );
	  
	}
    }
  wghypmng->stack= stack;
  
} /* end fill_edges */


static void
fill_states (
	     wghypmng_t  *wghypmng,
	     size_t       hyp,
	     tLWordGraph *wg
	     )
{
  
  struct wghypmng_stack stack;
  size_t p;
  struct wghypmng_node *v;
  const struct wghhnode *v2;
  struct wghhnode **hnodes;
  unsigned int N, n;
  tLWGState *states;
  size_t id;
  
  
  states= wg->states;
  hnodes= wghypmng->hnodes;
  v= wghypmng->v;
  stack= wghypmng->stack;
  states[0].t= 0;
  states[0].oedges= states[0].iedges= NULL;
  stack.v[0]= hyp; stack.N= 1;
  wg->start= &(states[0]);
  wg->end= &(states[v[hyp].id]);
  while ( stack.N != 0 )
    {
      p= stack.v[--stack.N];
      while ( p != WGNULLHYP && (N= v[p].N) == 0 ) p= v[p].ptr;
      if ( p == WGNULLHYP || (v[p].flag&0x2) ) continue;
      v[p].flag= 0x3;
      id= v[p].id;
      states[id].t= v[p].t;
      states[id].oedges= NULL; states[id].oedges= NULL;
      v2= hnodes[v[p].ptr];
      for ( n= 0; n < N; ++n )
	add_stack ( &stack, v2[n].ptr );
    }
  wghypmng->stack= stack;
  
} /* end fill_states */


static void
resize_nodes (
	      wghypmng_t *wghypmng
	      )
{
  
  size_t olds, i, s;
  struct wghypmng_node *v;
  
  
  olds= wghypmng->s;
  s= wghypmng->s*= 2;
  if ( s < olds )
    tL_error ( _( "Overflow of 'wghypmng->s'" ) );
  mt ( v= wghypmng->v= mem_realloc ( struct wghypmng_node,
				     wghypmng->v, s ) );
  for ( i= olds; i < s; ++i )
    {
      v[i].counter= 0;
      v[i].ptr= i+1;
    }
  
} /* end resize_nodes */


static void
resize_hnodes (
	       wghypmng_t *wghypmng
	       )
{
  
  size_t olds, i, s;
  struct wghhnode *aux, **hnodes;
  unsigned int K;
  
  
  olds= wghypmng->shnodes;
  s= wghypmng->shnodes*= 2;
  if ( s < olds )
    tL_error ( _( "Overflow of 'wghypmng->shnodes'" ) );
  mt ( hnodes= wghypmng->hnodes= mem_realloc ( struct wghhnode *,
					       wghypmng->hnodes, s ) );
  K= wghypmng->K;
  mt ( aux= hnodes[0]= mem_realloc ( struct wghhnode, hnodes[0], s*K ) );
  for ( i= 0; i < s; ++i, aux+= K ) hnodes[i]= aux;
  for ( i= olds; i < s; ++i )
    hnodes[i][0].ptr= i+1;
  
} /* end resize_hnodes */


static void
wghhnode_add (
	      struct wghhnode *v,
	      unsigned int    *N,
	      size_t           parent,
	      tLProb           p,
	      tLProb           hmmp,
	      tLProb           lmp,
	      unsigned long    w,
	      size_t           e
	      )
{
  
  unsigned int q, q2;
  
  
  for ( q= (*N)++;
	q != 0 && p < v[(q2=((q-1)>>1))].p;
	q= q2 )
    v[q]= v[q2];
  v[q].p= p;
  v[q].hmmp= hmmp;
  v[q].lmp= lmp;
  v[q].e= e;
  v[q].w= w;
  v[q].ptr= parent;
  
} /* end wghhnode_add */


static void
wghhnode_pop_add (
		  struct wghhnode *v,
		  unsigned int     N,
		  size_t           parent,
		  tLProb           p,
		  tLProb           hmmp,
		  tLProb           lmp,
		  unsigned long    w,
		  size_t           e
		  )
{
  
  unsigned int q, q2, k;
  
  
  for ( q= 0; (q2= ((q<<1)|1)) < N; q= q2 )
    {
      if ( (k=(q2+1)) < N && v[k].p < v[q2].p ) q2= k;
      if ( p > v[q2].p ) v[q]= v[q2];
      else break;
    }
  v[q].p= p;
  v[q].hmmp= hmmp;
  v[q].lmp= lmp;
  v[q].e= e;
  v[q].w= w;
  v[q].ptr= parent;
  
} /* wghhnode_pop_add */




/**********************/
/* FUNCIONS PÚBLIQUES */
/**********************/

wghypmng_t *
tL_create_wghypmng (
		    const unsigned int K
		    )
{
  
  wghypmng_t *new;
  
  
  assert ( K>0 );
  mt ( new= mem_alloc ( wghypmng_t, 1 ) );
  new->K= K;
  new->s= 2;
  new->shnodes= 1;
  new->empty= 1;
  new->emptyhnodes= 0;
  mt ( new->v= mem_alloc ( struct wghypmng_node, 2 ) );
  mt ( new->hnodes= mem_alloc ( struct wghhnode *, 1 ) );
  mt ( new->hnodes[0]= mem_alloc ( struct wghhnode, K ) );
  new->hnodes[0][0].ptr= 1;
  new->v[0].counter= 0;
  new->v[0].flag= 0;
  new->v[0].N= 0;
  new->v[0].ptr= 0;
  new->v[0].t= -1;
  new->v[1].ptr= 2;
  new->v[1].counter= 0;
  mt ( new->stack.v= mem_alloc ( size_t, 1 ) );
  new->stack.s= 1;
  new->stack.N= 0;
  
  return new;
  
} /* end tL_create_wghypmng */


void
tL_free_wghypmng (
		  wghypmng_t *wghypmng
		  )
{
  
  free ( wghypmng->stack.v );
  free ( wghypmng->hnodes[0] );
  free ( wghypmng->hnodes );
  free ( wghypmng->v );
  free ( wghypmng );
  
} /* end tL_free_wghypmng */


void
tL_free_wghyp (
	       wghypmng_t *wghypmng,
	       size_t      hyp
	       )
{
  
  size_t p, q, ptr;
  unsigned int N, n;
  struct wghypmng_stack stack;
  struct wghhnode *v;
  
  
  stack= wghypmng->stack;
  stack.v[0]= hyp; stack.N= 1;
  while ( stack.N != 0 )
    {
      p= stack.v[--stack.N];
      N= wghypmng->v[p].N;
      ptr= wghypmng->v[p].ptr;
      if ( N > 0 ) /* Té molts fills. */
	{
	  v= wghypmng->hnodes[ptr];
	  for ( n= 0; n < N; ++n )
	    {
	      q= v[n].ptr;
	      if ( --(wghypmng->v[q].counter) == 0 && q != 0 )
		add_stack ( &stack, q );
	    }
	  v[0].ptr= wghypmng->emptyhnodes;
	  wghypmng->emptyhnodes= ptr;
	}
      else /* És un node de segments. */
	{
	  if ( --(wghypmng->v[ptr].counter) == 0 && ptr != 0 )
	    add_stack ( &stack, ptr );
	}
       wghypmng->v[p].ptr= wghypmng->empty;
       wghypmng->empty= p;
    }
  wghypmng->stack= stack;
  
} /* end tL_free_wghyp */


size_t
tL_get_wghyp (
	      wghypmng_t        *wghypmng,
	      size_t             hyp,
	      unsigned long int *buffer,
	      const tLBool       align,
	      const tLBool       iscw,
	      const tLBool       wsil
	      )
{
  
  struct wghypmng_node *v;
  const struct wghhnode *v2;
  struct wghhnode **hnodes;
  size_t length, i, j, e, ptr;
  unsigned int N, n;
  int t, aux_t;
  tLProb max;
  unsigned long aux, lms;
  tLBool set_t, t_added;
  
  
  v= wghypmng->v;
  hnodes= wghypmng->hnodes;
  length= 0;
  set_t= FALSE;
  aux_t= -1; t_added= TRUE; /* CALLA. */
  while ( hyp != WGNULLHYP )
    {
      N= v[hyp].N;
      ptr= v[hyp].ptr;
      t= v[hyp].t;
      lms= v[hyp].lms;
      if ( N > 0 ) /* Node normal. */
	{
	  v2= hnodes[ptr];
	  max= -HUGE_VAL; e= NULL_ENTRY;
	  for ( n= 0; n < N; ++n )
	    if ( v2[n].p > max )
	      {
		max= v2[n].p;
		hyp= v2[n].ptr;
		e= v2[n].e;
	      }
	  if ( e != NULL_ENTRY )
	    {
	      buffer[length++]= (unsigned long) e;
	      if ( align )
		{
		  if ( iscw && !wsil )
		    {
		      t_added= FALSE;
		      aux_t= t;
		    }
		  else
		    {
		      buffer[length++]= (unsigned long) t;
		      t_added= TRUE;
		    }
		  set_t= TRUE;
		}
	    }
	}
      else /* Node segment. */
	{
	  if ( iscw && t == -1234 )
	    {
	      buffer[length++]= lms;
	      if ( set_t && !t_added )
		{
		  buffer[length++]= (unsigned long) aux_t;
		  t_added= TRUE;
		}
	    }
	  else if ( set_t ) buffer[length++]= (unsigned long) t;
	  hyp= ptr;
	}
    }
  
  /* Inverteix. */
  if ( length > 0 )
    for ( i= 0, j= length-1; i < j; ++i, --j )
      {
	aux= buffer[i];
	buffer[i]= buffer[j];
	buffer[j]= aux;
      }
  
  return length;
  
} /* end tL_get_wghyp */


size_t
tL_wghypmng_add (
		 wghypmng_t    *wghypmng,
		 size_t         parent,
		 unsigned long  lms,
		 int            t,
		 tLProb         p,
		 tLProb         hmmp,
		 tLProb         lmp,
		 unsigned long  w,
		 size_t         e
		 )
{
  
  struct wghypmng_node *v;
  struct wghhnode *v2, **hnodes;
  size_t new, new_ptr;
  
  
  if ( wghypmng->empty == wghypmng->s )
    resize_nodes ( wghypmng );
  if ( wghypmng->emptyhnodes == wghypmng->shnodes )
    resize_hnodes ( wghypmng );
  v= wghypmng->v;
  hnodes= wghypmng->hnodes;
  new= wghypmng->empty;
  wghypmng->empty= v[new].ptr;
  v[new].lms= lms;
  v[new].t= t;
  v[new].counter= 1;
  v[new].N= 1;
  v[new].flag= 0;
  v[new].ptr= new_ptr= wghypmng->emptyhnodes;
  wghypmng->emptyhnodes= hnodes[new_ptr][0].ptr;
  v2= hnodes[new_ptr];
  v2->p= p;
  v2->hmmp= hmmp;
  v2->lmp= lmp;
  v2->e= e;
  v2->w= w;
  v2->ptr= parent;
  ++(v[parent].counter);
  
  return new;
  
} /* end tL_wghypmng_add */


size_t
tL_wghypmng_add_seg (
		     wghypmng_t *wghypmng,
		     size_t      parent,
		     int         t
		     )
{
  
  struct wghypmng_node *v;
  size_t new;
  
  
  if ( wghypmng->empty == wghypmng->s )
    resize_nodes ( wghypmng );
  v= wghypmng->v;
  new= wghypmng->empty;
  wghypmng->empty= v[new].ptr;
  v[new].lms= 0;
  v[new].t= t;
  v[new].counter= 1;
  v[new].N= 0;
  v[new].flag= 0;
  v[new].ptr= parent;
  ++(v[parent].counter);
  
  return new;
  
} /* end tL_wghypmng_add_seg */


size_t
tL_wghypmng_add_sym (
		     wghypmng_t *wghypmng,
		     size_t      parent,
		     size_t      sym
		     )
{
  
  struct wghypmng_node *v;
  size_t new;
  
  
  if ( wghypmng->empty == wghypmng->s )
    resize_nodes ( wghypmng );
  v= wghypmng->v;
  new= wghypmng->empty;
  wghypmng->empty= v[new].ptr;
  v[new].lms= (unsigned long) sym;
  v[new].t= -1234;
  v[new].counter= 1;
  v[new].N= 0;
  v[new].flag= 0;
  v[new].ptr= parent;
  ++(v[parent].counter);
  
  return new;
  
} /* end tL_wghypmng_add_sym */


void
tL_wghypmng_insert (
		    wghypmng_t    *wghypmng,
		    size_t         hyp,
		    size_t         parent,
		    tLProb         p,
		    tLProb         hmmp,
		    tLProb         lmp,
		    unsigned long  w,
		    size_t         e
		    )
{
  
  struct wghypmng_node *v;
  struct wghhnode *v2;
  size_t aux;
  
  
  v= wghypmng->v; assert ( v[hyp].N>0 );
  v2= wghypmng->hnodes[v[hyp].ptr];
  if ( wghypmng->K > v[hyp].N )
    {
      wghhnode_add ( v2, &(v[hyp].N), parent, p, hmmp, lmp, w, e );
      ++(v[parent].counter);
    }
  else if ( p > v2->p )
    {
      aux= v2->ptr;
      if ( --(v[aux].counter) == 0 && aux != WGNULLHYP )
	tL_free_wghyp ( wghypmng, aux );
      wghhnode_pop_add ( v2, v[hyp].N, parent, p, hmmp, lmp, w, e );
      ++(v[parent].counter);
    }
  
} /* end tL_wghypmng_insert */


void
tL_wghypmng_get_wordgraph (
			   wghypmng_t       *wghypmng,
			   size_t            hyp,
			   const tLLM       *lm,
			   tLWordGraph     **wg,
			   const tLLexicon  *lexicon,
			   const tLBool      wsil,
			   const size_t      sil
			   )

{
  
  size_t nstates, nedges, nsegs;
  
  
  assert ( hyp != WGNULLHYP );
  label_states ( wghypmng, hyp, lexicon, &nstates, &nedges, &nsegs, wsil );
  if ( *wg == NULL ) *wg= tL_wordgraph_alloc ( nstates, nedges, nsegs );
  else tL_wordgraph_realloc ( *wg, nstates, nedges, nsegs );
  (*wg)->NS= nstates; (*wg)->NE= nedges; (*wg)->NSG= nsegs;
  (*wg)->gsf= lm->_gsf; (*wg)->wip= lm->_wip;
  fill_states ( wghypmng, hyp, *wg );
  fill_edges ( wghypmng, hyp, lexicon, *wg, wsil, sil );
  
} /* end tL_wghypmng_printwg */
