/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2009,2010,2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  wordnet.h - Llig un model de llenguatge en format 'wordnet'.
 *
 *  Adrià Giménez Pastor, 2009,2011.
 *
 */

#ifndef __LIBTL_WORDNET_H__
#define __LIBTL_WORDNET_H__

#include <stdio.h>
#include <zlib.h>

#include "lm_public.h"
#include "port.h"
#include "text_public.h"

/** \addtogroup LM
 *  @{
 */

/* Llig un model de llenguatge en format 'WORDNET' d'un fitxer. Se li
 * ha de passar un diccionari buit que omplirà amb el vocabulari del
 * model. Es necessari que s'especifiquen els símbols que indiquen
 * principi i final.
 */
/** Reads a language model from a Wordnet file. This function creates
 *  a new language model from a text description in Wordnet format
 *  stored in the file provided. A tLDict is provided to register the
 *  words. If the vocabulary is empty, new words are registered; else,
 *  an out-of-vocabulary word is treated as an error. The tokens used
 *  to represent the special initial and final words are required. The
 *  token used for the final word can also be used for the initial
 *  word.
 *
 *  @param from File where the text description is stored. 
 *  @param vocab The dictionary where the words are registered.
 *  @param emptyvocab Specifies if the provided dictionary is empty
 *  (new words must be registered) or not (new words are treated as
 *  errors).
 *  @param begin_sym The token for the special initial word.
 *  @param end_sym The token for the special final word.
 *  @param err Pointer to string variable. If not NULL, an error
 *  message is allocated in the variable in case of error.
 *  @return The language model, or NULL in case of error. 
 */
TL_API_FUNC(tLLM *)
tL_lm_new_from_wordnet_file (
			     gzFile         from,
			     tLDict        *vocab,
			     const tLBool   emptyvocab,
			     const char    *begin_sym,
			     const char    *end_sym,
			     char         **err
			     );

/** @} */

#endif /* __LIBTL_WORDNET_H__ */
