/*
 *  Copyright 2012,2013,2014 transLectures-UPV Team
 *  Copyright 2011 Adrià Giménez Pastor and Alfons Juan Císcar
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
/*
 *  tomix.c - Transforma un model de caràcters de tipus T a un model
 *            de caràcters equivalent de tipus mixtura de T.
 *
 *  Adrià Giménez Pastor, 2011,2012,2103.
 *
 */

#include <libTLK/TLK.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* i18n includes and defines */
#include <locale.h>
#include "intl.h"

#include "cmdl.h"
#include "get_amodel.h"
#include "write_amodel.h"

#include "sysinfo.h"




/*********/
/* TIPUS */
/*********/

struct opts
{
  
  int         verbose;
  const char *input_fn;
  const char *output_fn;
  tLBool      binary;
  
};




/************/
/* FUNCIONS */
/************/

static void
help (
      FILE       *to,
      const char *prog,
      tLBool      details
      )
{

  fprintf ( to, "%s [%s]\n\n", prog, _("options") );
  if ( !details ) return;
  fprintf ( to, "    %s:\n\n", _("Options") );
  fprintf ( to, "      -h,--help: %s\n", _("Show this help message") );
  fprintf ( to, "      -V,--version: %s\n", _("Version") );
  fprintf ( to, "      -v,--verbose: %s\n", _("Verbose mode") );
  fprintf ( to, "      -B,--binary: %s\n",
	    _("Print the model in binary format") );
  fprintf ( to, "      -i,--input <string>: %s\n",
	    _("File from which the model to be transformed will be read."
	      " Defaults to the standard input") );
  fprintf ( to, "      -o,--output <string>: %s\n",
	    _("File to which the resulting model will be written."
	      " Defaults to the standard output") );
  fprintf ( to, "\n" );

} /* end help */


static void
usage (
       int          argc,
       const char  *argv[],
       struct opts *opts
       )
{
  
  cmdl_t cmdl;
  int chelp, cversion, cbinary;
  const char *prog;
  
  
  /* Crea opcions. */
  cmdl= create_cmdl ();
  add_cmdl_counter ( cmdl, 'h', "help", &chelp );
  add_cmdl_counter ( cmdl, 'V', "version", &cversion );
  add_cmdl_counter ( cmdl, 'v', "verbose", &(opts->verbose) );
  add_cmdl_counter ( cmdl, 'B', "binary", &(cbinary) );
  add_cmdl_value ( cmdl, 'i', "input", &(opts->input_fn) );
  add_cmdl_value ( cmdl, 'o', "output", &(opts->output_fn) );
  
  /* Parseja opcions. */
  parse_cmdl ( cmdl, argc, argv, &prog );
  free_cmdl ( cmdl );

  /* Opcions. */
  if ( chelp )
    {
      help ( stdout, prog, TRUE );
      exit ( EXIT_SUCCESS );
    }
  if ( cversion )
    {
      printf ( Version );
      exit ( EXIT_SUCCESS );
    }
  opts->binary= (cbinary != 0);
  
} /* end usage */




/**********************/
/* PROGRAMA PRINCIPAL */
/**********************/

int main ( int argc, const char *argv[] )
{
  
  struct opts opts;
  tLDict *syms;
  tLAModel *amodel, *new;
  
  
  usage ( argc, argv, &opts );
  syms= tL_dict_new ();
  amodel= get_amodel ( opts.input_fn, syms, opts.verbose );
  if ( opts.verbose )
    fprintf ( stderr, _("Transforming\n") );
  if ( !strcmp ( amodel->type, "TiedStates" ) )
    new= tL_amodel_tiedstates_mixture_new ( amodel );
  else
    new= tL_amodel_mixture_new ( amodel );
  tL_amodel_free ( amodel );
  write_amodel ( opts.output_fn, new, syms, opts.binary );
  tL_amodel_free ( new );
  tL_dict_free ( syms );
  
  return EXIT_SUCCESS;
  
}
